﻿using System;
using System.Globalization;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Data;

namespace XLDLL_Financial_Demo
{
    /*
        ! You must add a reference to the XLeratorDLL/financial.dll file if it is not alreadly set!
        
        This file is located in the folder you specified during installation, 
        typically: C:\Program Files (x86)\WestClinTech LLC\XLeratorDLL-financial Trial\XLeratorDLL-financial.dll
        
    */

    using wct = XLeratorDLL_financial.XLeratorDLL_financial;
    using FinancialTypes = XLeratorDLL_financial.FinancialTypes;


    public class Test_financial
    {
        const string vbCrLf = "\r\n";
        const char vbTab = '\t';


        /* ---------------------------------------*/
        /* *** XLeratorDLL / financial            */
        /* ---------------------------------------*/

        public static void test_PRICE()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);         
            DateTime Maturity = new DateTime(2034, 6, 15);
            Double Rate = 0.025;                                    //Coupon Rate : 2.50%
            Double Yld = 0.0276;                                    //Yield : 2.76%
            Double Redemption = 100;                                //Redemption : 100
            Double Frequency = 2;                                   //Frequency : twice-yearly
            String Basis = "1";                                     //Basis : Actual/Actual
            Double result;

            result = wct.PRICE(Settlement, Maturity, Rate, Yld, Redemption, Frequency, Basis);

            Console.WriteLine("PRICE : " + result.ToString());
            File.WriteAllText("PRICE.csv", result.ToString());
        }

        public static void test_YIELD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);
            DateTime Maturity = new DateTime(2034, 6, 15);
            Double Rate = 0.025;                                    //Coupon Rate : 2.50%
            Double Yld = 0.0276;                                    //Yield : 2.76%
            Double Redemption = 100;                                //Redemption : 100
            Double Frequency = 2;                                   //Frequency : twice-yearly
            String Basis = "1";                                     //Basis : Actual/Actual
            Double result;

            result = wct.YIELD(Settlement, Maturity, Rate, Yld, Redemption, Frequency, Basis);

            Console.WriteLine("YIELD : " + result.ToString());
            File.WriteAllText("YIELD.csv", result.ToString());
        }

        public static void test_RPIFactors()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);                                    //Settlement Date : 2034-06-015
            DateTime Maturity = new DateTime(2034, 6, 15);                                     //Maturity Date : 2014-05-01
            Double Rate = 0.025;                                                               //Coupon Rate : 2.50%
            Double Price = 96.0;                                                               //Price : 96.00
            Double Yield = wct.NULL_DOUBLE;                                                    //Yield : NULL
            Double Redemption = 100;                                                           //Redemption : 100
            Double Frequency = 2;                                                              //Frequency : twice-yearly
            FinancialTypes.Basis2Type Basis = FinancialTypes.Basis2Type.Actual;                //Basis : Actual/Actual
            FinancialTypes.OutputRow_BONDRPI result;

            result = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPIFACTORS : " + "Previous Coupon Date : " + result.PrevCoup.ToShortDateString());
            Console.WriteLine("RPIFACTORS : " + "Next Coupon Date : " + result.NextCoup.ToShortDateString());
            Console.WriteLine("RPIFACTORS : " + "Accrued Days : " + result.A.ToString());
            Console.WriteLine("RPIFACTORS : " + "Days in Period : " + result.E.ToString());
            Console.WriteLine("RPIFACTORS : " + "Number of Coupons : " + result.N.ToString());
            Console.WriteLine("RPIFACTORS : " + "Coupon : " + result.C.ToString());
            Console.WriteLine("RPIFACTORS : " + "Price : " + result.P.ToString());
            Console.WriteLine("RPIFACTORS : " + "Accrued Interest : " + result.AI.ToString());
            Console.WriteLine("RPIFACTORS : " + "Yield : " + result.Y.ToString());

            File.WriteAllText("RPIFactors.csv", result.ToString());
        }

        public static void test_RPI()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);                         //Settlement Date : 2034-06-015
            DateTime Maturity = new DateTime(2034, 6, 15);
            Double Rate = 0.025;                                                    //Coupon Rate : 2.50%
            Double Price = 96.0;                                                    //Price : 96.00
            Double Yield = wct.NULL_DOUBLE;                                         //Yield : NULL
            Double Redemption = 100;                                                //Redemption : 100
            Double Frequency = 2;                                                   //Frequency : twice-yearly
            FinancialTypes.Basis2Type Basis = FinancialTypes.Basis2Type.Actual;     //Basis : Actual/Actual
            FinancialTypes.OutputRow_BONDRPI factors;

            factors = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPI : Yield : " + wct.RPI(factors.A, factors.DSC, factors.E, factors.N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString());
            File.WriteAllText("RPI.csv", "Yield : " + wct.RPI(factors.A, factors.DSC, factors.E, factors.N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Price = wct.NULL_DOUBLE;
            Yield = 0.0276;

            factors = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPI : Price : " + wct.RPI(factors.A, factors.DSC, factors.E, factors.N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption));
            File.AppendAllText("RPI.csv", vbCrLf + "Price : " + wct.RPI(factors.A, factors.DSC, factors.E, factors.N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString());
        }

        public static void test_PRICEMAT()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);       //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);        //Maturity Date : 2014-12-15
            DateTime Issue = new DateTime(2014, 7, 31);            //Issue Date : 2014-07-31
            Double Rate = 0.005;                                   //Rate : 0.5%
            Double Yield = 0.002;                                  //Yield : 0.2%
            String Basis = "3";                                    //Basis = Actual/365
            Double result;

            result = wct.PRICEMAT(Settlement, Maturity, Issue, Rate, Yield, Basis);

            Console.WriteLine("PRICEMAT : " + result.ToString());
            File.WriteAllText("PRICEMAT.csv", result.ToString());
        }

        public static void test_YIELDMAT()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);        //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);         //Maturity Date : 2014-12-15
            DateTime Issue = new DateTime(2014, 7, 31);             //Issue Date : 2014-07-31
            Double Rate = 0.005;                                    //Rate : 0.5%
            Double Price = 100.5;                                   //Yield : 0.2%
            String Basis = "3";                                     //Basis = Actual/365
            Double result;

            result = wct.YIELDMAT(Settlement, Maturity, Issue, Rate, Price, Basis);

            Console.WriteLine("YIELDMAT : " + result.ToString());
            File.WriteAllText("YIELDMAT.csv", result.ToString());
        }

        public static void test_IAMFACTORS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);                        //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);                         //Maturity Date : 2014-12-15
            DateTime Issue = new DateTime(2014, 7, 31);                             //Issue Date : 2014-07-31
            Double Rate = 0.005;                                                    //Rate : 0.5%
            Double Yield = 0.002;                                                   //Yield : 0.2%
            Double Price = wct.NULL_DOUBLE;                                         //Price : NULL
            FinancialTypes.Basis2Type Basis = FinancialTypes.Basis2Type.A365;       //Basis : A/365
            FinancialTypes.OutputRow_IAM result;

            result = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis);

            Console.WriteLine("IAMFACTORS : " + "Accrued Days : " + result.A.ToString());
            Console.WriteLine("IAMFACTORS : " + "Days in Year : " + result.B.ToString());
            Console.WriteLine("IAMFACTORS : " + "Days from Issue to Maturity : " + result.D_IM.ToString());
            Console.WriteLine("IAMFACTORS : " + "Days from Settlement to Maturity : " + result.DSM.ToString());
            Console.WriteLine("IAMFACTORS : " + "Price : " + result.P.ToString());
            Console.WriteLine("IAMFACTORS : " + "Accrued Interest : " + result.AI.ToString());
            Console.WriteLine("IAMFACTORS : " + "Yield : " + result.Y.ToString());
            Console.WriteLine("IAMFACTORS : " + "Total Interest : " + result.TI.ToString());
            Console.WriteLine("IAMFACTORS : " + "Dirty Price : " + result.DP.ToString());

            File.WriteAllText("IAMFACTORS.csv", result.ToString());
        }

        public static void test_IAM()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);                                 //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);                                  //Maturity Date : 2014-12-15
            DateTime Issue = new DateTime(2014, 7, 31);                                      //Issue Date : 2014-07-31
            Double Rate = 0.005;                                                             //Rate : 0.5%
            Double Yield = 0.002;                                                            //Yield : 0.2%
            Double Price = wct.NULL_DOUBLE;                                                  //Price : NULL
            FinancialTypes.Basis2Type Basis = FinancialTypes.Basis2Type.A365;                //Basis : A/365
            FinancialTypes.OutputRow_IAM factors;

            factors = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis);

            Console.WriteLine("IAM : Price : " + wct.IAM(factors.A, factors.B, factors.D_IM, factors.DSM, Rate, Price, Yield).ToString());
            File.WriteAllText("IAM.csv", "Price : " + wct.IAM(factors.A, factors.B, factors.D_IM, factors.DSM, Rate, Price, Yield).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Price = 100.06;
            Yield = wct.NULL_DOUBLE;

            factors = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis);

            Console.WriteLine("IAM : Yield : " + wct.IAM(factors.A, factors.B, factors.D_IM, factors.DSM, Rate, Price, Yield).ToString());
            File.AppendAllText("IAM.csv", vbCrLf + "Yield : " + wct.IAM(factors.A, factors.B, factors.D_IM, factors.DSM, Rate, Price, Yield).ToString());
        }

        public static void test_PRICEDISC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);          //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);           //Maturity Date : 2014-15-15
            Double Discount = 0.015;                                  //Discount Rate : 1.5%
            Double Redemption = 100;                                  //Redemption : 100
            String Basis = "3";                                       //Basis : A/365
            Double result;

            result = wct.PRICEDISC(Settlement, Maturity, Discount, Redemption, Basis);

            Console.WriteLine("PRICEDISC : " + result.ToString());
            File.WriteAllText("PRICEDISC.csv", result.ToString());
        }

        public static void test_YIELDDISC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);
            DateTime Maturity = new DateTime(2014, 12, 15);
            Double Price = 99.72;                                    //Discount Rate : 99.72
            Double Redemption = 100;                                 //Redemption : 100
            String Basis = "3";                                      //Basis : A/365
            Double result;

            result = wct.YIELDDISC(Settlement, Maturity, Price, Redemption, Basis);

            Console.WriteLine("YIELDDISC : " + result.ToString());
            File.WriteAllText("YIELDDISC.csv", result.ToString());
        }

        public static void test_DISC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);
            DateTime Maturity = new DateTime(2014, 12, 15);          
            Double Price = 99.72;                                    //Discount Rate : 99.72
            Double Redemption = 100;                                 //Redemption : 100
            String Basis = "3";                                      //Basis : A/365
            Double result;

            result = wct.DISC(Settlement, Maturity, Price, Redemption, Basis);

            Console.WriteLine("DISC : " + "Discount Rate : " + result.ToString());
            File.WriteAllText("DISC.csv", "Discount Rate : " + result.ToString());
        }

        public static void test_DISFACTORS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);         //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);          //Maturity Date : 2014-15-15
            Double Redemption = 100;                                 //Redemption : 100
            Double Discount = wct.NULL_DOUBLE;                       //Discount Rate : 
            Double Price = 99.72;                                    //Price : 99.72
            Double Yield = wct.NULL_DOUBLE;                          //Yield : 
            String Basis = "3";                                      //Basis : A/365
            FinancialTypes.OutputRow_DIS result;

            result = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, Yield, Basis);

            Console.WriteLine("DISFACTORS : " + "Days Settlement to Maturity : " + result.DSM.ToString());
            Console.WriteLine("DISFACTORS : " + "Days in Year : " + result.B.ToString());
            Console.WriteLine("DISFACTORS : " + "Price : " + result.P.ToString());
            Console.WriteLine("DISFACTORS : " + "Discount Rate : " + result.D.ToString());
            Console.WriteLine("DISFACTORS : " + "Yield : " + result.Y.ToString());
            Console.WriteLine("DISFACTORS : " + "Time : " + result.T.ToString());

            File.WriteAllText("DISFACTORS.csv", result.ToString());
        }

        public static void test_DIS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 7);          //Settlement Date : 2014-10-07
            DateTime Maturity = new DateTime(2014, 12, 15);           //Maturity Date : 2014-15-15                            
            Double Redemption = 100;                                  //Redemption : 100
            Double Discount = wct.NULL_DOUBLE;                        //Discount Rate : 
            Double Price = 99.72;                                     //Price : 99.72
            String Basis = "3";                                       //Basis : A/365
            FinancialTypes.OutputRow_DIS factors;

            factors = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, wct.NULL_DOUBLE, Basis);

            Console.WriteLine("DIS : " + "Discount Rate : " + wct.DIS(factors.DSM, Redemption, Price, Discount).ToString());
            File.WriteAllText("DIS.csv", "Discount Rate : " + wct.DIS(factors.DSM, Redemption, Price, Discount).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Discount = 0.015;
            Price = wct.NULL_DOUBLE;

            factors = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, wct.NULL_DOUBLE, Basis);

            Console.WriteLine("DIS : " + wct.DIS(factors.DSM, Redemption, Price, Discount).ToString());
            File.AppendAllText("DIS.csv", vbCrLf + wct.DIS(factors.DSM, Redemption, Price, Discount).ToString());
        }

        public static void test_ODDFPRICE()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);       //Settlement Date : 2034-06-015
            DateTime Maturity = new DateTime(2034, 6, 15);        //Maturity Date : 2014-05-01
            DateTime Issue = new DateTime(2014, 5, 1);            //Issue Duate : 2014-05-01
            DateTime FirstCoupon = new DateTime(2014, 6, 15);     //First Coupon Date : 2014-06-15
            Double Rate = 0.025;                                  //Coupon Rate : 2.50%
            Double Yld = 0.0276;                                  //Yield : 2.76%
            Double Redemption = 100;                              //Redemption : 100
            Double Frequency = 2;                                 //Frequency : twice-yearly
            String Basis = "1";                                   //Basis : Actual/Actual
            Double result;

            result = wct.ODDFPRICE(Settlement, Maturity, Issue, FirstCoupon, Rate, Yld, Redemption, Frequency, Basis);

            Console.WriteLine("ODDFPRICE : " + result.ToString());
            File.WriteAllText("ODDFPRICE.csv", result.ToString());
        }

        public static void test_ODDFYIELD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);
            DateTime Maturity = new DateTime(2034, 6, 15);
            DateTime Issue = new DateTime(2014, 5, 1);
            DateTime FirstCoupon = new DateTime(2014, 6, 15);
            Double Rate = 0.025;
            Double Price = 96.0076;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.ODDFYIELD(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("ODDFYIELD : " + result.ToString());
            File.WriteAllText("ODDFYIELD.csv", result.ToString());
        }

        public static void test_OFCFACTORS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);
            DateTime Maturity = new DateTime(2034, 6, 15);
            DateTime Issue = new DateTime(2014, 5, 1);
            DateTime FirstCoupon = new DateTime(2014, 6, 15);
            Double Rate = 0.025;
            Double Price = 96.0076;
            Double Yield = wct.NULL_DOUBLE;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            FinancialTypes.OutputRow_BONDOFC result;

            result = wct.OFCFACTORS(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFCFACTORS : " + "Coupon days : " + result.E.ToString());
            Console.WriteLine("OFCFACTORS : " + "Days settlement to next coupon : " + result.DSC.ToString());
            Console.WriteLine("OFCFACTORS : " + "Number of whole coupons : " + result.N.ToString());
            Console.WriteLine("OFCFACTORS : " + "Number of quasi-coupons in the odd period : " + result.NCF.ToString());
            Console.WriteLine("OFCFACTORS : " + "Coupons days in the first quasi-coupon period : " + result.A1.ToString());
            Console.WriteLine("OFCFACTORS : " + "Accrued days from issue date to the next quasi-coupon date : " + result.DFC1.ToString());
            Console.WriteLine("OFCFACTORS : " + "Normal length of the quasi-coupon period : " + result.NLF1.ToString());
            Console.WriteLine("OFCFACTORS : " + "Coupons days in the second quasi-coupon period : " + result.A2.ToString());
            Console.WriteLine("OFCFACTORS : " + "Accrued days from first quasi-coupon date to the issue date : " + result.DFC2.ToString());
            Console.WriteLine("OFCFACTORS : " + "Normal length of the period from quasi-coupon date to issue date : " + result.NLF2.ToString());
            Console.WriteLine("OFCFACTORS : " + "Number of whole coupons from the settlement date to the first coupon date : " + result.Nqf.ToString());
            Console.WriteLine("OFCFACTORS : " + "Calculated previous coupon date : " + result.quasistart.ToShortDateString());
            Console.WriteLine("OFCFACTORS : " + "Calculated next coupon date : " + result.quasicoup.ToShortDateString());
            Console.WriteLine("OFCFACTORS : " + "Coupon amount : " + result.C.ToString());
            Console.WriteLine("OFCFACTORS : " + "First coupon amount : " + result.FC.ToString());
            Console.WriteLine("OFCFACTORS : " + "Price : " + result.P.ToString());
            Console.WriteLine("OFCFACTORS : " + "Accrued interest : " + result.AI.ToString());
            Console.WriteLine("OFCFACTORS : " + "Yield : " + result.Y.ToString());

            File.WriteAllText("OFCFACTORS.csv", result.ToString());
        }

        public static void test_OFC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 5, 1);
            DateTime Maturity = new DateTime(2034, 6, 15);
            DateTime Issue = new DateTime(2014, 5, 1);
            DateTime FirstCoupon = new DateTime(2014, 6, 15);
            Double Rate = 0.025;
            Double Price = 96.0076;
            Double Yield = wct.NULL_DOUBLE;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            FinancialTypes.OutputRow_BONDOFC factors;

            factors = wct.OFCFACTORS(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFC : Yield : " + wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLF2), factors.A1, factors.DFC1, factors.NLF1, factors.A2, factors.DFC2, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.WriteAllText("OFC.csv", "Yield : " + wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLF2), factors.A1, factors.DFC1, factors.NLF1, factors.A2, factors.DFC2, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Yield = factors.Y;
            Price = wct.NULL_DOUBLE;

            Console.WriteLine("OFC : Price : " + wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLF2), factors.A1, factors.DFC1, factors.NLF1, factors.A2, factors.DFC2, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFC.csv", vbCrLf + "Price : " + wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLF2), factors.A1, factors.DFC1, factors.NLF1, factors.A2, factors.DFC2, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
        }

        public static void test_ODDLPRICE()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 1);
            DateTime Maturity = new DateTime(2014, 12, 15);
            DateTime LastInterest = new DateTime(2014, 9, 15);
            Double Rate = 0.0225;
            Double Yld = 0.001;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.ODDLPRICE(Settlement, Maturity, LastInterest, Rate, Yld, Redemption, Frequency, Basis);

            Console.WriteLine("ODDLPRICE : " + result.ToString());
            File.WriteAllText("ODDLPRICE.csv", result.ToString());
        }

        public static void test_ODDLYIELD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 1);
            DateTime Maturity = new DateTime(2014, 12, 15);
            DateTime LastInterest = new DateTime(2014, 9, 15);
            Double Rate = 0.0225;
            Double Price = 100.4453;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.ODDLYIELD(Settlement, Maturity, LastInterest, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("ODDLYIELD : " + result.ToString());
            File.WriteAllText("ODDLYIELD.csv", result.ToString());
        }

        public static void test_OLCFACTORS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 1);
            DateTime Maturity = new DateTime(2014, 12, 15);
            DateTime LastInterest = new DateTime(2014, 9, 15);
            Double Rate = 0.0225;
            Double Price = 100.4453;
            Double Yield = wct.NULL_DOUBLE;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            FinancialTypes.OutputRow_BONDOLC result;

            result = wct.OLCFACTORS(Settlement, Maturity, LastInterest, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OLCFACTORS : " + "Accrued Days : " + result.A.ToString());
            Console.WriteLine("OLCFACTORS : " + "Days from settlement to next coupon date : " + result.DSC.ToString());
            Console.WriteLine("OLCFACTORS : " + "Coupon days : " + result.E.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of coupons : " + result.N.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of quasi-coupons : " + result.NCL.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of accrued days in the first quasi-coupon period : " + result.A1.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of days in the first quasi-coupon period : " + result.DLC1.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of days from the settlement date to the quasi-coupon date : " + result.DSC1.ToString());
            Console.WriteLine("OLCFACTORS : " + "Normal length of the first quasi-coupon period : " + result.NLL1.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of accrued days in the second quasi-coupon period : " + result.A2.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of days in the second quasi-coupon period : " + result.DLC2.ToString());
            Console.WriteLine("OLCFACTORS : " + "Number of days from the quasi-cupon date to the maturity date : " + result.DSC2.ToString());
            Console.WriteLine("OLCFACTORS : " + "Normal length from quasi-coupon to maturity : " + result.NLL1.ToString());
            Console.WriteLine("OLCFACTORS : " + "Coupon : " + result.C.ToString());
            Console.WriteLine("OLCFACTORS : " + "Last coupon : " + result.LC.ToString());
            Console.WriteLine("OLCFACTORS : " + "Price : " + result.P.ToString());
            Console.WriteLine("OLCFACTORS : " + "Accrued interest : " + result.AI.ToString());
            Console.WriteLine("OLCFACTORS : " + "Yield : " + result.Y.ToString());
        
            File.WriteAllText("OLCFACTORS.csv", result.ToString());
        }

        public static void test_OLC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 1);
            DateTime Maturity = new DateTime(2014, 12, 15);
            DateTime LastInterest = new DateTime(2014, 9, 15);
            Double Rate = 0.0225;
            Double Price = 100.4453;
            Double Yield = wct.NULL_DOUBLE;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "1";
            FinancialTypes.OutputRow_BONDOLC factors;

            factors = wct.OLCFACTORS(Settlement, Maturity, LastInterest, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OLC : Yield : " + wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A, factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLL2), factors.A1, factors.DSC1, factors.DLC1, factors.NLL1, factors.A2, factors.DSC2, factors.DLC2, factors.NLL2).ToString());
            File.WriteAllText("OLC.csv", "Yield : " + wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A, factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLL2), factors.A1, factors.DSC1, factors.DLC1, factors.NLL1, factors.A2, factors.DSC2, factors.DLC2, factors.NLL2).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Yield = factors.Y;
            Price = wct.NULL_DOUBLE;

            Console.WriteLine("OLC : Price : " + wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A, factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLL2), factors.A1, factors.DSC1, factors.DLC1, factors.NLL1, factors.A2, factors.DSC2, factors.DLC2, factors.NLL2).ToString());
            File.AppendAllText("OLC.csv", vbCrLf + "Price : " + wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A, factors.E, factors.DSC, factors.N, wct.IsNull(factors.NLL2), factors.A1, factors.DSC1, factors.DLC1, factors.NLL1, factors.A2, factors.DSC2, factors.DLC2, factors.NLL2).ToString());
        }

        public static void test_PRICESTEP()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2013, 4, 15);
            DateTime Maturity = new DateTime(2019, 1, 15);
            Double Yield = 0.05;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "0";
            DateTime[] StepDates = new DateTime[3]; 
            StepDates[0] = new DateTime(2010, 1, 15);
            StepDates[1] = new DateTime(2013, 1, 15);
            StepDates[2] = new DateTime(2016, 1, 15);
            Double[] StepRates = { 0.05, 0.055, 0.06 };
            Double result;

            result = wct.PRICESTEP(Settlement, Maturity, Yield, Redemption, Frequency, Basis, StepDates, StepRates);

            Console.WriteLine("PRICESTEP : " + result.ToString());
            File.WriteAllText("PRICESTEP.csv", result.ToString());
        }

        public static void test_YIELDSTEP()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2013, 4, 15);
            DateTime Maturity = new DateTime(2019, 1, 15);
            Double Price = 103.6657;
            Double Redemption = 100;
            Double Frequency = 2;
            String Basis = "0";
            DateTime[] StepDates = new DateTime[3]; 
            StepDates[0] = new DateTime(2010, 1, 15);
            StepDates[1] = new DateTime(2013, 1, 15);
            StepDates[2] = new DateTime(2016, 1, 15);
            Double[] StepRates = { 0.05, 0.055, 0.06 };
            Double result;

            result = wct.YIELDSTEP(Settlement, Maturity, Price, Redemption, Frequency, Basis, StepDates, StepRates);

            Console.WriteLine("YIELDSTEP : " + result.ToString());
            File.WriteAllText("YIELDSTEP.csv", result.ToString());
        }

        public static void test_TBILLPRICE()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2016, 4, 1);
            DateTime Maturity = Settlement.AddDays(91);
            Double Discount = 0.001;
            Double result;

            result = wct.TBILLPRICE(Settlement, Maturity, Discount);

            Console.WriteLine("TBILLPRICE : " + result.ToString());
            File.WriteAllText("TBILLPRICE.csv", result.ToString());
        }

        public static void test_TBILLYIELD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2016, 4, 1);
            DateTime Maturity = Settlement.AddDays(91);
            Double Price = 99.9747;
            Double result;

            result = wct.TBILLYIELD(Settlement, Maturity, Price);

            Console.WriteLine("TBILLYIELD : " + result.ToString());
            File.WriteAllText("TBILLYIELD.csv", result.ToString());
        }

        public static void test_TBILLEQ()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2016, 4, 1);
            DateTime Maturity = Settlement.AddDays(91);
            Double Disc = 0.0028;
            Double result;

            result = wct.TBILLEQ(Settlement, Maturity, Disc);

            Console.WriteLine("TBILLEQ : " + "BOND EQUIVALENT YIELD : " + result.ToString());
            File.WriteAllText("TBILLEQ.csv", "BOND EQUIVALENT YIELD : " + result.ToString());
        }

        public static void test_PRICEACT()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 29);
            DateTime Maturity = new DateTime(2034, 1, 11);
            Double Rate = 0.11;
            Double Par = 100;
            Double Yield = 0.125;
            Double Frequency = 2;
            String Basis = "1";
            DateTime[] RepaymentDates; 
            Double[] RepaymentAmounts; 
            Double result;

            //with no repayment schedule
            result = wct.PRICEACT(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis);

            Console.WriteLine("PRICEACT : " + result.ToString());
            File.WriteAllText("PRICEACT.csv", result.ToString()); ;

            //=====================
            // EXAMPLE# 2
            //=====================

            //with a repayment schedule
            RepaymentDates = new DateTime[] { new DateTime(2012, 4, 30), new DateTime(2012, 10, 31), new DateTime(2013, 4, 30), new DateTime(2013, 10, 31), new DateTime(2014, 4, 30), new DateTime(2014, 10, 31), new DateTime(2015, 4, 30), new DateTime(2015, 10, 31), new DateTime(2016, 4, 30), new DateTime(2016, 10, 31), new DateTime(2017, 4, 30), new DateTime(2017, 10, 31), new DateTime(2018, 4, 30), new DateTime(2018, 10, 31), new DateTime(2019, 4, 30), new DateTime(2019, 10, 31) };
            RepaymentAmounts = new Double[] { 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25 }; ;
            Rate = 0.125;

            result = wct.PRICEACT(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts);

            Console.WriteLine("PRICEACT : " + result.ToString());
            File.WriteAllText("PRICEACT.csv", result.ToString());
        }

        public static void test_YIELDACT()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 29);
            DateTime Maturity = new DateTime(2014, 1, 11);
            Double Rate = 0.11;
            Double Par = 100;
            Double Price = 89.05835;
            Double Frequency = 2;
            String Basis = "1";
            DateTime[] RepaymentDates;
            Double[] RepaymentAmounts;
            Double result;

            //with no repayment schedule
            result = wct.YIELDACT(Settlement, Maturity, Rate, Par, Price, Frequency, Basis);

            Console.WriteLine("YIELDACT : " + result.ToString());
            File.WriteAllText("YIELDACT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //with a repayment schedule
            RepaymentDates = new DateTime[] { new DateTime(2012, 4, 30), new DateTime(2012, 10, 31), new DateTime(2013, 4, 30), new DateTime(2013, 10, 31), new DateTime(2014, 4, 30), new DateTime(2014, 10, 31), new DateTime(2015, 4, 30), new DateTime(2015, 10, 31), new DateTime(2016, 4, 30), new DateTime(2016, 10, 31), new DateTime(2017, 4, 30), new DateTime(2017, 10, 31), new DateTime(2018, 4, 30), new DateTime(2018, 10, 31), new DateTime(2019, 4, 30), new DateTime(2019, 10, 31) };
            RepaymentAmounts = new Double[] { 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25 };
            Rate = 0.125;
            Price = 99.9985;

            result = wct.YIELDACT(Settlement, Maturity, Rate, Par, Price, Frequency, Basis, RepaymentDates, RepaymentAmounts);

            Console.WriteLine("YIELDACT : " + result.ToString());
            File.WriteAllText("YIELDACT.csv", result.ToString());
        }

        public static void test_PRICEACTTV()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Par;
            Double Yield;
            Double Frequency;
            String Basis;
            DateTime[] RepaymentDates;
            Double[] RepaymentAmounts;
            FinancialTypes.PRICEACTTV_table result;

            //with no repayment schedule
            Settlement = new DateTime(2014, 10, 29);
            Maturity = new DateTime(2034, 11, 1);
            Rate = 0.11;
            Par = 100;
            Yield = 0.125;
            Frequency = 2;
            Basis = "1";

            result = wct.PRICEACTTV(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis);

            //Write the table output to a csv
            StringBuilder retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("PRICEACTTV : ex1 : " + retStr1.ToString());
            File.WriteAllText("PRICEACTTV.csv", "ex1 : " + retStr1.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //with a repayment schedule
            Settlement = new DateTime(2014, 10, 29);
            Maturity = new DateTime(2019, 10, 31);
            Rate = 0.125;
            Par = 100;
            Yield = 0.125;
            Frequency = 2;
            Basis = "1";
            RepaymentDates = new DateTime[] { new DateTime(2012, 4, 30), new DateTime(2012, 10, 31), new DateTime(2013, 4, 30), new DateTime(2013, 10, 31), new DateTime(2014, 4, 30), new DateTime(2014, 10, 31), new DateTime(2015, 4, 30), new DateTime(2015, 10, 31), new DateTime(2016, 4, 30), new DateTime(2016, 10, 31), new DateTime(2017, 4, 30), new DateTime(2017, 10, 31), new DateTime(2018, 4, 30), new DateTime(2018, 10, 31), new DateTime(2019, 4, 30), new DateTime(2019, 10, 31) };
            RepaymentAmounts = new Double[] { 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25 };

            result = wct.PRICEACTTV(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts);

            //Write the table output to a csv
            StringBuilder retStr2 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr2.Append(col.ColumnName + ",");
            }
            retStr2.Remove(retStr2.Length - 1, 1);
            retStr2.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr2.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr2.Append(",");
                    }
                    else
                    {
                        retStr2.AppendLine();

                    }
                }
            }
            Console.WriteLine("PRICEACTTV : ex2 : " + retStr2.ToString());
            File.AppendAllText("PRICEACTTV.csv", vbCrLf + "ex2 : " + retStr2.ToString());
        }

        public static void test_PRICEFR()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 16);
            DateTime Maturity = new DateTime(2034, 11, 1);
            Double Rate = 0.0475;
            Double Par = 100;
            Double Yield = 0.04;
            Double Frequency = 2;
            String Basis = "1";
            DateTime[] RepaymentDates;
            Double[] RepaymentAmounts;
            Double result;

            //repayment schedule
            RepaymentDates = new DateTime[] { new DateTime(2026, 11, 1), new DateTime(2028, 11, 1), new DateTime(2030, 11, 1), new DateTime(2032, 11, 1) };
            RepaymentAmounts = new Double[] { 20, 20, 20, 20 };

            result = wct.PRICEFR(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts);

            Console.WriteLine("PRICEFR : " + result.ToString());
            File.WriteAllText("PRICEFR.csv", result.ToString());
        }

        public static void test_YIELDFR()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2014, 10, 16);
            DateTime Maturity = new DateTime(2034, 11, 1);
            Double Rate = 0.0475;
            Double Par = 100;
            Double Price = 108.753442;
            Double Frequency = 2;
            String Basis = "1";
            DateTime[] RepaymentDates;
            Double[] RepaymentAmounts;
            Double result;

            //repayment schedule
            RepaymentDates = new DateTime[] { new DateTime(2026, 11, 1), new DateTime(2028, 11, 1), new DateTime(2030, 11, 1), new DateTime(2032, 11, 1) };
            RepaymentAmounts = new Double[] { 20, 20, 20, 20 };

            result = wct.YIELDFR(Settlement, Maturity, Rate, Par, Price, Frequency, Basis, RepaymentDates, RepaymentAmounts);

            Console.WriteLine("YIELDFR : " + result.ToString());
            File.WriteAllText("YIELDFR.csv", result.ToString());
        }

        public static void test_DIRTYPRICE()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            double Frequency;
            double Redemption;
            string Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Regular coupons
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : RPI : " + result.ToString());   //96.9453139716364
            File.WriteAllText("DIRTYPRICE.csv", "RPI : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Odd Short First
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2014, 5, 1);
            FirstCoupon = new DateTime(2014, 6, 15);

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : Odd Short First : " + result.ToString());   //96.0075631077824
            File.AppendAllText("DIRTYPRICE.csv", vbCrLf + "Odd Short First : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Odd Long First
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2014, 5, 1);
            FirstCoupon = new DateTime(2014, 12, 15);

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : Odd Long First : " + result.ToString());   //96.0033702877755
            File.AppendAllText("DIRTYPRICE.csv", vbCrLf + "Odd Long First : " + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Odd Short Last
            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2014, 12, 15);
            Rate = 0.0225;
            Yield = 0.001;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = new DateTime(2014, 9, 15);

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : Odd Short Last : " + result.ToString());                     //100.544776634675
            File.AppendAllText("DIRTYPRICE.csv", vbCrLf + "Odd Short Last : " + result.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Odd Long Last
            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2014, 12, 15);
            Rate = 0.0225;
            Yield = 0.001;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = new DateTime(2014, 3, 15);

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : Odd Long Last : " + result.ToString());                     //101.669543602845
            File.AppendAllText("DIRTYPRICE.csv", vbCrLf + "Odd Long Last : " + result.ToString());

            //=====================
            // EXAMPLE# 6
            //=====================

            //Odd long first & odd long last
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);

            result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYPRICE : Odd Long First & Last : " + result.ToString());             //104.420159644839
            File.AppendAllText("DIRTYPRICE.csv", vbCrLf + "Odd Long First & Last : " + result.ToString());
        }

        public static void test_DIRTYYIELD()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Price;
            double Frequency;
            double Redemption;
            string Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Regular coupons
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Price = 96.9453139716364;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : RPI : " + result.ToString());   //96.9453139716364
            File.WriteAllText("DIRTYYIELD.csv", "RPI : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Odd Short First
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Price = 96.0075631077824;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2014, 5, 1);
            FirstCoupon = new DateTime(2014, 6, 15);

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : Odd Short First : " + result.ToString());   //96.0075631077824
            File.AppendAllText("DIRTYYIELD.csv", vbCrLf + "Odd Short First : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Odd Long First
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Price = 96.0033702877755;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2014, 5, 1);
            FirstCoupon = new DateTime(2014, 12, 15);

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : Odd Long First : " + result.ToString());   //96.0033702877755
            File.AppendAllText("DIRTYYIELD.csv", vbCrLf + "Odd Long First : " + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Odd Short Last
            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2014, 12, 15);
            Rate = 0.0225;
            Price = 100.544776634675;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = new DateTime(2014, 9, 15);

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : Odd Short Last : " + result.ToString());   //100.544776634675
            File.AppendAllText("DIRTYYIELD.csv", vbCrLf + "Odd Short Last : " + result.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Odd Long Last
            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2014, 12, 15);
            Rate = 0.0225;
            Price = 101.669543602845;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = new DateTime(2014, 3, 15);

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : Odd Long Last : " + result.ToString());    //101.669543602845
            File.AppendAllText("DIRTYYIELD.csv", vbCrLf + "Odd Long Last : " + result.ToString());

            //=====================
            // EXAMPLE# 6
            //=====================

            //Odd long first & odd long last
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Rate = 0.03125;
            Price = 104.420159644839;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);

            result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            Console.WriteLine("DIRTYYIELD : Odd Long First & Last : " + result.ToString());
            File.AppendAllText("DIRTYYIELD.csv", vbCrLf + "Odd Long First & Last : " + result.ToString());
        }

        public static void test_OFLPRICE()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a bond with an odd short first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLPRICE : OSFOSL : " + result.ToString()); //102.00036224598
            File.WriteAllText("OFLPRICE.csv", "OSFOSL : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This is a bond with an odd long first coupon and an odd long last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLPRICE : OLFOLL : " + result.ToString()); //102.103433425767
            File.AppendAllText("OFLPRICE.csv", vbCrLf + "OLFOLL : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //This is a bond with an odd long first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLPRICE : OLFOSL : " + result.ToString());   //102.104790915433
            File.AppendAllText("OFLPRICE.csv", vbCrLf + "OLFOSL : " + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //This is an example of a bond paying interest every 26 weeks.
            Settlement = new DateTime(2014, 10, 4);
            Maturity = new DateTime(2029, 12, 12);
            Issue = new DateTime(2014, 7, 30);
            FirstCoupon = new DateTime(2015, 3, 18);
            LastCoupon = new DateTime(2029, 2, 28);
            Rate = 0.125;
            Yield = 0.11;
            Redemption = 100;
            Frequency = 182;
            Basis = "9";

            result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLPRICE : RPI : " + result.ToString());
            File.AppendAllText("OFLPRICE.csv", vbCrLf + "RPI : " + result.ToString());
        }

        public static void test_OFLYIELD()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            Double Rate;
            Double Price;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a bond with an odd short first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 102.00036224598;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("OFLYIELD : OSFOSL : " + result.ToString());      //0.02875
            File.WriteAllText("OFLYIELD.csv", "OSFOSL : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This is a bond with an odd long first coupon and an odd long last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 102.103433425767;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("OFLYIELD : OLFOLL : " + result.ToString());      //0.02875
            File.AppendAllText("OFLYIELD.csv", vbCrLf + "OLFOLL : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //This is a bond with an odd long first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 102.104790915433;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("OFLYIELD : OLFOSL : " + result.ToString());   //0.02875
            File.AppendAllText("OFLYIELD.csv", vbCrLf + "OLFOSL : " + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //This is an example of a bond paying interest every 26 weeks.
            Settlement = new DateTime(2014, 10, 4);
            Maturity = new DateTime(2029, 12, 12);
            Issue = new DateTime(2014, 7, 30);
            FirstCoupon = new DateTime(2015, 3, 18);
            LastCoupon = new DateTime(2029, 2, 28);
            Rate = 0.125;
            Price = 110.846098828263;
            Redemption = 100;
            Frequency = 182;
            Basis = "9";

            result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis);

            Console.WriteLine("OFLYIELD : RPI : " + result.ToString());   //0.11
            File.AppendAllText("OFLYIELD.csv", vbCrLf + "RPI : " + result.ToString());
        }

        public static void test_OFLFACTORS()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Price;
            Double Yield;
            Double Frequency;
            Double Redemption;
            String Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            FinancialTypes.OutputRow_BONDOFL factors;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a bond with an odd short first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = wct.NULL_DOUBLE;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLFACTORS : OSFOSL A1 : " + factors.A1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL A2 : " + factors.A2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL DSC : " + factors.DSC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL E : " + factors.E.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL N " + factors.N.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NCL : " + factors.NCL.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NCF : " + factors.NCF.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL DLC1 : " + factors.DLC1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL DLC2 : " + factors.DLC2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NLL1 : " + factors.NLL1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NLL2 : " + factors.NLL2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL DFC1 : " + factors.DFC1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL DFC2 : " + factors.DFC2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NLF1 " + factors.NLF1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL NLF2 : " + factors.NLF2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL Nqf : " + factors.Nqf.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL quasistart : " + factors.quasistart.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL quasicoupfirst : " + factors.quasicoupfirst.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL quasicouplast : " + factors.quasicouplast.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL quasimaturity : " + factors.quasimaturity.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL C : " + factors.C.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL LC : " + factors.LC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL FC : " + factors.FC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL P : " + factors.P.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL AI : " + factors.AI.ToString());
            Console.WriteLine("OFLFACTORS : OSFOSL Y : " + factors.Y.ToString());

            File.WriteAllText("OFLFACTORS.csv", "OSFOSL : " + factors.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This is a bond with an odd long first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 101.999004756314;
            Yield = wct.NULL_DOUBLE;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLFACTORS : OLFOSL A1 : " + factors.A1.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL A2 : " + factors.A2.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL DSC : " + factors.DSC.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL E : " + factors.E.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL N " + factors.N.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NCL : " + factors.NCL.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NCF : " + factors.NCF.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL DLC1 : " + factors.DLC1.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL DLC2 : " + factors.DLC2.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NLL1 : " + factors.NLL1.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NLL2 : " + factors.NLL2.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL DFC1 : " + factors.DFC1.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL DFC2 : " + factors.DFC2.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NLF1 " + factors.NLF1.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL NLF2 : " + factors.NLF2.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL Nqf : " + factors.Nqf.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL quasistart : " + factors.quasistart.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL quasicoupfirst : " + factors.quasicoupfirst.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL quasicouplast : " + factors.quasicouplast.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL quasimaturity : " + factors.quasimaturity.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL C : " + factors.C.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL LC : " + factors.LC.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL FC : " + factors.FC.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL P : " + factors.P.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL AI : " + factors.AI.ToString());
            Console.WriteLine("OFLFACTORS : OLFOSL Y : " + factors.Y.ToString());

            File.AppendAllText("OFLFACTORS.csv" , vbCrLf + "OLFOSL : " + factors.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //This is a bond with an odd short first coupon and an odd long last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 102.104790915433;
            Yield = wct.NULL_DOUBLE;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLFACTORS : OSFOLL A1 : " + factors.A1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL A2 : " + factors.A2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL DSC : " + factors.DSC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL E : " + factors.E.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL N " + factors.N.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NCL : " + factors.NCL.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NCF : " + factors.NCF.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL DLC1 : " + factors.DLC1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL DLC2 : " + factors.DLC2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NLL1 : " + factors.NLL1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NLL2 : " + factors.NLL2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL DFC1 : " + factors.DFC1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL DFC2 : " + factors.DFC2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NLF1 " + factors.NLF1.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL NLF2 : " + factors.NLF2.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL Nqf : " + factors.Nqf.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL quasistart : " + factors.quasistart.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL quasicoupfirst : " + factors.quasicoupfirst.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL quasicouplast : " + factors.quasicouplast.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL quasimaturity : " + factors.quasimaturity.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL C : " + factors.C.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL LC : " + factors.LC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL FC : " + factors.FC.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL P : " + factors.P.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL AI : " + factors.AI.ToString());
            Console.WriteLine("OFLFACTORS : OSFOLL Y : " + factors.Y.ToString());

            File.AppendAllText("OFLFACTORS.csv" , vbCrLf + "OSFOLL : " + factors.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //This is an example of a bond paying interest every 26 weeks.
            Settlement = new DateTime(2014, 10, 4);
            Maturity = new DateTime(2029, 12, 12);
            Issue = new DateTime(2014, 7, 30);
            FirstCoupon = new DateTime(2015, 3, 18);
            LastCoupon = new DateTime(2029, 2, 28);
            Rate = 0.125;
            Price = wct.NULL_DOUBLE;
            Yield = 0.11;
            Redemption = 100;
            Frequency = 182;
            Basis = "9";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLFACTORS : RPI A1 : " + factors.A1.ToString());
            Console.WriteLine("OFLFACTORS : RPI A2 : " + factors.A2.ToString());
            Console.WriteLine("OFLFACTORS : RPI DSC : " + factors.DSC.ToString());
            Console.WriteLine("OFLFACTORS : RPI E : " + factors.E.ToString());
            Console.WriteLine("OFLFACTORS : RPI N " + factors.N.ToString());
            Console.WriteLine("OFLFACTORS : RPI NCL : " + factors.NCL.ToString());
            Console.WriteLine("OFLFACTORS : RPI NCF : " + factors.NCF.ToString());
            Console.WriteLine("OFLFACTORS : RPI DLC1 : " + factors.DLC1.ToString());
            Console.WriteLine("OFLFACTORS : RPI DLC2 : " + factors.DLC2.ToString());
            Console.WriteLine("OFLFACTORS : RPI NLL1 : " + factors.NLL1.ToString());
            Console.WriteLine("OFLFACTORS : RPI NLL2 : " + factors.NLL2.ToString());
            Console.WriteLine("OFLFACTORS : RPI DFC1 : " + factors.DFC1.ToString());
            Console.WriteLine("OFLFACTORS : RPI DFC2 : " + factors.DFC2.ToString());
            Console.WriteLine("OFLFACTORS : RPI NLF1 " + factors.NLF1.ToString());
            Console.WriteLine("OFLFACTORS : RPI NLF2 : " + factors.NLF2.ToString());
            Console.WriteLine("OFLFACTORS : RPI Nqf : " + factors.Nqf.ToString());
            Console.WriteLine("OFLFACTORS : RPI quasistart : " + factors.quasistart.ToString());
            Console.WriteLine("OFLFACTORS : RPI quasicoupfirst : " + factors.quasicoupfirst.ToString());
            Console.WriteLine("OFLFACTORS : RPI quasicouplast : " + factors.quasicouplast.ToString());
            Console.WriteLine("OFLFACTORS : RPI quasimaturity : " + factors.quasimaturity.ToString());
            Console.WriteLine("OFLFACTORS : RPI C : " + factors.C.ToString());
            Console.WriteLine("OFLFACTORS : RPI LC : " + factors.LC.ToString());
            Console.WriteLine("OFLFACTORS : RPI FC : " + factors.FC.ToString());
            Console.WriteLine("OFLFACTORS : RPI P : " + factors.P.ToString());
            Console.WriteLine("OFLFACTORS : RPI AI : " + factors.AI.ToString());
            Console.WriteLine("OFLFACTORS : RPI Y : " + factors.Y.ToString());

            File.AppendAllText("OFLFACTORS.csv", vbCrLf + "RPI : " + factors.ToString());
        }

        public static void test_OFL()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Price;
            Double Yield;
            Double Frequency;
            Double Redemption;
            String Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            FinancialTypes.OutputRow_BONDOFL factors;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a bond with an odd short first coupon and an odd short last coupon.

            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = wct.NULL_DOUBLE;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFL : OSFOSL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.WriteAllText("OFL.csv" , "OSFOSL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            Price = factors.P;
            Yield = wct.NULL_DOUBLE;

            Console.WriteLine("OFL : OSFOSL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv", vbCrLf + "OSFOSL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This is a bond with an odd long first coupon and an odd short last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            Issue = new DateTime(2012, 6, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 101.999004756314;
            Yield = wct.NULL_DOUBLE;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFL : OLFOSL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv" , vbCrLf + "OLFOSL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            Yield = factors.Y;
            Price = wct.NULL_DOUBLE;

            Console.WriteLine("OFL : OLFOSL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv", vbCrLf + "OLFOSL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //This is a bond with an odd short first coupon and an odd long last coupon.
            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 11, 28);
            Issue = new DateTime(2012, 12, 7);
            FirstCoupon = new DateTime(2013, 3, 15);
            LastCoupon = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Price = 102.104790915433;
            Yield = wct.NULL_DOUBLE;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFL : OSFOLL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv" , vbCrLf + "OSFOLL Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            Yield = factors.Y;
            Price = wct.NULL_DOUBLE;

            Console.WriteLine("OFL : OSFOLL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv", vbCrLf + "OSFOLL Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //This is an example of a bond paying interest every 26 weeks.
            Settlement = new DateTime(2014, 10, 4);
            Maturity = new DateTime(2029, 12, 12);
            Issue = new DateTime(2014, 7, 30);
            FirstCoupon = new DateTime(2015, 3, 18);
            LastCoupon = new DateTime(2029, 2, 28);
            Rate = 0.125;
            Price = wct.NULL_DOUBLE;
            Yield = 0.11;
            Redemption = 100;
            Frequency = 182;
            Basis = "9";

            factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFL : RPI Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv" , vbCrLf + "RPI Price : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());

            Price = factors.P;
            Yield = wct.NULL_DOUBLE;

            Console.WriteLine("OFL : RPI Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
            File.AppendAllText("OFL.csv" , vbCrLf + "RPI Yield : " + wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), factors.A1, factors.A2, factors.DSC, factors.E, factors.N, (factors.NCF == 1), (factors.NCL == 1), factors.DLC1, factors.DLC2, factors.NLL1, factors.NLL2, factors.DFC1, factors.DFC2, factors.NLF1, factors.NLF2, Convert.ToInt32(factors.Nqf)).ToString());
        }

        public static void test_BONDCF()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            FinancialTypes.BONDCF_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            /*
             In this example we generate the cash flows for bond which pays regular period interest and is maturing on 2034-06-15. 
             The settlement date is 2014-05-01, the yield is 2.76%, the coupon rate is 2.50%, the redemption value is 100, 
             the coupon is paid twice-yearly and the day-count convention is actual/actual.
             */
            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            result = wct.BONDCF(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            //Write the table output to a csv
            StringBuilder retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("BONDCF : " + retStr1.ToString());
            File.WriteAllText("BONDCF.csv", retStr1.ToString());
        }

        public static void test_STEPCF()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime Issue;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            System.Array Steps;
            FinancialTypes.BONDCF_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            /*
             In this example we generate the cash flows for bond maturing on 2034-06-15. The settlement date is 2014-05-01, the yield is 2.76%. 
             The coupon is paid twice-yearly, the redemption value is 100 and the day-count convention is actual/actual. 
             The step-rate schedule is passed in using a derived table.
            */

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);

            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            Steps = new object[,] {   
                                    {new DateTime(2029,12,15) , new DateTime(2024,12,15), new DateTime(2020,12,15), new DateTime(2016,12,15), new DateTime(2012,12,15)},
                                    {0.015,                     0.014,                    0.013,                    0.012,                    0.011 }
                                  };

            result = wct.STEPCF(Settlement, Maturity, Steps, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            //Write the table output to a csv
            StringBuilder retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("STEPCF : " + retStr1.ToString());
            File.WriteAllText("STEPCF.csv", retStr1.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            /*
             In this example we use the same data but use two typed arrays instead of 2-dimnsional object array.
            */

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            DateTime[] Steps_date = {new DateTime(2029,12,15) , new DateTime(2024,12,15), new DateTime(2020,12,15), new DateTime(2016,12,15), new DateTime(2012,12,15)};
            Double[] Steps_rate = {0.015, 0.014, 0.013, 0.012, 0.011};

            result = wct.STEPCF(Settlement, Maturity, Steps_date, Steps_rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            //Write the table output to a csv
            retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("STEPCF : " + retStr1.ToString());
            File.WriteAllText("STEPCF.csv", retStr1.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            /*
             In this example, we have a bond with zero first step. We use a ado datatable which typically would be populated with input data queried from a database.
            */

            //SELECT
            //   *
            //FROM
            //   wct.STEPCF(
            //        '2016-07-01' --@Settlement
            //       ,'2044-06-15' --@Maturity
            //       ,'SELECT * FROM #t' --@Step
            //       ,0.0290       --@Yield
            //       ,100          --@Redemption
            //       ,2            --@Frequency
            //       ,1            --@Basis
            //       ,NULL         --@Issue
            //       ,NULL         --@FirstCoupon
            //       ,NULL         --@LastCoupon
            //       )
            Settlement = new DateTime(2016, 7, 1);
            Maturity = new DateTime(2044, 6, 15);
            Yield = 0.0290;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";
            Issue = wct.NULL_DATE;
            FirstCoupon = wct.NULL_DATE;
            LastCoupon = wct.NULL_DATE;

            //SELECT
            //   *
            //INTO
            //   #t
            //FROM (VALUES
            //   ('2040-12-15', .018),
            //   ('2036-12-15', .017),
            //   ('2032-12-15', .016),
            //   ('2028-12-15', .015),
            //   ('2024-12-15', .014)
            //   )n(date_step, rate_step)
            DataTable Steps_data = new DataTable("Steps_data");
            Steps_data.Columns.Add("date_step", (new DateTime()).GetType());
            Steps_data.Columns.Add("rate_step", (new double()).GetType());
            Steps_data.Rows.Add(new DateTime(2040, 12, 15), 0.018);
            Steps_data.Rows.Add(new DateTime(2036, 12, 15), 0.017);
            Steps_data.Rows.Add(new DateTime(2032, 12, 15), 0.016);
            Steps_data.Rows.Add(new DateTime(2028, 12, 15), 0.015);
            Steps_data.Rows.Add(new DateTime(2024, 12, 15), 0.014);

            result = wct.STEPCF(Settlement, Maturity, Steps_data, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon);

            //Write the table output to a csv
            retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("STEPCF : " + retStr1.ToString());
            File.WriteAllText("STEPCF.csv", retStr1.ToString());
        }

        public static void test_COUPDAYBS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.COUPDAYBS(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPDAYBS : " + result.ToString());
            File.WriteAllText("COUPDAYBS.csv", result.ToString());
        }

        public static void test_COUPDAYS()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.COUPDAYS(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPDAYS : " + result.ToString());
            File.WriteAllText("COUPDAYS.csv", result.ToString());
        }

        public static void test_COUPDAYSNC()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.COUPDAYSNC(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPDAYSNC : " + result.ToString());
            File.WriteAllText("COUPDAYSNC.csv", result.ToString());
        }

        public static void test_COUPNUM()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            Double result;

            result = wct.COUPNUM(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPNUM : " + result.ToString());
            File.WriteAllText("COUPNUM.csv", result.ToString());
        }

        public static void test_COUPNCD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            DateTime result;

            result = wct.COUPNCD(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPNCD : " + result.ToShortDateString());
            File.WriteAllText("COUPNCD.csv", result.ToShortDateString());
        }

        public static void test_COUPPCD()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            DateTime Settlement = new DateTime(2007, 12, 16);
            DateTime Maturity = new DateTime(2027, 3, 1);
            Double Frequency = 2;
            String Basis = "1";
            DateTime result;

            result = wct.COUPPCD(Settlement, Maturity, Frequency, Basis);

            Console.WriteLine("COUPPCD : " + result.ToShortDateString());
            File.WriteAllText("COUPPCD.csv", result.ToShortDateString());
        }

        public static void test_CFCONVEXITY()
        {
            Double Disc_rate;
            DateTime VDate;
            Double[] CFAmt;
            DateTime[] CFDate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Disc_rate = 0.050602946;
            VDate = new DateTime(2012, 12, 21);
            CFDate = new DateTime[] { new DateTime(2013, 5, 15), new DateTime(2013, 11, 15), new DateTime(2014, 5, 15), new DateTime(2014, 11, 15), new DateTime(2015, 5, 15), new DateTime(2015, 11, 15), new DateTime(2016, 5, 15), new DateTime(2016, 11, 15), new DateTime(2017, 5, 15), new DateTime(2017, 11, 15) };
            CFAmt = new Double[] { 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5 };

            result = wct.CFCONVEXITY(Disc_rate, VDate, CFAmt, CFDate);

            Console.WriteLine("CFCOVNEXITY : " + result.ToString());     //22.6933974996726
            File.WriteAllText("CFCOVNEXITY.csv", result.ToString());
        }

        public static void test_CFDURATION()
        {
            //=====================
            // EXAMPLE# 1
            //=====================

            Double Disc_rate;
            DateTime VDate;
            Double[] CFAmt;
            DateTime[] CFDate;
            Double result;

            Disc_rate = 0.050602946;
            VDate = new DateTime(2012, 12, 21);
            CFDate = new DateTime[] { new DateTime(2013, 5, 15), new DateTime(2013, 11, 15), new DateTime(2014, 5, 15), new DateTime(2014, 11, 15), new DateTime(2015, 5, 15), new DateTime(2015, 11, 15), new DateTime(2016, 5, 15), new DateTime(2016, 11, 15), new DateTime(2017, 5, 15), new DateTime(2017, 11, 15) };
            CFAmt = new Double[] { 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5 };

            result = wct.CFDURATION(Disc_rate, VDate, CFAmt, CFDate);

            Console.WriteLine("CFDURATION : " + result.ToString());      //4.38864089754494
            File.WriteAllText("CFDURATION.csv", result.ToString());
        }

        public static void test_CFMDURATION()
        {
            Double Disc_rate;
            DateTime VDate;
            Double[] CFAmt;
            DateTime[] CFDate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Disc_rate = 0.050602946;
            VDate = new DateTime(2012, 12, 21);
            CFDate = new DateTime[] { new DateTime(2013, 5, 15), new DateTime(2013, 11, 15), new DateTime(2014, 5, 15), new DateTime(2014, 11, 15), new DateTime(2015, 5, 15), new DateTime(2015, 11, 15), new DateTime(2016, 5, 15), new DateTime(2016, 11, 15), new DateTime(2017, 5, 15), new DateTime(2017, 11, 15) };
            CFAmt = new Double[] { 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5 };

            result = wct.CFMDURATION(Disc_rate, VDate, CFAmt, CFDate);

            Console.WriteLine("CFMDURATION : " + result.ToString());      //4.17725927216746
            File.WriteAllText("CFMDURATION.csv", result.ToString());
        }

        public static void test_CONVEXITY()
        {

            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            int Frequency;
            String Basis; ;
            Double Par;
            Double Redemption;
            DateTime IssueDate;
            DateTime FirstInterestDate;
            DateTime LastInterestDate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2011, 12, 12);
            Maturity = new DateTime(2018, 4, 20);
            Rate = 0.02;
            Yield = 0.0217;
            Frequency = 2;
            Basis = "1";
            Par = 100;
            Redemption = 100;
            IssueDate = wct.NULL_DATE;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;

            result = wct.CONVEXITY(Settlement, Maturity, Rate, Yield, Frequency, Basis, Par, Redemption, IssueDate, FirstInterestDate, LastInterestDate);

            Console.WriteLine("CONVEXITY : " + result.ToString());      //39.3347575344594
            File.WriteAllText("CONVEXITY.csv", result.ToString());
        }

        public static void test_DURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 10);
            Maturity = new DateTime(2033, 11, 30);
            Rate = 0.03;
            Yield = 0.0315;
            Frequency = 2;
            Basis = "1";

            result = wct.DURATION(Settlement, Maturity, Rate, Yield, Frequency, Basis);

            Console.WriteLine("DURATION : " + result.ToString());     //14.4816608104738
            File.WriteAllText("DURATION.csv", result.ToString());
        }

        public static void test_MDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 10);
            Maturity = new DateTime(2033, 11, 30);
            Rate = 0.03;
            Yield = 0.0315;
            Frequency = 2;
            Basis = "1";

            result = wct.MDURATION(Settlement, Maturity, Rate, Yield, Frequency, Basis);

            Console.WriteLine("MDURATION : " + result.ToString());                //14.2571113073825
            File.WriteAllText("MDURATION.csv", result.ToString());
        }

        public static void test_OFCCONVEXITY()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            IssueDate = new DateTime(2014, 5, 1);
            FirstCouponDate = new DateTime(2014, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFCCONVEXITY(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFCCONVEXITY : " + result.ToString());   //2.91355672454477
            File.WriteAllText("OFCCONVEXITY.csv", result.ToString());
        }

        public static void test_OFCDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            IssueDate = new DateTime(2014, 5, 1);
            FirstCouponDate = new DateTime(2014, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFCDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFCDURATION : " + result.ToString());                //15.8299356059947
            File.WriteAllText("OFCDURATION.csv", result.ToString());
        }

        public static void test_OFCMDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            IssueDate = new DateTime(2014, 5, 1);
            FirstCouponDate = new DateTime(2014, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFCMDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFCMDURATION : " + result.ToString());               //15.6144561116539
            File.WriteAllText("OFCMDURATION.csv", result.ToString());
        }

        public static void test_OFLCONVEXITY()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            IssueDate = new DateTime(2012, 12, 7);
            FirstCouponDate = new DateTime(2013, 3, 15);
            LastCouponDate = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLCONVEXITY(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLCONVEXITY : " + result.ToString());                //0.715521292648933
            File.WriteAllText("OFLCONVEXITY.csv", result.ToString());
        }

        public static void test_OFLDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            IssueDate = new DateTime(2012, 12, 7);
            FirstCouponDate = new DateTime(2013, 3, 15);
            LastCouponDate = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLDURATION : " + result.ToString());               //7.98265325301858
            File.WriteAllText("OFLDURATION.csv", result.ToString());
        }

        public static void test_OFLMDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime IssueDate;
            DateTime FirstCouponDate;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2013, 3, 4);
            Maturity = new DateTime(2022, 4, 28);
            IssueDate = new DateTime(2012, 12, 7);
            FirstCouponDate = new DateTime(2013, 3, 15);
            LastCouponDate = new DateTime(2022, 3, 15);
            Rate = 0.03125;
            Yield = 0.02875;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OFLMDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OFLMDURATION : " + result.ToString());                //7.86952877685135
            File.WriteAllText("OFLMDURATION.csv", result.ToString());
        }

        public static void test_OLCCONVEXITY()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2034, 12, 15);
            LastCouponDate = new DateTime(2034, 9, 15);
            Rate = 0.0425;
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OLCCONVEXITY(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OLCCONVEXITY : " + result.ToString());                 //2.3881623279505
            File.WriteAllText("OLCCONVEXITY.csv", result.ToString());
        }

        public static void test_OLCDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2034, 12, 15);
            LastCouponDate = new DateTime(2034, 9, 15);
            Rate = 0.0425;
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OLCDURATION(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OLCDURATION : " + result.ToString());                 //13.8535032850962
            File.WriteAllText("OLCDURATION.csv", result.ToString());
        }

        public static void test_OLCMDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime LastCouponDate;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2034, 12, 15);
            LastCouponDate = new DateTime(2034, 9, 15);
            Rate = 0.0425;
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.OLCMDURATION(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("OLCMDURATION : " + result.ToString());               //13.5818659657806
            File.WriteAllText("OLCMDURATION.csv", result.ToString());
        }

        public static void test_RPICONVEXITY()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.RPICONVEXITY(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPICONVEXITY : " + result.ToString());                //2.88538118059157
            File.WriteAllText("RPICONVEXITY.csv", result.ToString());
        }

        public static void test_RPIDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.RPIDURATION(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPIDURATION : " + result.ToString());                //15.6780086644317
            File.WriteAllText("RPIDURATION.csv", result.ToString());
        }

        public static void test_RPIMDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 5, 1);
            Maturity = new DateTime(2034, 6, 15);
            Rate = 0.025;
            Yield = 0.0276;
            Redemption = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.RPIMDURATION(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis);

            Console.WriteLine("RPIMDURATION : " + result.ToString());               //15.4645972227576
            File.WriteAllText("RPIMDURATION.csv", result.ToString());
        }

        public static void test_STEPCONVEXITY()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime[] CoupDates;
            Double[] Coupons;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 4, 21);
            Maturity = new DateTime(2019, 1, 15);
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "0";
            CoupDates = new DateTime[] { new DateTime(2010, 1, 15), new DateTime(2013, 1, 15), new DateTime(2016, 1, 15) };
            Coupons = new Double[] { 0.05, 0.055, 0.06 };

            result = wct.STEPCONVEXITY(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons);

            Console.WriteLine("STEPCONVEXITY : " + result.ToString());                //0.202295869292095
            File.WriteAllText("STEPCONVEXITY.csv", result.ToString());
        }

        public static void test_STEPDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime[] CoupDates;
            Double[] Coupons;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 4, 21);
            Maturity = new DateTime(2019, 1, 15);
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "0";
            CoupDates = new DateTime[] { new DateTime(2010, 1, 15), new DateTime(2013, 1, 15), new DateTime(2016, 1, 15) };
            Coupons = new Double[] { 0.05, 0.055, 0.06 };

            result = wct.STEPDURATION(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons);

            Console.WriteLine("STEPDURATION : " + result.ToString());                //4.18487140205481
            File.WriteAllText("STEPDURATION.csv", result.ToString());
        }

        public static void test_STEPMDURATION()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Yield;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime[] CoupDates;
            Double[] Coupons;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 4, 21);
            Maturity = new DateTime(2019, 1, 15);
            Yield = 0.04;
            Redemption = 100;
            Frequency = 2;
            Basis = "0";
            CoupDates = new DateTime[] { new DateTime(2010, 1, 15), new DateTime(2013, 1, 15), new DateTime(2016, 1, 15) };
            Coupons = new Double[] { 0.05, 0.055, 0.06 };

            result = wct.STEPMDURATION(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons);

            Console.WriteLine("STEPMDURATION : " + result.ToString());              //4.10281510005374
            File.WriteAllText("STEPMDURATION.csv", result.ToString());
        }

        public static void test_ACCINTACT()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Par;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 29);
            Maturity = new DateTime(2034, 11, 1);
            Rate = 0.11;
            Par = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.ACCINTACT(Settlement, Maturity, Rate, Par, Frequency, Basis);

            Console.WriteLine("ACCINTACT : " + result.ToString());  //5.45479452054794
            File.WriteAllText("ACCINTACT.csv", result.ToString());
        }

        public static void test_ACCRINT()
        {
            DateTime Issue;
            DateTime FirstInterest;
            DateTime Settlement;
            Double Rate;
            Double Par;
            Double Frequency;
            String Basis;
            bool CalcMethod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Issue = new DateTime(2008, 3, 1);
            FirstInterest = new DateTime(2008, 8, 31);
            Settlement = new DateTime(2008, 3, 3);
            Rate = 0.1;
            Par = 1000;
            Frequency = 2;
            Basis = "0";
            CalcMethod = false;

            result = wct.ACCRINT(Issue, FirstInterest, Settlement, Rate, Par, Frequency, Basis, CalcMethod);

            Console.WriteLine("ACCRINT : " + result.ToString());                //0.555555555555556
            File.WriteAllText("ACCRINT.csv", result.ToString());
        }

        public static void test_ACCRINTM()
        {
            DateTime Issue;
            DateTime Settlement;
            Double Rate;
            Double Par;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Issue = new DateTime(2008, 4, 1);
            Settlement = new DateTime(2008, 6, 15);
            Rate = 0.1;
            Par = 1000;
            Basis = "3";

            result = wct.ACCRINTM(Issue, Settlement, Rate, Par, Basis);

            Console.WriteLine("ACCRINTM : " + result.ToString()); //20.5479452054795
            File.WriteAllText("ACCRINTM.csv", result.ToString());
        }

        public static void test_AIFACTOR()
        {
            FinancialTypes.BasisType Basis;
            Double Rate;
            DateTime Maturity;
            DateTime Settlement;
            int Frequency;
            DateTime FirstInterestDate;
            DateTime LastInterestDate;
            DateTime IssueDate;
            DateTime[] Holidays;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Regular periodic Interest
            Basis = FinancialTypes.BasisType.Bond;
            Rate = 0.05;
            Maturity = new DateTime(2015, 5, 15);
            Settlement = new DateTime(2012, 2, 15);
            Frequency = 2;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;
            IssueDate = wct.NULL_DATE;

            result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate);

            Console.WriteLine("AIFACTOR : BOND_RPI : " + result.ToString());            //0.0125
            File.WriteAllText("AIFACTOR.csv", "BOND_RPI : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Compare different daycount conventions
            Rate = 0.05;
            Maturity = new DateTime(2015, 5, 15);
            Settlement = new DateTime(2012, 2, 29);
            Frequency = 2;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;
            IssueDate = wct.NULL_DATE;

            result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate);

            for (int i = 0; i <= 9; i++)
            {
                Console.WriteLine("AIFACTOR : BOND_DC : daycount convention " + i.ToString() + " : " + wct.AIFACTOR(i.ToString(), Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate).ToString());
                File.AppendAllText("AIFACTOR.csv", vbCrLf + "BOND_DC : daycount convention " + i.ToString() + " : " + wct.AIFACTOR(i.ToString(), Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate).ToString() + vbCrLf);
            }

            //=====================
            // EXAMPLE# 3
            //=====================

            //BUS/252 example
            Basis = FinancialTypes.BasisType.BUS252;
            Rate = 0.05;
            Maturity = new DateTime(2012, 6, 30);
            Settlement = new DateTime(2012, 1, 31);
            Frequency = 2;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;
            IssueDate = wct.NULL_DATE;
            Holidays = new DateTime[] { new DateTime(2011, 1, 1), new DateTime(2011, 3, 7), new DateTime(2011, 3, 8), new DateTime(2011, 4, 21), new DateTime(2011, 4, 22), new DateTime(2011, 5, 1), new DateTime(2011, 6, 23), new DateTime(2011, 9, 7), new DateTime(2011, 10, 12), new DateTime(2011, 11, 2), new DateTime(2011, 11, 15), new DateTime(2011, 12, 25), new DateTime(2012, 1, 1), new DateTime(2012, 2, 20), new DateTime(2012, 2, 21), new DateTime(2012, 4, 6), new DateTime(2012, 4, 21), new DateTime(2012, 5, 1), new DateTime(2012, 6, 7), new DateTime(2012, 9, 7), new DateTime(2012, 10, 12), new DateTime(2012, 11, 2), new DateTime(2012, 11, 15), new DateTime(2012, 12, 25) };

            result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate, Holidays);

            Console.WriteLine("AIFACTOR : BUS/252 : " + result.ToString());          //0.00414877286812127
            File.AppendAllText("AIFACTOR.csv", vbCrLf + "BUS252 : " + result.ToString());
        }

        public static void test_AIFACTOR_IAM()
        {
            String Basis;
            Double Rate;
            DateTime IssueDate;
            DateTime Settlement;
            DateTime Maturity;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Basis = "2";
            Rate = 0.005;
            IssueDate = new DateTime(2012, 3, 1);
            Settlement = new DateTime(2012, 5, 21);
            Maturity = new DateTime(2012, 7, 1);

            result = wct.AIFACTOR_IAM(Basis, Rate, IssueDate, Settlement, Maturity);

            Console.WriteLine("AIFACTOR_IAM : " + result.ToString());       //0.001125
            File.WriteAllText("AIFACTOR_IAM.csv", result.ToString());
        }

        public static void test_AIFACTOR_OFC()
        {

            FinancialTypes.BasisType[] Basis;
            Double Rate;
            DateTime IssueDate;
            DateTime Settlement;
            DateTime FirstInterestDate;
            int Frequency;
            DateTime[] Holidays;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Returns a result for each of the interest bases
            Basis = new FinancialTypes.BasisType[] {FinancialTypes.BasisType.Bond, 
                                                    FinancialTypes.BasisType.Actual, 
                                                    FinancialTypes.BasisType.A360,                            
                                                    FinancialTypes.BasisType.A365, 
                                                    FinancialTypes.BasisType.EBOND, 
                                                    FinancialTypes.BasisType.ISDA_30_360, 
                                                    FinancialTypes.BasisType.NLACT, 
                                                    FinancialTypes.BasisType.NL360, 
                                                    FinancialTypes.BasisType.NL365, 
                                                    FinancialTypes.BasisType.A364, 
                                                    FinancialTypes.BasisType.Bond_NON_EOM, 
                                                    FinancialTypes.BasisType.Actual_NON_EOM, 
                                                    FinancialTypes.BasisType.A360_NON_EOM, 
                                                    FinancialTypes.BasisType.A365_NON_EOM, 
                                                    FinancialTypes.BasisType.EBOND_NON_EOM, 
                                                    FinancialTypes.BasisType.ISDA_30_360_NON_EOM, 
                                                    FinancialTypes.BasisType.NLACT_NON_EOM, 
                                                    FinancialTypes.BasisType.NL360_NON_EOM, 
                                                    FinancialTypes.BasisType.NL365_NON_EOM, 
                                                    FinancialTypes.BasisType.A364_NON_EOM, 
                                                    FinancialTypes.BasisType.BUS252, 
                                                    FinancialTypes.BasisType.Actual_ISDA, 
                                                    FinancialTypes.BasisType.Actual_ISMA, 
                                                    FinancialTypes.BasisType.Actual_365L, 
                                                    FinancialTypes.BasisType.Actual_AFB, 
                                                    FinancialTypes.BasisType.BUS252_NON_EOM};
            Holidays = new DateTime[] { new DateTime(2011, 1, 1), new DateTime(2011, 3, 7), new DateTime(2011, 3, 8), new DateTime(2011, 4, 21), new DateTime(2011, 4, 22), new DateTime(2011, 5, 1), new DateTime(2011, 6, 23), new DateTime(2011, 9, 7), new DateTime(2011, 10, 12), new DateTime(2011, 11, 2), new DateTime(2011, 11, 15), new DateTime(2011, 12, 25), new DateTime(2012, 1, 1), new DateTime(2012, 2, 20), new DateTime(2012, 2, 21), new DateTime(2012, 4, 6), new DateTime(2012, 4, 21), new DateTime(2012, 5, 1), new DateTime(2012, 6, 7), new DateTime(2012, 9, 7), new DateTime(2012, 10, 12), new DateTime(2012, 11, 2), new DateTime(2012, 11, 15), new DateTime(2012, 12, 25) };
            Rate = 0.1;
            IssueDate = new DateTime(2011, 7, 15);
            Settlement = new DateTime(2012, 6, 30);
            FirstInterestDate = new DateTime(2012, 6, 30);
            Frequency = 2;

            File.WriteAllText("AIFACTOR_OFC.csv", "");

            for (int i = 0; i <= Basis.GetUpperBound(0); i++)
            {
                switch (Basis[i])//Case Basis(i)
                {
                    case FinancialTypes.BasisType.BUS252 : 

                    case FinancialTypes.BasisType.BUS252_NON_EOM : 
                        result = wct.AIFACTOR_OFC(Basis[i], Rate, IssueDate, Settlement, FirstInterestDate, Frequency, Holidays) * 10000;
                        break;
                    default : 
                        result = wct.AIFACTOR_OFC(Basis[i], Rate, IssueDate, Settlement, FirstInterestDate, Frequency) * 10000;
                        break;
                }
                Console.WriteLine("AIFACTOR_OFC : " + Basis[i].ToString() + " : " + result.ToString());
                File.AppendAllText("AIFACTOR_OFC.csv", Basis[i].ToString() + " : " + result.ToString() + vbCrLf);
            }
        }

        public static void test_AIFACTOR_OLC()
        {
            FinancialTypes.BasisType[] Basis;
            Double Rate;
            DateTime LastCouponDate;
            DateTime Settlement;
            DateTime Maturity;
            int Frequency;
            DateTime[] Holidays;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Returns a result for each of the interest bases
            Basis = new FinancialTypes.BasisType[] {FinancialTypes.BasisType.Bond, 
                                                    FinancialTypes.BasisType.Actual, 
                                                    FinancialTypes.BasisType.A360, 
                                                    FinancialTypes.BasisType.A365, 
                                                    FinancialTypes.BasisType.EBOND, 
                                                    FinancialTypes.BasisType.ISDA_30_360, 
                                                    FinancialTypes.BasisType.NLACT, 
                                                    FinancialTypes.BasisType.NL360, 
                                                    FinancialTypes.BasisType.NL365, 
                                                    FinancialTypes.BasisType.A364, 
                                                    FinancialTypes.BasisType.Bond_NON_EOM, 
                                                    FinancialTypes.BasisType.Actual_NON_EOM, 
                                                    FinancialTypes.BasisType.A360_NON_EOM, 
                                                    FinancialTypes.BasisType.A365_NON_EOM, 
                                                    FinancialTypes.BasisType.EBOND_NON_EOM, 
                                                    FinancialTypes.BasisType.ISDA_30_360_NON_EOM, 
                                                    FinancialTypes.BasisType.NLACT_NON_EOM, 
                                                    FinancialTypes.BasisType.NL360_NON_EOM, 
                                                    FinancialTypes.BasisType.NL365_NON_EOM, 
                                                    FinancialTypes.BasisType.A364_NON_EOM, 
                                                    FinancialTypes.BasisType.BUS252, 
                                                    FinancialTypes.BasisType.Actual_ISDA, 
                                                    FinancialTypes.BasisType.Actual_ISMA, 
                                                    FinancialTypes.BasisType.Actual_365L, 
                                                    FinancialTypes.BasisType.Actual_AFB, 
                                                    FinancialTypes.BasisType.BUS252_NON_EOM};
            Holidays = new DateTime[] { new DateTime(2011, 1, 1), new DateTime(2011, 3, 7), new DateTime(2011, 3, 8), new DateTime(2011, 4, 21), new DateTime(2011, 4, 22), new DateTime(2011, 5, 1), new DateTime(2011, 6, 23), new DateTime(2011, 9, 7), new DateTime(2011, 10, 12), new DateTime(2011, 11, 2), new DateTime(2011, 11, 15), new DateTime(2011, 12, 25), new DateTime(2012, 1, 1), new DateTime(2012, 2, 20), new DateTime(2012, 2, 21), new DateTime(2012, 4, 6), new DateTime(2012, 4, 21), new DateTime(2012, 5, 1), new DateTime(2012, 6, 7), new DateTime(2012, 9, 7), new DateTime(2012, 10, 12), new DateTime(2012, 11, 2), new DateTime(2012, 11, 15), new DateTime(2012, 12, 25) };
            Rate = 0.1;
            LastCouponDate = new DateTime(2011, 7, 15);
            Settlement = new DateTime(2012, 6, 30);
            Maturity = new DateTime(2012, 6, 30);
            Frequency = 2;

            File.WriteAllText("AIFACTOR_OLC.csv", "");
            for (int i = 0; i <= Basis.GetUpperBound(0); i++)
            {
                switch (Basis[i])
                {
                    case FinancialTypes.BasisType.BUS252 : 
                    case FinancialTypes.BasisType.BUS252_NON_EOM : 
                        result = wct.AIFACTOR_OLC(Basis[i], Rate, LastCouponDate, Settlement, Maturity, Frequency, Holidays) * 10000;
                        break;
                    default : 
                        result = wct.AIFACTOR_OLC(Basis[i], Rate, LastCouponDate, Settlement, Maturity, Frequency) * 10000;
                        break;
                }
                Console.WriteLine("AIFACTOR_OLC : " + Basis[i].ToString() + " : " + result.ToString());
                File.AppendAllText("AIFACTOR_OLC.csv", Basis[i].ToString() + " : " + result.ToString() + vbCrLf);
            }
        }

        public static void test_AIFACTOR_RPI()
        {
            FinancialTypes.BasisType[] Basis;
            Double Rate;
            DateTime PrevCoupDate;
            DateTime NextCoupDate;
            DateTime Settlement;
            DateTime Maturity;
            int Frequency;
            DateTime[] Holidays;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Returns a result for each of the interest bases
            Basis = new FinancialTypes.BasisType[] {FinancialTypes.BasisType.Bond, 
                                                    FinancialTypes.BasisType.Actual, 
                                                    FinancialTypes.BasisType.A360, 
                                                    FinancialTypes.BasisType.A365, 
                                                    FinancialTypes.BasisType.EBOND, 
                                                    FinancialTypes.BasisType.ISDA_30_360, 
                                                    FinancialTypes.BasisType.NLACT, 
                                                    FinancialTypes.BasisType.NL360, 
                                                    FinancialTypes.BasisType.NL365, 
                                                    FinancialTypes.BasisType.A364, 
                                                    FinancialTypes.BasisType.Bond_NON_EOM, 
                                                    FinancialTypes.BasisType.Actual_NON_EOM, 
                                                    FinancialTypes.BasisType.A360_NON_EOM, 
                                                    FinancialTypes.BasisType.A365_NON_EOM, 
                                                    FinancialTypes.BasisType.EBOND_NON_EOM, 
                                                    FinancialTypes.BasisType.ISDA_30_360_NON_EOM, 
                                                    FinancialTypes.BasisType.NLACT_NON_EOM, 
                                                    FinancialTypes.BasisType.NL360_NON_EOM, 
                                                    FinancialTypes.BasisType.NL365_NON_EOM, 
                                                    FinancialTypes.BasisType.A364_NON_EOM, 
                                                    FinancialTypes.BasisType.BUS252, 
                                                    FinancialTypes.BasisType.Actual_ISDA, 
                                                    FinancialTypes.BasisType.Actual_ISMA, 
                                                    FinancialTypes.BasisType.Actual_365L, 
                                                    FinancialTypes.BasisType.Actual_AFB, 
                                                    FinancialTypes.BasisType.BUS252_NON_EOM};
            Holidays = new DateTime[] { new DateTime(2011, 1, 1), new DateTime(2011, 3, 7), new DateTime(2011, 3, 8), new DateTime(2011, 4, 21), new DateTime(2011, 4, 22), new DateTime(2011, 5, 1), new DateTime(2011, 6, 23), new DateTime(2011, 9, 7), new DateTime(2011, 10, 12), new DateTime(2011, 11, 2), new DateTime(2011, 11, 15), new DateTime(2011, 12, 25), new DateTime(2012, 1, 1), new DateTime(2012, 2, 20), new DateTime(2012, 2, 21), new DateTime(2012, 4, 6), new DateTime(2012, 4, 21), new DateTime(2012, 5, 1), new DateTime(2012, 6, 7), new DateTime(2012, 9, 7), new DateTime(2012, 10, 12), new DateTime(2012, 11, 2), new DateTime(2012, 11, 15), new DateTime(2012, 12, 25) };
            Rate = 0.1;
            PrevCoupDate = new DateTime(2003, 11, 1);
            Settlement = new DateTime(2004, 5, 1);
            NextCoupDate = new DateTime(2004, 5, 1);
            Maturity = new DateTime(2009, 11, 1);
            Frequency = 2;

            File.WriteAllText("AIFACTOR_RPI.csv", "");
            for (int i = 0; i <= Basis.GetUpperBound(0); i++)
            {
                switch (Basis[i])
                {
                    case FinancialTypes.BasisType.BUS252 : 
                    case FinancialTypes.BasisType.BUS252_NON_EOM : 
                        result = wct.AIFACTOR_RPI(Basis[i], Rate, PrevCoupDate, Settlement, NextCoupDate, Frequency, Maturity, Holidays) * 10000;
                        break;
                    default : 
                        result = wct.AIFACTOR_RPI(Basis[i], Rate, PrevCoupDate, Settlement, NextCoupDate, Frequency, Maturity) * 10000;
                        break;
                }
                Console.WriteLine("AIFACTOR_RPI : " + Basis[i].ToString() + " : " + result.ToString());
                File.AppendAllText("AIFACTOR_RPI.csv", Basis[i].ToString() + " : " + result.ToString() + vbCrLf);
            }
        }

        public static void test_BONDINT()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double Par;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2006, 1, 15);
            Maturity = new DateTime(2010, 12, 15);
            Rate = 0.04;
            Par = 100;
            Frequency = 2;
            Basis = "1";

            result = wct.BONDINT(Settlement, Maturity, Rate, Par, Frequency, Basis);

            Console.WriteLine("BONDINT : " + result.ToString());        //0.340659340659341
            File.WriteAllText("BONDINT.csv", result.ToString());
        }

        public static void test_COMPINT()
        {
            String Basis;
            Double Rate;
            DateTime Issue;
            DateTime Settlement;
            DateTime Maturity;
            int CompFreq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Basis = "1";
            Rate = 0.0175;
            Issue = new DateTime(2013, 11, 15);
            Settlement = new DateTime(2013, 11, 29);
            Maturity = new DateTime(2016, 11, 15);
            CompFreq = 12;

            result = wct.COMPINT(Basis, Rate, Issue, Settlement, Maturity, CompFreq);

            Console.WriteLine("COMPINT : " + result.ToString());        //0.000680555555555573
            File.WriteAllText("COMPINT.csv", result.ToString());
        }

        public static void test_ODDCOMPINT()
        {
            String Basis;
            Double Rate;
            DateTime Issue;
            DateTime Settlement;
            DateTime Maturity;
            DateTime FirstCoupon;
            DateTime LastCoupon;
            int CompFreq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Basis = "1";
            Rate = 0.0175;
            Issue = new DateTime(2013, 11, 15);
            Settlement = new DateTime(2013, 11, 29);
            Maturity = new DateTime(2016, 11, 30);
            FirstCoupon = new DateTime(2013, 11, 30);
            LastCoupon = wct.NULL_DATE;
            CompFreq = 12;

            result = wct.ODDCOMPINT(Basis, Rate, Issue, Settlement, Maturity, FirstCoupon, LastCoupon, CompFreq);

            Console.WriteLine("ODDCOMPINT : " + result.ToString());        //0.000680555555555556
            File.WriteAllText("ODDCOMPINT.csv", result.ToString());
        }

        public static void test_ODDFINT()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime Issue;
            DateTime FirstCoupon;
            Double Rate;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2010, 11, 1);
            Maturity = new DateTime(2020, 7, 15);
            Issue = new DateTime(2010, 7, 23);
            FirstCoupon = new DateTime(2011, 1, 15);
            Rate = 0.03;
            Frequency = 2;
            Basis = "1";

            result = wct.ODDFINT(Settlement, Maturity, Issue, FirstCoupon, Rate, Frequency, Basis);

            Console.WriteLine("ODDFINT : " + result.ToString());
            File.WriteAllText("ODDFINT.csv", result.ToString());
        }

        public static void test_ODDLINT()
        {
            DateTime Settlement;
            DateTime Maturity;
            DateTime LastCoupon;
            Double Rate;
            Double Frequency;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 10, 1);
            Maturity = new DateTime(2014, 12, 15);
            LastCoupon = new DateTime(2014, 9, 15);
            Rate = 0.0225;
            Frequency = 2;
            Basis = "1";

            result = wct.ODDLINT(Settlement, Maturity, LastCoupon, Rate, Frequency, Basis);

            Console.WriteLine("ODDLINT : " + result.ToString());
            File.WriteAllText("ODDLINT.csv", result.ToString());
        }

        public static void test_STEPACCINT()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Frequency;
            String Basis;
            DateTime[] CoupDates;
            Double[] Coupons;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Settlement = new DateTime(2014, 4, 21);
            Maturity = new DateTime(2019, 1, 15);
            Frequency = 2;
            Basis = "0";
            CoupDates = new DateTime[] { new DateTime(2010, 1, 15), new DateTime(2013, 1, 15), new DateTime(2016, 1, 15) };
            Coupons = new Double[] { 0.05, 0.055, 0.06 };

            result = wct.STEPACCINT(Settlement, Maturity, Frequency, Basis, CoupDates, Coupons);

            Console.WriteLine("STEPACCINT : " + result.ToString());
            File.WriteAllText("STEPACCINT.csv", result.ToString());
        }

        public static void test_PV()
        {
            Double rate;
            Double nper;
            Double pmt;
            Double FV;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.08 / 12.0;
            nper = 20.0 * 12.0;
            pmt = 500;
            FV = 0;
            pay_type = 0;

            result = wct.PV(rate, nper, pmt, FV, pay_type);

            Console.WriteLine("PV : " + result.ToString());     //-59777.1458511878
            File.WriteAllText("PV.csv", result.ToString());
        }

        public static void test_PMT()
        {
            Double rate;
            Double nper;
            Double pv;
            Double FV;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.08 / 12.0;
            nper = 20.0 * 12.0;
            pv = -59777.1458511878;
            FV = 0;
            pay_type = 0;

            result = wct.PMT(rate, nper, pv, FV, pay_type);

            Console.WriteLine("PMT : " + result.ToString());     //500
            File.WriteAllText("PMT.csv", result.ToString());
        }
        
        public static void test_RATE()
        {
            Double nper;
            Double pmt;
            Double pv;
            Double FV;
            int Pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            nper = 20.0 * 12.0;
            pmt = 500;
            pv = -59777.1458511878;
            FV = 0;
            Pay_type = 0;

            result = wct.RATE(nper, pmt, pv, FV, Pay_type, wct.NULL_DOUBLE);

            Console.WriteLine("RATE : " + result.ToString());       //0.0066666666666667
            File.WriteAllText("RATE.csv", result.ToString());
        }
        
        public static void test_NPER()
        {
            Double rate;
            Double pmt;
            Double pv;
            Double FV;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.08 / 12.0;
            pmt = 500;
            pv = -59777.1458511878;
            FV = 0;
            pay_type = 0;

            result = wct.NPER(rate, pmt, pv, FV, pay_type);

            Console.WriteLine("NPER : " + result.ToString());
            File.WriteAllText("NPER.csv", result.ToString());
        }
        
        public static void test_LPMT()
        {
            Double PV;
            DateTime LoanDate;
            Double Rate;
            DateTime FirstPayDate;
            int NumPmts;
            int PmtPerYear;
            Double DaysInYear;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            /*
            payment amount for a 50,000, 5-year loan starting on 1 November 
            with payments due on the first of every month. 
            The rate on the loan is 6 per cent.
            */
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            PmtPerYear = 12;
            DaysInYear = wct.NULL_DOUBLE;
            FV = 0;
            IntRule = "A";

            result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule);

            Console.WriteLine("LPMT : " + result.ToString());      //966.640076471413
            File.WriteAllText("LPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            /*
            payment amount for a 50,000, 5-year loan starting on 1 November 
            with payments due every 4 weeks, starting on 1 December. 
            The rate on the loan is 6 per cent.
            */
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 65;
            PmtPerYear = 13;
            DaysInYear = 365;
            FV = 0;
            IntRule = "A";

            result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule);

            Console.WriteLine("LPMT : " + result.ToString());    //892.076053138519
            File.AppendAllText("LPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            /*
             payment amount for a 50,000, 5-year loan starting on 1 November 
             with payments due semi-monthly, starting on 1 December. 
             The rate on the loan is 6 per cent and the loan has a balloon payment of 10,000 at the end.
             */
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 120;
            PmtPerYear = 24;
            DaysInYear = 360;
            FV = 10000;
            IntRule = "A";

            result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule);

            Console.WriteLine("LPMT : " + result.ToString());    //412.446978104019
            File.AppendAllText("LPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            /*
             Calculate the payment amount for a 150,000 4-year loan at 12 percent interest 
             using the US Rule made on 13 October with no payments due until 4 January and 
             with payments due every 4 weeks after that.
             */
            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPayDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            PmtPerYear = 13;
            DaysInYear = 365;
            FV = 0;
            IntRule = "U";

            result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule);

            Console.WriteLine("LPMT : " + result.ToString());    //412.446978104019
            File.AppendAllText("LPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_ODDPV()
        {
            Double Rate;
            int NumPmts;
            Double pmt;
            Double FV;
            int pmtperyear;
            DateTime LoanDate;
            DateTime FirstPayDate;
            Double DaysInYear;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the present value, using a rate of 6.0%, for a 5-year loan starting on 1 November 
            //with payments due on the first of every month, 
            //having a monthly payment of 966.64. 
            //The rate on the loan is 6 per cent.
            Rate = 0.06;
            NumPmts = 60;
            pmt = 966.64;
            FV = 0;
            pmtperyear = 12;
            LoanDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 12, 1);
            DaysInYear = wct.NULL_INT;

            result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear);

            Console.WriteLine("ODDPV : " + result.ToString());      //-49999.9960444733
            File.WriteAllText("ODDPV.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the present value, using a rate of 6.0%, for a 5-year loan starting on 1 November 
            //with payments due every 4 weeks, 
            //starting on 1 December. 
            //Each payment is 892.08
            Rate = 0.06;
            NumPmts = 65;
            pmt = 892.08;
            FV = 0;
            pmtperyear = 13;
            LoanDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 12, 1);
            DaysInYear = wct.NULL_INT;

            result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear);

            Console.WriteLine("ODDPV : " + result.ToString());     //-50000.2212177688
            File.AppendAllText("ODDPV.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the present value using a rate of 6.0% for a 5-year loan starting on 1 November 
            //with payments due semi-monthly,
            //starting on 1 December. 
            //The loan has a balloon payment of 10,000 at the end and the periodic payment is 412.45.
            Rate = 0.06;
            NumPmts = 120;
            pmt = 412.45;
            FV = 10000;
            pmtperyear = 24;
            LoanDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 12, 1);
            DaysInYear = 360;

            result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear);

            Console.WriteLine("ODDPV : " + result.ToString());//      -50000.3121743558
            File.AppendAllText("ODDPV.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Calculate the present value using a rate of 12.0% for a 4-year loan made on 13 October 
            //with no payments due until 4 January and 
            //with payments of 3,708.09 due every 4 weeks after that.
            Rate = 0.12;
            NumPmts = 52;
            pmt = 3708.09;
            FV = 0;
            pmtperyear = 13;
            LoanDate = new DateTime(2010, 10, 13);
            FirstPayDate = new DateTime(2011, 1, 4);
            DaysInYear = 365;

            result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear);

            Console.WriteLine("ODDPV : " + result.ToString());     // -150084.652551611
            File.AppendAllText("ODDPV.csv", vbCrLf + result.ToString());
        }
        
        public static void test_ODDFPMT()
        {
            Double rate;
            int nper;
            Double pv;
            Double fv;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //periodic payment for an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
            //The price of the annuity is 11,500 and there is no cash value at the end of the annuity. 
            //The first period is 1 and 5/6th longer than the other periods.
            rate = 0.005;
            nper = 36;
            pv = -11500;
            fv = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod);

            Console.WriteLine("ODDFPMT : " + result.ToString());        //351.309392324512
            File.WriteAllText("ODDFPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //periodic payment for an annuity assuming a periodic rate of 0.5%, with 180 periodic payments. 
            //The price of the annuity is 250,000 and there is a 50,000 cash value at the end of the annuity. 
            //The first period is one-half as long as the other periods.
            rate = 0.005;
            nper = 180;
            pv = -250000;
            fv = 50000;
            firstperiod = 0.5;

            result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod);

            Console.WriteLine("ODDFPMT : " + result.ToString());        //1932.45924676706
            File.AppendAllText("ODDFPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //weekly payment for an automobile lease with a term of 3 years and an annual interest rate of 25%. 
            //The amount to be financed is 11,000 and the residual value at the end of the lease is 3,500. 
            //The first payment is due 2014-11-25.
            rate = 0.25 * 7.0 / 365.0;
            nper = 156;
            pv = -11000;
            fv = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod);

            Console.WriteLine("ODDFPMT : " + result.ToString()); //       85.5110616087362
            File.AppendAllText("ODDFPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_ODDFPV()
        {
            Double rate;
            int nper;
            Double pmt;
            Double FV;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //present value of an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
            //The periodic payment is 351.31 and there is no cash value at the end of the annuity. 
            //The first period is 1 and 5/6th longer than the other periods.
            rate = 0.005;
            nper = 36;
            pmt = 351.31;
            FV = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod);

            Console.WriteLine("ODDFPV : " + result.ToString());         //-11500.0198920617
            File.WriteAllText("ODDFPV.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //present value of an annuity assuming a periodic rate of 0.5%, with 180 periodic payments of 1932.46. 
            //There is a 50,000 cash value at the end of the annuity. 
            //The first period is one-half as long as the other periods.
            rate = 0.005;
            nper = 180;
            pmt = 1932.46;
            FV = 50000;
            firstperiod = 0.5;

            result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod);

            Console.WriteLine("ODDFPV : " + result.ToString());         //-250000.089483625
            File.AppendAllText("ODDFPV.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //present value of an automobile lease with a term of 3 years and an annual interest rate of 25%. 
            //Then weekly payment is 85.51 and the residual value at the end of the lease is 3,500. 
            //The first payment is due 2014-11-25.
            rate = 0.25 * 7.0 / 365.0;
            nper = 156;
            pmt = 85.51;
            FV = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod);

            Console.WriteLine("ODDFPV : " + result.ToString());         //-10999.8839704582
            File.AppendAllText("ODDFPV.csv", vbCrLf + result.ToString());
        }
        
        public static void test_ODDFRATE()
        {
            int nper;
            Double pmt;
            Double pv;
            Double fv;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //rate for an annuity with 36 periodic payments of 351.31 and a current value of 11,500. 
            //There is no cash value at the end of the annuity. 
            //The first period is 1 and 5/6th longer than the other periods.
            nper = 36;
            pmt = 351.31;
            pv = -11500;
            fv = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod);

            Console.WriteLine("ODDFRATE : " + result.ToString());          //0.00500009249040195
            File.WriteAllText("ODDFRATE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //rate for an annuity with a current value of 250,000 and 180 periodic payments of 1932.46. 
            //There is a 50,000 cash value at the end of the annuity. 
            //The first period is one-half as long as the other periods.
            nper = 180;
            pmt = 1932.46;
            pv = -250000;
            fv = 50000;
            firstperiod = 0.5;

            result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod);

            Console.WriteLine("ODDFRATE : " + result.ToString());//0.00500000422661209
            File.AppendAllText("ODDFRATE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //annual rate for an automobile lease having a current value of 11000 with a term of 3 years and weekly payments of 85.51. 
            //The residual value at the end of the lease is 3,500. 
            //The first payment is due 2014-11-25.
            nper = 156;
            pmt = 85.51;
            pv = -11000;
            fv = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod) * 365.0 / 7.0;

            Console.WriteLine("ODDFRATE : " + result.ToString());
            File.AppendAllText("ODDFRATE.csv", vbCrLf + result.ToString());
        }
        
        public static void test_IPMT()
        {
            Double rate;
            Double per;
            Double nper;
            Double pv;
            Double fv;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06 / 12.0;
            per = 13;
            nper = 12 * 20;
            pv = -400000;
            fv = 100000;
            pay_type = 0;

            result = wct.IPMT(rate, per, nper, pv, fv, pay_type);

            Console.WriteLine("IPMT : " + result.ToString());     //1959.95301768065
            File.WriteAllText("IPMT.csv", result.ToString());
        }
        
        public static void test_PPMT()
        {
            Double rate;
            Double per;
            Double nper;
            Double pv;
            Double fv;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06 / 12.0;
            per = 13;
            nper = 12 * 20;
            pv = -400000;
            fv = 100000;
            pay_type = 0;

            result = wct.PPMT(rate, per, nper, pv, fv, pay_type);

            Console.WriteLine("PPMT : " + result.ToString());       //689.340157753871
            File.WriteAllText("PPMT.csv", result.ToString());
        }
        
        public static void test_ODDFIPMT()
        {
            Double rate;
            int per;
            int nper;
            Double pv;
            Double FV;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.005;
            per = 1;
            nper = 36;
            pv = -11500;
            FV = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFIPMT : " + result.ToString()); //105.636223805979
            File.WriteAllText("ODDFIPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            rate = 0.005;
            per = 1;
            nper = 180;
            pv = -250000;
            FV = 50000;
            firstperiod = 0.5;

            result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFIPMT : " + result.ToString());       //624.220697042766
            File.AppendAllText("ODDFIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            rate = 0.25 * 7.0 / 365.0;
            per = 1;
            nper = 156;
            pv = -11000;
            FV = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFIPMT : " + result.ToString());
            File.AppendAllText("ODDFIPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_ODDFPPMT()
        {
            Double rate;
            int per;
            int nper;
            Double pv;
            Double FV;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.005;
            per = 1;
            nper = 36;
            pv = -11500;
            FV = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFPPMT : " + result.ToString()); //245.673168518522
            File.WriteAllText("ODDFPPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            rate = 0.005;
            per = 1;
            nper = 180;
            pv = -250000;
            FV = 50000;
            firstperiod = 0.5;

            result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFPPMT : " + result.ToString());       //1308.23854972428
            File.AppendAllText("ODDFPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            rate = 0.25 * 7.0 / 365.0;
            per = 1;
            nper = 156;
            pv = -11000;
            FV = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod);

            Console.WriteLine("ODDFPPMT : " + result.ToString());//-5.0546399988707
            File.AppendAllText("ODDFPPMT.csv", vbCrLf + result.ToString());
        }

        public static void test_CUMIPMT()
        {
            Double rate;
            Double nper;
            Double pv;
            int start_period;
            int end_period;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.0075;
            nper = 360;
            pv = -125000;
            start_period = 349;
            end_period = 360;
            pay_type = 0;

            result = wct.CUMIPMT(rate, nper, pv, start_period, end_period, pay_type);

            Console.WriteLine("CUMIPMT : " + result.ToString());//568.35255468914
            File.WriteAllText("CUMIPMT.csv", result.ToString());
        }
        
        public static void test_CUMPRINC()
        {
            Double rate;
            Double nper;
            Double pv;
            int start_period;
            int end_period;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.0075;
            nper = 360;
            pv = -125000;
            start_period = 349;
            end_period = 360;
            pay_type = 0;

            result = wct.CUMPRINC(rate, nper, pv, start_period, end_period, pay_type);

            Console.WriteLine("CUMIPRINC : " + result.ToString());//11500.9866994826
            File.WriteAllText("CUMPRINC.csv", result.ToString());
        }
        
        public static void test_CUMODDFIPMT()
        {
            Double rate;
            int nper;
            Double pv;
            Double FV;
            int startperiod;
            int endperiod;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.005;
            nper = 36;
            pv = -11500;
            FV = 0;
            startperiod = 1;
            endperiod = 10;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFIPMT : " + result.ToString());           //558.349885303963
            File.WriteAllText("CUMODDFIPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            rate = 0.005;
            nper = 180;
            pv = -250000;
            FV = 50000;
            startperiod = 171;
            endperiod = 180;
            firstperiod = 0.5;
            result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFIPMT : " + result.ToString());          //2953.57105763922
            File.AppendAllText("CUMODDFIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            rate = 0.25 * 7.0 / 365.0;
            nper = 156;
            pv = -11000;
            FV = 3500;
            startperiod = 1;
            endperiod = 10;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFIPMT : " + result.ToString());//559.725417921967
            File.AppendAllText("CUMODDFIPMT.csv", vbCrLf + result.ToString());
        }

        public static void test_CUMODDFPPMT()
        {
            Double rate;
            int nper;
            Double pv;
            Double FV;
            int startperiod;
            int endperiod;
            Double firstperiod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.005;
            nper = 36;
            pv = -11500;
            FV = 0;
            startperiod = 1;
            endperiod = 10;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFPPMT : " + result.ToString()); //2954.74403794116
            File.WriteAllText("CUMODDFPPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            rate = 0.005;
            nper = 180;
            pv = -250000;
            FV = 50000;
            startperiod = 171;
            endperiod = 180;
            firstperiod = 0.5;

            result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFPPMT : " + result.ToString()); //66371.0214100313
            File.AppendAllText("CUMODDFPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            rate = 0.25 * 7.0 / 365.0;
            nper = 156;
            pv = -11000;
            FV = 3500;
            startperiod = 1;
            endperiod = 10;
            firstperiod = (new DateTime(2014, 11, 15) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod);

            Console.WriteLine("CUMODDFPPMT : " + result.ToString());        //295.385198165395
            File.AppendAllText("CUMODDFPPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_FV()
        {
            Double rate;
            Double nper;
            Double pmt;
            Double pv;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.12;
            nper = 12;
            pmt = -1000;
            pv = -5000;
            pay_type = 0;

            result = wct.FV(rate, nper, pmt, pv, pay_type);

            Console.WriteLine("FV : " + result.ToString()); //43613.0132339597
            File.WriteAllText("FV.csv", result.ToString());
        }
        
        public static void test_PVGA()
        {
            Double pmt;
            Double pgr;
            Double nper;
            Double rate;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pmt = 3191.76;
            pgr = 0.03;
            nper = 40;
            rate = 0.07;
            pay_type = 1;

            result = wct.PVGA(pmt, pgr, nper, rate, pay_type);

            Console.WriteLine("PVGA : " + result.ToString());//66780.481480667
            File.WriteAllText("PVGA.csv", result.ToString());
        }
        
        public static void test_PMTGA()
        {
            Double fv;
            Double pgr;
            int nper;
            Double rate;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            fv = 1000000;
            pgr = 0.03;
            nper = 40;
            rate = 0.07;
            pay_type = 1;

            result = wct.PMTGA(fv, pgr, nper, rate, pay_type);

            Console.WriteLine("PMTGA : " + result.ToString());         //3191.75519827819
            File.WriteAllText("PMTGA.csv", result.ToString());
        }
        
        public static void test_FVGA()
        {
            Double pmt;
            Double pgr;
            int nper;
            Double rate;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pmt = 3191.76;
            pgr = 0.03;
            nper = 40;
            rate = 0.07;
            pay_type = 1;

            result = wct.FVGA(pmt, pgr, nper, rate, pay_type);

            Console.WriteLine("FVGA : " + result.ToString());      //1000001.50441419
            File.WriteAllText("FVGA.csv", result.ToString());           
        }

        public static void test_NPERGA()
        {
            Double fv;
            Double pgr;
            Double pmt;
            Double rate;
            int pay_type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            fv = 1000000;
            pgr = 0.03;
            pmt = 3191.76;
            rate = 0.07;
            pay_type = 1;

            result = wct.NPERGA(fv, pgr, pmt, rate, pay_type);

            Console.WriteLine("NPERGA : " + result.ToString());         //40
            File.WriteAllText("NPERGA.csv", result.ToString());
        }
        
        public static void test_ODDFPMTSCHED()
        {
            Double rate;
            int nper;
            Double pv;
            Double fv;
            Double firstperiod;
            FinancialTypes.ODDFPMTSCHED_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Schedule for an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
            //The price of the annuity is 11,500 and there is no cash value at the end of the annuity. 
            //The first period is 1 and 5/6th longer than the other periods.
            rate = 0.005;
            nper = 36;
            pv = -11500;
            fv = 0;
            firstperiod = 1 + 5.0 / 6.0;

            result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod);

            //Write the table output to a csv
            StringBuilder retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("ODDFPMTSCHED : ex1 : " + retStr1.ToString());
            File.WriteAllText("ODDFPMTSCHED.csv", "ex1 : " + retStr1.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            // Schedule for an annuity assuming a periodic rate of 0.5%, with 180 periodic payments. 
            // The price of the annuity is 250,000 and there is a 50,000 cash value at the end of the annuity. 
            //The first period is one-half as long as the other periods.
            rate = 0.005;
            nper = 180;
            pv = -250000;
            fv = 50000;
            firstperiod = 0.5;

            result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod);

            //Write the table output to a csv
            StringBuilder retStr2 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr2.Append(col.ColumnName + ",");
            } retStr2.Remove(retStr2.Length - 1, 1);
            retStr2.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr2.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr2.Append(",");
                    }
                    else
                    {
                        retStr2.AppendLine();
                    }
                }
            }
            Console.WriteLine("ODDFPMTSCHED : ex2 : " + retStr2.ToString());
            File.AppendAllText("ODDFPMTSCHED.csv", vbCrLf + "ex2 : " + retStr2.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Schedule for a weekly payment for an automobile lease with a term of 3 years and an annual interest rate of 25%. 
            //The amount to be financed is 11,000 and the residual value at the end of the lease is 3,500. 
            //The first payment is due 2014-11-25.
            rate = 0.25 * 7.0 / 365.0;
            nper = 156;
            pv = -11000;
            fv = 3500;
            firstperiod = (new DateTime(2014, 11, 25) - new DateTime(2014, 11, 13)).TotalDays / 7.0;

            result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod);

            //Write the table output to a csv
            StringBuilder retStr3 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr3.Append(col.ColumnName + ",");
            }
            retStr3.Remove(retStr3.Length - 1, 1);
            retStr3.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr3.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr3.Append(",");
                    }
                    else
                    {
                        retStr3.AppendLine();
                    }
                }
            }
            Console.WriteLine("ODDFPMTSCHED : ex3 : " + retStr3.ToString());
            File.AppendAllText("ODDFPMTSCHED.csv", vbCrLf + "ex3 : " + retStr3.ToString());
        }
        
        public static void test_PMTSCHED()
        {
            Double pv;
            Double pmt;
            int nper;
            Double fv;
            int pay_type;
            FinancialTypes.PMTSCHED_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pv = 100000;
            pmt = 8606.64;
            nper = 12;
            fv = 0;
            pay_type = 0;
            result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type);

            //Write the table output to a csv
            StringBuilder retStr1 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr1.Append(col.ColumnName + ",");
            }
            retStr1.Remove(retStr1.Length - 1, 1);
            retStr1.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr1.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr1.Append(",");
                    }
                    else
                    {
                        retStr1.AppendLine();
                    }
                }
            }
            Console.WriteLine("PMTSCHED : ex1 " + retStr1.ToString());
            File.WriteAllText("PMTSCHED.csv", "ex1 : " + retStr1.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            pv = 32000;
            pmt = 600.95;
            nper = 60;
            fv = 0;
            pay_type = 0;

            result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type);

            //Write the table output to a csv
            StringBuilder retStr2 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr2.Append(col.ColumnName + ",");
            }
            retStr2.Remove(retStr2.Length - 1, 1);
            retStr2.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr2.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr2.Append(",");
                    }
                    else
                    {
                        retStr2.AppendLine();
                    }
                }
            }
            Console.WriteLine("PMTSCHED : ex2 : " + retStr2.ToString());
            File.AppendAllText("PMTSCHED.csv", vbCrLf + "ex2 : " + retStr2.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            pv = 250000;
            pmt = 6533.74;
            nper = 36;
            fv = 50000;
            pay_type = 0;

            result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type);

            //Write the table output to a csv
            StringBuilder retStr3 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr3.Append(col.ColumnName + ",");
            }
            retStr3.Remove(retStr3.Length - 1, 1);
            retStr3.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr3.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr3.Append(",");
                    }
                    else
                    {
                        retStr3.AppendLine();
                    }
                }
            }
            Console.WriteLine("PMTSCHED : ex3 : " + retStr3.ToString());
            File.AppendAllText("PMTSCHE.csv", vbCrLf + "ex3 : " + retStr3.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            pv = 10000;
            pmt = 869.95;
            nper = 12;
            fv = 0;
            pay_type = 1;

            result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type);

            //Write the table output to a csv
            StringBuilder retStr4 = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr4.Append(col.ColumnName + ",");
            }
            retStr4.Remove(retStr4.Length - 1, 1);
            retStr4.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr4.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr4.Append(",");
                    }
                    else
                    {
                        retStr4.AppendLine();
                    }
                }
            }
            Console.WriteLine("PMTSCHED : ex4 : " + retStr4.ToString());
            File.AppendAllText("PMTSCHED.csv", vbCrLf + "ex4 : " + retStr4.ToString());
        }
        
        public static void test_IRR()
        {
            Double[] CF;
            int[] per;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            CF = new Double[] { -10000, 2000, 1500, 3000, 3800, 2500, 2500 };
            per = new int[] { 1, 2, 5, 6, 10, 14, 14 };

            result = wct.IRR(CF, per, wct.NULL_DOUBLE);

            Console.WriteLine("IRR : " + result.ToString());            //0.0586199750678698
            File.WriteAllText("IRR.csv", result.ToString());

            //Calculate the NPV value using the IRR.
            if (wct.IsNull(result))
            {
                Console.WriteLine("IRR : NPV : " + Math.Round(wct.NPV(result, CF, per), 4).ToString("F4"));   //0.0000
                File.AppendAllText("IRR.csv" , vbCrLf + "NPV : " + Math.Round(wct.NPV(result, CF, per), 4).ToString("F4"));
            }
        }
        
        public static void test_MIRR()
        {
            Double[] cf;
            int[] per;
            Double FinanceRate;
            Double ReinvestRate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -1000, -4000, -6000, -10000, -12000, 1000, 7000, 12000, 15000, 22000 };
            per = new int[] { 0, 1, 2, 4, 6, 6, 8, 11, 14, 17 };
            FinanceRate = 0.008333;
            ReinvestRate = 0.01;

            result = wct.MIRR(cf, per, FinanceRate, ReinvestRate);

            Console.WriteLine("MIRR : " + result.ToString());           //0.0366852516799792
            File.WriteAllText("MIRR.csv", result.ToString());
        }

        public static void test_XIRR()
        {
            Double[] cf;
            DateTime[] cfdate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            cfdate = new DateTime[] { new DateTime(2011, 11, 30), new DateTime(2012, 3, 15), new DateTime(2012, 7, 18), new DateTime(2012, 11, 30), new DateTime(2013, 1, 23), new DateTime(2013, 4, 30), new DateTime(2014, 2, 6) };

            result = wct.XIRR(cf, cfdate, wct.NULL_DOUBLE);

            Console.WriteLine("XIRR : " + result.ToString());           //0.249256702747075
            File.WriteAllText("XIRR.csv", result.ToString());

            //Calculate the XNPV value using the XIRR.
            if (!wct.IsNull(result))
            {
                Console.WriteLine("XIRR : XNPV : " + Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"));         //0.0000
                File.AppendAllText("XIRR.csv", vbCrLf + "XNPV : " + Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"));
            }

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important -- in this example the dates are in descending order in the Array
            Array.Resize(ref cf, 10);
            Array.Resize(ref cfdate, 10);
            cfdate[0] = new DateTime(2017, 3, 31);
            cf[0] = -49436.82;
            for (int i = 1; i < 10; i++)
            {
                cfdate[i] = wct.EOMONTH(cfdate[0], -i);
                cf[i] = 5000;
            }

            result = wct.XIRR(cf, cfdate, wct.NULL_DOUBLE);

            Console.WriteLine("XIRR : " + result.ToString());          //-0.0245351885742908
            File.AppendAllText("XIRR.csv", vbCrLf + result.ToString());

            //Calculate the XNPV value using the XIRR.
            if (!wct.IsNull(result))
            {
                Console.WriteLine("XIRR : XNPV : " + Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"));         //0.0000
                File.AppendAllText("XIRR.csv", vbCrLf + "XNPV : " + Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"));
            }
        }

        public static void test_XIRR30360()
        {
            Double[] cf;
            DateTime[] cfdate;
            int freq;
            int days360type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            cfdate = new DateTime[] { new DateTime(2011, 11, 30), new DateTime(2012, 3, 15), new DateTime(2012, 7, 18), new DateTime(2012, 11, 30), new DateTime(2013, 1, 23), new DateTime(2013, 4, 30), new DateTime(2014, 2, 6) };
            freq = 2;
            days360type = 0;

            result = wct.XIRR30360(cf, cfdate, freq, days360type, wct.NULL_DOUBLE);

            Console.WriteLine("XIRR30360 : " + result.ToString());          //0.235942247765748
            File.WriteAllText("XIRR30360.csv", result.ToString());

            //Calculate the XNPV30360 value using the XIRR30360.
            if (!wct.IsNull(result))
            {
                Console.WriteLine("XIRR30360 : XNPV30360 : " + wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString());//-6.57348937238567E-05
                File.AppendAllText("XIRR30360.csv", vbCrLf + "XNPV30360 : " + wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString());
            }

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important -- in this example the dates are in descending order in the Array
            Array.Resize(ref cf, 10);
            Array.Resize(ref cfdate, 10);
            cfdate[0] = new DateTime(2017, 3, 31);
            cf[0] = -49436.82;
            for (int i = 1; i < 10; i++)
            {
                cfdate[i] = wct.EOMONTH(cfdate[0], -i);
                cf[i] = 5000;
            }
            freq = 2;
            days360type = 0;

            result = wct.XIRR30360(cf, cfdate, freq, days360type, wct.NULL_DOUBLE);

            Console.WriteLine("XIRR30360 : " + result.ToString());           //-0.024570360778599
            File.AppendAllText("XIRR30360.csv", vbCrLf + result.ToString());

            //Calculate the XNPV30360 value using the XIRR30360.
            if (!wct.IsNull(result))
            {
                Console.WriteLine("XIRR30360 : XNPV30360 : " + wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString());         //5.5313139455393E-06
                File.AppendAllText("XIRR30360.csv", vbCrLf + "XNPV30360 : " + wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString());
            }
        }

        public static void test_XIRRT()
        {
            Double[] cf;
            Double[] T;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            T = new Double[] { 0, 0.29041095890411, 0.632876712328767, 1.0027397260274, 1.15068493150685, 1.41643835616438, 2.18904109589041 };

            result = wct.XIRRT(cf, T, wct.NULL_DOUBLE);

            Console.WriteLine("XIRRT : " + result.ToString());           //0.24925670279801
            File.WriteAllText("XIRRT.csv", result.ToString());

            //Calculate the XNPVT value using the XIRRT.
            if (result != null)
            {
                Console.WriteLine("XIRRT : XNPVT : " + wct.XNPVT(result, cf, T).ToString());         //-9.19715239433572E-06
                File.AppendAllText("XIRRT.csv", vbCrLf + "XNPVT : " + wct.XNPVT(result, cf, T).ToString());
            }

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important
            cf = new Double[] { 90000, 75000, 37500, 12500, -2500, -50000, -100000 };
            T = new Double[] { 2.18754277891855, 1.41514598540146, 1.14963503649635, 1, 0.631147540983607, 0.289617486338798, 0 };

            result = wct.XIRRT(cf, T, wct.NULL_DOUBLE);

            Console.WriteLine("XIRRT : " + result.ToString());           //0.249465703484356
            File.AppendAllText("XIRRT.csv", vbCrLf + result.ToString());

            //Calculate the XNPVT value using the XIRRT.
            if (result != null)
            {
                Console.WriteLine("XIRRT : XNPVT : " + wct.XNPVT(result, cf, T).ToString());         //-9.2460832092911E-06
                File.AppendAllText("XIRRT.csv", vbCrLf + "XNPVT : " + wct.XNPVT(result, cf, T).ToString());
            }
        }

        public static void test_XMIRR()
        {
            Double[] cf;
            DateTime[] cfdate;
            Double financerate;
            Double reinvestrate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -1000, -4000, -6000, -10000, -12000, 1000, 7000, 12000, 15000, 22000 };
            cfdate = new DateTime[] { new DateTime(2011, 1, 31), new DateTime(2011, 3, 15), new DateTime(2011, 4, 19), new DateTime(2011, 6, 6), new DateTime(2011, 7, 2), new DateTime(2011, 7, 28), new DateTime(2011, 8, 26), new DateTime(2011, 11, 16), new DateTime(2012, 3, 17), new DateTime(2012, 6, 19) };
            financerate = 0.1;
            reinvestrate = 0.12;

            result = wct.XMIRR(cf, cfdate, financerate, reinvestrate);           //0.555762831092004

            Console.WriteLine("XMIRR : " + result.ToString());
            File.WriteAllText("XMIRR.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This changes the date to which the cash flows are discounted to 12/31/2010 from 1/31/2011.
            Array.Resize(ref cf, (cf.GetUpperBound(0) + 1));
            Array.Resize(ref cfdate, (cfdate.GetUpperBound(0) + 1));
            cfdate[cfdate.GetUpperBound(0)] = new DateTime(2010, 12, 31);

            result = wct.XMIRR(cf, cfdate, financerate, reinvestrate);           //0.524881753702055

            Console.WriteLine("XMIRR : " + result.ToString());
            File.AppendAllText("XMIRR.csv", vbCrLf + result.ToString());
        }

        public static void test_AMORTIZECASHFLOWS()
        {
            Double[] cf;
            DateTime[] cfdate;
            FinancialTypes.AMORTIZECASHFLOWS_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -1350, 18.89, 18.89, 18.89, 18.89, 18.89, 18.89, 18.89, 1418.89 };
            cfdate = new DateTime[] { new DateTime(2013, 1, 15), new DateTime(2013, 4, 15), new DateTime(2013, 7, 15), new DateTime(2013, 11, 15), new DateTime(2013, 1, 15), new DateTime(2013, 4, 15), new DateTime(2013, 7, 15), new DateTime(2013, 11, 15), new DateTime(2015, 1, 15) };

            result = wct.AMORTIZECASHFLOWS(cf, cfdate);

            //Write the table output to a csv
            StringBuilder retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }

            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("AMORTIZECASHFLOWS : " + retStr.ToString());
            File.WriteAllText("AMORTIZECASHFLOWS.csv", retStr.ToString());
        }

        public static void test_EFV()
        {
            Double startper;
            Double per;
            Double endper;
            Double cashflowrate;
            Double endrate;
            Double cashflow;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            startper = 0;
            per = 1;
            endper = 12;
            cashflowrate = 0.00142 / 12.0;
            endrate = 0.00246 / 12.0;
            cashflow = 100000;

            result = wct.EFV(startper, per, endper, cashflowrate, endrate, cashflow); //100234.416482003

            Console.WriteLine("EFV : " + result.ToString());
            File.WriteAllText("EFV.csv", result.ToString());
        }

        public static void test_ENPV()
        {
            Double rate;
            Double[] cf;
            int[] per;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.0075;
            cf = new Double[] { -10000, 2000, 1500, 3000, 3800, 2500, 2500 };
            per = new int[] { 1, 2, 5, 6, 10, 14, 14 };

            result = wct.ENPV(rate, cf, per);

            Console.WriteLine("ENPV : " + result.ToString());            //4420.95266037402
            File.WriteAllText("ENPV.csv", result.ToString());
        }

        public static void test_EPV()
        {
            Double rate;
            Double startPer;
            Double endper;
            Double cashflow;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.01 / 12.0;
            startPer = 0;
            endper = 1;
            cashflow = 10000;

            result = wct.EPV(rate, startPer, endper, cashflow);

            Console.WriteLine("EPV : " + result.ToString());         //9991.67360532889
            File.WriteAllText("EPV.csv", result.ToString());
        }

        public static void test_NFV()
        {
            Double rate;
            Double[] cf;
            int[] per;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.0075;
            cf = new Double[] { -10000, 2000, 1500, 3000, 3800, 2500, 2500 };
            per = new int[] { 1, 2, 5, 6, 10, 14, 14 };

            result = wct.NFV(rate, cf, per);

            Console.WriteLine("NFV : " + result.ToString());         //4871.93602805044
            File.WriteAllText("NFV.csv", result.ToString());
        }

        public static void test_NPV()
        {
            Double rate;
            Double[] cf;
            int[] per;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.0075;
            cf = new Double[] { -10000, 2000, 1500, 3000, 3800, 2500, 2500 };
            per = new int[] { 1, 2, 5, 6, 10, 14, 14 };

            result = wct.NPV(rate, cf, per);

            Console.WriteLine("NPV : " + result.ToString());         //4388.04234280299
            File.WriteAllText("NPV.csv", result.ToString());
        }

        public static void test_XDCF()
        {
            Double discrate;
            DateTime vdate;
            Double[] cf;
            DateTime[] cfdate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            discrate = 0.06;
            vdate = new DateTime(2012, 12, 21);
            cf = new Double[] { -87268.05, 70231.96, -457.02, -40603.18, 83598.77, -30994.43, 34666.49, 49029.24, -93685.98, -59199.55 };
            cfdate = new DateTime[] { new DateTime(2016, 4, 19), new DateTime(2019, 11, 13), new DateTime(2021, 6, 13), new DateTime(2016, 10, 29), new DateTime(2021, 3, 16), new DateTime(2014, 3, 15), new DateTime(2022, 3, 30), new DateTime(2020, 4, 30), new DateTime(2018, 2, 25), new DateTime(2015, 4, 10) };

            result = wct.XDCF(discrate, vdate, cf, cfdate);

            Console.WriteLine("XDCF : " + result.ToString());            // -103643.043186434
            File.WriteAllText("XDCF.csv", result.ToString());
        }

        public static void test_XFV()
        {
            DateTime startdate;
            DateTime cashflowdate;
            DateTime enddate;
            Double cashflowrate;
            Double endrate;
            Double cashflow;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            startdate = new DateTime(2011, 2, 1);
            cashflowdate = new DateTime(2011, 3, 1);
            enddate = new DateTime(2012, 2, 1);
            cashflowrate = 0.00142;
            endrate = 0.00246;
            cashflow = 100000;

            result = wct.XFV(startdate, cashflowdate, enddate, cashflowrate, endrate, cashflow);

            Console.WriteLine("XFV : " + result.ToString());
            File.WriteAllText("XFV.csv", result.ToString());
        }

        public static void test_XNFV()
        {
            Double rate;
            Double[] cf;
            DateTime[] cfdate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06;
            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            cfdate = new DateTime[] { new DateTime(2011, 11, 30), new DateTime(2012, 3, 15), new DateTime(2012, 7, 18), new DateTime(2012, 11, 30), new DateTime(2013, 1, 23), new DateTime(2013, 4, 30), new DateTime(2014, 2, 6) };

            result = wct.XNFV(rate, cf, cfdate);

            Console.WriteLine("XNFV : " + result.ToString());         //49496.1812211834
            File.WriteAllText("XNFV.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important -- in this example the dates are in descending order in the Array
            Array.Resize(ref cf, 10);
            Array.Resize(ref cfdate, 10);
            cfdate[0] = new DateTime(2017, 3, 31);
            cf[0] = -49436.82;
            for (int i = 1; i < 10; i++)
            {
                cfdate[i] = wct.EOMONTH(cfdate[0], -i);
                cf[i] = 5000;
            }
            rate = -0.0025;

            result = wct.XNFV(rate, cf, cfdate);

            Console.WriteLine("XNFV : " + result.ToString());        //506.061147540191
            File.AppendAllText("XNFV.csv", vbCrLf + result.ToString());
        }

        public static void test_XNPV()
        {
            Double rate;
            Double[] cf;
            DateTime[] cfdate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06;
            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            cfdate = new DateTime[] { new DateTime(2011, 11, 30), new DateTime(2012, 3, 15), new DateTime(2012, 7, 18), new DateTime(2012, 11, 30), new DateTime(2013, 1, 23), new DateTime(2013, 4, 30), new DateTime(2014, 2, 6) };

            result = wct.XNPV(rate, cf, cfdate);

            Console.WriteLine("XNPV : " + result.ToString());         //43568.8517245775
            File.WriteAllText("XNPV.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important -- in this example the dates are in descending order in the Array
            Array.Resize(ref cf, 10);
            Array.Resize(ref cfdate, 10);
            cfdate[0] = new DateTime(2017, 3, 31);
            cf[0] = -49436.82;
            for (int i = 1; i < 10; i++)
            {
                cfdate[i] = wct.EOMONTH(cfdate[0], -i);
                cf[i] = 5000;
            }
            rate = -0.0025;

            result = wct.XNPV(rate, cf, cfdate);

            Console.WriteLine("XNPV : " + result.ToString());        //507.117283780321
            File.AppendAllText("XNPV.csv", vbCrLf + result.ToString());
        }

        public static void test_XPV()
        {
            Double rate;
            DateTime startdate;
            DateTime enddate;
            Double cashflow;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.01;
            startdate = new DateTime(2011, 2, 1);
            enddate = new DateTime(2011, 3, 1);
            cashflow = 10000;

            result = wct.XPV(rate, startdate, enddate, cashflow);

            Console.WriteLine("XPV : " + result.ToString());        //9992.36978197548
            File.WriteAllText("XPV.csv", result.ToString());
        }

        public static void test_XNPV30360()
        {
            Double rate;
            Double[] cf;
            DateTime[] cfdate;
            int freq;
            int days360type;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06;
            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            cfdate = new DateTime[] { new DateTime(2011, 11, 30), new DateTime(2012, 3, 15), new DateTime(2012, 7, 18), new DateTime(2012, 11, 30), new DateTime(2013, 1, 23), new DateTime(2013, 4, 30), new DateTime(2014, 2, 6) };
            freq = 2;
            days360type = 0;

            result = wct.XNPV30360(rate, cf, cfdate, freq, days360type);

            Console.WriteLine("XNPV30360 : " + result.ToString());          //43346.5387538406
            File.WriteAllText("XNPV30360.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important -- in this example the dates are in descending order in the Array
            Array.Resize(ref cf, 10);
            Array.Resize(ref cfdate, 10);
            cfdate[0] = new DateTime(2017, 3, 11);
            cf[0] = -49436.82;
            for (int i = 1; i < 10; i++)
            {
                cfdate[i] = wct.EOMONTH(cfdate[0], -i);
                cf[i] = 5000;
            }
            freq = 2;
            days360type = 0;
            rate = -0.0025;

            result = wct.XNPV30360(rate, cf, cfdate, freq, days360type);

            Console.WriteLine("XNPV30360 : " + result.ToString());           //506.880388631596
            File.AppendAllText("XNPV30360.csv", vbCrLf + result.ToString());
        }

        public static void test_XNPVT()
        {
            Double rate;
            Double[] cf;
            Double[] T;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = 0.06;
            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };
            T = new Double[] { 0, 0.29041095890411, 0.632876712328767, 1.0027397260274, 1.15068493150685, 1.41643835616438, 2.18904109589041 };

            result = wct.XNPVT(rate, cf, T);

            Console.WriteLine("XNPVT : " + result.ToString());           //43568.8517245775
            File.WriteAllText("XNPVT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Order is not important
            cf = new Double[] { 90000, 75000, 37500, 12500, -2500, -50000, -100000 };
            T = new Double[] { 2.18754277891855, 1.41514598540146, 1.14963503649635, 1, 0.631147540983607, 0.289617486338798, 0 };

            result = wct.XNPVT(rate, cf, T);

            Console.WriteLine("XNPVT : " + result.ToString());           //43582.4812123601
            File.AppendAllText("XNPVT.csv", vbCrLf + result.ToString());
        }

        public static void test_EMDIETZ()
        {
            Double[] tranamt;
            DateTime[] trandate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            tranamt = new Double[] { 20000, 4000, 1750, -3500, -2500, 1250, -3750, -18500 };
            trandate = new DateTime[] { new DateTime(2009, 12, 31), new DateTime(2010, 1, 6), new DateTime(2010, 1, 28), new DateTime(2010, 2, 3), new DateTime(2010, 2, 18), new DateTime(2010, 3, 1), new DateTime(2010, 3, 15), new DateTime(2010, 3, 30) };

            result = wct.EMDIETZ(tranamt, trandate);

            Console.WriteLine("EMDIETZ : " + result.ToString());         //0.0584871328307772
            File.WriteAllText("EMDIETZ.csv", result.ToString());
        }

        public static void test_GTWRR()
        {
            Double[] tranamt;
            DateTime[] trandate;
            bool[] mv;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //(EMV - CF) / BMV
            tranamt = new Double[] { 100000.0, 5500.0, -5254.0, 105000.0, 4556.0, -9754.0, 110250.0, 886.0, -9525.0, 9775.0, -983.0, 115762.5, -6734.0, 3913.0, 1284.0, -6015.0, 121550.63 };
            trandate = new DateTime[] { new DateTime(2011, 12, 31), new DateTime(2012, 1, 10), new DateTime(2012, 1, 20), new DateTime(2012, 1, 31), new DateTime(2012, 2, 10), new DateTime(2012, 2, 20), new DateTime(2012, 2, 29), new DateTime(2012, 3, 10), new DateTime(2012, 3, 10), new DateTime(2012, 3, 20), new DateTime(2012, 3, 20), new DateTime(2012, 3, 31), new DateTime(2012, 4, 10), new DateTime(2012, 4, 20), new DateTime(2012, 4, 30), new DateTime(2012, 4, 30), new DateTime(2012, 4, 30) };
            mv = new Boolean[] { true, false, false, true, false, false, true, false, false, false, false, true, false, false, false, false, true };

            result = wct.GTWRR(tranamt, trandate, mv, 1);

            Console.WriteLine("GTWRR : " + result.ToString());           //0.346944952211272
            File.WriteAllText("GTWRR.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //EMV / (BMV + CF)
            result = wct.GTWRR(tranamt, trandate, mv, 2);

            Console.WriteLine("GTWRR : " + result.ToString());          //0.362813519089841
            File.AppendAllText("GTWRR.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //(EMV + W) / (BMV + D)
            result = wct.GTWRR(tranamt, trandate, mv, 0);

            Console.WriteLine("GTWRR : " + result.ToString());         //0.327310011739417
            File.AppendAllText("GTWRR.csv", vbCrLf + result.ToString());
        }

        public static void test_MDIETZ()
        { 
            DateTime[] trandate;
            Double[] tranamt;
            DateTime StartDate;
            DateTime EndDate;
            Double BMV;
            Double EMV;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //calculate the performance on an account

            //Date        Description     Value
            //12/31/2009  Market Value    20,000
            //1/6/2010    Deposit         4,000
            //1/28/2010   Deposit         1,750
            //2/3/2010    Withdrawal      -3,500
            //2/18/2010   Withdrawal      -2,500
            //3/1/2010    Deposit         1,250
            //3/15/2010   Withdrawal      -3,750
            //3/31/2010   Market Value    18,500

            trandate = new DateTime[] {new DateTime(2009, 12, 31), new DateTime(2010, 1, 6), new DateTime(2010, 1, 28), new DateTime(2010, 2, 3), new DateTime(2010, 2, 18), new DateTime(2010, 3, 1), new DateTime(2010, 3, 15), new DateTime(2010, 3, 31)};
            tranamt = new Double[] {20000.0, 4000.0, 1750.0, -3500.0, -2500.0, 1250.0, -3750.0, 18500.0};
            StartDate = new DateTime(2009, 12, 31);
            EndDate = new DateTime(2010, 3, 31);
            BMV = 20000;
            EMV = 18500;

            result = wct.MDIETZ(tranamt, trandate, StartDate, EndDate, BMV, EMV);

            Console.WriteLine("MDIETZ : " + result.ToString());           //'0.0584871328307772
            File.WriteAllText("MDIETZ.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Date        Description     Value
            //12/31/2009  Market Value    20,000
            //1/6/2010    Deposit         4,000
            //1/28/2010   Deposit         1,750
            //2/3/2010    Withdrawal      -3,500
            //2/18/2010   Withdrawal      -2,500
            //3/1/2010    Deposit         1,250
            //3/15/2010   Withdrawal      -3,750
            //3/31/2010   Market Value    18,500

            trandate = new DateTime[] {new DateTime(2009, 12, 31), new DateTime(2010, 1, 6), new DateTime(2010, 1, 28), new DateTime(2010, 2, 3), new DateTime(2010, 2, 18), new DateTime(2010, 3, 1), new DateTime(2010, 3, 15), new DateTime(2010, 3, 31)};
            tranamt = new Double[] {20000.0, 4000.0, 1750.0, -3500.0, -2500.0, 1250.0, -3750.0, 18500.0};

            result = wct.MDIETZ(tranamt, trandate, StartDate, EndDate, BMV, EMV);

            Console.WriteLine("MDIETZ : " + result.ToString());           //0.0584871328307772
            File.AppendAllText("MDIETZ.csv", result.ToString());

        }

        public static void test_LMDIETZ()
        {
            Double[] tranamt;
            DateTime[] trandate;
            bool[] mv;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            tranamt = new Double[] { 498987.32, -993.58, 1000.3, -954.15, 839.55, 36124.27, 493997.45, 503977.19, -930.18, 828.23, -938.37, 1090.9, -48246.35, 498937.42, 509016.96, -922.09, 1090.67, -916.77, 1044.57, 35643.6, 503926.79, 514107.13 };
            trandate = new DateTime[] { new DateTime(2012, 6, 28), new DateTime(2012, 7, 5), new DateTime(2012, 7, 10), new DateTime(2012, 7, 16), new DateTime(2012, 7, 20), new DateTime(2012, 7, 25), new DateTime(2012, 7, 25), new DateTime(2012, 7, 31), new DateTime(2012, 8, 3), new DateTime(2012, 8, 9), new DateTime(2012, 8, 14), new DateTime(2012, 8, 20), new DateTime(2012, 8, 24), new DateTime(2012, 8, 24), new DateTime(2012, 8, 29), new DateTime(2012, 9, 4), new DateTime(2012, 9, 7), new DateTime(2012, 9, 13), new DateTime(2012, 9, 18), new DateTime(2012, 9, 24), new DateTime(2012, 9, 24), new DateTime(2012, 9, 28) };
            mv = new Boolean[] { true, false, false, false, false, false, true, true, false, false, false, false, false, true, true, false, false, false, false, false, true, true };

            result = wct.LMDIETZ(tranamt, trandate, mv);

            Console.WriteLine("LMDIETZ : " + result.ToString());        //-0.0272594273584181
            File.WriteAllText("LMDIETZ.csv", result.ToString());
        }

        public static void test_TWROR()
        {
            Double[] tranamt;
            DateTime[] trandate;
            String[] ttype;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            tranamt = new Double[] { -100000.0, -111111.0, 10000.0, -110666.31, -500.0, -89759.65, -20000.0, -90298.21, -500.0, -94079.6, 500.0, 11000.0, -8000.0, 660.0, -94173.68 };
            trandate = new DateTime[] { new DateTime(2012, 1, 31), new DateTime(2012, 2, 14), new DateTime(2012, 2, 14), new DateTime(2012, 2, 29), new DateTime(2012, 2, 29), new DateTime(2012, 3, 15), new DateTime(2012, 3, 15), new DateTime(2012, 3, 31), new DateTime(2012, 3, 31), new DateTime(2012, 4, 15), new DateTime(2012, 4, 15), new DateTime(2012, 4, 15), new DateTime(2012, 4, 15), new DateTime(2012, 4, 15), new DateTime(2012, 4, 30) };
            ttype = new String[] { "M", "M", "", "M", "B", "M", "", "M", "E", "M", "B", "", "", "E", "M" };

            result = wct.TWROR(tranamt, trandate, ttype);

            Console.WriteLine("TWROR : " + result.ToString());           //-0.0808595339518893
            File.WriteAllText("TWROR.csv", result.ToString());
        }

        public static void test_TWRR()
        {
            Double[] tranamt;
            DateTime[] trandate;
            bool[] mv;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            tranamt = new Double[] { 15000, -15477.3, 436.49, 0, -15115.7, 0, -495.97, -14878.1, 297.92, -609.07, -14197.3, 157.06, -880.28, -14275.1, 0, 0 };
            trandate = new DateTime[] { new DateTime(2012, 1, 3), new DateTime(2012, 1, 3), new DateTime(2012, 1, 3), new DateTime(2012, 1, 3), new DateTime(2012, 1, 10), new DateTime(2012, 1, 10), new DateTime(2012, 1, 10), new DateTime(2012, 1, 17), new DateTime(2012, 1, 17), new DateTime(2012, 1, 17), new DateTime(2012, 1, 24), new DateTime(2012, 1, 24), new DateTime(2012, 1, 24), new DateTime(2012, 1, 31), new DateTime(2012, 1, 31), new DateTime(2012, 1, 31) };
            mv = new bool[] { false, true, false, false, true, false, false, true, false, false, true, false, false, true, false, false };

            result = wct.TWRR(tranamt, trandate, mv);

            Console.WriteLine("TWRR : " + result.ToString());            //0.0246243629116802
            File.WriteAllText("TWRR.csv", result.ToString());
        }

        public static void test_BetaCoKurt()
        {
            Double[] Ra;
            Double[] Rb;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };

            result = wct.BetaCoKurt(Ra, Rb);

            Console.WriteLine("BetaCoKurt : " + result.ToString());       //0.439447084098378    
            File.WriteAllText("BetaCoKurt.csv", result.ToString());       
        }

        public static void test_BetaCoSkew()
        {
            Double[] Ra;
            Double[] Rb;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };

            result = wct.BetaCoSkew(Ra, Rb);

            Console.WriteLine("BetaCoSkew : " + result.ToString());          //0.703968314287678   
            File.WriteAllText("BetaCoSkew.csv", result.ToString());
        }

        public static void test_BetaCoVar()
        {
            Double[] Ra;
            Double[] Rb;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };

            result = wct.BetaCoVar(Ra, Rb);

            Console.WriteLine("BetaCoVar : " + result.ToString());          //0.234187219388425 
            File.WriteAllText("BetaCoVar.csv", result.ToString());
        }

        public static void test_DownsideDeviation()
        {
            Double[] Ra;
            Double MAR;
            bool Full;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;
            Full = true;

            result = wct.DownsideDeviation(Ra, MAR, Full);

            Console.WriteLine("DownsideDeviation : " + result.ToString());          //0.0076206946799754 
            File.WriteAllText("DownsideDeviation.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Full = false;

            result = wct.DownsideDeviation(Ra, MAR, Full);

            Console.WriteLine("DownsideDeviation : " + result.ToString());          //0.00988289278133041 
            File.AppendAllText("DownsideDeviation.csv", vbCrLf + result.ToString());
        }

        public static void test_DownsideFrequency()
        {
            Double[] Ra;
            Double MAR;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;

            result = wct.DownsideFrequency(Ra, MAR);

            Console.WriteLine("DownsideFrequency : " + result.ToString());          //0.594594594594595
            File.WriteAllText("DownsideFrequency.csv", result.ToString());
        }

        public static void test_DownsidePotential()
        {
            Double[] Ra;
            Double MAR;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;

            result = wct.DownsidePotential(Ra, MAR);

            Console.WriteLine("DownsidePotential : " + result.ToString());          //0.00474210810810811
            File.WriteAllText("DownsidePotential.csv", result.ToString());
        }

        public static void test_EQALPHA()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double[] bValue;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] { new DateTime(2012, 12, 11), new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1) };
            pValue = new Double[] { 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29 };
            bValue = new Double[] { 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59 };

            result = wct.EQALPHA(pDate, pValue, bValue);

            Console.WriteLine("EQALPHA : " + result.ToString());        //-0.000395022617631785
            File.WriteAllText("EQALPHA.csv", result.ToString());
        }

        public static void test_EQBETA()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double[] bValue;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] { new DateTime(2012, 12, 11), new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1) };
            pValue = new Double[] { 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29 };
            bValue = new Double[] { 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59 };

            result = wct.EQBETA(pDate, pValue, bValue);

            Console.WriteLine("EQBETA : " + result.ToString());        //0.902353333082278
            File.WriteAllText("EQBETA.csv", result.ToString());
        }

        public static void test_EQVOLATILITY()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double scale;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] { new DateTime(2012, 12, 18), new DateTime(2012, 12, 17), new DateTime(2012, 12, 14), new DateTime(2012, 12, 13), new DateTime(2012, 12, 12), new DateTime(2012, 12, 11), new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1), new DateTime(2012, 10, 31) };
            pValue = new Double[] { 195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68 };
            scale = 252;

            result = wct.EQVOLATILITY(pDate, pValue, scale);

            Console.WriteLine("EQVOLATILITY : " + result.ToString());        //0.141437834070616
            File.WriteAllText("EQVOLATILITY.csv", result.ToString());
        }

        public static void test_FinCoKurt()
        {
            Double[] Ra;
            Double[] Rb;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };

            result = wct.FinCoKurt(Ra, Rb);

            Console.WriteLine("FinCoKurt : " + result.ToString());          //7.86818968887937E-10
            File.WriteAllText("FinCoKurt.csv", result.ToString());
        }
        
        public static void test_FinCoSkew()
        {
            Double[] Ra;
            Double[] Rb;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };

            result = wct.FinCoSkew(Ra, Rb);

            Console.WriteLine("FinCoSkew : " + result.ToString());          //1.77056901645221E-08
            File.WriteAllText("FinCoSkew.csv", result.ToString());
        }
        
        public static void test_INFORATIO()
        {
            Double[] r;
            Double[] rb;
            Double scale;
            bool geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            r = new Double[] { 0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135 };
            rb = new Double[] { 0.0115, 0.0119, -0.0041, -0.0063, 0.0004, 0.0065, 0.0003, 0.0029, 0.0033, 0.0016, -0.0017, -0.0047, 0.0002, 0.0043, 0.0079, -0.0052, -0.002, 0.013, 0.0023, 0.0007, 0.0199, 0.0048, -0.0016, -0.0139, -0.004, 0.0001, 0.0017, -0.0122, -0.0237, 0.0079, 0.0022, -0.0094, 0.0109 };
            scale = 252;
            geometric = false;

            result = wct.INFORATIO(r, rb, scale, geometric);

            Console.WriteLine("INFORATIO : " + result.ToString());           //-1.46734740387312
            File.WriteAllText("INFORATIO.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            geometric = true;

            result = wct.INFORATIO(r, rb, scale, geometric);

            Console.WriteLine("INFORATIO : " + result.ToString());          //-1.6901913539965
            File.AppendAllText("INFORATIO.csv", vbCrLf + result.ToString());
        }
        
        public static void test_INFORATIO2()
        {
            DateTime[] vDate;
            Double[] pr;
            Double[] prb;
            Double scale;
            Boolean geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            vDate = new DateTime[] { new DateTime(2012, 12, 18), new DateTime(2012, 12, 17), new DateTime(2012, 12, 14), new DateTime(2012, 12, 13), new DateTime(2012, 12, 12), new DateTime(2012, 12, 11), new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1), new DateTime(2012, 10, 31) };
            pr = new Double[] { 195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68 };
            prb = new Double[] { 1446.79, 1430.36, 1413.58, 1419.45, 1428.48, 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59, 1412.16 };
            scale = 252;
            geometric = false;

            result = wct.INFORATIO2(vDate, pr, prb, scale, geometric);

            Console.WriteLine("INFORATIO2 : " + result.ToString());          //-1.47787598470777
            File.WriteAllText("INFORATIO2.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            geometric = true;

            result = wct.INFORATIO2(vDate, pr, prb, scale, geometric);

            Console.WriteLine("INFORATIO2 : " + result.ToString());          //-1.70044976017884
            File.AppendAllText("INFORATIO2.csv", vbCrLf + result.ToString());
        }
        
        public static void test_MAXDD()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] { new DateTime(2011, 12, 31), new DateTime(2012, 1, 31), new DateTime(2012, 2, 29), new DateTime(2012, 3, 31), new DateTime(2012, 4, 30), new DateTime(2012, 5, 31), new DateTime(2012, 6, 30), new DateTime(2012, 7, 31), new DateTime(2012, 8, 31), new DateTime(2012, 9, 30), new DateTime(2012, 10, 31), new DateTime(2012, 11, 30), new DateTime(2012, 12, 31), new DateTime(2013, 1, 31), new DateTime(2013, 2, 28), new DateTime(2013, 3, 31), new DateTime(2013, 4, 30), new DateTime(2013, 5, 31), new DateTime(2013, 6, 30), new DateTime(2013, 7, 31), new DateTime(2013, 8, 31), new DateTime(2013, 9, 30), new DateTime(2013, 10, 31), new DateTime(2013, 11, 30), new DateTime(2013, 12, 31), new DateTime(2014, 1, 31), new DateTime(2014, 2, 28), new DateTime(2014, 3, 31), new DateTime(2014, 4, 30), new DateTime(2014, 5, 31), new DateTime(2014, 6, 30), new DateTime(2014, 7, 31), new DateTime(2014, 8, 31), new DateTime(2014, 9, 30), new DateTime(2014, 10, 31), new DateTime(2014, 11, 30), new DateTime(2014, 12, 31) };
            pValue = new Double[] { 109.39, 102.46, 102.98, 102.38, 113.23, 93.06, 90.13, 79.55, 108.59, 108.39, 103.31, 101.83, 131.3, 127.31, 92.56, 65.05, 108.19, 95.3, 76.85, 84.59, 82.59, 107.36, 101.79, 143.82, 124.53, 109.66, 95.17, 108.4, 104.61, 84.24, 86.99, 92.34, 99.92, 128.53, 108.63, 101.59, 82.25 };

            result = wct.MAXDD(pDate, pValue);

            Console.WriteLine("MAXDD : " + result.ToString());           //-0.504569687738005
            File.WriteAllText("MAXDD.csv", result.ToString());
        }

        public static void test_MAXDD2()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] { new DateTime(2012, 1, 31), new DateTime(2012, 2, 29), new DateTime(2012, 3, 31), new DateTime(2012, 4, 30), new DateTime(2012, 5, 31), new DateTime(2012, 6, 30), new DateTime(2012, 7, 31), new DateTime(2012, 8, 31), new DateTime(2012, 9, 30), new DateTime(2012, 10, 31), new DateTime(2012, 11, 30), new DateTime(2012, 12, 31), new DateTime(2013, 1, 31), new DateTime(2013, 2, 28), new DateTime(2013, 3, 31), new DateTime(2013, 4, 30), new DateTime(2013, 5, 31), new DateTime(2013, 6, 30), new DateTime(2013, 7, 31), new DateTime(2013, 8, 31), new DateTime(2013, 9, 30), new DateTime(2013, 10, 31), new DateTime(2013, 11, 30), new DateTime(2013, 12, 31), new DateTime(2014, 1, 31), new DateTime(2014, 2, 28), new DateTime(2014, 3, 31), new DateTime(2014, 4, 30), new DateTime(2014, 5, 31), new DateTime(2014, 6, 30), new DateTime(2014, 7, 31), new DateTime(2014, 8, 31), new DateTime(2014, 9, 30), new DateTime(2014, 10, 31), new DateTime(2014, 11, 30), new DateTime(2014, 12, 31) };
            pValue = new Double[] { -0.063351, 0.005075, -0.005826, 0.105978, -0.178133, -0.031485, -0.117386, 0.365053, -0.001842, -0.046868, -0.014326, 0.289404, -0.030388, -0.272956, -0.297213, 0.663182, -0.119142, -0.193599, 0.100716, -0.023643, 0.299915, -0.051882, 0.412909, -0.134126, -0.119409, -0.132136, 0.139014, -0.034963, -0.194723, 0.032645, 0.061501, 0.082088, 0.286329, -0.154828, -0.064807, -0.190373 };

            result = wct.MAXDD2(pDate, pValue);

            Console.WriteLine("MAXDD2 : " + result.ToString());           //-0.504569891864632
            File.WriteAllText("MAXDD2.csv", result.ToString());
        }
        
        public static void test_MOIC()
        {
            Double[] cf;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            cf = new Double[] { -100000, -50000, -2500, 12500, 37500, 75000, 90000 };

            result = wct.MOIC(cf);

            Console.WriteLine("MOIC : " + result.ToString());            //1.40983606557377
            File.WriteAllText("MOIC.csv", result.ToString());
        }
        
        public static void test_Omega()
        {
            Double[] Ra;
            Double MAR;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;

            result = wct.Omega(Ra, MAR);

            Console.WriteLine("Omega : " + result.ToString());           //0.644154156550285
            File.WriteAllText("Omega.csv", result.ToString());
        }
        
        public static void test_OmegaExcessReturns()
        {
            Double[] Ra;
            Double[] Rb;
            Double MAR;
            int freq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };
            MAR = 0.005;
            freq = 12;

            result = wct.OmegaExcessReturn(Ra, Rb, MAR, freq);

            Console.WriteLine("OmegaExcessReturn : " + result.ToString());         //0.0378469204243685    
            File.WriteAllText("OmegaExcessReturn.csv", result.ToString());
        }
        
        public static void test_OmegaSharpeRatio()
        {
            Double[] Ra;
            Double MAR;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;

            result = wct.OmegaSharpeRatio(Ra, MAR);

            Console.WriteLine("OmegaSharpeRatio : " + result.ToString());          //-0.35584584344971
            File.WriteAllText("OmegaSharpeRatio.csv", result.ToString());
        }
        
        public static void test_SemiDeviation()
        {
            Double[] Ra;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };

            result = wct.SemiDeviation(Ra);

            Console.WriteLine("SemiDeviation : " + result.ToString());          //0.00660706168061328
            File.WriteAllText("SemiDeviation.csv", result.ToString());
        }
        
        public static void test_SemiVariance()
        {
            Double[] Ra;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };

            result = wct.SemiVariance(Ra);

            Console.WriteLine("SemiVariance : " + result.ToString());          //0.00922003188116091
            File.WriteAllText("SemiVariance.csv", result.ToString());
        }
        
        public static void test_SHARPE()
        {
            Double[] r;
            Double[] rf = null;
            Double scale;
            Boolean geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            r = new Double[] { 0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135 };
            Array.Resize (ref rf, (r.GetUpperBound(0)));
            rf = new Double[r.Length];
            for (int i = 0; i < r.Length ; i++)
            {
                rf[i] = 0.001 / 252.0;
            }
            scale = 252;
            geometric = false;

            result = wct.SHARPE(r, rf, scale, geometric);

            Console.WriteLine("SHARPE : " + result.ToString());          // 0.630146290350507
            File.WriteAllText("SHARPE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            r = new Double[] { 0.009416, 0.013579, 0.009315, 0.018145, 0.007068, 0.006657, 0.006028, 0.004719, 0.018247, 0.001731, 0.002923, 0.018072, 0.0102, 0.014619, 0.017633, 0.010065, 0.012939, 0.008044, 0.017269, 0.010842, 0.009544, 0.008508, 0.000504, 0.015876, 0.00385, 0.002251, 0.013098, 0.013069, 0.006818, 0.013184, 0.013902, 0.007957, 0.003052, 0.013372, 0.014231, 0.011182 };
            rf = new Double[] { 0.004986, 0.005009, 0.00495, 0.005082, 0.005112, 0.005066, 0.004967, 0.005018, 0.005011, 0.004863, 0.004825, 0.004895, 0.00507, 0.005088, 0.004956, 0.005025, 0.004736, 0.004933, 0.004969, 0.004814, 0.004959, 0.005083, 0.004977, 0.004992, 0.005003, 0.004997, 0.004815, 0.005026, 0.005013, 0.005044, 0.004822, 0.005019, 0.00487, 0.005153, 0.004994, 0.004966 };
            scale = 12;

            result = wct.SHARPE(r, rf, scale, geometric);

            Console.WriteLine("SHARPE : " + result.ToString());          // 3.58446805338759
            File.AppendAllText("SHARPE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            r = new Double[] { 0.0173, -0.001, 0.0099, -0.0177, 0.035, -0.0142, -0.0153, 0.0008, -0.0005, -0.0695, -0.0133, 0.0151, 0.0072, -0.004, 0.0367, 0.0239, -0.0148, -0.0171, 0.0097, 0.0082, 0.0108, 0.0204, 0.0347, -0.0282, -0.0213, 0.0097, -0.0271, 0.0203, 0.032, -0.0268, -0.0081, -0.0263, -0.0145, -0.0088, 0.0361, -0.0158, -0.013, -0.0152, 0.0154, -0.0026, 0.0269, 0.0091, 0.0053, 0.0224, 0.0052, -0.0024, 0.0167, 0.0103, 0.0523, -0.0185 };
            rf = new Double[] { 0.000195, 0.000194, 0.000192, 0.000193, 0.00019, 0.000192, 0.000193, 0.000192, 0.000192, 0.000193, 0.000193, 0.00019, 0.000189, 0.000189, 0.000199, 0.000189, 0.000191, 0.000193, 0.000192, 0.000195, 0.000192, 0.000192, 0.000192, 0.000194, 0.000192, 0.00019, 0.00019, 0.000194, 0.00019, 0.000192, 0.000192, 0.000195, 0.000196, 0.000191, 0.000192, 0.000193, 0.000196, 0.00019, 0.000195, 0.000191, 0.000188, 0.000191, 0.000191, 0.000194, 0.000193, 0.000194, 0.000191, 0.000193, 0.000191, 0.000189 };
            scale = 52;
            geometric = true;

            result = wct.SHARPE(r, rf, scale, geometric);

            Console.WriteLine("SHARPE : " + result.ToString());           //0.478892083019103
            File.AppendAllText("SHARPE.csv", vbCrLf + result.ToString());
        }
        
        public static void test_SHARPE2()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double[] RF;
            Double scale;
            Boolean Geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            pDate = new DateTime[] {new DateTime(2012, 12, 18), new DateTime(2012, 12, 17), new DateTime(2012, 12, 14), new DateTime(2012, 12, 13), new DateTime(2012, 12, 12), new DateTime(2012, 12, 11), 
            new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), 
            new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), 
            new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), 
            new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1), new DateTime(2012, 10, 31) };
            //I may have copy-pasted something I shouldn't have, so, just in case, I'm keeping the original dates here for reference
            //#12/18/2012), new DateTime(12/17/2012), new DateTime(12/14/2012), new DateTime(12/13/2012), new DateTime(12/12/2012), new DateTime(12/11/2012), new DateTime(12/10/2012), new DateTime(12/7/2012), new DateTime(12/6/2012), new DateTime(12/5/2012), new DateTime(12/4/2012), new DateTime(12/3/2012), new DateTime(11/30/2012), new DateTime(11/29/2012), new DateTime(11/28/2012), new DateTime(11/27/2012), new DateTime(11/26/2012), new DateTime(11/23/2012), new DateTime(11/21/2012), new DateTime(11/20/2012), new DateTime(11/19/2012), new DateTime(11/16/2012), new DateTime(11/15/2012), new DateTime(11/14/2012), new DateTime(11/13/2012), new DateTime(11/12/2012), new DateTime(11/9/2012), new DateTime(11/8/2012), new DateTime(11/7/2012), new DateTime(11/6/2012), new DateTime(11/5/2012), new DateTime(11/2/2012), new DateTime(11/1/2012), new DateTime(10/31/2012#};
            pValue = new Double[] { 195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68 };
            RF = new Double[pValue.Length];
            for (int i = 0; i < pValue.Length; i++)
            {
                RF[i] = 0.001 / 252.0;
            }
            scale = 252;
            Geometric = false;

            result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric);

            Console.WriteLine("SHARPE2 : " + result.ToString());         //0.618904269463314
            File.WriteAllText("SHARPE2.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            pDate = new DateTime[] { new DateTime(2010, 12, 31), new DateTime(2011, 1, 31), new DateTime(2011, 2, 28), new DateTime(2011, 3, 31), new DateTime(2011, 4, 30), new DateTime(2011, 5, 31), new DateTime(2011, 6, 30), new DateTime(2011, 7, 31), new DateTime(2011, 8, 31), new DateTime(2011, 9, 30), new DateTime(2011, 10, 31), new DateTime(2011, 11, 30), new DateTime(2011, 12, 31), new DateTime(2012, 1, 31), new DateTime(2012, 2, 29), new DateTime(2012, 3, 31), new DateTime(2012, 4, 30), new DateTime(2012, 5, 31), new DateTime(2012, 6, 30), new DateTime(2012, 7, 31), new DateTime(2012, 8, 31), new DateTime(2012, 9, 30), new DateTime(2012, 10, 31), new DateTime(2012, 11, 30), new DateTime(2012, 12, 31), new DateTime(2013, 1, 31), new DateTime(2013, 2, 28), new DateTime(2013, 3, 31), new DateTime(2013, 4, 30), new DateTime(2013, 5, 31), new DateTime(2013, 6, 30), new DateTime(2013, 7, 31), new DateTime(2013, 8, 31), new DateTime(2013, 9, 30), new DateTime(2013, 10, 31), new DateTime(2013, 11, 30), new DateTime(2013, 12, 31) };
            pValue = new Double[] { 100000.0, 100457.78, 101518.83, 102055.61, 103216.4, 103970.62, 104915.46, 105640.67, 107643.46, 108017.35, 109334.07, 110373.01, 111550.51, 112898.34, 113909.43, 114682.07, 114877.46, 115731.44, 117754.23, 118578.8, 120634.65, 121411.91, 123689.75, 124274.22, 125307.78, 126672.4, 128171.03, 129291.6, 131008.02, 132464.56, 133378.55, 133769.46, 135724.66, 136462.27, 137830.14, 139773.62, 142001.01 };
            RF = new Double[] { 0.005202, 0.004891, 0.005097, 0.004959, 0.005014, 0.004995, 0.005035, 0.004929, 0.005027, 0.004865, 0.005033, 0.005042, 0.004938, 0.005016, 0.00499, 0.005084, 0.004889, 0.004898, 0.005133, 0.004719, 0.004908, 0.00496, 0.004986, 0.004853, 0.004995, 0.00499, 0.00497, 0.004814, 0.00503, 0.005208, 0.005126, 0.00493, 0.004932, 0.00486, 0.004946, 0.004971, 0.00495 };
            scale = 12;
            Geometric = false;

            result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric);

            Console.WriteLine("SHARPE2 : " + result.ToString());         //3.76729327551418
            File.AppendAllText("SHARPE2.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //                                    #12/17/2012#,                 #12/10/2012#,           #12/3/2012#,                #11/26/2012#,               #11/19/2012#,               #11/12/2012#,               #11/5/2012#,                #10/31/2012#,               #10/22/2012#,               #10/15/2012#,               #10/8/2012#,                #10/1/2012#,             #9/24/2012#,               #9/17/2012#,                #9/10/2012#,              #9/4/2012#,               #8/27/2012#,               #8/20/2012#,                 #8/13/2012#,             #8/6/2012#,               #7/30/2012#,               #7/23/2012#,               #7/16/2012#,               #7/9/2012#,               #7/2/2012#,               #6/25/2012#,               #6/18/2012#,               #6/11/2012#,               #6/4/2012#,               #5/29/2012#,               #5/21/2012#,               #5/14/2012#,               #5/7/2012#,              #4/30/2012#,                 #4/23/2012#,              #4/16/2012#,               #4/9/2012#,               #4/2/2012#,               #3/26/2012#,               #3/19/2012#,              #3/12/2012#,                #3/5/2012#,               #2/27/2012#,               #2/21/2012#,               #2/13/2012#,                #2/6/2012#,              #1/30/2012#,               #1/23/2012#,              #1/17/2012#,                #1/9/2012#,                #1/3/2012#
            pDate = new DateTime[] { new DateTime(2012, 12, 17), new DateTime(2012, 12, 10), new DateTime(2012, 12, 3), new DateTime(2012, 11, 26), new DateTime(2012, 11, 19), new DateTime(2012, 11, 12), new DateTime(2012, 11, 5), new DateTime(2012, 10, 31), new DateTime(2012, 10, 22), new DateTime(2012, 10, 15), new DateTime(2012, 10, 8), new DateTime(2012, 10, 1), new DateTime(2012, 9, 24), new DateTime(2012, 9, 17), new DateTime(2012, 9, 10), new DateTime(2012, 9, 4), new DateTime(2012, 8, 27), new DateTime(2012, 8, 20), new DateTime(2012, 8, 13), new DateTime(2012, 8, 6), new DateTime(2012, 7, 30), new DateTime(2012, 7, 23), new DateTime(2012, 7, 16), new DateTime(2012, 7, 9), new DateTime(2012, 7, 2), new DateTime(2012, 6, 25), new DateTime(2012, 6, 18), new DateTime(2012, 6, 11), new DateTime(2012, 6, 4), new DateTime(2012, 5, 29), new DateTime(2012, 5, 21), new DateTime(2012, 5, 14), new DateTime(2012, 5, 7), new DateTime(2012, 4, 30), new DateTime(2012, 4, 23), new DateTime(2012, 4, 16), new DateTime(2012, 4, 9), new DateTime(2012, 4, 2), new DateTime(2012, 3, 26), new DateTime(2012, 3, 19), new DateTime(2012, 3, 12), new DateTime(2012, 3, 5), new DateTime(2012, 2, 27), new DateTime(2012, 2, 21), new DateTime(2012, 2, 13), new DateTime(2012, 2, 6), new DateTime(2012, 1, 30), new DateTime(2012, 1, 23), new DateTime(2012, 1, 17), new DateTime(2012, 1, 9), new DateTime(2012, 1, 3) };
            pValue = new Double[] { 195.08, 191.76, 191.95, 190.07, 193.49, 186.94, 189.64, 192.59, 192.43, 192.52, 206.89, 209.67, 206.55, 205.08, 205.91, 198.63, 194.0, 196.91, 200.34, 198.42, 196.81, 194.7, 190.8, 184.41, 189.77, 193.9, 192.04, 197.39, 193.46, 187.46, 192.63, 194.2, 199.44, 202.38, 204.18, 197.06, 200.22, 202.86, 205.99, 202.86, 203.39, 198.07, 196.28, 195.24, 190.96, 189.97, 190.43, 187.31, 185.4, 176.19, 179.52 };
            RF = new Double[] { 0.000191, 0.000192, 0.000192, 0.000193, 0.000194, 0.000191, 0.000192, 0.000194, 0.000188, 0.000195, 0.00019, 0.000194, 0.000196, 0.000189, 0.000193, 0.000194, 0.000192, 0.000191, 0.000192, 0.000191, 0.000194, 0.000196, 0.000191, 0.000193, 0.000194, 0.000192, 0.000192, 0.000193, 0.000195, 0.00019, 0.000191, 0.000191, 0.000194, 0.000193, 0.000191, 0.00019, 0.000191, 0.000194, 0.000193, 0.000196, 0.000195, 0.000189, 0.000191, 0.000194, 0.000193, 0.000193, 0.00019, 0.000193, 0.000188, 0.000193, 0.000191 };
            scale = 52;
            Geometric = true;

            result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric);

            Console.WriteLine("SHARPE2 : " + result.ToString());         //0.47823560953273
            File.AppendAllText("SHARPE2.csv", vbCrLf + result.ToString());
        }
        
        public static void test_SORTINO()
        {
            Double[] R;
            Double MAR;
            Boolean Full;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            R = new Double[] { 0.0089, 0.0012, -0.002, 0.01, -0.0002, 0.02, 0.03, 0.01, -0.003, 0.01, 0.0102, -0.01 };
            MAR = Math.Pow(1.1, 1.0 / 12.0) - 1;
            Full = true;

            result = wct.SORTINO(R, MAR, Full);

            Console.WriteLine("SORTINO : " + result.ToString());        //-0.11938313404921
            File.WriteAllText("SORTINO.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Full = false;

            result = wct.SORTINO(R, MAR, Full);

            Console.WriteLine("SORTINO : " + result.ToString());         //-0.0770614816651062
            File.AppendAllText("SORTINO.csv", vbCrLf + result.ToString());
        }
        
        public static void test_SORTINO2()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double MAR;
            Boolean Full;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //                                   #12/31/2013#,                #1/31/2014#,               #2/28/2014#,                #3/31/2014#,                #4/30/2014#,                #5/31/2014#,            #6/30/2014#,                #7/31/2014#,                #8/31/2014#,             #9/30/2014#,                #10/31/2014#,              #11/30/2014#,                #12/31/2014#
            pDate = new DateTime[] { new DateTime(2013, 12, 31), new DateTime(2014, 1, 31), new DateTime(2014, 2, 28), new DateTime(2014, 3, 31), new DateTime(2014, 4, 30), new DateTime(2014, 5, 31), new DateTime(2014, 6, 30), new DateTime(2014, 7, 31), new DateTime(2014, 8, 31), new DateTime(2014, 9, 30), new DateTime(2014, 10, 31), new DateTime(2014, 11, 30), new DateTime(2014, 12, 31) };
            pValue = new Double[] { 1000000.0, 1008900.0, 1010110.68, 1008090.46, 1018171.36, 1017967.73, 1038327.08, 1069476.9, 1080171.66, 1076931.15, 1087700.46, 1098795.01, 1087807.06 };
            MAR = Math.Pow(1.1, 1.0 / 12.0) - 1.0;
            Full = true;

            result = wct.SORTINO2(pDate, pValue, MAR, Full);

            Console.WriteLine("SORTINO2 : " + result.ToString());         //-0.11938313404921
            File.WriteAllText("SORTINO2.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Full = false;

            result = wct.SORTINO2(pDate, pValue, MAR, Full);

            Console.WriteLine("SORTINO2 : " + result.ToString());         //-0.0770614816651062
            File.AppendAllText("SORTINO2.csv", vbCrLf + result.ToString());
        }
        
        public static void test_SpecificRisk()
        {
            Double[] Ra;
            Double[] Rb;
            Double[] Rf;
            int Freq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };
            Rf = new Double[] { 0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332 };
            Freq = 12;

            result = wct.SpecificRisk(Ra, Rb, Rf, Freq);

            Console.WriteLine("SpecificRisk : " + result.ToString());            //0.0324179638746894
            File.WriteAllText("SpecificRisk.csv", result.ToString());
        }
        
        public static void test_SystematicRisk()
        {
            Double[] Ra;
            Double[] Rb;
            Double[] Rf;
            int Freq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };
            Rf = new Double[] { 0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332 };
            Freq = 12;

            result = wct.SystematicRisk(Ra, Rb, Rf, Freq);

            Console.WriteLine("SystematicRisk : " + result.ToString());          //0.00514725126745008
            File.WriteAllText("SystematicRisk.csv", result.ToString());
        }
        
        public static void test_TotalRisk()
        {
            Double[] Ra;
            Double[] Rb;
            Double[] Rf;
            int Freq;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            Rb = new Double[] { -0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025 };
            Rf = new Double[] { 0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332 };
            Freq = 12;

            result = wct.TotalRisk(Ra, Rb, Rf, Freq);

            Console.WriteLine("TotalRisk : " + result.ToString());            //0.0328240548590654
            File.WriteAllText("TotalRisk.csv", result.ToString());
        }
        
        public static void test_TREYNOR()
        {
            Double[] r;
            Double[] rb;
            Double Rf;
            Double scale;
            Boolean geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            r = new Double[] { 0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135 };
            rb = new Double[] { 0.0115, 0.0119, -0.0041, -0.0063, 0.0004, 0.0065, 0.0003, 0.0029, 0.0033, 0.0016, -0.0017, -0.0047, 0.0002, 0.0043, 0.0079, -0.0052, -0.002, 0.013, 0.0023, 0.0007, 0.0199, 0.0048, -0.0016, -0.0139, -0.004, 0.0001, 0.0017, -0.0122, -0.0237, 0.0079, 0.0022, -0.0094, 0.0109 };
            Rf = 0.001 / 252.0;
            scale = 252;
            geometric = false;

            result = wct.TREYNOR(r, rb, Rf, scale, geometric);

            Console.WriteLine("TREYNOR : " + result.ToString());           //0.000388271591630151
            File.WriteAllText("TREYNOR.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            r = new Double[] { 0.0173, -0.001, 0.0099, -0.0177, 0.035, -0.0142, -0.0153, 0.0008, -0.0005, -0.0695, -0.0133, 0.0151, 0.0072, -0.004, 0.0367, 0.0239, -0.0148, -0.0171, 0.0097, 0.0082, 0.0108, 0.0204, 0.0347, -0.0282, -0.0213, 0.0097, -0.0271, 0.0203, 0.032, -0.0268, -0.0081, -0.0263, -0.0145, -0.0088, 0.0361, -0.0158, -0.013, -0.0152, 0.0154, -0.0026, 0.0269, 0.0091, 0.0053, 0.0224, 0.0052, -0.0024, 0.0167, 0.0103, 0.0523, -0.0185 };
            rb = new Double[] { 0.0157, -0.0032, 0.0013, 0.005, 0.0362, -0.0145, -0.0243, 0.0016, -0.0148, 0.0032, -0.0221, 0.0141, -0.0133, -0.0038, 0.0194, 0.0223, -0.0032, -0.005, 0.0087, 0.0107, 0.0036, 0.0171, 0.0043, 0.0016, -0.0055, 0.0203, -0.0058, 0.013, 0.0373, -0.0302, 0.0174, -0.043, -0.0115, -0.0244, 0.018, 0.006, -0.0199, -0.0074, 0.0081, -0.005, 0.0243, 0.0009, 0.0028, 0.0033, 0.0138, -0.0017, 0.0217, 0.0007, 0.0204, 0.0088 };
            Rf = 0.001 / 52.0;
            scale = 52;
            geometric = true;

            result = wct.TREYNOR(r, rb, Rf, scale, geometric);

            Console.WriteLine("TREYNOR : " + result.ToString());         //0.103411484610504
            File.AppendAllText("TREYNOR.csv", vbCrLf + result.ToString());
        }
        
        public static void test_TREYNOR2()
        {
            DateTime[] pDate;
            Double[] pValue;
            Double[] bValue;
            Double Rf;
            Double scale;
            Boolean geometric;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

//          pDate = New Date() {                 #12/18/2012#,               #12/17/2012#,                #12/14/2012#,              #12/13/2012#,               #12/12/2012#,               #12/11/2012#,               #12/10/2012#,               #12/7/2012#,               #12/6/2012#,               #12/5/2012#,               #12/4/2012#,               #12/3/2012#,                #11/30/2012#,               #11/29/2012#,               #11/28/2012#,              #11/27/2012#,               #11/26/2012#,               #11/23/2012#,                #11/21/2012#,               #11/20/2012#,               #11/19/2012#,               #11/16/2012#,              #11/15/2012#,               #11/14/2012#,                #11/13/2012#,               #11/12/2012#,              #11/9/2012#,                #11/8/2012#,              #11/7/2012#,               #11/6/2012#,               #11/5/2012#,                #11/2/2012#,              #11/1/2012#,               #10/31/2012#}
            pDate = new DateTime[] { new DateTime(2012, 12, 18), new DateTime(2012, 12, 17), new DateTime(2012, 12, 14), new DateTime(2012, 12, 13), new DateTime(2012, 12, 12), new DateTime(2012, 12, 11), new DateTime(2012, 12, 10), new DateTime(2012, 12, 7), new DateTime(2012, 12, 6), new DateTime(2012, 12, 5), new DateTime(2012, 12, 4), new DateTime(2012, 12, 3), new DateTime(2012, 11, 30), new DateTime(2012, 11, 29), new DateTime(2012, 11, 28), new DateTime(2012, 11, 27), new DateTime(2012, 11, 26), new DateTime(2012, 11, 23), new DateTime(2012, 11, 21), new DateTime(2012, 11, 20), new DateTime(2012, 11, 19), new DateTime(2012, 11, 16), new DateTime(2012, 11, 15), new DateTime(2012, 11, 14), new DateTime(2012, 11, 13), new DateTime(2012, 11, 12), new DateTime(2012, 11, 9), new DateTime(2012, 11, 8), new DateTime(2012, 11, 7), new DateTime(2012, 11, 6), new DateTime(2012, 11, 5), new DateTime(2012, 11, 2), new DateTime(2012, 11, 1), new DateTime(2012, 10, 31) };
            //Just in case my copy-paste was wrong, I'm keeping these values around : #12/18/2012), new DateTime(12/17/2012), new DateTime(12/14/2012), new DateTime(12/13/2012), new DateTime(12/12/2012), new DateTime(12/11/2012), new DateTime(12/10/2012), new DateTime(12/7/2012), new DateTime(12/6/2012), new DateTime(12/5/2012), new DateTime(12/4/2012), new DateTime(12/3/2012), new DateTime(11/30/2012), new DateTime(11/29/2012), new DateTime(11/28/2012), new DateTime(11/27/2012), new DateTime(11/26/2012), new DateTime(11/23/2012), new DateTime(11/21/2012), new DateTime(11/20/2012), new DateTime(11/19/2012), new DateTime(11/16/2012), new DateTime(11/15/2012), new DateTime(11/14/2012), new DateTime(11/13/2012), new DateTime(11/12/2012), new DateTime(11/9/2012), new DateTime(11/8/2012), new DateTime(11/7/2012), new DateTime(11/6/2012), new DateTime(11/5/2012), new DateTime(11/2/2012), new DateTime(11/1/2012), new DateTime(10/31/2012#
            pValue = new Double[] { 195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68 };
//          bValue = New Double() { 1446.79, 1430.36, 1413.58, 1419.45, 1428.48, 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59, 1412.16 }
            bValue = new Double[] { 1446.79, 1430.36, 1413.58, 1419.45, 1428.48, 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59, 1412.16 };
            Rf = 0.001 / 252.0;
            scale = 252;
            geometric = false;

            result = wct.TREYNOR2(pDate, pValue, bValue, Rf, scale, geometric);

            Console.WriteLine("TREYNOR2 : " + result.ToString());           //0.000381122001119534
            File.WriteAllText("TREYNOR2.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            pDate = new DateTime[] { new DateTime(2012, 12, 17), new DateTime(2012, 12, 10), new DateTime(2012, 12, 3), new DateTime(2012, 11, 26), new DateTime(2012, 11, 19), new DateTime(2012, 11, 12), new DateTime(2012, 11, 5), new DateTime(2012, 10, 31), new DateTime(2012, 10, 22), new DateTime(2012, 10, 15), new DateTime(2012, 10, 8), new DateTime(2012, 10, 1), new DateTime(2012, 9, 24), new DateTime(2012, 9, 17), new DateTime(2012, 9, 10), new DateTime(2012, 9, 4), new DateTime(2012, 8, 27), new DateTime(2012, 8, 20), new DateTime(2012, 8, 13), new DateTime(2012, 8, 6), new DateTime(2012, 7, 30), new DateTime(2012, 7, 23), new DateTime(2012, 7, 16), new DateTime(2012, 7, 9), new DateTime(2012, 7, 2), new DateTime(2012, 6, 25), new DateTime(2012, 6, 18), new DateTime(2012, 6, 11), new DateTime(2012, 6, 4), new DateTime(2012, 5, 29), new DateTime(2012, 5, 21), new DateTime(2012, 5, 14), new DateTime(2012, 5, 7), new DateTime(2012, 4, 30), new DateTime(2012, 4, 23), new DateTime(2012, 4, 16), new DateTime(2012, 4, 9), new DateTime(2012, 4, 2), new DateTime(2012, 3, 26), new DateTime(2012, 3, 19), new DateTime(2012, 3, 12), new DateTime(2012, 3, 5), new DateTime(2012, 2, 27), new DateTime(2012, 2, 21), new DateTime(2012, 2, 13), new DateTime(2012, 2, 6), new DateTime(2012, 1, 30), new DateTime(2012, 1, 23), new DateTime(2012, 1, 17), new DateTime(2012, 1, 9), new DateTime(2012, 1, 3) };
            pValue = new Double[] { 195.08, 191.76, 191.95, 190.07, 193.49, 186.94, 189.64, 192.59, 192.43, 192.52, 206.89, 209.67, 206.55, 205.08, 205.91, 198.63, 194.0, 196.91, 200.34, 198.42, 196.81, 194.7, 190.8, 184.41, 189.77, 193.9, 192.04, 197.39, 193.46, 187.46, 192.63, 194.2, 199.44, 202.38, 204.18, 197.06, 200.22, 202.86, 205.99, 202.86, 203.39, 198.07, 196.28, 195.24, 190.96, 189.97, 190.43, 187.31, 185.4, 176.19, 179.52 };
            bValue = new Double[] { 1435.81, 1413.58, 1418.07, 1416.18, 1409.15, 1359.88, 1379.85, 1414.2, 1411.94, 1433.19, 1428.59, 1460.93, 1440.67, 1460.15, 1465.77, 1437.92, 1406.58, 1411.13, 1418.16, 1405.87, 1390.99, 1385.97, 1362.66, 1356.78, 1354.68, 1362.16, 1335.02, 1342.84, 1325.66, 1278.04, 1317.82, 1295.22, 1353.39, 1369.1, 1403.36, 1378.53, 1370.26, 1398.08, 1408.47, 1397.11, 1404.17, 1370.87, 1369.63, 1365.74, 1361.23, 1342.64, 1344.9, 1316.33, 1315.38, 1289.09, 1277.81 };
            Rf = 0.001 / 52.0;
            scale = 52;
            geometric = true;

            result = wct.TREYNOR2(pDate, pValue, bValue, Rf, scale, geometric);

            Console.WriteLine("TREYNOR2 : " + result.ToString());           //0.103301285778773    
            File.AppendAllText("TREYNOR2.csv", vbCrLf + result.ToString());           
        }
        
        public static void test_UpsideFrequency()
        {
            Double[] Ra;
            Double MAR;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;

            result = wct.UpsideFrequency(Ra, MAR);

            Console.WriteLine("UpsideFrequency : " + result.ToString());         // 0.405405405405405
            File.WriteAllText("UpsideFrequency.csv", result.ToString());
        }
        
        public static void test_UpsidePotentialRatio()
        {
            Double[] Ra;
            Double MAR;
            Boolean Full;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;
            Full = false;

            result = wct.UpsidePotentialRatio(Ra, MAR, Full);

            Console.WriteLine("UpsidePotentialRatio : " + result.ToString());          //0.762408352161206
            File.WriteAllText("UpsidePotentialRatio.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Full = true;

            result = wct.UpsidePotentialRatio(Ra, MAR, Full);

            Console.WriteLine("UpsidePotentialRatio : " + result.ToString());          //0.400835983716186
            File.AppendAllText("UpsidePotentialRatio.csv", vbCrLf + result.ToString());
        }
        
        public static void test_UpsideRisk()
        {
            Double[] Ra;
            Double MAR;
            Boolean Full;
            String State;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Ra = new Double[] { 0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088 };
            MAR = 0.005;
            Full = false;
            State = "POTENTIAL";

            result = wct.UpsideRisk(Ra, MAR, Full, State);

            Console.WriteLine("UpsideRisk : " + result.ToString());         //0.0075348
            File.WriteAllText("UpsideRisk.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            State = "RISK";

            result = wct.UpsideRisk(Ra, MAR, Full, State);

            Console.WriteLine("UpsideRisk : " + result.ToString());         //0.00916366023668126
            File.AppendAllText("UpsideRisk.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            State = "VARIANCE";

            result = wct.UpsideRisk(Ra, MAR, Full, State);

            Console.WriteLine("UpsideRisk : " + result.ToString());          //8.39726689333333E-05 
            File.AppendAllText("UpsideRisk.csv", vbCrLf + result.ToString());
        }
        
        public static void test_CUMLIPMT()
        {
            Double PV;
            DateTime LoanDate;
            Double Rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int StartPer;
            int EndPer;
            int DaysInYr;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first year’s cumulative interest for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            StartPer = 1;
            EndPer = 12;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLIPMT : " + result.ToString());            //2759.52255542426
            File.WriteAllText("CUMLIPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the last year’s interest for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 percent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            StartPer = 49;
            EndPer = 60;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLIPMT : " + result.ToString());            //368.355536073601
            File.AppendAllText("CUMLIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the cumulative interest payments for the first two periods on a 150,000 4-year loan 
            // at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
            //  and with payments due every 4 weeks after that.

            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPayDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            Pmtpyr = 13;
            StartPer = 1;
            EndPer = 2;
            DaysInYr = 365;
            FV = 0;
            IntRule = "U";

            result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLIPMT : " + result.ToString());            //5473.97260273973
            File.AppendAllText("CUMLIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same cumulative interest payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLIPMT : " + result.ToString());            //5477.51646742669    
            File.AppendAllText("CUMLIPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_CUMLPPMT()
        {
            Double PV;
            DateTime LoanDate;
            Double Rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int StartPer;
            int EndPer;
            int DaysInYr;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first year’s cumulative principal for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            StartPer = 1;
            EndPer = 12;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLPPMT : " + result.ToString());            //8840.15836223269
            File.WriteAllText("CUMLPPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the last year’s principal payments for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            StartPer = 49;
            EndPer = 60;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLPPMT : " + result.ToString());           // 11231.3253815834
            File.AppendAllText("CUMLPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the cumulative principal payments for the first two periods on a 150,000 4-year loan 
            //at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
            //and with payments due every 4 weeks after that.

            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPayDate = new DateTime(2010, 10, 13);
            NumPmts = 52;
            Pmtpyr = 13;
            StartPer = 1;
            EndPer = 2;
            DaysInYr = 365;
            FV = 0;
            IntRule = "U";

            result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLPPMT : " + result.ToString());            //1942.2129309029
            File.AppendAllText("CUMLPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same cumulative principal payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule);

            Console.WriteLine("CUMLPPMT : " + result.ToString());            //1938.83811031561
            File.AppendAllText("CUMLPPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_EFFECT()
        {
            Double NominalRate;
            int npery;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            NominalRate = 0.04;
            npery = 12;

            result = wct.EFFECT(NominalRate, npery);

            Console.WriteLine("EFFECT : " + result.ToString());
            File.WriteAllText("EFFECT.csv", result.ToString());
        }
        
        public static void test_FVSCHEDULE()
        {
            Double[] rate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            rate = new Double[] { 0.00025, 0.00025, 0.000167, 0.0005, 0.000333, 0.0005, 0.00075, 0.00025, 0.0005, 0.00025, 0.000333, 0.000417 };

            result = wct.FVSCHEDULE(rate) * 100000;

            Console.WriteLine("FVSCHEDULE : " + result.ToString());          //100450.914303657
            File.WriteAllText("FVSCHEDULE.csv", result.ToString());
        }
        
        public static void test_LIPMT()
        {
            Double PV;
            DateTime LoanDate;
            Double Rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int Per;
            int DaysInYr;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 12, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            Per = 1;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LIPMT : " + result.ToString());            //250
            File.WriteAllText("LIPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //  Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //  The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            Per = 60;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LIPMT : " + result.ToString());            //4.80915460931035
            File.AppendAllText("LIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the first interest payment for a 150,000 4-year loan 
            // at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
            //and with payments due every 4 weeks after that.

            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPayDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            Pmtpyr = 13;
            Per = 1;
            DaysInYr = 365;
            FV = 0;
            IntRule = "U";

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LIPMT : " + result.ToString());           // 3708.09276682132
            File.AppendAllText("LIPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LIPMT : " + result.ToString());            //4093.15068493151    
            File.AppendAllText("LIPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_LPPMT()
        {
            Double PV;
            DateTime LoanDate;
            Double Rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int Per;
            int DaysInYr;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first principal payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 12, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            Per = 1;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LPPMT : " + result.ToString());            //716.640076471413
            File.WriteAllText("LPPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //  Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //  The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            Per = 60;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LPPMT : " + result.ToString());       //961.830921862103
            File.AppendAllText("LPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the first principal payment for a 150,000 4-year loan 
            //at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
            //and with payments due every 4 weeks after that.

            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPayDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            Pmtpyr = 13;
            Per = 1;
            DaysInYr = 365;
            FV = 0;
            IntRule = "U";

            result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LPPMT : " + result.ToString());            //0
            File.AppendAllText("LPPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same principal payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule);

            Console.WriteLine("LPPMT : " + result.ToString());            //-384.973396060356
            File.AppendAllText("LPPMT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_LPMTSCHED()
        {
            Double PV;
            DateTime LoanDate;
            Double pmt;
            Double rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int DaysInYr;
            Double FV;
            FinancialTypes.InterestRuleType IntRule;
            int Decimals;
            int LastPmtNum;
            FinancialTypes.LPMTSCHED_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //amortization schedule for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The periodic payment is 966.64.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            rate = wct.NULL_DOUBLE;
            pmt = 966.64;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = 0;
            IntRule = FinancialTypes.InterestRuleType.Actuarial;
            Decimals = 2;
            LastPmtNum = wct.NULL_INT;

            result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("LPMTSCHED : ex1 : " + retStr.ToString());
            File.WriteAllText("LPMTSCHED.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //amortization schedule for a 50,000, 5-year loan starting on 1 November with payments of 1,161.41 
            //due on the fifteenth of every month starting 15-Apr-2011. 
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            rate = wct.NULL_DOUBLE;
            pmt = 1161.41;
            FirstPayDate = new DateTime(2011, 4, 5);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = 0;
            IntRule = FinancialTypes.InterestRuleType.Actuarial;
            Decimals = 2;
            LastPmtNum = wct.NULL_INT;

            result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum);

            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("LPMTSCHED : ex2 : " + retStr.ToString());
            File.AppendAllText("LPMTSCHED.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //In this example, the US Rule is applied, so there is no negative amortization. 
            //The monthly payment is 1160.82, and all other parameters are from the previous example.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            rate = wct.NULL_DOUBLE;
            pmt = 1160.82;
            FirstPayDate = new DateTime(2011, 4, 15);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = 0;
            IntRule = FinancialTypes.InterestRuleType.US;
            Decimals = 2;
            LastPmtNum = wct.NULL_INT;

            result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum);

            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("LPMTSCHED : ex3 : " + retStr.ToString());
            File.AppendAllText("LPMTSCHED.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //In this example, we have a 7,125,000 loan that pays monthly and is being amortized over 25 years, 
            //but the loan is paid off in full at the end of 10 years. 
            //The monthly payment is 41,554.10
            PV = 7125000;
            LoanDate = new DateTime(2008, 11, 18);
            rate = wct.NULL_DOUBLE;
            pmt = 41554.1;
            FirstPayDate = new DateTime(2008, 12, 1);
            NumPmts = 300;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = 0;
            IntRule = FinancialTypes.InterestRuleType.Actuarial;
            Decimals = 2;
            LastPmtNum = 120;

            result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum);

            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("LPMTSCHED : ex4 : " + retStr.ToString());
            File.AppendAllText("LPMTSCHED.csv", vbCrLf + "ex4 : " + retStr.ToString());
        }
        
        public static void test_LRATE()
        {
            Double PV;
            DateTime LoanDate;
            Double pmt;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            Double DaysInYr;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            pmt = 966.64;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_DOUBLE;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE);

            Console.WriteLine("LRATE : " + result.ToString());           //0.06
            File.WriteAllText("LRATE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            pmt = 892.42;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 65;
            Pmtpyr = 13;
            DaysInYr = wct.NULL_DOUBLE;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE);

            Console.WriteLine("LRATE : " + result.ToString());           //0.0601605094753396
            File.AppendAllText("LRATE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            pmt = 412.45;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 120;
            Pmtpyr = 24;
            DaysInYr = wct.NULL_DOUBLE;
            FV = 10000;
            IntRule = wct.NULL_STRING;

            result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE);

            Console.WriteLine("LRATE : " + result.ToString());           //0.0600022174769416
            File.AppendAllText("LRATE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            PV = 150000;
            LoanDate = new DateTime(2010, 10, 13);
            pmt = 3710.27;
            FirstPayDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            Pmtpyr = 13;
            DaysInYr = wct.NULL_DOUBLE;
            FV = wct.NULL_DOUBLE;
            IntRule = wct.NULL_STRING;

            result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE);

            Console.WriteLine("LRATE : " + result.ToString());           //0.120281649456623
            File.AppendAllText("LRATE.csv", vbCrLf + result.ToString());
        }
        
        public static void test_NOMINAL()
        {
            Double Rate;
            int npery;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Rate = 0.0407415429197906;
            npery = 12;

            result = wct.NOMINAL(Rate, npery);

            Console.WriteLine("NOMINAL : " + result.ToString());         //0.0400000000000009
            File.WriteAllText("NOMINAL.csv", result.ToString());
        }
        
        public static void test_NUMPMTS()
        {
            DateTime firstpaymentdate;
            DateTime lastpaymentdate;
            int pmtperyr;
            int result;

            //=====================
            // EXAMPLE# 1
            //=====================

            firstpaymentdate = new DateTime(2014, 1, 31);
            lastpaymentdate = new DateTime(2019, 12, 31);
            pmtperyr = 12;

            result = wct.NUMPMTS(firstpaymentdate, lastpaymentdate, pmtperyr);

            Console.WriteLine("NUMPMTS : " + result.ToString());         //72
            File.WriteAllText("NUMPMTS.csv", result.ToString());
        }

        public static void test_TOTALINT()
        {
            Double nper;
            Double pmt;
            Double pv;
            Double fv;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            nper = 15;
            pmt = 215;
            pv = 3000;
            fv = 0;
            result = wct.TOTALINT(nper, pmt, pv, fv);

            Console.WriteLine("TOTALINT : " + result.ToString());            //225
            File.WriteAllText("TOTALINT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            nper = 48;
            pmt = 595.02;
            pv = 30000;
            fv = 5000;

            result = wct.TOTALINT(nper, pmt, pv, fv);

            Console.WriteLine("TOTALINT : " + result.ToString());            //3560.96    
            File.AppendAllText("TOTALINT.csv", vbCrLf + result.ToString());
        }
        
        public static void test_AMORTRATE()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double rate;
            Double FaceAmount;
            Double CleanPrice;
            Double Redemption;
            Double Frequency;
            FinancialTypes.BasisType Basis;
            DateTime IssueDate;
            DateTime FirstInterestDate;
            DateTime LastInterestDate;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //We buy 1,000,000 in face value of a bond on 2012-05-03 at a price of 999000. 
            //The bond matures on 2012-06-30 and accrues interest using the Actual/365 day-count method. The interest rate is 5%
            Settlement = new DateTime(2015, 5, 3);
            Maturity = new DateTime(2015, 6, 30);
            rate = 0.05;
            FaceAmount = 1000000.0;
            CleanPrice = 999000.0;
            Redemption = wct.NULL_DOUBLE;
            Frequency = 2;
            Basis = FinancialTypes.BasisType.A365;
            IssueDate = wct.NULL_DATE;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;

            result = wct.AMORTRATE(Settlement, Maturity, rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            Console.WriteLine("AMORTRATE : " + result.ToString());           //0.000154306279086793
            File.WriteAllText("AMORTRATE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //In this example we change the basis to the US 30/360 day-count convention.
            Basis = FinancialTypes.BasisType.Bond;

            result = wct.AMORTRATE(Settlement, Maturity, rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            Console.WriteLine("AMORTRATE : " + result.ToString());            //0.000156512494013327
            File.AppendAllText("AMORTRATE.csv", vbCrLf + result.ToString());
        }

        public static void test_BONDAMORT()
        {
            DateTime Settlement;
            DateTime Maturity;
            Double Rate;
            Double FaceAmount;
            Double CleanPrice;
            Double Redemption;
            Double Frequency;
            String Basis;
            DateTime IssueDate;
            DateTime FirstInterestDate;
            DateTime LastInterestDate;
            FinancialTypes.BONDAMORT_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            // We buy 1,000,000 in face value of a bond on 2012-05-03 at a price of 999000. 
            // The bond matures on 2012-06-30 and accrues interest using the Actual/365 day-count method. 
            // The interest rate is 5%.

            Settlement = new DateTime(2012,5,3);
            Maturity = new DateTime(2012,6,30);
            Rate = 0.05;
            FaceAmount = 1000000;
            CleanPrice = 999000;
            Redemption = wct.NULL_DOUBLE;
            Frequency = 2;
            Basis = "3";
            IssueDate = wct.NULL_DATE;
            FirstInterestDate = wct.NULL_DATE;
            LastInterestDate = wct.NULL_DATE;

            result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            // Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("BONDAMORT : ex1 : " + retStr.ToString());
            File.WriteAllText("BONDAMORT.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            // In this example we will change the basis to 0, to reflect the US 30/360 day-count convention.
            // Notice the row for the 31-May-12. There is no daily coupon accrual, reflecting the market convention for this day-count convention.
            // Since there is no daily coupon accrual, there is no need to adjust the interest for this day and there is no daily amortization.

            Basis = "0";

            result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            // Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("BONDAMORT : ex2 : " + retStr.ToString());
            File.WriteAllText("BONDAMORT.csv", "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            // Let’s look at what happens with a US 30/360 bond over a February month end.
            // Notice that for 2012-02-29 the daily coupon is actually two days of coupon interest (again, reflecting market practice). 
            //  Thus, the daily amortization also needs to reflect two days of amortization.

            Settlement = new DateTime(2012, 2, 15);
            Maturity = new DateTime(2012, 3, 15);

            result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            // Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("BONDAMORT : ex3 : " + retStr.ToString());
            File.WriteAllText("BONDAMORT.csv", "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            // Here’s an example for NL/365 in which Feb-29 is not included in the date calculations (NL meaning No Leap year).
            //  On 29-Feb-12 there is no coupon accrual and therefore there is no amortization.

            Settlement = new DateTime(2012, 2, 15);
            Maturity = new DateTime(2012, 3, 15);
            Basis = "7";

            result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            // Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("BONDAMORT : ex4 : " + retStr.ToString());
            File.WriteAllText("BONDAMORT.csv", "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            // Let’s look at a bond which accrues on an actual/actual basis.
            // Notice the on 2012-03-16 the dly_coup changed from 137.3626 to 135.8696 and the dly_amort changed from 1.6317 to 3.1250. 
            // The dly_eff_rate, however, only changed slightly, from 138.9943 to 138.9945 and the (calculated) amort_rate did not change at all. 
            // In fact, if we were to look at the last 10 days of the amortization schedule we would see that the (calculated) amortization rate is still the same.

            Settlement = new DateTime(2012, 3, 10);
            Maturity = new DateTime(2022, 3, 15);
            CleanPrice = 990000.0;
            Basis = "1";

            result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate);

            // Write the table output to a csv
            // We only write out the rows where amort_date < 2012-03-20
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                    if ((DateTime)row["amort_date"] >= new DateTime(2012, 3, 20)) break;
                }
            }
            Console.WriteLine("BONDAMORT : ex5 : " + retStr.ToString());
            File.WriteAllText("BONDAMORT.csv", "ex5 : " + retStr.ToString());
        }
        
        public static void test_AMORTSCHED()
        {
            Double PV;
            DateTime LoanDate;
            Double rate;
            DateTime FirstPayDate;
            int NumPmts;
            int Pmtpyr;
            int DaysInYr;
            Double FV;
            FinancialTypes.InterestRuleType IntRule;
            FinancialTypes.AMORTSCHED_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //amortization schedule for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
            //The rate on the loan is 6 per cent.
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            rate = 0.06;
            FirstPayDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = FinancialTypes.InterestRuleType.Actuarial;

            result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("AMORTSCHED : ex1 : " + retStr.ToString());
            File.WriteAllText("AMORTSCHED.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This is an example of a loan that negatively amortizes principal using the actuarial rule.. 
            PV = 50000;
            LoanDate = new DateTime(2010, 11, 1);
            rate = 0.12;
            FirstPayDate = new DateTime(2011, 4, 15);
            NumPmts = 60;
            Pmtpyr = 12;
            DaysInYr = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            IntRule = FinancialTypes.InterestRuleType.Actuarial;

            result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule);

            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("AMORTSCHED : ex2 : " + retStr.ToString());
            File.AppendAllText("AMORTSCHED.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //The same loan, except that the US Rule is applied, so there is no negative amortization.
            IntRule = FinancialTypes.InterestRuleType.US;

            result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule);

            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("AMORTSCHED : ex3 : " + retStr.ToString());
            File.AppendAllText("AMORTSCHED.csv", vbCrLf + "ex3 : " + retStr.ToString());
        }
        
        public static void test_Balloon()
        {
            Double OutstandingAmount;
            FinancialTypes.InterestBasisType InterestBasis;
            Double InterestRate;
            int PaymentFrequency;
            DateTime MaturityDate;
            DateTime ReferenceDate;
            DateTime PrevPayDate;
            DateTime StartDate;
            DateTime FirstPayDate;
            DateTime GracePeriodStartDate;
            DateTime GracePeriodEndDate;
            FinancialTypes.Balloon_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a simple 5-year loan with quarterly payments of interest.
            OutstandingAmount = 100000;
            InterestBasis = FinancialTypes.InterestBasisType.A365;
            InterestRate = 0.04;
            PaymentFrequency = 3;
            MaturityDate = new DateTime(2019, 9, 15);
            ReferenceDate = new DateTime(2014, 9, 15);
            PrevPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;

            result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Balloon : ex1 : " + retStr.ToString());
            File.WriteAllText("Balloon.csv", "ex1 : " +retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //In this example, we modify the first payment date so that the interest will not be paid until 31-Mar-2015.
            FirstPayDate = new DateTime(2015, 3, 15);

            result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Balloon : ex2 : " + retStr.ToString());
            File.AppendAllText("Balloon.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //No interest payments in 2018
            GracePeriodStartDate = new DateTime(2018, 1, 1);
            GracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Balloon : ex3 : " + retStr.ToString());
            File.AppendAllText("Balloon.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Get rid of the first payment date and set previous payment date.
            FirstPayDate = wct.NULL_DATE;
            PrevPayDate = new DateTime(2014, 8, 15);

            result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Balloon : ex4 : " + retStr.ToString());
            File.AppendAllText("Balloon.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Eliminate the grace period and the previous payment date and add a start date.
            PrevPayDate = wct.NULL_DATE;
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;
            StartDate = new DateTime(2011, 9, 15);

            result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Balloon : ex5 : " + retStr.ToString());
            File.AppendAllText("Balloon.csv", vbCrLf + "ex5 : " + retStr.ToString());
        }
        
        public static void test_Bullet()
        {
            Double OutstandingAmount;
            FinancialTypes.InterestBasisType InterestBasis;
            Double InterestRate;
            int Frequency;
            DateTime MaturityDate;
            DateTime ReferenceDate;
            FinancialTypes.Bullet_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            OutstandingAmount = 6000000;
            InterestBasis = FinancialTypes.InterestBasisType.A360;
            InterestRate = 0.07;
            Frequency = 3;
            MaturityDate = new DateTime(2015, 7, 5);
            ReferenceDate = new DateTime(2014, 6, 30);

            result = wct.Bullet(OutstandingAmount, InterestBasis, InterestRate, Frequency, MaturityDate, ReferenceDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("Bullet : " + retStr.ToString());
            File.WriteAllText("Bullet.csv", retStr.ToString());
        }
        
        public static void test_CDRCashFlow()
        {
            Double PrinAmt;
            Double InterestRate;
            int NumPmts;
            int LastPmtNum;
            int PmtPerYr;
            Boolean InterestOnly;
            int PrinPaymentMultiple;
            int FirstPrinPayNo;
            Double PmtPayPct;
            StringBuilder retStr;
            FinancialTypes.CDRCashflow_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            System.Array LSRates_array = new double[,] { { 1, 0.15 }, { 91, 0.16 }, { 181, 0.14 }, { 271, 0.17 }, { 361, 0.13 } };
            System.Array CPRRates_array = new double[,] {{1, 0.002}, {25, 0.012}, {61, 0.02}, { 181, 0.06}};
            System.Array CDRRates_array = new double[,] {{1, 0.1}, {61, 0.08}, {181, 0.05}};

            //In this example we amortize a 100,000,000 loan with a 5% interest rate over 360 months 

            PrinAmt = 100000000;
            InterestRate = 0.05;
            NumPmts = 360;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 12;
            InterestOnly = false;
            PrinPaymentMultiple = wct.NULL_INT;
            FirstPrinPayNo = wct.NULL_INT;
            PmtPayPct = wct.NULL_DOUBLE;

            result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CDRCashflow : ex1 : " + retStr.ToString());
            File.WriteAllText("CDRCashflow.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //In this example, we have a 10-year, 20,000,000 loan, with monthly payments calculated as though it were a 25-year loan.

            PrinAmt = 20000000;
            InterestRate = 0.05;
            NumPmts = 300;
            LastPmtNum = 120;
            PmtPerYr = 12;

            result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CDRCashflow : ex2 : " + retStr.ToString());
            File.AppendAllText("CDRCashflow.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //In this example we have a 10-year, 25,000,000 interest-only loan with quarterly payments

            PrinAmt = 25000000;
            InterestRate = 0.04;
            NumPmts = 40;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 4;

            int[] LSRates_mo = {1, 91, 181, 271, 361};                  //{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
            Double[] LSRates_rate = {0.15, 0.16, 0.14, 0.17, 0.13};     //{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
            int[] CPRRates_mo = {1, 25, 61, 181};                       //{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
            Double[] CPRRates_rate = {0.002, 0.012, 0.02, 0.06};        //{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
            int[] CDRRates_mo = {1, 61, 181};                           //{{1, 0.1}, {61, 0.08}, {181, 0.05}}
            Double[] CDRRates_rate = {0.1, 0.08, 0.05};                 //{{1, 0.1}, {61, 0.08}, {181, 0.05}}

            result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_mo, LSRates_rate, CPRRates_mo, CPRRates_rate, CDRRates_mo, CDRRates_rate, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CDRCashflow : ex3 : " + retStr.ToString());
            File.AppendAllText("CDRCashflow.csv", vbCrLf + "ex3 : " + retStr.ToString());
        }

        public static void test_ConstantCashFlow()
        {
            Double OutstandingAmount;
            Double LastPrinPayAmount;
            Double InterestRate;
            int PaymentFrequency;
            DateTime MaturityDate;
            DateTime ReferenceDate;
            DateTime PrevPayDate;
            DateTime StartDate;
            DateTime FirstPayDate;
            DateTime GracePeriodStartDate;
            DateTime GracePeriodEndDate;
            FinancialTypes.ConstantCashFlow_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a simple a 300,000 loan with quarterly payments and an interest rate of 6.0%.
            OutstandingAmount = 300000;
            LastPrinPayAmount = 0;
            InterestRate = 0.06;
            PaymentFrequency = 3;
            MaturityDate = new DateTime(2019, 12, 15);
            ReferenceDate = new DateTime(2014, 12, 15);
            PrevPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantCashFlow.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Modify the terms to reflect a final principal payment of 172,000.
            LastPrinPayAmount = 172000;

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlow.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Add a first payment date
            FirstPayDate = new DateTime(2015, 6, 15);

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlow.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //No payments in 2018
            GracePeriodStartDate = new DateTime(2018, 1, 1);
            GracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlow.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Don't use first payment date; previous payment date is 2014-11-15
            FirstPayDate = wct.NULL_DATE;
            PrevPayDate = new DateTime(2014, 11, 15);

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlow.csv", vbCrLf + "ex5 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 6
            //=====================

            //Get rid of the grace period an add a start date.
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;
            StartDate = new DateTime(2014, 10, 15);

            result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlow : ex6 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlow.csv", vbCrLf + "ex6 : " + retStr.ToString());
        }
                
        public static void test_ConstantCashFlowFR()
        {
            Double OutstandingAmount;
            Double LastPrinPayAmount;
            int PaymentFrequency;
            DateTime MaturityDate;
            DateTime ReferenceDate; ;
            DateTime PrevPayDate;
            DateTime StartDate;
            DateTime FirstPayDate;
            DateTime GracePeriodStartDate;
            DateTime GracePeriodEndDate;
            DateTime[] FutureDates;
            Double[] FutureRates;
            FinancialTypes.ConstantCashFlowFR_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            FutureDates = new DateTime[] { new DateTime(2014, 12, 31), new DateTime(2015, 1, 31), new DateTime(2015, 2, 28), new DateTime(2015, 3, 31), new DateTime(2015, 4, 30), new DateTime(2015, 5, 31), new DateTime(2015, 6, 30), new DateTime(2015, 7, 31), new DateTime(2015, 8, 31), new DateTime(2015, 9, 30), new DateTime(2015, 10, 31), new DateTime(2015, 11, 30), new DateTime(2015, 12, 31), new DateTime(2016, 1, 31), new DateTime(2016, 2, 29), new DateTime(2016, 3, 31), new DateTime(2016, 4, 30), new DateTime(2016, 5, 31), new DateTime(2016, 6, 30), new DateTime(2016, 7, 31), new DateTime(2016, 8, 31), new DateTime(2016, 9, 30), new DateTime(2016, 10, 31), new DateTime(2016, 11, 30), new DateTime(2016, 12, 31), new DateTime(2017, 1, 31), new DateTime(2017, 2, 28), new DateTime(2017, 3, 31), new DateTime(2017, 4, 30), new DateTime(2017, 5, 31), new DateTime(2017, 6, 30), new DateTime(2017, 7, 31), new DateTime(2017, 8, 31), new DateTime(2017, 9, 30), new DateTime(2017, 10, 31), new DateTime(2017, 11, 30), new DateTime(2017, 12, 31), new DateTime(2018, 1, 31), new DateTime(2018, 2, 28), new DateTime(2018, 3, 31), new DateTime(2018, 4, 30), new DateTime(2018, 5, 31), new DateTime(2018, 6, 30), new DateTime(2018, 7, 31), new DateTime(2018, 8, 31), new DateTime(2018, 9, 30), new DateTime(2018, 10, 31), new DateTime(2018, 11, 30), new DateTime(2018, 12, 31), new DateTime(2019, 1, 31), new DateTime(2019, 2, 28), new DateTime(2019, 3, 31), new DateTime(2019, 4, 30), new DateTime(2019, 5, 31), new DateTime(2019, 6, 30), new DateTime(2019, 7, 31), new DateTime(2019, 8, 31), new DateTime(2019, 9, 30), new DateTime(2019, 10, 31), new DateTime(2019, 11, 30), new DateTime(2019, 12, 31) };
            FutureRates = new Double[] { 0.025, 0.025102, 0.025205, 0.025308, 0.02541, 0.025513, 0.025615, 0.025718, 0.02582, 0.025923, 0.026025, 0.026128, 0.026231, 0.026333, 0.026436, 0.026539, 0.026641, 0.026744, 0.026847, 0.026949, 0.027052, 0.027155, 0.027257, 0.02736, 0.027463, 0.027566, 0.027668, 0.027771, 0.027874, 0.027977, 0.028079, 0.028182, 0.028285, 0.028388, 0.028491, 0.028594, 0.028696, 0.028799, 0.028902, 0.029005, 0.029108, 0.029211, 0.029314, 0.029417, 0.02952, 0.029623, 0.029726, 0.029829, 0.029932, 0.030035, 0.030138, 0.030241, 0.030344, 0.030447, 0.03055, 0.030653, 0.030756, 0.030859, 0.030962, 0.031065, 0.031168 };

            //This is a simple a 300,000 loan with quarterly payments and an interest rate of 6.0%.
            OutstandingAmount = 300000;
            LastPrinPayAmount = 0;
            PaymentFrequency = 3;
            MaturityDate = new DateTime(2019, 12, 15);
            ReferenceDate = new DateTime(2014, 12, 15);
            PrevPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR: ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantCashFlowFR.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Modify the terms to reflect a final principal payment of 172,000.
            LastPrinPayAmount = 172000;

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Add a first payment date and set the last principal payment amount to 0.
            FirstPayDate = new DateTime(2015, 6, 15);
            LastPrinPayAmount = 0;

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //No payemnts in 2018
            GracePeriodStartDate = new DateTime(2018, 1, 1);
            GracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Don't use first payment date; previous payment date is 2014-11-15
            FirstPayDate = wct.NULL_DATE;
            PrevPayDate = new DateTime(2014, 11, 15);

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf + "ex5 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 6
            //=====================

            //Get rid of the grace period an add a starte date.
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;
            StartDate = new DateTime(2014, 10, 15);

            result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantCashFlowFR : ex6 : " + retStr.ToString());
            File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf + "ex6 : " + retStr.ToString());
        }
        
        public static void test_ConstantPaymentAmount()
        {
            Double OutstandingAmount;
            FinancialTypes.InterestBasisType InterestBasis;
            Double InterestRate;
            int PaymentFrequency;
            Double PaymentAmount;
            DateTime ReferenceDate;
            DateTime PrevPayDate;
            DateTime StartDate;
            DateTime FirstPayDate;
            DateTime GracePeriodStartDate;
            DateTime GracePeriodEndDate;
            FinancialTypes.ConstantPaymentAmount_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a simple 300,000 loan with quarterly payments of 17,500 and an interest rate of 5%.
            OutstandingAmount = 300000;
            InterestBasis = FinancialTypes.InterestBasisType.A360;
            InterestRate = 0.05;
            PaymentFrequency = 3;
            PaymentAmount = 17500;
            ReferenceDate = new DateTime(2014, 12, 15);
            PrevPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPaymentAmount : ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantPaymentAmount.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Add a first payment date so that the first payment will not be paid until 30-June-2015.
            FirstPayDate = new DateTime(2015, 6, 30);

            result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPaymentAmount : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //No payemnts in 2018
            GracePeriodStartDate = new DateTime(2018, 1, 1);
            GracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPaymentAmount : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Don't use first payment date; previous payment date is 2014-11-15
            FirstPayDate = wct.NULL_DATE;
            PrevPayDate = new DateTime(2014, 11, 15);

            result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPaymentAmount : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Get rid of the grace period, the previous payment date and add a start date.
            GracePeriodStartDate = wct.NULL_DATE;
            GracePeriodEndDate = wct.NULL_DATE;
            PrevPayDate = wct.NULL_DATE;
            StartDate = new DateTime(2014, 10, 15);

            result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPaymentAmount : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf + "ex5 : " + retStr.ToString());
        }

        public static void test_ConstantPrincipal()
        {
            Double OutstandingAmount;
            String InterestBasis;
            Double InterestRate;
            int PrincipalPaymentFrequency;
            int InterestPaymentFrequency;
            Double LastPrinPayAmount;
            DateTime MaturityDate;
            DateTime ReferenceDate;
            DateTime PrevPrincipalPayDate;
            DateTime PrevInterestPayDate;
            DateTime StartDate;
            DateTime FirstPrincipalPayDate;
            DateTime FirstInterestPayDate;
            DateTime PrincipalGracePeriodStartDate;
            DateTime PrincipalGracePeriodEndDate;
            DateTime InterestGracePeriodStartDate;
            DateTime InterestGracePeriodEndDate;
            StringBuilder retStr;
            FinancialTypes.ConstantPrincipal_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a simple 5-year 100,000 loan with quarterly payments of principal and interest and an interest rate of 4.5%.

            OutstandingAmount = 100000;
            InterestBasis = "30/360";
            InterestRate = 0.045;
            PrincipalPaymentFrequency = 3;
            InterestPaymentFrequency = 3;
            LastPrinPayAmount = 10000.0;
            MaturityDate = new DateTime(2019, 10, 1);
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = wct.NULL_DATE;
            PrevInterestPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipal : ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantPrincipal.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //monthly payments of principal and quarterly payments of interest

            OutstandingAmount = 100000;
            InterestBasis = "30/360";
            InterestRate = 0.045;
            PrincipalPaymentFrequency = 1;
            InterestPaymentFrequency = 3;
            LastPrinPayAmount = 10000.0;
            MaturityDate = new DateTime(2019, 10, 1);
            ReferenceDate = new DateTime(2014, 10, 1);

            result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //'Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipal : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipal.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.

            OutstandingAmount = 100000;
            InterestBasis = "30/360";
            InterestRate = 0.045;
            PrincipalPaymentFrequency = 1;
            InterestPaymentFrequency = 3;
            LastPrinPayAmount = 10000.0;
            MaturityDate = new DateTime(2019, 10, 1);
            ReferenceDate = new DateTime(2014, 10, 1);
            FirstPrincipalPayDate = new DateTime(2015, 3, 1);
            FirstInterestPayDate = new DateTime(2015, 1, 1);

            result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipal : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipal.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //there are no payments in 2018

            OutstandingAmount = 100000;
            InterestBasis = "30/360";
            InterestRate = 0.045;
            PrincipalPaymentFrequency = 1;
            InterestPaymentFrequency = 3;
            LastPrinPayAmount = 10000.0;
            MaturityDate = new DateTime(2019, 10, 1);
            ReferenceDate = new DateTime(2014, 10, 1);
            //,NULL              --@PrevPrincipalPayDate
            //,NULL              --@PrevInterestPayDate
            //,NULL              --@StartDate
            FirstPrincipalPayDate = new DateTime(2015, 3, 1);
            FirstInterestPayDate = new DateTime(2015, 1, 1);
            PrincipalGracePeriodStartDate = new DateTime(2018, 1, 1);
            PrincipalGracePeriodEndDate = new DateTime(2019, 1, 1);
            InterestGracePeriodStartDate = new DateTime(2018, 1, 1);
            InterestGracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipal : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipal.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //The principal payment frequency to every 5 months, the interest payment frequency to every 6 months, remove the grace periods, and calculate the first payment dates from the previous interest and previous principal payemnt dates.

            OutstandingAmount = 100000;
            InterestBasis = "30/360";
            InterestRate = 0.045;
            PrincipalPaymentFrequency = 5;
            InterestPaymentFrequency = 6;
            LastPrinPayAmount = 10000.0;
            MaturityDate = new DateTime(2019, 10, 1);
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = new DateTime(2014, 6, 1);
            PrevInterestPayDate = new DateTime(2014, 5, 1);
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipal : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipal.csv", vbCrLf + "ex5 : " + retStr.ToString());
        }
        
        public static void test_ConstantPrincipalAmount()
        {
            Double OutstandingAmount;
            FinancialTypes.InterestBasisType InterestBasis;
            Double InterestRate;
            int FreqPayPrincipal;
            int FreqPayInterest;
            Double PrinPayAmount;
            DateTime ReferenceDate;
            DateTime PrevPrincipalPayDate;
            DateTime PrevInterestPayDate;
            DateTime StartDate;
            DateTime FirstPrincipalPayDate;
            DateTime FirstInterestPayDate;
            DateTime PrincipalGracePeriodStartDate; ;
            DateTime PrincipalGracePeriodEndDate;
            DateTime InterestGracePeriodStartDate;
            DateTime InterestGracePeriodEndDate;
            FinancialTypes.ConstantPrincipalAmount_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a 100,000 loan with quarterly payments of principal and interest and an interest rate of 3.0%. 
            //The principal payment is 5,250.
            OutstandingAmount = 100000;
            InterestBasis = FinancialTypes.InterestBasisType.A365;
            InterestRate = 0.03;
            FreqPayPrincipal = 3;
            FreqPayInterest = 3;
            PrinPayAmount = 5250.0;
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = wct.NULL_DATE;
            PrevInterestPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalAmount : ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantPrincipalAmount.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Modify the loan to have monthly payments of principal and semi-annual payments of interest
            FreqPayPrincipal = 1;
            FreqPayInterest = 6;

            result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalAmount : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Modify the loan so that first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.
            FirstPrincipalPayDate = new DateTime(2015, 3, 1);
            FirstInterestPayDate = new DateTime(2015, 1, 1);

            result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalAmount : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //No payments in 2016
            PrincipalGracePeriodStartDate = new DateTime(2016, 1, 1);
            PrincipalGracePeriodEndDate = new DateTime(2017, 1, 1);
            InterestGracePeriodStartDate = new DateTime(2016, 1, 1);
            InterestGracePeriodEndDate = new DateTime(2017, 1, 1);

            result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalAmount : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Modify the loan so that the principal payment frequency will be every 5 months and 
            //the first payment dates are calculated from the previous interest and previous principal payment dates.
            FreqPayPrincipal = 5;
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = new DateTime(2014, 6, 1);
            PrevInterestPayDate = new DateTime(2014, 9, 1);
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalAmount : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf + "ex5 : " + retStr.ToString());
        }
        
        public static void test_ConstantPrincipalRate()
        {
            Double OutstandingAmount;
            FinancialTypes.InterestBasisType InterestBasis;
            Double InterestRate;
            int FreqPayPrincipal;
            int FreqPayInterest;
            Double AmortizationRate;
            Double MinimumPayment;
            DateTime ReferenceDate;
            DateTime PrevPrincipalPayDate;
            DateTime PrevInterestPayDate;
            DateTime StartDate;
            DateTime FirstPrincipalPayDate;
            DateTime FirstInterestPayDate;
            DateTime PrincipalGracePeriodStartDate;
            DateTime PrincipalGracePeriodEndDate;
            DateTime InterestGracePeriodStartDate;
            DateTime InterestGracePeriodEndDate;
            FinancialTypes.ConstantPrincipalRate_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //This is a 100,000 loan with quarterly payments of principal and interest and an interest rate of 3.0%. 
            //The principal payments are calculated as 13.0% of the outstanding principal balance, with a minimum payment of 2,500.00.
            OutstandingAmount = 100000;
            InterestBasis = FinancialTypes.InterestBasisType.A365;
            InterestRate = 0.03;
            FreqPayPrincipal = 3;
            FreqPayInterest = 3;
            AmortizationRate = 0.13;
            MinimumPayment = 2500;
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = wct.NULL_DATE;
            PrevInterestPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalRate : ex1 : " + retStr.ToString());
            File.WriteAllText("ConstantPrincipalRate.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Modify the loan to have monthly payments of principal and semi-annual payments of interest
            FreqPayPrincipal = 1;
            FreqPayInterest = 6;

            result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalRate : ex2 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            // Modify the loan so that first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.
            FirstPrincipalPayDate = new DateTime(2015, 3, 1);
            FirstInterestPayDate = new DateTime(2015, 1, 1);

            result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalRate : ex3 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //No payments in 2016
            PrincipalGracePeriodStartDate = new DateTime(2016, 1, 1);
            PrincipalGracePeriodEndDate = new DateTime(2017, 1, 1);
            InterestGracePeriodStartDate = new DateTime(2016, 1, 1);
            InterestGracePeriodEndDate = new DateTime(2017, 1, 1);

            result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalRate : ex4 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            // Modify the loan so that the principal payment frequency will be every 5 months and 
            // the first payment dates are calculated from the previous interest and previous principal payment dates.
            FreqPayPrincipal = 5;
            ReferenceDate = new DateTime(2014, 10, 1);
            PrevPrincipalPayDate = new DateTime(2014, 6, 1);
            PrevInterestPayDate = new DateTime(2014, 9, 1);
            StartDate = wct.NULL_DATE;
            FirstPrincipalPayDate = wct.NULL_DATE;
            FirstInterestPayDate = wct.NULL_DATE;
            PrincipalGracePeriodStartDate = wct.NULL_DATE;
            PrincipalGracePeriodEndDate = wct.NULL_DATE;
            InterestGracePeriodStartDate = wct.NULL_DATE;
            InterestGracePeriodEndDate = wct.NULL_DATE;

            result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("ConstantPrincipalRate : ex5 : " + retStr.ToString());
            File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf + "ex5 : " + retStr.ToString());
        }
        
        public static void test_CONSTPRINAMORT()
        {
            Double pv;
            Double rate;
            DateTime LoanDate;
            int NumPmtsPerYear;
            DateTime FirstPaymentDate;
            int DaysInYear;
            int NumberOfPayments;
            int LastPaymentNumber;
            int FirstPrinPayNo;
            Double FV;
            Double PPMT;
            Boolean eom;
            FinancialTypes.CONSTPRINAMORT_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //A 1,000,000 loan dated 2014-05-14 with 50 monthly payments commencing on 2014-06-15. The interest rate is 6.0%.
            pv = 1000000;
            rate = 0.06;
            LoanDate = new DateTime(2014, 5, 15);
            NumPmtsPerYear = 12;
            FirstPaymentDate = new DateTime(2014, 6, 15);
            DaysInYear = 360;
            NumberOfPayments = 50;
            LastPaymentNumber = wct.NULL_INT;
            FirstPrinPayNo = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            PPMT = wct.NULL_DOUBLE;
            eom = true;

            result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CONSTPRINAMORT : ex1 : " + retStr.ToString());
            File.WriteAllText("CONSTPRINAMORT.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Same loan except the first payment date is changed to 2014-06-30
            FirstPaymentDate = new DateTime(2014, 6, 30);

            result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CONSTPRINAMORT : ex2 : " + retStr.ToString());
            File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //In this example there are 26 payments per year and the principal is amortized as though the loan is maturing in 5 years, 
            //but it will be paid off, in full, in 2 years
            pv = 1000000;
            rate = 0.06;
            LoanDate = new DateTime(2014, 5, 15);
            NumPmtsPerYear = 26;
            FirstPaymentDate = new DateTime(2014, 6, 30);
            DaysInYear = 365;
            NumberOfPayments = 130;
            LastPaymentNumber = 52;
            FirstPrinPayNo = wct.NULL_INT;
            FV = wct.NULL_DOUBLE;
            PPMT = wct.NULL_DOUBLE;
            eom = true;

            result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CONSTPRINAMORT : ex3 : " + retStr.ToString());
            File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //In this example, no interest is due until the 14th installment and then the principal will be 
            //paid down by 25,000 with each payment with balance paid off at maturity.
            pv = 1000000;
            rate = 0.06;
            LoanDate = new DateTime(2014, 5, 15);
            NumPmtsPerYear = 26;
            FirstPaymentDate = new DateTime(2014, 6, 30);
            DaysInYear = 365;
            NumberOfPayments = 52;
            LastPaymentNumber = wct.NULL_INT;
            FirstPrinPayNo = 14;
            FV = wct.NULL_DOUBLE;
            PPMT = 25000;
            eom = true;

            result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("CONSTPRINAMORT : ex4 : " + retStr.ToString());
            File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf + "ex4 : " + retStr.ToString());
        }
        
        public static void test_NPD()
        {
            DateTime SettleDate;
            DateTime FirstPayDate;
            int PmtPerYr;
            int NumPmts;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the next payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
            SettleDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 9, 15);
            PmtPerYr = 12;
            NumPmts = 60;

            result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts);

            Console.WriteLine("NPD : " + result.ToString());         //11/15/2010
            File.WriteAllText("NPD.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 7, 29);
            PmtPerYr = 26;

            result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPD : " + result.ToString());         //11/3/2010
            File.AppendAllText("NPD.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 3, 15);
            PmtPerYr = 24;

            result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPD : " + result.ToString());         //10/31/2010
            File.AppendAllText("NPD.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Calculate the maturity date on a loan by entering a date far in the future.
            SettleDate = new DateTime(2099, 12, 31);
            FirstPayDate = new DateTime(2009, 3, 15);
            PmtPerYr = 26;
            NumPmts = 78;

            result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts);

            Console.WriteLine("NPD : " + result.ToString());         //2/26/2012
            File.AppendAllText("NPD.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
            SettleDate = new DateTime(2010, 12, 6);
            FirstPayDate = new DateTime(2010, 12, 15);
            PmtPerYr = 12;

            result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPD : " + result.ToString());         //12/15/2010
            File.AppendAllText("NPD.csv", vbCrLf + result.ToString());
        }
        
        public static void test_NPNO()
        {
            DateTime SettleDate;
            DateTime FirstPayDate;
            int PmtPerYr;
            int NumPmts;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the next payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
            SettleDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 9, 15);
            PmtPerYr = 12;
            NumPmts = 60;

            result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr, NumPmts);

            Console.WriteLine("NPNO : " + result.ToString());         //3
            File.WriteAllText("NPNO.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 7, 29);
            PmtPerYr = 26;

            result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPNO : " + result.ToString());         //34
            File.AppendAllText("NPNO.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 3, 15);
            PmtPerYr = 24;

            result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPNO : " + result.ToString());         //40
            File.AppendAllText("NPNO.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
            SettleDate = new DateTime(2010, 12, 6);
            FirstPayDate = new DateTime(2010, 12, 15);
            PmtPerYr = 12;

            result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("NPNO : " + result.ToString());         //1
            File.AppendAllText("NPNO.csv", vbCrLf + result.ToString());
        }
        
        public static void test_PAYMENTPERIODS()
        {
            DateTime ReferenceDate;
            int PaymentFrequency;
            DateTime PrevPayDate;
            DateTime StartDate;
            DateTime FirstPayDate;
            DateTime InterimGracePeriodStartDate;
            DateTime InterimGracePeriodEndDate;
            DateTime MaturityDate;
            FinancialTypes.OutputRow_PAYMENTPERIODS result;

            //=====================
            // EXAMPLE# 1
            //=====================

            // loan with quarterly payments, a Reference Date of 2014-09-15 and a Maturity Date of 2019-09-15.
            ReferenceDate = new DateTime(2014, 9, 15);
            PaymentFrequency = 3;
            PrevPayDate = wct.NULL_DATE;
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            InterimGracePeriodStartDate = wct.NULL_DATE;
            InterimGracePeriodEndDate = wct.NULL_DATE;
            MaturityDate = new DateTime(2019, 9, 15);

            result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate);

            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "InitialGracePeriod:" + result.InitialGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "InterimGracePeriodMonthStart : " + result.InterimGracePeriodMonthStart);
            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "InterimGracePeriodMonthEnd : " + result.InterimGracePeriodMonthEnd);
            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "MonthsUntilFirstPayment" + result.MonthsUntilFirstPayment);
            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "InterimGracePeriod : " + result.InterimGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex1 : " + "NumberOfPayments : " + result.NumberOfPayments);

            File.WriteAllText("PAYMENTPERIODS.csv", "ex1 : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Add a first payment date
            FirstPayDate = new DateTime(2015, 3, 15);

            result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate);

            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "InitialGracePeriod:" + result.InitialGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "InterimGracePeriodMonthStart : " + result.InterimGracePeriodMonthStart);
            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "InterimGracePeriodMonthEnd : " + result.InterimGracePeriodMonthEnd);
            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "MonthsUntilFirstPayment" + result.MonthsUntilFirstPayment);
            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "InterimGracePeriod : " + result.InterimGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex2 : " + "NumberOfPayments : " + result.NumberOfPayments);

            File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf + "ex2 : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //No payments in 2018
            InterimGracePeriodStartDate = new DateTime(2018, 1, 1);
            InterimGracePeriodEndDate = new DateTime(2019, 1, 1);

            result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate);

            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "InitialGracePeriod:" + result.InitialGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "InterimGracePeriodMonthStart : " + result.InterimGracePeriodMonthStart);
            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "InterimGracePeriodMonthEnd : " + result.InterimGracePeriodMonthEnd);
            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "MonthsUntilFirstPayment" + result.MonthsUntilFirstPayment);
            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "InterimGracePeriod : " + result.InterimGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex3 : " + "NumberOfPayments : " + result.NumberOfPayments);

            File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf + "ex3 : " + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Clear out every date except Reference Date and Maturity Date and add a previous payment date.
            PrevPayDate = new DateTime(2014, 8, 15);
            StartDate = wct.NULL_DATE;
            FirstPayDate = wct.NULL_DATE;
            InterimGracePeriodStartDate = wct.NULL_DATE;
            InterimGracePeriodEndDate = wct.NULL_DATE;

            result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate);

            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "InitialGracePeriod:" + result.InitialGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "InterimGracePeriodMonthStart : " + result.InterimGracePeriodMonthStart);
            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "InterimGracePeriodMonthEnd : " + result.InterimGracePeriodMonthEnd);
            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "MonthsUntilFirstPayment" + result.MonthsUntilFirstPayment);
            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "InterimGracePeriod : " + result.InterimGracePeriod);
            Console.WriteLine("PAYMENTPERIODS : ex4 : " + "NumberOfPayments : " + result.NumberOfPayments);

            File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf + "ex4 : " + result.ToString());
        }
        
        public static void test_PERIODRATE()
        {
            Double Nominal;
            Double ComPeriod;
            Double PayPeriod;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Nominal = 0.07;
            ComPeriod = 2;           //semi-annual compounding
            PayPeriod = 12;          //monthly compounding

            result = wct.PERIODRATE(Nominal, ComPeriod, PayPeriod);

            Console.WriteLine("PERIODRATE : " + result.ToString());          //0.0690004739713013
            File.WriteAllText("PERIODRATE.csv", result.ToString());
        }
        
        public static void test_PPD()
        {
            DateTime SettleDate;
            DateTime FirstPayDate;
            int PmtPerYr;
            int NumPmts;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the previous payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
            SettleDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 9, 15);
            PmtPerYr = 12;
            NumPmts = 60;

            result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts);

            Console.WriteLine("PPD : " + result.ToString());         //10/15/2010
            File.WriteAllText("PPD.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the previous payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 7, 29);
            PmtPerYr = 26;

            result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPD : " + result.ToString());         //10/20/2010
            File.AppendAllText("PPD.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the previous payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 3, 15);
            PmtPerYr = 24;

            result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPD : " + result.ToString());         //10/31/2010
            File.AppendAllText("PPD.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Calculate the previous payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
            SettleDate = new DateTime(2010, 12, 6);
            FirstPayDate = new DateTime(2010, 12, 15);
            PmtPerYr = 12;

            result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPD : " + (wct.IsNull(result) ? "NULL" : result.ToString()));         //NULL
            File.AppendAllText("PPD.csv", vbCrLf + result.ToString());
        }
        
        public static void test_PPNO()
        {
            DateTime SettleDate;
            DateTime FirstPayDate;
            int PmtPerYr;
            int NumPmts;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the previous payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
            SettleDate = new DateTime(2010, 11, 1);
            FirstPayDate = new DateTime(2010, 9, 15);
            PmtPerYr = 12;
            NumPmts = 60;

            result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr, NumPmts);

            Console.WriteLine("PPNO : " + result.ToString());         //2
            File.WriteAllText("PPNO.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 7, 29);
            PmtPerYr = 26;

            result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPNO : " + result.ToString());         //33
            File.AppendAllText("PPNO.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
            SettleDate = new DateTime(2010, 10, 29);
            FirstPayDate = new DateTime(2009, 3, 15);
            PmtPerYr = 24;

            result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPNO : " + result.ToString());         //39
            File.AppendAllText("PPNO.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
            SettleDate = new DateTime(2010, 12, 6);
            FirstPayDate = new DateTime(2010, 12, 15);
            PmtPerYr = 12;

            result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr);

            Console.WriteLine("PPNO : " + result.ToString());         //NULL
            File.AppendAllText("PPNO.csv", vbCrLf + result.ToString());
        }
        
        public static void test_SMMAmort()
        {
            Double PrinAmt;
            Double InterestRate;
            int NumPmts;
            int LastPmtNum;
            int PmtPerYr;
            Boolean InterestOnly;
            int PrinPaymentMultiple;
            int FirstPrinPayNo;
            Double PmtPayPct;
            StringBuilder retStr;
            FinancialTypes.CDRCashflow_table result;

            //=====================
            // EXAMPLE# 1
            //=====================

            System.Array LSRates_array = new double[,] { { 1, 0.15 }, { 91, 0.16 }, { 181, 0.14 }, { 271, 0.17 }, { 361, 0.13 } };
            System.Array CPRRates_array = new double[,] { { 1, 0.002 }, { 25, 0.012 }, { 61, 0.02 }, { 181, 0.06 } };
            System.Array CDRRates_array = new double[,] { { 1, 0.1 }, { 61, 0.08 }, { 181, 0.05 } };

            //In this example we amortize a 100,000,000 loan with a 5% interest rate over 360 months 

            PrinAmt = 100000000;
            InterestRate = 0.05;
            NumPmts = 360;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 12;
            InterestOnly = false;
            PrinPaymentMultiple = wct.NULL_INT;
            FirstPrinPayNo = wct.NULL_INT;
            PmtPayPct = wct.NULL_DOUBLE;

            result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SMMAmort : ex1 : " + retStr.ToString());
            File.WriteAllText("SMMAmort.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //In this example, we have a 10-year, 20,000,000 loan, with monthly payments calculated as though it were a 25-year loan.

            PrinAmt = 20000000;
            InterestRate = 0.05;
            NumPmts = 300;
            LastPmtNum = 120;
            PmtPerYr = 12;

            result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SMMAmort : ex2 : " + retStr.ToString());
            File.AppendAllText("SMMAmort.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //In this example we have a 10-year, 25,000,000 interest-only loan with quarterly payments

            PrinAmt = 25000000;
            InterestRate = 0.04;
            NumPmts = 40;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 4;

            int[] LSRates_mo = { 1, 91, 181, 271, 361 };                  //{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
            Double[] LSRates_rate = { 0.15, 0.16, 0.14, 0.17, 0.13 };     //{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
            int[] CPRRates_mo = { 1, 25, 61, 181 };                       //{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
            Double[] CPRRates_rate = { 0.002, 0.012, 0.02, 0.06 };        //{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
            int[] CDRRates_mo = { 1, 61, 181 };                           //{{1, 0.1}, {61, 0.08}, {181, 0.05}}
            Double[] CDRRates_rate = { 0.1, 0.08, 0.05 };                 //{{1, 0.1}, {61, 0.08}, {181, 0.05}}

            result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_mo, LSRates_rate, CPRRates_mo, CPRRates_rate, CDRRates_mo, CDRRates_rate, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SMMAmort : ex3 : " + retStr.ToString());
            File.AppendAllText("SMMAmort.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //In this example we have a 10,000,000 loan with monthly payments of interest and quarterly payments of principal. The loan matures in 120 months.

            PrinAmt = 10000000;
            InterestRate = 0.06;
            NumPmts = 120;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 12;
            InterestOnly = false;
            PrinPaymentMultiple = 3;
            FirstPrinPayNo = wct.NULL_INT;
            PmtPayPct = wct.NULL_DOUBLE;

            result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SMMAmort : ex4 : " + retStr.ToString());
            File.AppendAllText("SMMAmort.csv", vbCrLf + "ex4 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 5
            //=====================

            //In this example we have a 10,000,000 loan with monthly payments of interest and quarterly payments of principal. The loan matures in 120 months.

            PrinAmt = 10000000;
            InterestRate = 0.06;
            NumPmts = 119;
            LastPmtNum = wct.NULL_INT;
            PmtPerYr = 12;
            InterestOnly = false;
            PrinPaymentMultiple = 3;
            FirstPrinPayNo = 2;
            PmtPayPct = wct.NULL_DOUBLE;

            result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SMMAmort : ex5 : " + retStr.ToString());
            File.AppendAllText("SMMAmort.csv", vbCrLf + "ex5 : " + retStr.ToString());

        }
        
        public static void test_UNEQUALLOANPAYMENTS()
        {
            Double pv;
            Double rate;
            DateTime loandate;
            int interestfrequency;
            DateTime firstpaymentdate;
            int daysinyear;
            int prinpaymentmultiple;
            int firstprinpayno;
            int numberofpayments;
            int lastpaymentnumber;
            Double fv;
            Boolean IsRegPay;
            FinancialTypes.UNEQUALLOANPAYMENTS_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            //A 1,000,000 loan dated 2014-01-15 with monthly payments commencing on 2014-02-15 and 
            //quarterly principal payments commencing with the third interest payment. 
            //Notice that the quarterly payments are constant at 130,690.30 but 
            //that interest payments very from quarter to quarter though they are the same for each month between principal payments.
            pv = 1000000;
            rate = 0.12;
            loandate = new DateTime(2014, 1, 15);
            interestfrequency = 12;
            firstpaymentdate = new DateTime(2014, 2, 15);
            daysinyear = wct.NULL_INT;
            prinpaymentmultiple = 3;
            firstprinpayno = 3;
            numberofpayments = 24;
            lastpaymentnumber = wct.NULL_INT;
            fv = 0;
            IsRegPay = true;

            result = wct.UNEQUALLOANPAYMENTS(pv, rate, loandate, interestfrequency, firstpaymentdate, daysinyear, prinpaymentmultiple, firstprinpayno, numberofpayments, lastpaymentnumber, fv, IsRegPay);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("UNEQUALLOANPAYMENTS : ex1 : " + retStr.ToString());
            File.WriteAllText("UNEQUALLOANPAYMENTS.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //This loan, has monthly payments and is interest only for the first 6 months, with payments calculated assuming a 25-year maturity but payoff is at the end of 5 years.
            pv = 1000000;
            rate = 0.05;
            loandate = new DateTime(2014, 1, 15);
            interestfrequency = 12;
            firstpaymentdate = new DateTime(2014, 2, 15);
            daysinyear = wct.NULL_INT;
            prinpaymentmultiple = 1;
            firstprinpayno = 7;
            numberofpayments = 300;
            lastpaymentnumber = 60;
            fv = 0;
            IsRegPay = true;

            result = wct.UNEQUALLOANPAYMENTS(pv, rate, loandate, interestfrequency, firstpaymentdate, daysinyear, prinpaymentmultiple, firstprinpayno, numberofpayments, lastpaymentnumber, fv, IsRegPay);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("UNEQUALLOANPAYMENTS : ex2 : " + retStr.ToString());
            File.AppendAllText("UNEQUALLOANPAYMENTS.csv", vbCrLf + "ex2 : " + retStr.ToString());
        }
        
        public static void test_R78IPMT()
        {
            Double LoanAmount;
            DateTime LoanStartDate;
            Double Rate;
            DateTime FirstPaymentDate;
            int NumPmts;
            int PmtPerYear;
            int NumPeriod;
            int DaysInYear;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
            LoanAmount = 50000;
            LoanStartDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPaymentDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            PmtPerYear = 12;
            NumPeriod = 1;
            DaysInYear = wct.NULL_INT;
            FV = 0;
            IntRule = "A";

            result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78IPMT : " + result.ToString());         //262.242773386387
            File.WriteAllText("R78IPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
            LoanAmount = 50000;
            LoanStartDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPaymentDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            PmtPerYear = 12;
            NumPeriod = 60;
            DaysInYear = wct.NULL_INT;
            FV = 0;
            IntRule = "A";

            result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78IPMT : " + result.ToString());       //4.37071288977312
            File.AppendAllText("R78IPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the first interest payment for a 150,000 4-year loan at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January and with payments due every 4 weeks after that.
            LoanAmount = 150000;
            LoanStartDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPaymentDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            PmtPerYear = 13;
            NumPeriod = 1;
            DaysInYear = 365;
            FV = 0;
            IntRule = "U";

            result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78IPMT : " + result.ToString());         //1615.88014621541
            File.AppendAllText("R78IPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78IPMT : " + result.ToString());         //1616.04600080377    
            File.AppendAllText("R78IPMT.csv", vbCrLf + result.ToString());
        }

        public static void test_R78PPMT()
        {
            Double LoanAmount;
            DateTime LoanStartDate;
            Double Rate;
            DateTime FirstPaymentDate;
            int NumPmts;
            int PmtPerYear;
            int NumPeriod;
            int DaysInYear;
            Double FV;
            String IntRule;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the first interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
            LoanAmount = 50000;
            LoanStartDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPaymentDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            PmtPerYear = 12;
            NumPeriod = 1;
            DaysInYear = wct.NULL_INT;
            FV = 0;
            IntRule = "A";

            result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78PPMT : " + result.ToString());         //704.397303085026
            File.WriteAllText("R78PPMT.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
            LoanAmount = 50000;
            LoanStartDate = new DateTime(2010, 11, 1);
            Rate = 0.06;
            FirstPaymentDate = new DateTime(2010, 12, 1);
            NumPmts = 60;
            PmtPerYear = 12;
            NumPeriod = 60;
            DaysInYear = wct.NULL_INT;
            FV = 0;
            IntRule = "A";

            result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78PPMT : " + result.ToString());       //962.26936358164
            File.AppendAllText("R78PPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate the first interest payment for a 150,000 4-year loan at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January and with payments due every 4 weeks after that.
            LoanAmount = 150000;
            LoanStartDate = new DateTime(2010, 10, 13);
            Rate = 0.12;
            FirstPaymentDate = new DateTime(2011, 1, 4);
            NumPmts = 52;
            PmtPerYear = 13;
            NumPeriod = 1;
            DaysInYear = 365;
            FV = 0;
            IntRule = "U";

            result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78PPMT : " + result.ToString());         //2092.2126206059
            File.AppendAllText("R78PPMT.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
            IntRule = "A";

            result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule);

            Console.WriteLine("R78PPMT : " + result.ToString());         //2092.13128806738    
            File.AppendAllText("R78PPMT.csv", vbCrLf + result.ToString());
        }

        public static void test_R78PAYOFF()
        {
            Double LoanAmount;
            Double FV;
            Double IntAmt;
            int NumPmts;
            int PeriodNo;
            Double pmt;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the payoff amount, using the Rule-of-78, for a 24,000 loan that originated on 8/15/2009 at a rate of 5.9%, with payments on the first of every month for 36 months, commencing on 9/1/2009, paid off on 12/1/2010. The monthly payment is 727.37
            FV = 0;
            NumPmts = 36;
            PeriodNo = 16;
            pmt = 727.37;
            LoanAmount = 24000;
            IntAmt = wct.TOTALINT(NumPmts, pmt, LoanAmount, FV);

            result = wct.R78PAYOFF(IntAmt, NumPmts, PeriodNo, pmt);

            Console.WriteLine("R78PAYOFF : " + result.ToString());           //14585.7051351351
            File.WriteAllText("R78PAYOFF.csv", result.ToString());
        }
        
        public static void test_R78REBATE()
        {
            Double IntAmt;
            int NumPmts;
            int PeriodNo;
            Double pmt;
            Double prin;
            Double fv;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Calculate the rebate amount, using the Rule of 78, for a 24,000 loan that originated on 8/15/2009 at a rate of 5.9%, with payments on the first of every month for 36 months, commencing on 9/1/2009, paid off on 12/1/2010. The monthly payment is 727.37
            fv = 0;
            prin = 24000;
            pmt = 727.37;
            NumPmts = 36;
            PeriodNo = 16;
            IntAmt = wct.TOTALINT(NumPmts, pmt, prin, fv);

            result = wct.R78REBATE(IntAmt, NumPmts, PeriodNo);

            Console.WriteLine("R78REBATE : " + result.ToString());           //689.064864864865
            File.WriteAllText("R78REBATE.csv", result.ToString());
        }
        
        public static void test_DB()
        {
            Double Cost;
            Double Salvage;
            Double Life;
            int Per;
            int MonthsInFirstYear;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Cost = 60000;
            Salvage = 0;
            Life = 4;
            Per = 1;
            MonthsInFirstYear = 3;

            result = wct.DB(Cost, Salvage, Life, Per, MonthsInFirstYear);

            Console.WriteLine("DB : " + result.ToString());          //15000    
            File.WriteAllText("DB.csv", result.ToString());
        }
        
        public static void test_DDB()
        {
            Double Cost;
            Double Salvage;
            Double Life;
            int Per;
            Double Factor;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Cost = 60000;
            Salvage = 0;
            Life = 4;
            Per = 1;
            Factor = 2.25;

            result = wct.DDB(Cost, Salvage, Life, Per, Factor);

            Console.WriteLine("DDB : " + result.ToString());          //337500
            File.WriteAllText("DDB.csv", result.ToString());
        }

        public static void test_SLN()
        {
            Double Cost;
            Double Salvage;
            Double Life;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Cost = 600000;
            Salvage = 25000;
            Life = 40;

            result = wct.SLN(Cost, Salvage, Life);

            Console.WriteLine("SLN : " + result.ToString());         //14375
            File.WriteAllText("SLN.csv", result.ToString());
        }

        public static void test_SYD()
        {
            Double Cost;
            Double Salvage;
            Double Life;
            Double Per;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Cost = 30000;
            Salvage = 7500;
            Life = 10;
            Per = 1;

            result = wct.SYD(Cost, Salvage, Life, Per);

            Console.WriteLine("SYD : " + result.ToString());         //4090.90909090909    
            File.WriteAllText("SYD.csv", result.ToString());
        }

        public static void test_VDB()
        {
            Double Cost;
            Double Salvage;
            Double Life;
            Double start_period;
            Double end_period;
            Double Factor;
            Boolean No_switch;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Cost = 2400;
            Salvage = 300;
            Life = 10;
            start_period = 6;
            end_period = 7;
            Factor = 1.5;
            No_switch = true;

            result = wct.VDB(Cost, Salvage, Life, start_period, end_period, Factor, No_switch);

            Console.WriteLine("VDB : " + result.ToString());         //135.773825625
            File.WriteAllText("VDB.csv", result.ToString());
        }

        public static void test_DFINTERP()
        {
            DateTime[] dfdate;
            Double[] df;
            DateTime newdate;
            DateTime startdate;
            FinancialTypes.ZCReturnValues RV;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            dfdate = new DateTime[] { new DateTime(2013, 1, 16), new DateTime(2013, 1, 17), new DateTime(2013, 1, 25), new DateTime(2013, 1, 31), new DateTime(2013, 2, 18), new DateTime(2013, 3, 18), new DateTime(2013, 4, 17), new DateTime(2013, 6, 19), new DateTime(2013, 9, 18), new DateTime(2013, 12, 18), new DateTime(2014, 3, 19), new DateTime(2014, 6, 18), new DateTime(2014, 9, 17), new DateTime(2014, 12, 17), new DateTime(2015, 1, 19), new DateTime(2016, 1, 18), new DateTime(2017, 4, 17), new DateTime(2013, 1, 17), new DateTime(2020, 1, 17), new DateTime(2023, 1, 17), new DateTime(2013, 1, 19) };
            df = new Double[] { 0.999995555575309, 0.99999111117037, 0.999956112706425, 0.999916450742048, 0.999804481000583, 0.999574621744643, 0.999241679910437, 0.998800609148515, 0.998022836090921, 0.997197057207847, 0.996311568695976, 0.995354720378904, 0.994289565586446, 0.993104681356623, 0.992402694592988, 0.98508478388398, 0.973098042807202, 0.955265832115111, 0.906604451702898, 0.820620615064395, 0.385646181323946 };
            newdate = new DateTime(2013, 3, 20);
            startdate = new DateTime(2013, 1, 15);

            //return the interpolated discount factor
            RV = FinancialTypes.ZCReturnValues.DF;

            result = wct.DFINTERP(dfdate, df, newdate, startdate, RV);

            Console.WriteLine("DFINTERP : " + result.ToString());            //0.999555003248605
            File.WriteAllText("DFINTERP.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            newdate = new DateTime(2013, 6, 30);

            //Discount Factor
            RV = FinancialTypes.ZCReturnValues.DF;

            result = wct.DFINTERP(dfdate, df, newdate, startdate, RV);

            Console.WriteLine("DFINTERP : " + result.ToString());           //0.998709482483852
            File.AppendAllText("DFINTERP.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Zero Coupon
            RV = FinancialTypes.ZCReturnValues.Zero;

            result = wct.DFINTERP(dfdate, df, newdate, startdate, RV);

            Console.WriteLine("DFINTERP : " + result.ToString());            //0.00283941624767274
            File.AppendAllText("DFINTERP.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Continuously compounded zero coupon
            RV = FinancialTypes.ZCReturnValues.CCZero;

            result = wct.DFINTERP(dfdate, df, newdate, startdate, RV);

            Console.WriteLine("DFINTERP : " + result.ToString());            //0.00284125037852567    
            File.AppendAllText("DFINTERP.csv", vbCrLf + result.ToString());
        }

        public static void test_ED_FUT_CONV_ADJ_HL()
        {
            Double Price;
            Double Vol;
            Double T1;
            Double T2;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Price = 99.675;
            Vol = 0.0008;
            T1 = wct.ED_FUTYF("M3", new DateTime(2013, 3, 25), null);
            T2 = wct.NULL_DOUBLE;

            result = wct.ED_FUT_CONV_ADJ_HL(Price, Vol, T1, T2);

            Console.WriteLine("D_FUT_CONV_ADJ_HL : " + result.ToString());           //0.003294    
            File.WriteAllText("D_FUT_CONV_ADJ_HL.csv", result.ToString());
        }

        public static void test_INTERPDFACT()
        {
            DateTime[] dfdate;
            Double[] df;
            DateTime iStartDate;
            DateTime iEndDate;
            DateTime StartDate;
            FinancialTypes.INTERPDFACT_table result;
            StringBuilder retStr;

            //=====================
            // EXAMPLE# 1
            //=====================

            dfdate = new DateTime[] { new DateTime(2013, 1, 16), new DateTime(2013, 1, 17), new DateTime(2013, 1, 25), new DateTime(2013, 1, 31), new DateTime(2013, 2, 18), new DateTime(2013, 3, 18), new DateTime(2013, 4, 17), new DateTime(2013, 6, 19), new DateTime(2013, 9, 18), new DateTime(2013, 12, 18), new DateTime(2014, 3, 19), new DateTime(2014, 6, 18), new DateTime(2014, 9, 17), new DateTime(2014, 12, 17), new DateTime(2015, 1, 19), new DateTime(2016, 1, 18), new DateTime(2017, 4, 17), new DateTime(2013, 1, 17), new DateTime(2020, 1, 17), new DateTime(2023, 1, 17), new DateTime(2013, 1, 19) };
            df = new Double[] { 0.999995555575309, 0.99999111117037, 0.999956112706425, 0.999916450742048, 0.999804481000583, 0.999574621744643, 0.999241679910437, 0.998800609148515, 0.998022836090921, 0.997197057207847, 0.996311568695976, 0.995354720378904, 0.994289565586446, 0.993104681356623, 0.992402694592988, 0.98508478388398, 0.973098042807202, 0.955265832115111, 0.906604451702898, 0.820620615064395, 0.385646181323946 };
            iStartDate = new DateTime(2013, 1, 16);
            iEndDate = new DateTime(2013, 2, 13);
            StartDate = new DateTime(2013, 1, 15);

            result = wct.INTERPDFACT(dfdate, df, iStartDate, iEndDate, StartDate);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("INTERPDFACT : " + retStr.ToString());
            File.WriteAllText("INTERPDFACT.csv", retStr.ToString());
        }

        public static void test_SWAPCURVE()
        {
            DateTime[] mDate;
            Double[] cRate;
            String[] iType;
            DateTime StartDate;
            int Frequency;
            DateTime SpotDate;
            String CashBasis;
            String FuturesBasis;
            String SwapsBasis;
            String InterpMethod;
            String DateRoll;
            DateTime[] Holidays;
            FinancialTypes.SWAPCURVE_table result;
            StringBuilder retStr;
            Object[,] curve;
            Object[,] futures;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2013, 3, 7);
            SpotDate = new DateTime(2013, 3, 11);
            Frequency = 2;
            CashBasis = "2";
            FuturesBasis = "2";
            SwapsBasis = "0";
            InterpMethod = "L";
            DateRoll = "M";
            curve = new Object[,] { { "1M", "3M", "6M", "1Y", "2Y", "3Y", "4Y", "5Y", "7Y", "10Y", "30Y" }, { 0.0023, 0.0028, 0.0044, 0.0031, 0.0039, 0.0054, 0.0074, 0.01, 0.015, 0.0207, 0.0304 }, { "C", "C", "C", "S", "S", "S", "S", "S", "S", "S", "S" } };

            //Convert the tenor codes into dates
            for (int i = 0; i <= curve.GetUpperBound(1); i++)
            {
                curve[0, i] = wct.TENOR2DATE(curve[0, i].ToString(), StartDate, SpotDate);
            }

            result = wct.SWAPCURVE(curve, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SWAPCURVE : ex1 : " + retStr.ToString());
            File.WriteAllText("SWAPCURVE.csv", "ex1 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Use one of the overloads
            mDate = new DateTime[(curve.GetUpperBound(1) + 1)];
            cRate = new Double[(curve.GetUpperBound(1) + 1)];
            iType = new String[(curve.GetUpperBound(1) + 1)];
            for (int i = 0; i <= curve.GetUpperBound(1); i++)
            {
                mDate[i] = (DateTime)(curve[0, i]);
                cRate[i] = (Double)(curve[1, i]);
                iType[i] = (String)(curve[2, i]);
            }

            result = wct.SWAPCURVE(mDate, cRate, iType, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SWAPCURVE : ex2 : " + retStr.ToString());
            File.AppendAllText("SWAPCURVE.csv", vbCrLf + "ex2 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Add the futures to the existing curve

            futures = new Object[,] { { "M3",   "U3",   "Z3",   "H4",  "M4",  "U4",   "Z4" }, 
                                      { 99.705, 99.685, 99.645, 99.61, 99.56, 99.495, 99.405 }, 
                                      { "F",    "F",    "F",    "F",   "F",   "F",    "F" } };
            int r = curve.GetUpperBound(1) + 1;
            curve = Redim<object>(curve, curve.GetUpperBound(0), curve.GetUpperBound(1) + futures.GetUpperBound(1) + 1, true);//ReDim Preserve curve(curve.GetUpperBound(0), curve.GetUpperBound(1) + futures.GetUpperBound(1) + 1);
            for (int i = 0; i <= futures.GetUpperBound(1); i++)
            {
                curve[0, i + r] = wct.ED_FUT2DATE(futures[0, i].ToString(), StartDate);
                curve[1, i + r] = (100.0 - (Double)(futures[1, i])) / 100.0;
                curve[2, i + r] = futures[2, i];
            }

            result = wct.SWAPCURVE(curve, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SWAPCURVE : ex3 : " + retStr.ToString());
            File.AppendAllText("SWAPCURVE.csv", vbCrLf + "ex3 : " + retStr.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            //Use one of the overloads
            Array.Resize(ref mDate, (curve.GetUpperBound(1) + 1));
            Array.Resize(ref cRate, (curve.GetUpperBound(1) + 1));
            Array.Resize(ref iType, (curve.GetUpperBound(1) + 1));
            for (int i = 0; i <= curve.GetUpperBound(1); i++)
            {
                mDate[i] = (DateTime)(curve[0, i]);
                cRate[i] = (Double)(curve[1, i]);
                iType[i] = (String)(curve[2, i]);
            }

            result = wct.SWAPCURVE(mDate, cRate, iType, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            //Write the table output to a csv
            retStr = new StringBuilder();
            foreach (DataColumn col in result.Columns)
            {
                retStr.Append(col.ColumnName + ",");
            }
            retStr.Remove(retStr.Length - 1, 1);
            retStr.AppendLine();
            foreach (DataRow row in result.Rows)
            {
                for (int i = 0; i < result.Columns.Count; i++)
                {
                    retStr.Append(row[i].ToString());
                    if (i < result.Columns.Count - 1)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
            }
            Console.WriteLine("SWAPCURVE : ex4 : " + retStr.ToString());
            File.AppendAllText("SWAPCURVE.csv", vbCrLf + "ex4 : " + retStr.ToString());
        }

        public static void test_ZEROCOUPON()
        {
            DateTime vDate;
            String RV;
            DateTime StartDate;
            int Frequency;
            DateTime SpotDate;
            String CashBasis;
            String FuturesBasis;
            String SwapsBasis;
            String InterpMethod;
            String DateRoll;
            DateTime[] Holidays;
            Double result;
            Object[,] curve;
            Object[,] futures;

            //=====================
            // EXAMPLE# 1
            //=====================

            vDate = new DateTime(2013, 9, 11);
            RV = "ZC";
            StartDate = new DateTime(2013, 3, 7);
            Frequency = 2;
            SpotDate = new DateTime(2013, 3, 11);
            CashBasis = "2";
            FuturesBasis = "2";
            SwapsBasis = "0";
            InterpMethod = "L";
            DateRoll = "MP";
            curve = new Object[,] { { "1M",   "3M",   "6M",   "1Y",   "2Y",   "3Y",   "4Y",   "5Y", "7Y",  "10Y",  "30Y" }, 
                                    { 0.0023, 0.0028, 0.0044, 0.0031, 0.0039, 0.0054, 0.0074, 0.01, 0.015, 0.0207, 0.0304 }, 
                                    { "C",    "C",    "C",    "S",    "S",    "S",    "S",    "S",  "S",   "S",    "S" } };

            //Convert the tenor codes into dates
            for (int i = 0; i <= curve.GetUpperBound(1); i++)
            {
                curve[0, i] = wct.TENOR2DATE(curve[0, i].ToString(), StartDate, SpotDate);
            }

            result = wct.ZEROCOUPON(curve, vDate, RV, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            Console.WriteLine("ZEROCOUPON : " + result.ToString());          //0.00441592127318121
            File.WriteAllText("ZEROCOUPON.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //Add the futures to the existing curve

            futures = new Object[,] { { "M3",   "U3",   "Z3",   "H4",  "M4",  "U4",   "Z4" }, 
                                      { 99.705, 99.685, 99.645, 99.61, 99.56, 99.495, 99.405 }, 
                                      { "F",    "F",    "F",    "F",   "F",   "F",    "F" } };
            int r = curve.GetUpperBound(1) + 1;
            curve = Redim<object>(curve, curve.GetUpperBound(0), curve.GetUpperBound(1) + futures.GetUpperBound(1) + 1, true);
            for (int i = 0; i <= futures.GetUpperBound(1); i++)
            {
                curve[0, i + r] = wct.ED_FUT2DATE(futures[0, i].ToString(), StartDate);
                curve[1, i + r] = (100.0 - (Double)(futures[1, i])) / 100.0;
                curve[2, i + r] = futures[2, i];
            }

            result = wct.ZEROCOUPON(curve, vDate, RV, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll);

            Console.WriteLine("ZEROCOUPON : " + result.ToString());         //0.00296280638269212
            File.AppendAllText("ZEROCOUPON.csv", vbCrLf + result.ToString());
        }

        public static void test_NELSONSIEGEL()
        {
            Double Maturity;
            Double[] T;
            Double[] R;
            FinancialTypes.NScoef coefficients;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Maturity = 1;
            T = new Double[] { 1, 2, 3, 5, 7, 10, 20, 30 };
            R = new Double[] { 0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406 };
            coefficients = wct.NSCOEF(T, R);

            result = wct.NELSONSIEGEL(Maturity, coefficients.B0, coefficients.B1, coefficients.B2, coefficients.Tau);

            Console.WriteLine("NELSONSIEGEL : " + result.ToString());            //0.00298376016908077
            File.WriteAllText("NELSONSIEGEL.csv", result.ToString());
        }

        public static void test_NSCOEF()
        {
            Double[] T;
            Double[] R;
            FinancialTypes.NScoef result;

            //=====================
            // EXAMPLE# 1
            //=====================

            T = new Double[] { 1, 2, 3, 5, 7, 10, 20, 30 };
            R = new Double[] { 0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406 };

            result = wct.NSCOEF(T, R);

            Console.WriteLine("NSCOEF : B0 : " + result.B0);           //0.0466701067626488
            Console.WriteLine("NSCOEF : B1 : " + result.B1);            //-0.0409398575156674
            Console.WriteLine("NSCOEF : B2 : " + result.B2);            //-0.0635075128703469
            Console.WriteLine("NSCOEF : Tau : " + result.Tau);          //1.6545
            Console.WriteLine("NSCOEF : RMSE : " + result.RMSE);        //9.8912932859058E-07

            File.WriteAllText("NSCOEF.csv", result.ToString());
        }

        public static void test_NSCOEF2()
        {
            Double[] T;
            Double[] R;
            int NumSteps;
            Double Tau_min;
            Double Tau_max;
            Double B0_min;
            Double B0_max;
            Double B1_min;
            Double B1_max;
            Double B2_min;
            Double B2_max;
            FinancialTypes.NScoef result;

            //=====================
            // EXAMPLE# 1
            //=====================

            T = new Double[] { 1, 2, 3, 5, 7, 10, 20, 30 };
            R = new Double[] { 0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406 };
            NumSteps = 900;
            Tau_min = 0.5;
            Tau_max = 9.5;
            B0_min = -0.5;
            B0_max = 0.15;
            B1_min = -0.15;
            B1_max = 0.3;
            B2_min = -0.3;
            B2_max = 0.3;

            result = wct.NSCOEF2(T, R, NumSteps, Tau_min, Tau_max, B0_min, B0_max, B1_min, B1_max, B2_min, B2_max);

            Console.WriteLine("NSCOEF2 : B0 : " + result.B0);            //0.0466701067626488
            Console.WriteLine("NSCOEF2 : B1 : " + result.B1);            //-0.0409398575156674
            Console.WriteLine("NSCOEF2 : B2 : " + result.B2);            //-0.0635075128703469
            Console.WriteLine("NSCOEF2 : Tau : " + result.Tau);          //1.6545
            Console.WriteLine("NSCOEF2 : RMSE : " + result.RMSE);        //9.8912932859058E-07

            File.WriteAllText("NSCOEF2.csv", result.ToString());
        }

        public static void test_ED_FUTYF()
        {
            String DeliveryCode;
            DateTime StartDate;
            String Basis;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            DeliveryCode = "M3";
            StartDate = new DateTime(2013, 3, 25);
            Basis = "2";

            result = wct.ED_FUTYF(DeliveryCode, StartDate, Basis);

            Console.WriteLine("ED_FUTYF : " + result.ToString());            //0.238888888888889
            File.WriteAllText("ED_FUTYF.csv", result.ToString());
        }

        public static void test_ED_FUT2DATE()
        {
            String DeliveryCode;
            DateTime StartDate;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            DeliveryCode = "M3";
            StartDate = new DateTime(2013, 3, 25);

            result = wct.ED_FUT2DATE(DeliveryCode, StartDate);

            Console.WriteLine("ED_FUT2DATE : " + result.ToString());
            File.WriteAllText("ED_FUT2DATE.csv", result.ToString());
        }
        
        public static void test_TENOR2DATE()
        {
            String[] Tenors;
            DateTime startDate;
            DateTime spotDate;

            //=====================
            // EXAMPLE# 1
            //=====================

            Tenors = new String[] { "ON", "TN", "SN", "1W", "2W", "1M", "3M", "6M", "1Y", "2Y", "5Y", "10Y" };
            startDate = new DateTime(2013, 3, 25);
            spotDate = wct.NULL_DATE;
            File.WriteAllText("TENOR2DATE.tsv", "");
            for (int i = 0; i <= Tenors.GetUpperBound(0); i++)
            {
                Console.WriteLine("TENOR2DATE : " + Tenors[i] + vbTab + wct.TENOR2DATE(Tenors[i], startDate, spotDate));
                File.AppendAllText("TENOR2DATE.tsv", Tenors[i] + vbTab + wct.TENOR2DATE(Tenors[i], startDate, spotDate).ToString() + vbCrLf);
            }

            //=====================
            // EXAMPLE# 2
            //=====================

            //Add some holidays
            List<DateTime> Holidays = new List<DateTime>(); //As New List(Of Date)
            for (int i = 2013; i <= 2023; i++)
            {
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 1, 1), "D", 0, "G"));        //New Year's Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 1, 1), "Mon").AddDays(14));  //MLK Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 2, 1), "Mon").AddDays(14));  //President's Day
                Holidays.Add(wct.LASTWEEKDAY(new DateTime(i, 5, 1), "Mon"));               //Memorial Day
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 7, 4), "D", 0, "G"));        //Independence Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 9, 1), "Mon"));              //Labor Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 10, 1), "Mon").AddDays(7));  //Columbus Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 11, 1), "Thu").AddDays(21)); //Thanksgiving
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 12, 25), "D", 0, "G"));      //Christmas Day
            }
            startDate = new DateTime(2013, 3, 27);

            for (int i = 0; i <= Tenors.GetUpperBound(0); i++)
            {
                Console.WriteLine("TENOR2DATE : " + Tenors[i] + vbTab + wct.TENOR2DATE(Tenors[i], startDate, spotDate, Holidays));
                File.AppendAllText("TENOR2DATE.tsv", Tenors[i] + vbTab + wct.TENOR2DATE(Tenors[i], startDate, spotDate, Holidays).ToString() + vbCrLf);
            }
        }
        
        public static void test_BUSDAYS()
        {
            List<DateTime> Holidays = new List<DateTime>();
            DateTime[] StartDate;
            DateTime[] EndDate;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime[] { new DateTime(2015, 8, 1), new DateTime(2014, 11, 1), new DateTime(2010, 3, 16), new DateTime(2009, 10, 21), new DateTime(2014, 7, 22), new DateTime(2013, 2, 3), new DateTime(2013, 4, 26), new DateTime(2013, 1, 7), new DateTime(2015, 8, 4), new DateTime(2013, 12, 16), new DateTime(2015, 1, 2), new DateTime(2009, 12, 2), new DateTime(2014, 6, 14), new DateTime(2010, 1, 15), new DateTime(2015, 1, 19), new DateTime(2014, 9, 20), new DateTime(2009, 11, 9), new DateTime(2011, 7, 8), new DateTime(2010, 11, 7), new DateTime(2013, 1, 8), new DateTime(2012, 7, 12), new DateTime(2012, 7, 3) };
            EndDate = new DateTime[] { new DateTime(2014, 11, 19), new DateTime(2014, 4, 12), new DateTime(2015, 12, 12), new DateTime(2011, 11, 24), new DateTime(2015, 10, 13), new DateTime(2012, 2, 27), new DateTime(2015, 1, 23), new DateTime(2011, 6, 7), new DateTime(2010, 9, 23), new DateTime(2015, 8, 25), new DateTime(2013, 8, 7), new DateTime(2011, 10, 7), new DateTime(2015, 10, 23), new DateTime(2015, 3, 13), new DateTime(2010, 6, 7), new DateTime(2012, 12, 31), new DateTime(2014, 9, 1), new DateTime(2015, 9, 24), new DateTime(2011, 6, 8), new DateTime(2014, 11, 26), new DateTime(2015, 2, 4), new DateTime(2012, 8, 30) };

            Holidays = new List<DateTime>();
            for (int i = 2010; i <= 2020; i++)
            {
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 1, 1), "D", 0, "G"));        //New Year's Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 1, 1), "Mon").AddDays(14));  //MLK Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 2, 1), "Mon").AddDays(14));  //President's Day
                Holidays.Add(wct.LASTWEEKDAY(new DateTime(i, 5, 1), "Mon"));               //Memorial Day
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 7, 4), "D", 0, "G"));        //Independence Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 9, 1), "Mon"));              //Labor Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 10, 1), "Mon").AddDays(7));  //Columbus Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 11, 1), "Thu").AddDays(21)); //Thanksgiving
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 12, 25), "D", 0, "G"));      //Christmas Day
            }

            File.WriteAllText("BUSDAYS.tsv", "");
            for (int i = 0; i <= StartDate.GetUpperBound(0); i++)
            {
                Console.WriteLine("BUSDAYS : " + StartDate[i].ToString() + vbTab + EndDate[i] + vbTab + wct.BUSDAYS(StartDate[i], EndDate[i], Holidays) + vbTab + (EndDate[i] - StartDate[i]).TotalDays);
                File.AppendAllText("BUSDAYS.tsv", StartDate[i].ToString() + vbTab + EndDate[i].ToString() + vbTab + wct.BUSDAYS(StartDate[i], EndDate[i], Holidays) + vbTab + (EndDate[i] - StartDate[i]).TotalDays + vbCrLf);
            }
        }
                
        public static void test_BUSDAYSWE()
        {
            DateTime[] Holidays;
            DateTime StartDate;
            DateTime EndDate;
            int WE1;
            int WE2;
            int result;
            int i;

            //=====================
            // EXAMPLE# 1
            //=====================

            //In this example we calculate the number of business days from Wednesday to Wednesday assuming that the only weekend day is Sunday.
            StartDate = new DateTime(2014, 5, 1);
            EndDate = new DateTime(2014, 5, 8);
            WE1 = 0;

            result = wct.BUSDAYSWE(StartDate, EndDate, "", WE1, wct.NULL_INT);

            Console.WriteLine("BUSDAYSWE : " + result.ToString());       //6
            File.WriteAllText("BUSDAYSWE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //In this example, we add some holidays, and we calculate the number number of business days in the month of May, assuming that the only weekend day is Sunday.
            Holidays = new DateTime[] { new DateTime(2014, 1, 1), new DateTime(2014, 3, 7), new DateTime(2014, 3, 8), new DateTime(2014, 4, 21), new DateTime(2014, 4, 22), new DateTime(2014, 5, 1), new DateTime(2014, 6, 23), new DateTime(2014, 9, 7), new DateTime(2014, 10, 12), new DateTime(2014, 11, 2), new DateTime(2014, 11, 15), new DateTime(2014, 12, 25), new DateTime(2014, 1, 1), new DateTime(2012, 2, 20), new DateTime(2012, 2, 21), new DateTime(2012, 4, 6), new DateTime(2012, 4, 21), new DateTime(2012, 5, 1), new DateTime(2012, 6, 7), new DateTime(2012, 9, 7), new DateTime(2012, 10, 12), new DateTime(2012, 11, 2), new DateTime(2012, 11, 15), new DateTime(2012, 12, 25) };//#1/1/2014), new DateTime(1/20/2014), new DateTime(2/17/2014), new DateTime(5/26/2014), new DateTime(7/4/2014), new DateTime(9/1/2014), new DateTime(10/13/2014), new DateTime(11/11/2014), new DateTime(11/27/2014), new DateTime(12/25/2014#}
            StartDate = new DateTime(2014, 5, 1);
            EndDate = new DateTime(2014, 6, 1);

            result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, wct.NULL_INT);

            Console.WriteLine("BUSINESSWE : ex1 : " + result.ToString());          //26
            File.AppendAllText("BUSDAYSWE.csv", vbCrLf + "ex1 : " + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //In this example, we calculate the number of business days for each month in 2014 assuming that the only weekend day is Sunday.
            for (i = 1; i <= 12; i++)
            {
                StartDate = new DateTime(2014, i, 1);
                EndDate = StartDate.AddMonths(1);
                result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, wct.NULL_INT);
                Console.WriteLine("BUSDAYSWE : ex2 : " + MonthName(i) + vbTab + result.ToString());
                File.AppendAllText("BUSDAYSWE.csv", vbCrLf + "ex2 : " + MonthName(i) + vbTab + result.ToString());
            }

            //=====================
            // EXAMPLE# 4
            //=====================

            //In this example, the weekend days are Monday and Tuesday.
            WE1 = 1;
            WE2 = 2;
            for (i = 1; i <= 12; i++)
            {
                StartDate = new DateTime(2014, i, 1);
                EndDate = StartDate.AddMonths(1);
                result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, WE2);
                Console.WriteLine("BUSDAYSWE : ex3 : " + MonthName(i) + vbTab + result.ToString());
                File.AppendAllText("BUSDAYSWE.csv", vbCrLf + "ex3 : " + MonthName(i) + vbTab + result.ToString());
            }
        }
        
        public static void test_BUSINESSDATE()
        {
            List<DateTime> Holidays;
            DateTime StartDate;
            String DatePart;
            int Num;
            String DateRollRule;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //Create some holidays
            Holidays = new List<DateTime>();
            for (int i = 2010; i <= 2020; i++)
            {
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 1, 1), "D", 0, "G"));        //New Year's Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 1, 1), "Mon").AddDays(14));  //MLK Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 2, 1), "Mon").AddDays(14));  //President's Day
                Holidays.Add(wct.LASTWEEKDAY(new DateTime(i, 5, 1), "Mon"));               //Memorial Day
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 7, 4), "D", 0, "G"));        //Independence Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 9, 1), "Mon"));              //Labor Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 10, 1), "Mon").AddDays(7));  //Columbus Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 11, 1), "Thu").AddDays(21)); //Thanksgiving
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 12, 25), "D", 0, "G"));      //Christmas Day
            }

            //In this example, we calculate 2 business days from 2012-05-04.
            StartDate = new DateTime(2012, 5, 4);
            DatePart = "D";
            Num = 2;
            DateRollRule = "F";

            result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays);

            Console.WriteLine("BUSINESSDATE : " + result.ToString());           //5/8/2012
            File.WriteAllText("BUSINESSDATE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //calculate 5 business days from 2012-05-21
            StartDate = new DateTime(2012, 5, 21);
            Num = 5;

            result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays);

            Console.WriteLine("BUSINESSDATE : " + result.ToString());            //5/29/2012
            File.AppendAllText("BUSINESSDATE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            //Calculate 3 months from 2012-02-28
            StartDate = new DateTime(2012, 2, 28);
            DatePart = "M";
            Num = 3;

            result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays);

            Console.WriteLine("BUSINESSDATE : " + result.ToString());           //5/29/2012
            File.AppendAllText("BUSINESSDATE.csv", vbCrLf + result.ToString());
        }

        public static void test_BUSINESSDATEWE()
        {
            List<DateTime> Holidays;
            DateTime StartDate;
            String DatePart;
            int Num;
            String DateRollRule;
            int WE1;
            int WE2;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            // Create some holidays
            Holidays = new List<DateTime>();
            //for i As Integer = 2010 To 2020
            for (int i = 2010; i <= 2020; i++)
            {
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 1, 1), "D", 0, "G"));        //New Year's Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 1, 1), "Mon").AddDays(14));  //MLK Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 2, 1), "Mon").AddDays(14));  //President's Day
                Holidays.Add(wct.LASTWEEKDAY(new DateTime(i, 5, 1), "Mon"));               //Memorial Day
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 7, 4), "D", 0, "G"));        //Independence Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 9, 1), "Mon"));              //Labor Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 10, 1), "Mon").AddDays(7));  //Columbus Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 11, 1), "Thu").AddDays(21)); //Thanksgiving
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 12, 25), "D", 0, "G"));      //Christmas Day
            }

            // In this example, we calculate 2 business days from 2012-05-04.

            StartDate = new DateTime(2012, 5, 4);
            DatePart = "D";
            Num = 2;
            DateRollRule = "F";
            WE1 = wct.NULL_INT;
            WE2 = wct.NULL_INT;

            result = wct.BUSINESSDATEWE(StartDate, DatePart, Num, DateRollRule, Holidays, WE1, WE2);

            Console.WriteLine("BUSINESSDATEWE : " + result.ToString());            // 5/8/2012
            File.WriteAllText("BUSINESSDATEWE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            // What if we want to count Saturday as a business day?

            WE1 = 6;
            result = wct.BUSINESSDATEWE(StartDate, DatePart, Num, DateRollRule, Holidays, WE1, WE2);

            Console.WriteLine("BUSINESSDATEWE : " + result.ToString());            // 5/8/2012
            File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 3
            //=====================

            // calculate 5 business days from 2012-05-21

            WE1 = wct.NULL_INT;
            StartDate = new DateTime(2012, 5, 21);
            Num = 5;

            result = wct.BUSINESSDATEWE(StartDate, DatePart, Num, DateRollRule, Holidays, WE1, WE2);

            Console.WriteLine("BUSINESSDATEWE : " + result.ToString());
            File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf + result.ToString());

            //=====================
            // EXAMPLE# 4
            //=====================

            // Calculate 3 months from 2012-02-28

            StartDate = new DateTime(2012, 2, 28);
            DatePart = "M";
            Num = 3;

            result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays);

            Console.WriteLine("BUSINESSDATEWE : " + result.ToString());
            File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf + result.ToString());
        }

        public static void test_T360()
        {
            DateTime Maturity;
            DateTime Settlement;
            DateTime CFDate;
            int Freq;
            int Method30360;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Maturity = new DateTime(2022, 3, 16);
            Settlement = new DateTime(2015, 3, 12);
            CFDate = new DateTime(2015, 3, 31);
            Freq = 2;
            Method30360 = 0;

            result = wct.T360(Maturity, Settlement, CFDate, Freq, Method30360);

            Console.WriteLine("T360 : " + result.ToString());            // 5/8/2012
            File.WriteAllText("T360.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            Maturity = new DateTime(2018, 7, 3);
            CFDate = Maturity;
            Settlement = new DateTime(2015, 3, 18);
            do 
            {
                Console.WriteLine("T360 : " + CFDate + vbTab + wct.T360(Maturity, Settlement, CFDate, Freq, Method30360).ToString());
                File.AppendAllText("T360.csv", vbCrLf + CFDate + vbTab + wct.T360(Maturity, Settlement, CFDate, Freq, Method30360).ToString());

                CFDate = CFDate.AddMonths(-6);

            } while (CFDate > Settlement);
        }

        public static void test_CALCDATE()
        {
            int Y;
            int M;
            int D;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Y = 2011;
            D = 7;
            M = 2;

            result = wct.CALCDATE(Y, M, D);

            Console.WriteLine("CALCDATE : " + result.ToString());            // 2/7/2011
            File.WriteAllText("CALCDATE.csv", result.ToString());
        }

        public static void test_DATEFLOAT()
        {
            int Y;
            int M;
            int D;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Y = 2011;
            D = 7;
            M = 2;

            result = wct.DATEFLOAT(Y, M, D);

            Console.WriteLine("DATEFLOAT : " + result.ToString());            // 40579
            File.WriteAllText("DATEFLOAT.csv", result.ToString());
        }

        public static void test_DATEINT()
        {
            int Y;
            int M;
            int D;
            int result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Y = 2011;
            D = 7;
            M = 2;

            result = wct.DATEINT(Y, M, D);

            Console.WriteLine("DATEINT : " + result.ToString());            // 40579
            File.WriteAllText("DATEINT.csv", result.ToString());
        }

        public static void test_DAYS360()
        {
            DateTime[] StartDate;
            DateTime[] EndDate;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime[] { new DateTime(2007, 1, 15), new DateTime(2007, 1, 15), new DateTime(2007, 1, 15), new DateTime(2007, 9, 30), new DateTime(2007, 9, 30), new DateTime(2007, 9, 30), new DateTime(2007, 1, 15), new DateTime(2007, 1, 31), new DateTime(2007, 2, 28), new DateTime(2006, 8, 31), new DateTime(2007, 2, 28), new DateTime(2007, 2, 14), new DateTime(2007, 2, 26), new DateTime(2008, 2, 29), new DateTime(2008, 2, 29), new DateTime(2008, 2, 29), new DateTime(2007, 2, 28), new DateTime(2007, 10, 31), new DateTime(2007, 8, 31), new DateTime(2008, 2, 29), new DateTime(2008, 8, 31), new DateTime(2009, 2, 28) };
            EndDate = new DateTime[] { new DateTime(2007, 1, 30), new DateTime(2007, 2, 15), new DateTime(2007, 7, 15), new DateTime(2008, 3, 31), new DateTime(2007, 10, 31), new DateTime(2008, 9, 30), new DateTime(2007, 1, 31), new DateTime(2007, 2, 28), new DateTime(2007, 3, 31), new DateTime(2007, 2, 28), new DateTime(2007, 8, 31), new DateTime(2007, 2, 28), new DateTime(2008, 2, 29), new DateTime(2009, 2, 28), new DateTime(2008, 3, 30), new DateTime(2008, 3, 31), new DateTime(2007, 3, 5), new DateTime(2007, 11, 28), new DateTime(2008, 2, 29), new DateTime(2008, 8, 31), new DateTime(2009, 2, 28), new DateTime(2009, 8, 31) };

            Console.WriteLine("DAYS360 : " + "Start Date" + vbTab + "End Date" + vbTab + "30/360 US" + vbTab + "30E/360" + "30E360 ISDA" + vbTab + "Actual");
            File.WriteAllText("DAYS360.tsv", "Start Date" + vbTab + "End Date" + vbTab + "30/360 US" + vbTab + "30E/360" + "30E360 ISDA" + vbTab + "Actual");

            for (int i = 0; i <= StartDate.GetUpperBound(0); i++)
            {
                Console.WriteLine("DAYS360 : " + StartDate[i].ToString() + vbTab + EndDate[i].ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 0).ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 1).ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 2).ToString() + vbTab + (EndDate[i] - StartDate[i]).TotalDays.ToString());
                File.AppendAllText("DAYS360.tsv", vbCrLf + StartDate[i].ToString() + vbTab + EndDate[i].ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 0).ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 1).ToString() + vbTab + wct.DAYS360(StartDate[i], EndDate[i], 2).ToString() + vbTab + (EndDate[i] - StartDate[i]).TotalDays.ToString());
            }
        }

        public static void test_DAYSINMONTH()
        {
            DateTime StartDate;
            int result;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2014, 9, 18);

            result = wct.DAYSINMONTH(StartDate);

            Console.WriteLine("DAYSINMONTH : " + result.ToString());            // 30
            File.WriteAllText("DAYSINMONTH.csv", result.ToString());
        }

        public static void test_DAYSINYEAR()
        {
            DateTime StartDate;
            int result;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2014, 9, 18);

            result = wct.DAYSINYEAR(StartDate);

            Console.WriteLine("DAYSINYEAR : " + result.ToString());            // 365
            File.WriteAllText("DAYSINYEAR.csv", result.ToString());
        }

        public static void test_DAYSNL()
        {
            DateTime[] StartDate;
            DateTime[] EndDate;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime[] { new DateTime(2007, 1, 15), new DateTime(2007, 1, 15), new DateTime(2007, 1, 15), new DateTime(2007, 9, 30), new DateTime(2007, 9, 30), new DateTime(2007, 9, 30), new DateTime(2007, 1, 15), new DateTime(2007, 1, 31), new DateTime(2007, 2, 28), new DateTime(2006, 8, 31), new DateTime(2007, 2, 28), new DateTime(2007, 2, 14), new DateTime(2007, 2, 26), new DateTime(2008, 2, 29), new DateTime(2008, 2, 29), new DateTime(2008, 2, 29), new DateTime(2007, 2, 28), new DateTime(2007, 10, 31), new DateTime(2007, 8, 31), new DateTime(2008, 2, 29), new DateTime(2008, 8, 31), new DateTime(2009, 2, 28) };
            EndDate = new DateTime[] { new DateTime(2007, 1, 30), new DateTime(2007, 2, 15), new DateTime(2007, 7, 15), new DateTime(2008, 3, 31), new DateTime(2007, 10, 31), new DateTime(2008, 9, 30), new DateTime(2011, 1, 31), new DateTime(2011, 2, 28), new DateTime(2011, 3, 31), new DateTime(2011, 2, 28), new DateTime(2011, 8, 31), new DateTime(2011, 2, 28), new DateTime(2012, 2, 29), new DateTime(2013, 2, 28), new DateTime(2012, 3, 30), new DateTime(2012, 3, 31), new DateTime(2011, 3, 5), new DateTime(2011, 11, 28), new DateTime(2012, 2, 29), new DateTime(2012, 8, 31), new DateTime(2012, 2, 28), new DateTime(2012, 8, 31) };

            Console.WriteLine("DAYS360 : " + "Start Date" + vbTab + "End Date" + vbTab + "DAYSNL" + vbTab + "Actual");
            File.WriteAllText("DAYS360.tsv", "Start Date" + vbTab + "End Date" + vbTab + "DAYSNL" + vbTab + "Actual");

            for (int i = 0; i <= StartDate.GetUpperBound(0); i++)
            {
                Console.WriteLine("DAYS360 : " + StartDate[i].ToString() + vbTab + EndDate[i].ToString() + vbTab + wct.DAYSNL(StartDate[i], EndDate[i]).ToString() + vbTab + (EndDate[i] - StartDate[i]).TotalDays.ToString());
                File.AppendAllText("DAYS360.tsv", vbCrLf + StartDate[i].ToString() + vbTab + EndDate[i].ToString() + vbTab + wct.DAYSNL(StartDate[i], EndDate[i]).ToString() + vbTab + (EndDate[i] - StartDate[i]).TotalDays.ToString());
            }
        }

        public static void test_EASTER()
        {
            int Y;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Y = 2016;

            result = wct.EASTER(Y);

            Console.WriteLine("EASTER : " + result.ToString());            // 03/27/2016
            File.WriteAllText("EASTER.csv", result.ToString());
        }

        public static void test_EDATE()
        {
            DateTime StartDate;
            int Months;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2016, 8, 31);
            Months = 6;

            result = wct.EDATE(StartDate, Months);

            Console.WriteLine("EDATE : " + result.ToString());
            File.WriteAllText("EDATE.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            result = wct.EDATE(StartDate, -Months);

            Console.WriteLine("EDATE : " + result.ToString());
            File.AppendAllText("EDATE.csv", vbCrLf + result.ToString());
        }

        public static void test_EOMONTH()
        {
            DateTime StartDate;
            int Months;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2016, 8, 15);
            Months = 6;

            result = wct.EOMONTH(StartDate, Months);

            Console.WriteLine("EOMONTH : " + result.ToString());
            File.WriteAllText("EOMONTH.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            result = wct.EOMONTH(StartDate, -Months);

            Console.WriteLine("EOMONTH : " + result.ToString());
            File.AppendAllText("EOMONTH.csv", vbCrLf + result.ToString());
        }

        public static void test_FIRSTWEEKDAY()
        {
            DateTime startDate;
            String DayOfWeek;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            // First Wednesday in January 2013

            startDate =  new DateTime(2013, 1, 1);
            DayOfWeek = "4";

            result = wct.FIRSTWEEKDAY(startDate, DayOfWeek);

            Console.WriteLine("FIRSTWEEKDAY : " + result.ToString());        // 01/02/2013
            File.WriteAllText("FIRSTWEEKDAY.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            DayOfWeek = "Wed";

            result = wct.FIRSTWEEKDAY(startDate, DayOfWeek);

            Console.WriteLine("FIRSTWEEKDAY : " + result.ToString());       // 01/02/2013
            File.AppendAllText("FIRSTWEEKDAY.csv", vbCrLf + result.ToString());
        }

        public static void test_ISREGULARPAY()
        {
            DateTime IssueDate;
            DateTime FirstPaymentDate;
            int PmtPerYear;
            StringBuilder retStr;
            Boolean result;

            //=====================
            // EXAMPLE# 1
            //=====================

            //A loan with weekly repayments with the first payment 7 days after the issue date.

            IssueDate = new DateTime(2014, 1, 13);
            FirstPaymentDate = new DateTime(2014, 1, 20);
            PmtPerYear = 52;

            result = wct.ISREGULARPAY(IssueDate, FirstPaymentDate, PmtPerYear);

            Console.WriteLine("ISREGULARPAY : ex1 : " + result.ToString());               //True
            File.WriteAllText("ISREGULARPAY.csv", "ex1 : " + result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            //The following example uses 2013-12-31 as @IssueDate and then compares all the days in January 2014 
            //(using the SeriesDate function from the XLeratorDB math library) 
            //as @FirstPaymentDate and pivots the results for each possible value of @PmtPerYear.

            int[] pmt = {1, 2, 3, 4, 6, 12, 13, 24, 26, 52, 365};
            retStr = new StringBuilder();
            retStr.Append("FirstPaymentDate");
            for (int i = 0; i <= 10; i++)
            {
                retStr.Append(",");
                retStr.Append(pmt[i].ToString());
            }
            retStr.AppendLine();

            IssueDate = new DateTime(2013, 12, 31);
            FirstPaymentDate = new DateTime(2013, 12, 31);
            do
            {
                retStr.Append(FirstPaymentDate.ToString());
                for (int i = 0; i <= 10; i++)
                {
                    result = wct.ISREGULARPAY(IssueDate, FirstPaymentDate, pmt[i]);
                    retStr.Append(result.ToString());
                    if (i < 10)
                    {
                        retStr.Append(",");
                    }
                    else
                    {
                        retStr.AppendLine();
                    }
                }
                FirstPaymentDate = FirstPaymentDate.AddDays(1);
            } while (FirstPaymentDate <= new DateTime(2014, 1, 31));

            Console.WriteLine("ISREGULARPAY : ex2 : " + retStr.ToString());
            File.AppendAllText("ISREGULARPAY.csv", vbCrLf + "ex2 : " + retStr.ToString());
        }  

        public static void test_LASTWEEKDAY()
        {
            DateTime startDate;
            String DayOfWeek;
            DateTime result;

            //=====================
            // EXAMPLE# 1
            //=====================

            // Last Wednesday in January 2013

            startDate = new DateTime(2013, 1, 1);
            DayOfWeek = "4";

            result = wct.LASTWEEKDAY(startDate, DayOfWeek);

            Console.WriteLine("LASTWEEKDAY : " + result.ToString());       // 01/30/2013
            File.WriteAllText("LASTWEEKDAY.csv", result.ToString());

            //=====================
            // EXAMPLE# 2
            //=====================

            DayOfWeek = "Wed";

            result = wct.LASTWEEKDAY(startDate, DayOfWeek);

            Console.WriteLine("LASTWEEKDAY : " + result.ToString());       // 01/30/2013
            File.AppendAllText("LASTWEEKDAY.csv", vbCrLf + result.ToString());
        }

        public static void test_NBD()
        {
            List<DateTime> Holidays;
            String result;

            //=====================
            // EXAMPLE# 1
            //=====================

            // Create some holidays
            Holidays = new List<DateTime>();
            for (int i = 2010; i <= 2020; i++)
            {
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 1, 1), "D", 0, "G"));        //New Year's Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 1, 1), "Mon").AddDays(14));  //MLK Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 2, 1), "Mon").AddDays(14));  //President's Day
                Holidays.Add(wct.LASTWEEKDAY(new DateTime(i, 5, 1), "Mon"));               //Memorial Day
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 7, 4), "D", 0, "G"));        //Independence Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 9, 1), "Mon"));              //Labor Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 10, 1), "Mon").AddDays(7));  //Columbus Day
                Holidays.Add(wct.FIRSTWEEKDAY(new DateTime(i, 11, 1), "Thu").AddDays(21)); //Thanksgiving
                Holidays.Add(wct.BUSINESSDATE(new DateTime(i, 12, 25), "D", 0, "G"));      //Christmas Day
            }
            
            result = wct.NBD(Holidays);

            Console.WriteLine("NBD : " + result.ToString());
            File.WriteAllText("NBD.csv", result.ToString());
        }

        public static void test_NUMMONTHS()
        {
            DateTime StartDate;
            DateTime EndDate;
            int result;

            StartDate = new DateTime(2014, 9, 18);
            EndDate = new DateTime(2014, 10, 1);

            result = wct.NUMMONTHS(StartDate, EndDate);

            Console.WriteLine("NUMMONTHS : " + result.ToString());
            File.WriteAllText("NUMMONTHS.csv", result.ToString());
        }

        public static void test_YEARFRAC()
        {
            DateTime StartDate;
            DateTime EndDate;

            //=====================
            // EXAMPLE# 1
            //=====================

            StartDate = new DateTime(2015, 8, 30);
            EndDate = new DateTime (2016,4,30);

            File.WriteAllText("YEARFRAC.csv", "");
            for (int i = 0; i <= 4; i++)
            {
                Console.WriteLine("YEARFRAC : " + StartDate + " to " + EndDate + " Basis = " + i + " : " + wct.YEARFRAC(StartDate, EndDate, i.ToString()));
                File.AppendAllText("YEARFRAC.csv", StartDate + " to " + EndDate + " Basis = " + i + " : " + wct.YEARFRAC(StartDate, EndDate, i.ToString()) + vbCrLf);
            }
        }

        public static void test_DOLLARDE()
        {
            Double Fraction_dollar;
            int Fraction;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Fraction_dollar = 1.59;
            Fraction = 64;

            result = wct.DOLLARDE(Fraction_dollar, Fraction);            // 1.921875

            Console.WriteLine("DOLLARDE : " + result.ToString());
            File.WriteAllText("DOLLARDE.csv", result.ToString());
        }

        public static void test_DOLLARFR()
        {
            Double Decimal_dollar;
            int Fraction ;
            Double result;

            //=====================
            // EXAMPLE# 1
            //=====================

            Decimal_dollar = 1.921875;
            Fraction = 64;

            result = wct.DOLLARFR(Decimal_dollar, Fraction);

            Console.WriteLine("DOLLARFR : " + result.ToString());       // 1.59
            File.WriteAllText("DOLLARFR.csv", result.ToString());
        }

        public static void test_BondPriceFromZeroes()
        {
            // BondPriceFromZeroes - example coming soon - refer to code example in XLeratorDLL/finanicial DEMO VB.NET project
        }

        public static void test_CMTCurve()
        {
            // CMTCurve - example coming soon - refer to code example in XLeratorDLL/finanicial DEMO VB.NET project
        }

        public static void test_LogNormalIRLattice()
        {
            // LogNormalIRLattice - example coming soon - refer to code example in XLeratorDLL/finanicial DEMO VB.NET project
        }

        public static void test_OAC()
        {
            // OAC - example coming soon
        }

        public static void test_OAD()
        {
            // OAD - example coming soon
        }

        public static void test_OAS()
        {
            // OAS - example coming soon
        }

        public static void test_PriceFromIRLattice()
        {
            // PriceFromIRLattice - example coming soon
        }

        public static void test_PriceFromZeroesTVF()
        {
            // PriceFromZeroesTVF - example coming soon
        }

        public static void test_ZSPREAD()
        {
            // ZSPREAD - example coming soon
        }
        

        public static void test_XLDLL_FINANCIAL_VERSION()
        {
            String result;

            //=====================
            // EXAMPLE# 1
            //=====================

            result = wct.XLDLL_FINANCIAL_VERSION(wct.VersionInfoType.LicenseInformation);

            Console.WriteLine("XLDLL_FINANCIAL_VERSION : " + result);
            File.WriteAllText("XLDLL_FINANCIAL_VERSION.csv", result);
        }
    

        //--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


        //==========================
        // *** HELPER FUNCTIONS
        //==========================

        public static string MonthName(int i)
        {
            string mo = "";
            switch (i)
            {
                case 1 : 
                    mo = "January";
                    break;
                case 2 : 
                    mo = "Febuary";
                    break;
                case 3 : 
                    mo = "March";
                    break;
                case 4 : 
                    mo = "April";
                    break;
                case 5 : 
                    mo = "May";
                    break;
                case 6 : 
                    mo = "June";
                    break;
                case 7 : 
                    mo = "July";
                    break;
                case 8 : 
                    mo = "August";
                    break;
                case 9 : 
                    mo = "September";
                    break;
                case 10 : 
                    mo = "October";
                    break;
                case 11 : 
                    mo = "November";
                    break;
                case 12 : 
                    mo = "December";
                    break;
            }
            return mo;
        }
        
        static T[,] Redim<T>(T[,] arr, bool preserved)
        {
            int Ubound0 = arr.GetUpperBound(0);
            int Ubound1 = arr.GetUpperBound(1);
            return Redim<T>(arr, Ubound0, Ubound1, preserved);
        }

        static T[,] Redim<T>(T[,] arr, int newUbound0, int newUbound1, bool preserved)
        {
            T[,] arrRedimed = new T[newUbound0 + 1, newUbound1 + 1];
            if (preserved)
            {
                int Ubound0 = Math.Min(arr.GetUpperBound(0), newUbound0);
                int Ubound1 = Math.Min(arr.GetUpperBound(1), newUbound1);
                for (int j = 0; j <= Ubound1; j++)
                {
                    for (int i = 0; i <= Ubound0; i++)
                    {
                        arrRedimed[i, j] = arr[i, j];
                    }
                }
            }
            return arrRedimed;
        }


    }   /* end: class Test_financial */

}  /* end: namespace XLDLL_Financial_Demo */
