﻿Option Explicit On
Option Strict On

Imports System.Reflection
Imports System.Text
Imports System.IO
Imports wct = XLeratorDLL_financial.XLeratorDLL_financial
Imports XLeratorDLL_financial.FinancialTypes
Imports XLeratorDLL_financial.OptionTypes

'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
'NOTE: If naming conflicts are encountered simply adjust the module's 'Imports' and qualify XLeratorDLL function and type references appropriately.
'!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


Public Class Test_financial

    '---------------------------------------
    '*** XLeratorDLL / financial
    '---------------------------------------

    Public Shared Sub test_PRICE()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Yld As Double = 0.0276                                  'Yield: 2.76%
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.PRICE(Settlement, Maturity, Rate, Yld, Redemption, Frequency, Basis)

        Console.WriteLine("PRICE : " & result.ToString())
        File.WriteAllText("PRICE.csv", result.ToString())

    End Sub

    Public Shared Sub test_YIELD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0                                  'Price: 96.00
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.YIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("YIELD : " & result.ToString())
        File.WriteAllText("YIELD.csv", result.ToString())

    End Sub

    Public Shared Sub test_RPIFactors()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0                                  'Price: 96.00
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As Basis2Type = Basis2Type.Actual                 'Basis: Actual/Actual
        Dim result As OutputRow_BONDRPI

        result = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis)

        With result
            Console.WriteLine("RPIFACTORS : " & "Previous Coupon Date : " & .PrevCoup.ToShortDateString)
            Console.WriteLine("RPIFACTORS : " & "Next Coupon Date : " & .NextCoup.ToShortDateString)
            Console.WriteLine("RPIFACTORS : " & "Accrued Days : " & .A.ToString())
            Console.WriteLine("RPIFACTORS : " & "Days in Period : " & .E.ToString())
            Console.WriteLine("RPIFACTORS : " & "Number of Coupons : " & .N.ToString())
            Console.WriteLine("RPIFACTORS : " & "Coupon : " & .C.ToString())
            Console.WriteLine("RPIFACTORS : " & "Price : " & .P.ToString())
            Console.WriteLine("RPIFACTORS : " & "Accrued Interest : " & .AI.ToString())
            Console.WriteLine("RPIFACTORS : " & "Yield : " & .Y.ToString())
        End With

        File.WriteAllText("RPIFACTORS.csv", result.ToString())

    End Sub

    Public Shared Sub test_RPI()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0                                  'Price: 96.00
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As Basis2Type = Basis2Type.Actual                 'Basis: Actual/Actual
        Dim factors As OutputRow_BONDRPI

        factors = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis)

        With factors
            Console.WriteLine("RPI : Yield : " & wct.RPI(.A, .DSC, .E, .N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString())
            File.WriteAllText("RPI.csv", "Yield : " & wct.RPI(.A, .DSC, .E, .N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        Price = wct.NULL_DOUBLE
        Yield = 0.0276

        factors = wct.RPIFACTORS(Settlement, Maturity, Rate, Price, Yield, Redemption, Frequency, Basis)

        With factors
            Console.WriteLine("RPI : Price : " & wct.RPI(.A, .DSC, .E, .N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString())
            File.AppendAllText("RPI.csv", vbCrLf + "Price : " + wct.RPI(.A, .DSC, .E, .N, Rate, Yield, Price, Convert.ToInt32(Frequency), Redemption).ToString())
        End With

    End Sub

    Public Shared Sub test_PRICEMAT()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/7/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-12-15
        Dim Issue As Date = #7/31/2014#                                 'Issue Date: 2014-07-31
        Dim Rate As Double = 0.005                                      'Rate: 0.5%
        Dim Yield As Double = 0.002                                     'Yield: 0.2%
        Dim Basis As String = "3"                                       'Basis = Actual/365
        Dim result As Double

        result = wct.PRICEMAT(Settlement, Maturity, Issue, Rate, Yield, Basis)

        Console.WriteLine("PRICEMAT : " & result.ToString())
        File.WriteAllText("PRICEMAT.csv", result.ToString())

    End Sub

    Public Shared Sub test_YIELDMAT()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/7/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-12-15
        Dim Issue As Date = #7/31/2014#                                 'Issue Date: 2014-07-31
        Dim Rate As Double = 0.005                                      'Rate: 0.5%
        Dim Price As Double = 100.05                                    'Price: 100.05
        Dim Basis As String = "3"                                       'Basis = Actual/365
        Dim result As Double

        result = wct.YIELDMAT(Settlement, Maturity, Issue, Rate, Price, Basis)

        Console.WriteLine("YIELDMAT : " & result.ToString())
        File.WriteAllText("YIELDMAT.csv", result.ToString())

    End Sub

    Public Shared Sub test_IAMFACTORS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/7/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-12-15
        Dim Issue As Date = #7/31/2014#                                 'Issue Date: 2014-07-31
        Dim Rate As Double = 0.005                                      'Rate: 0.5%
        Dim Yield As Double = 0.002                                     'Yield: 0.2%
        Dim Price As Double = wct.NULL_DOUBLE                           'Price: NULL
        Dim Basis As Basis2Type = Basis2Type.A365                       'Basis: A/365
        Dim result As OutputRow_IAM

        result = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis)

        With result
            Console.WriteLine("IAMFACTORS : " & "Accrued Days : " & .A.ToString())
            Console.WriteLine("IAMFACTORS : " & "Days in Year : " & .B.ToString())
            Console.WriteLine("IAMFACTORS : " & "Days from Issue to Maturity : " & .D_IM.ToString())
            Console.WriteLine("IAMFACTORS : " & "Days from Settlement to Maturity : " & .DSM.ToString())
            Console.WriteLine("IAMFACTORS : " & "Price : " & .P.ToString())
            Console.WriteLine("IAMFACTORS : " & "Accrued Interest : " & .AI.ToString())
            Console.WriteLine("IAMFACTORS : " & "Yield : " & .Y.ToString())
            Console.WriteLine("IAMFACTORS : " & "Total Interest : " & .TI.ToString())
            Console.WriteLine("IAMFACTORS : " & "Dirty Price : " & .DP.ToString())
        End With

        File.WriteAllText("IAMFACTORS.csv", result.ToString())

    End Sub

    Public Shared Sub test_IAM()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/7/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-12-15
        Dim Issue As Date = #7/31/2014#                                 'Issue Date: 2014-07-31
        Dim Rate As Double = 0.005                                      'Rate: 0.5%
        Dim Yield As Double = 0.002                                     'Yield: 0.2%
        Dim Price As Double = wct.NULL_DOUBLE                           'Price: NULL
        Dim Basis As Basis2Type = Basis2Type.A365                       'Basis: A/365
        Dim factors As OutputRow_IAM

        factors = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis)

        With factors
            Console.WriteLine("IAM : Price : " & wct.IAM(.A, .B, .D_IM, .DSM, Rate, Price, Yield).ToString())
            File.WriteAllText("IAM.csv", "Price : " & wct.IAM(.A, .B, .D_IM, .DSM, Rate, Price, Yield).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        Price = 100.06
        Yield = wct.NULL_DOUBLE

        factors = wct.IAMFACTORS(Settlement, Maturity, Issue, Rate, Price, Yield, Basis)

        With factors
            Console.WriteLine("IAM : Yield : " & wct.IAM(.A, .B, .D_IM, .DSM, Rate, Price, Yield).ToString())
            File.WriteAllText("IAM.csv", "Yield : " & wct.IAM(.A, .B, .D_IM, .DSM, Rate, Price, Yield).ToString())
        End With

    End Sub

    Public Shared Sub test_PRICEDISC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #7/10/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-15-15
        Dim Discount As Double = 0.015                                  'Discount Rate: 1.5%
        Dim Redemption As Double = 100                                  'Redemption: 100
        Dim Basis As String = "3"                                       'Basis: A/365
        Dim result As Double

        result = wct.PRICEDISC(Settlement, Maturity, Discount, Redemption, Basis)

        Console.WriteLine("PRICEDISC : " & result.ToString())
        File.WriteAllText("PRICEDISC.csv", result.ToString())

    End Sub

    Public Shared Sub test_YIELDDISC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #7/10/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-15-15
        Dim Price As Double = 99.72                                     'Discount Rate: 99.72
        Dim Redemption As Double = 100                                  'Redemption: 100
        Dim Basis As String = "3"                                       'Basis: A/365
        Dim result As Double

        result = wct.YIELDDISC(Settlement, Maturity, Price, Redemption, Basis)

        Console.WriteLine("YIELDDISC : " & result.ToString())
        File.WriteAllText("YIELDDISC.csv", result.ToString())

    End Sub

    Public Shared Sub test_DISC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #7/10/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-15-15
        Dim Price As Double = 99.72                                     'Price: 99.72
        Dim Redemption As Double = 100                                  'Redemption: 100
        Dim Basis As String = "3"                                       'Basis: A/365
        Dim result As Double

        result = wct.DISC(Settlement, Maturity, Price, Redemption, Basis)

        Console.WriteLine("DISC : " & result.ToString())
        File.WriteAllText("DISC.csv", result.ToString())

    End Sub

    Public Shared Sub test_DISFACTORS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #7/10/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-15-15
        Dim Redemption As Double = 100                                  'Redemption: 100
        Dim Discount As Double = wct.NULL_DOUBLE                        'Discount Rate: NULL
        Dim Price As Double = 99.72                                     'Price: 99.72
        Dim Yield As Double = wct.NULL_DOUBLE                           'Yield: NULL
        Dim Basis As String = "3"                                       'Basis: A/365
        Dim result As OutputRow_DIS

        result = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, Yield, Basis)

        With result
            Console.WriteLine("DISFACTORS : " & "Days Settlement to Maturity : " & .DSM.ToString())
            Console.WriteLine("DISFACTORS : " & "Days in Year : " & .B.ToString())
            Console.WriteLine("DISFACTORS : " & "Price : " & .P.ToString())
            Console.WriteLine("DISFACTORS : " & "Discount Rate : " & .D.ToString())
            Console.WriteLine("DISFACTORS : " & "Yield : " & .Y.ToString())
            Console.WriteLine("DISFACTORS : " & "Time : " & .T.ToString())
        End With

        File.WriteAllText("DISFACTORS.csv", result.ToString())

    End Sub

    Public Shared Sub test_DIS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #7/10/2014#                            'Settlement Date: 2014-10-07
        Dim Maturity As Date = #12/15/2014#                             'Maturity Date: 2014-15-15
        Dim Redemption As Double = 100                                  'Redemption: 100
        Dim Discount As Double = wct.NULL_DOUBLE                        'Discount Rate:
        Dim Price As Double = 99.72                                     'Price: 99.72
        Dim Basis As String = "3"                                       'Basis: A/365
        Dim factors As XLeratorDLL_financial.FinancialTypes.OutputRow_DIS

        factors = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, wct.NULL_DOUBLE, Basis)

        With factors
            Console.WriteLine("DIS : " & wct.DIS(.DSM, Redemption, Price, Discount).ToString())
            File.WriteAllText("DIS.csv", wct.DIS(.DSM, Redemption, Price, Discount).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        Discount = 0.015
        Price = wct.NULL_DOUBLE

        factors = wct.DISFACTORS(Settlement, Maturity, Redemption, Discount, Price, wct.NULL_DOUBLE, Basis)

        With factors
            Console.WriteLine("DIS : " & wct.DIS(.DSM, Redemption, Price, Discount).ToString())
            File.AppendAllText("DIS.csv", vbCrLf & wct.DIS(.DSM, Redemption, Price, Discount).ToString())
        End With

    End Sub

    Public Shared Sub test_ODDFPRICE()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Issue As Date = #5/1/2014#                              'Issue Duate: 2014-05-01
        Dim FirstCoupon As Date = #6/15/2014#                       'First Coupon Date: 2014-06-15
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Yld As Double = 0.0276                                  'Yield: 2.76%
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.ODDFPRICE(Settlement, Maturity, Issue, FirstCoupon, Rate, Yld, Redemption, Frequency, Basis)

        Console.WriteLine("ODDFPRICE : " & result.ToString())
        File.WriteAllText("ODDFPRICE.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDFYIELD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Issue As Date = #5/1/2014#                              'Issue Duate: 2014-05-01
        Dim FirstCoupon As Date = #6/15/2014#                       'First Coupon Date: 2014-06-15
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0076                               'Price: 96.0076
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.ODDFYIELD(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("ODDFYIELD : " & result.ToString())
        File.WriteAllText("ODDFYIELD.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFCFACTORS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Issue As Date = #5/1/2014#                              'Issue Duate: 2014-05-01
        Dim FirstCoupon As Date = #6/15/2014#                       'First Coupon Date: 2014-06-15
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0076                               'Price: 96.0076
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As OutputRow_BONDOFC

        result = wct.OFCFACTORS(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)

        With result
            Console.WriteLine("OFCFACTORS : " & "Coupon days : " & result.E.ToString())
            Console.WriteLine("OFCFACTORS : " & "Days settlement to next coupon : " & result.DSC.ToString())
            Console.WriteLine("OFCFACTORS : " & "Number of whole coupons : " & result.N.ToString())
            Console.WriteLine("OFCFACTORS : " & "Number of quasi-coupons in the odd period : " & result.NCF.ToString())
            Console.WriteLine("OFCFACTORS : " & "Coupons days in the first quasi-coupon period : " & result.A1.ToString())
            Console.WriteLine("OFCFACTORS : " & "Accrued days from issue date to the next quasi-coupon date : " & result.DFC1.ToString())
            Console.WriteLine("OFCFACTORS : " & "Normal length of the quasi-coupon period : " & result.NLF1.ToString())
            Console.WriteLine("OFCFACTORS : " & "Coupons days in the second quasi-coupon period : " & result.A2.ToString())
            Console.WriteLine("OFCFACTORS : " & "Accrued days from first quasi-coupon date to the issue date : " & result.DFC2.ToString())
            Console.WriteLine("OFCFACTORS : " & "Normal length of the period from quasi-coupon date to issue date : " & result.NLF2.ToString())
            Console.WriteLine("OFCFACTORS : " & "Number of whole coupons from the settlement date to the first coupon date : " & result.Nqf.ToString())
            Console.WriteLine("OFCFACTORS : " & "Calculated previous coupon date : " & result.quasistart.ToShortDateString)
            Console.WriteLine("OFCFACTORS : " & "Calculated next coupon date : " & result.quasicoup.ToShortDateString)
            Console.WriteLine("OFCFACTORS : " & "Coupon amount : " & result.C.ToString())
            Console.WriteLine("OFCFACTORS : " & "First coupon amount : " & result.FC.ToString())
            Console.WriteLine("OFCFACTORS : " & "Price : " & result.P.ToString())
            Console.WriteLine("OFCFACTORS : " & "Accrued interest : " & result.AI.ToString())
            Console.WriteLine("OFCFACTORS : " & "Yield : " & result.Y.ToString())
        End With

        File.WriteAllText("OFCFACTORS.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #5/1/2014#                         'Settlement Date: 2034-06-015
        Dim Maturity As Date = #6/15/2034#                          'Maturity Date: 2014-05-01
        Dim Issue As Date = #5/1/2014#                              'Issue Duate: 2014-05-01
        Dim FirstCoupon As Date = #6/15/2014#                       'First Coupon Date: 2014-06-15
        Dim Rate As Double = 0.025                                  'Coupon Rate: 2.50%
        Dim Price As Double = 96.0076                               'Price: 96.0076
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim factors As OutputRow_BONDOFC

        factors = wct.OFCFACTORS(Settlement, Maturity, Issue, FirstCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)

        With factors
            Console.WriteLine("OFC : Yield : " & wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .E, .DSC, .N, wct.IsNull(.NLF2), .A1, .DFC1, .NLF1, .A2, .DFC2, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.WriteAllText("OFC.csv", "Yield : " & wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .E, .DSC, .N, wct.IsNull(.NLF2), .A1, .DFC1, .NLF1, .A2, .DFC2, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        Yield = factors.Y
        Price = wct.NULL_DOUBLE

        With factors
            Console.WriteLine("OFC : Price : " & wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .E, .DSC, .N, wct.IsNull(.NLF2), .A1, .DFC1, .NLF1, .A2, .DFC2, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFC.csv", vbCrLf & "Price : " & wct.OFC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .E, .DSC, .N, wct.IsNull(.NLF2), .A1, .DFC1, .NLF1, .A2, .DFC2, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

    End Sub

    Public Shared Sub test_ODDLPRICE()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/1/2014#                        'Settlement Date: 2014-10-01
        Dim Maturity As Date = #12/15/2014#                         'Maturity Date: 2014-12-15
        Dim LastInterest As Date = #9/15/2014#                      'Last Interest Date: 2014-09-15
        Dim Rate As Double = 0.0225                                 'Coupon Rate: 2.250%
        Dim Yld As Double = 0.001                                   'Yield: 0.10%
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.ODDLPRICE(Settlement, Maturity, LastInterest, Rate, Yld, Redemption, Frequency, Basis)

        Console.WriteLine("ODDLPRICE : " & result.ToString())
        File.WriteAllText("ODDLPRICE.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDLYIELD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/1/2014#                        'Settlement Date: 2014-10-01
        Dim Maturity As Date = #12/15/2014#                         'Maturity Date: 2014-12-15
        Dim LastInterest As Date = #9/15/2014#                      'Last Interest Date: 2014-09-15
        Dim Rate As Double = 0.0225                                 'Coupon Rate: 2.250%
        Dim Price As Double = 100.4453                              'Price: 100.4453
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As Double

        result = wct.ODDLYIELD(Settlement, Maturity, LastInterest, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("ODDLYIELD : " & result.ToString())
        File.WriteAllText("ODDLYIELD.csv", result.ToString())

    End Sub

    Public Shared Sub test_OLCFACTORS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/1/2014#                        'Settlement Date: 2014-10-01
        Dim Maturity As Date = #12/15/2014#                         'Maturity Date: 2014-12-15
        Dim LastInterest As Date = #9/15/2014#                      'Last Interest Date: 2014-09-15
        Dim Rate As Double = 0.0225                                 'Coupon Rate: 2.250%
        Dim Price As Double = 100.4453                              'Price: 100.4453
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim result As OutputRow_BONDOLC

        result = wct.OLCFACTORS(Settlement, Maturity, LastInterest, Rate, Price, Yield, Redemption, Frequency, Basis)

        With result
            Console.WriteLine("OLCFACTORS : " & "Accrued Days : " & result.A.ToString())
            Console.WriteLine("OLCFACTORS : " & "Days from settlement to next coupon date : " & result.DSC.ToString())
            Console.WriteLine("OLCFACTORS : " & "Coupon days : " & result.E.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of coupons : " & result.N.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of quasi-coupons : " & result.NCL.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of accrued days in the first quasi-coupon period : " & result.A1.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of days in the first quasi-coupon period : " & result.DLC1.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of days from the settlement date to the quasi-coupon date : " & result.DSC1.ToString())
            Console.WriteLine("OLCFACTORS : " & "Normal length of the first quasi-coupon period : " & result.NLL1.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of accrued days in the second quasi-coupon period : " & result.A2.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of days in the second quasi-coupon period : " & result.DLC2.ToString())
            Console.WriteLine("OLCFACTORS : " & "Number of days from the quasi-cupon date to the maturity date : " & result.DSC2.ToString())
            Console.WriteLine("OLCFACTORS : " & "Normal length from quasi-coupon to maturity : " & result.NLL1.ToString())
            Console.WriteLine("OLCFACTORS : " & "Coupon : " & result.C.ToString())
            Console.WriteLine("OLCFACTORS : " & "Last coupon : " & result.LC.ToString())
            Console.WriteLine("OLCFACTORS : " & "Price : " & result.P.ToString())
            Console.WriteLine("OLCFACTORS : " & "Accrued interest : " & result.AI.ToString())
            Console.WriteLine("OLCFACTORS : " & "Yield : " & result.Y.ToString())
        End With

        File.WriteAllText("OLCFACTORS.csv", result.ToString())

    End Sub

    Public Shared Sub test_OLC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/1/2014#                        'Settlement Date: 2014-10-01
        Dim Maturity As Date = #12/15/2014#                         'Maturity Date: 2014-12-15
        Dim LastInterest As Date = #9/15/2014#                      'Last Interest Date: 2014-09-15
        Dim Rate As Double = 0.0225                                 'Coupon Rate: 2.250%
        Dim Price As Double = 100.4453                              'Price: 100.4453
        Dim Yield As Double = wct.NULL_DOUBLE                       'Yield: NULL
        Dim Redemption As Double = 100                              'Redemption: 100
        Dim Frequency As Double = 2                                 'Frequency: twice-yearly
        Dim Basis As String = "1"                                   'Basis: Actual/Actual
        Dim factors As OutputRow_BONDOLC

        factors = wct.OLCFACTORS(Settlement, Maturity, LastInterest, Rate, Price, Yield, Redemption, Frequency, Basis)

        With factors
            Console.WriteLine("OLC : Yield : " & wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A, .E, .DSC, .N, wct.IsNull(.NLL2), .A1, .DSC1, .DLC1, .NLL1, .A2, .DSC2, .DLC2, .NLL2).ToString())
            File.WriteAllText("OLC.csv", "Yield : " & wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A, .E, .DSC, .N, wct.IsNull(.NLL2), .A1, .DSC1, .DLC1, .NLL1, .A2, .DSC2, .DLC2, .NLL2).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        Yield = factors.Y
        Price = wct.NULL_DOUBLE

        With factors
            Console.WriteLine("OLC : Price : " & wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A, .E, .DSC, .N, wct.IsNull(.NLL2), .A1, .DSC1, .DLC1, .NLL1, .A2, .DSC2, .DLC2, .NLL2).ToString())
            File.AppendAllText("OLC.csv", vbCrLf & "Price : " & wct.OLC(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A, .E, .DSC, .N, wct.IsNull(.NLL2), .A1, .DSC1, .DLC1, .NLL1, .A2, .DSC2, .DLC2, .NLL2).ToString())
        End With

    End Sub

    Public Shared Sub test_PRICESTEP()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #4/15/2013#
        Dim Maturity As Date = #1/15/2019#
        Dim Yield As Double = 0.05
        Dim Redemption As Double = 100
        Dim Frequency As Double = 2
        Dim Basis As String = "0"
        Dim StepDates As Date() = {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Dim StepRates As Double() = {0.05, 0.055, 0.06}
        Dim result As Double

        result = wct.PRICESTEP(Settlement, Maturity, Yield, Redemption, Frequency, Basis, StepDates, StepRates)

        Console.WriteLine("PRICESTEP : " & result.ToString())
        File.WriteAllText("PRICESTEP.csv", result.ToString())

    End Sub

    Public Shared Sub test_YIELDSTEP()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #4/15/2013#
        Dim Maturity As Date = #1/15/2019#
        Dim Price As Double = 103.6657
        Dim Redemption As Double = 100
        Dim Frequency As Double = 2
        Dim Basis As String = "0"
        Dim StepDates As Date() = {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Dim StepRates As Double() = {0.05, 0.055, 0.06}
        Dim result As Double

        result = wct.YIELDSTEP(Settlement, Maturity, Price, Redemption, Frequency, Basis, StepDates, StepRates)

        Console.WriteLine("YIELDSTEP : " & result.ToString())
        File.WriteAllText("YIELDSTEP.csv", result.ToString())

    End Sub

    Public Shared Sub test_TBILLPRICE()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #4/1/2016#
        Dim Maturity As Date = Settlement.AddDays(91)
        Dim Discount As Double = 0.001
        Dim result As Double

        result = wct.TBILLPRICE(Settlement, Maturity, Discount)

        Console.WriteLine("TBILLPRICE : " & result.ToString())
        File.WriteAllText("TBILLPRICE.csv", result.ToString())

    End Sub

    Public Shared Sub test_TBILLYIELD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #4/1/2016#
        Dim Maturity As Date = Settlement.AddDays(91)
        Dim Price As Double = 99.9747
        Dim result As Double

        result = wct.TBILLYIELD(Settlement, Maturity, Price)

        Console.WriteLine("TBILLYIELD : " & result.ToString())
        File.WriteAllText("TBILLYIELD.csv", result.ToString())

    End Sub

    Public Shared Sub test_TBILLEQ()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #4/1/2016#
        Dim Maturity As Date = Settlement.AddDays(91)
        Dim Disc As Double = 0.0028
        Dim result As Double

        result = wct.TBILLEQ(Settlement, Maturity, Disc)

        Console.WriteLine("TBILLEQ : " & "BOND EQUIVALENT YIELD : " & result.ToString())
        File.WriteAllText("TBILLEQ.csv", result.ToString())

    End Sub

    Public Shared Sub test_PRICEACT()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/29/2014#
        Dim Maturity As Date = #1/11/2014#
        Dim Rate As Double = 0.11
        Dim Par As Double = 100
        Dim Yield As Double = 0.125
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim RepaymentDates As Date()
        Dim RepaymentAmounts As Double()
        Dim result As Double

        'with no repayment schedule
        result = wct.PRICEACT(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis)

        Console.WriteLine("PRICEACT : " & result.ToString())
        File.WriteAllText("PRICEACT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'with a repayment schedule
        RepaymentDates = New Date() {#4/30/2012#, #10/31/2012#, #4/30/2013#, #10/31/2013#, #4/30/2014#, #10/31/2014#, #4/30/2015#, #10/31/2015#, #4/30/2016#, #10/31/2016#, #4/30/2017#, #10/31/2017#, #4/30/2018#, #10/31/2018#, #4/30/2019#, #10/31/2019#}
        RepaymentAmounts = New Double() {6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25}
        Rate = 0.125

        result = wct.PRICEACT(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts)

        Console.WriteLine("PRICEACT : " & result.ToString())
        File.AppendAllText("PRICEACT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_YIELDACT()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/29/2014#
        Dim Maturity As Date = #1/11/2014#
        Dim Rate As Double = 0.11
        Dim Par As Double = 100
        Dim Price As Double = 89.05835
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim RepaymentDates As Date()
        Dim RepaymentAmounts As Double()
        Dim result As Double

        'with no repayment schedule
        result = wct.YIELDACT(Settlement, Maturity, Rate, Par, Price, Frequency, Basis)

        Console.WriteLine("YIELDACT : " & result.ToString())
        File.WriteAllText("YIELDACT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'with a repayment schedule
        RepaymentDates = New Date() {#4/30/2012#, #10/31/2012#, #4/30/2013#, #10/31/2013#, #4/30/2014#, #10/31/2014#, #4/30/2015#, #10/31/2015#, #4/30/2016#, #10/31/2016#, #4/30/2017#, #10/31/2017#, #4/30/2018#, #10/31/2018#, #4/30/2019#, #10/31/2019#}
        RepaymentAmounts = New Double() {6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25}

        Rate = 0.125
        Price = 99.9985

        result = wct.YIELDACT(Settlement, Maturity, Rate, Par, Price, Frequency, Basis, RepaymentDates, RepaymentAmounts)

        Console.WriteLine("YIELDACT : " & result.ToString())
        File.AppendAllText("YIELDACT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_PRICEACTTV()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Par As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim RepaymentDates As Date()
        Dim RepaymentAmounts As Double()
        Dim result As XLeratorDLL_financial.FinancialTypes.PRICEACTTV_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'with no repayment schedule
        Settlement = #10/29/2014#
        Maturity = #11/1/2034#
        Rate = 0.11
        Par = 100
        Yield = 0.125
        Frequency = 2
        Basis = "1"

        result = wct.PRICEACTTV(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis)

        'Write the table output to a csv
        Dim retStr1 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName & ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next

        Console.WriteLine("PRICEACTTV : " & retStr1.ToString())
        File.WriteAllText("PRICEACTTV.csv", retStr1.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'with a repayment schedule
        Settlement = #10/29/2014#
        Maturity = #10/31/2019#
        Rate = 0.125
        Par = 100
        Yield = 0.125
        Frequency = 2
        Basis = "1"
        RepaymentDates = New Date() {#4/30/2012#, #10/31/2012#, #4/30/2013#, #10/31/2013#, #4/30/2014#, #10/31/2014#, #4/30/2015#, #10/31/2015#, #4/30/2016#, #10/31/2016#, #4/30/2017#, #10/31/2017#, #4/30/2018#, #10/31/2018#, #4/30/2019#, #10/31/2019#}
        RepaymentAmounts = New Double() {6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25, 6.25}

        result = wct.PRICEACTTV(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts)

        'Write the table output to a csv
        Dim retStr2 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr2.Append(col.ColumnName & ",")
        Next
        retStr2.Remove(retStr2.Length - 1, 1)
        retStr2.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr2.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr2.Append(",")
                Else
                    retStr2.AppendLine()
                End If
            Next
        Next

        Console.WriteLine("PRICEACTTV : " & retStr2.ToString())
        File.AppendAllText("PRICEACTTV.csv", vbCrLf & retStr2.ToString())

    End Sub

    Public Shared Sub test_PRICEFR()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/16/2014#
        Dim Maturity As Date = #11/1/2034#
        Dim Rate As Double = 0.0475
        Dim Par As Double = 100
        Dim Yield As Double = 0.04
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim RepaymentDates As Date()
        Dim RepaymentAmounts As Double()
        Dim result As Double

        'repayment schedule
        RepaymentDates = New Date() {#11/1/2026#, #11/1/2028#, #11/1/2030#, #11/1/2032#}
        RepaymentAmounts = New Double() {20, 20, 20, 20}

        result = wct.PRICEFR(Settlement, Maturity, Rate, Par, Yield, Frequency, Basis, RepaymentDates, RepaymentAmounts)

        Console.WriteLine("PRICEFR : " & result.ToString())
        File.WriteAllText("PRICEFR.csv", result.ToString())

    End Sub

    Public Shared Sub test_YIELDFR()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #10/16/2014#
        Dim Maturity As Date = #11/1/2034#
        Dim Rate As Double = 0.0475
        Dim Par As Double = 100
        Dim Price As Double = 108.753442
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim RepaymentDates As Date()
        Dim RepaymentAmounts As Double()
        Dim result As Double

        'repayment schedule
        RepaymentDates = New Date() {#11/1/2026#, #11/1/2028#, #11/1/2030#, #11/1/2032#}
        RepaymentAmounts = New Double() {20, 20, 20, 20}

        result = wct.YIELDFR(Settlement, Maturity, Rate, Par, Price, Frequency, Basis, RepaymentDates, RepaymentAmounts)

        Console.WriteLine("YIELDFR : " & result.ToString())
        File.WriteAllText("YIELDFR.csv", result.ToString())

    End Sub

    Public Shared Sub test_DIRTYPRICE()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Redemption As Double
        Dim Basis As String
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Regular coupons
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : RPI : " & result.ToString())    '96.9453139716364
        File.WriteAllText("DIRTYPRICE.csv", "RPI : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Odd Short First
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #5/1/2014#
        FirstCoupon = #6/15/2014#

        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : Odd Short First : " & result.ToString())    '96.0075631077824
        File.AppendAllText("DIRTYPRICE.csv", vbCrLf & "OSF : " & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Odd Long First
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #5/1/2014#
        FirstCoupon = #12/15/2014#

        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : Odd Long First : " & result.ToString())     '96.0033702877755
        File.AppendAllText("DIRTYPRICE.csv", vbCrLf & "OLF : " & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Odd Short Last
        Settlement = #10/1/2014#
        Maturity = #12/15/2014#
        Rate = 0.0225
        Yield = 0.001
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = #9/15/2014#

        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : Odd Short Last : " & result.ToString())     '100.544776634675
        File.AppendAllText("DIRTYPRICE.csv", vbCrLf & "OSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Odd Long Last
        Settlement = #10/1/2014#
        Maturity = #12/15/2014#
        Rate = 0.0225
        Yield = 0.001
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = #3/15/2014#
        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : Odd Long Last : " & result.ToString())      '101.669543602845
        File.AppendAllText("DIRTYPRICE.csv", vbCrLf & "OLL : " & result.ToString())

        '=====================
        ' EXAMPLE# 6
        '=====================

        'Odd long first & odd long last
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#

        result = wct.DIRTYPRICE(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYPRICE : Odd Long First & Last : " & result.ToString())  '104.420159644839
        File.AppendAllText("DIRTYPRICE.csv", vbCrLf & "OLFOLL : " & result.ToString())

    End Sub

    Public Shared Sub test_DIRTYYIELD()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Price As Double
        Dim Frequency As Double
        Dim Redemption As Double
        Dim Basis As String
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Regular coupons
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Price = 96.9453139716364
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : RPI : " & result.ToString())    '0.0276
        File.WriteAllText("DIRTYYIELD.csv", "RPI : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Odd Short First
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Price = 96.0075631077824
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #5/1/2014#
        FirstCoupon = #6/15/2014#

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : Odd Short First : " & result.ToString())    '0.0276
        File.AppendAllText("DIRTYYIELD.csv", vbCrLf & "OSF : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Odd Long First
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Price = 96.0033702877755
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #5/1/2014#
        FirstCoupon = #12/15/2014#

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : Odd Long First : " & result.ToString())     '0.0276
        File.AppendAllText("DIRTYYIELD.csv", vbCrLf & "OLF : " & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Odd Short Last
        Settlement = #10/1/2014#
        Maturity = #12/15/2014#
        Rate = 0.0225
        Price = 100.544776634675
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = #9/15/2014#

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : Odd Short Last : " & result.ToString())     '0.001
        File.AppendAllText("DIRTYYIELD.csv", vbCrLf & "OSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Odd Long Last
        Settlement = #10/1/2014#
        Maturity = #12/15/2014#
        Rate = 0.0225
        Price = 101.669543602845
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = #3/15/2014#

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : Odd Long Last : " & result.ToString())      '0.001
        File.AppendAllText("DIRTYYIELD.csv", vbCrLf & "OLL : " & result.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Odd long first & odd long last
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Rate = 0.03125
        Price = 104.420159644839
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#

        result = wct.DIRTYYIELD(Settlement, Maturity, Rate, Price, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        Console.WriteLine("DIRTYYIELD : Odd Long First & Last : " & result.ToString())  '0.02875
        File.AppendAllText("DIRTYYIELD.csv", vbCrLf & "OLFL : " & result.ToString())

    End Sub

    Public Shared Sub test_OFLPRICE()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a bond with an odd short first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLPRICE : " & "OSFOSL : " & result.ToString())      '102.00036224598
        File.WriteAllText("OFLPRICE.csv", "OSFOSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This is a bond with an odd long first coupon and an odd long last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLPRICE : " & "OLFOLL : " & result.ToString())      '102.103433425767
        File.AppendAllText("OFLPRICE.csv", vbCrLf & "OLFOLL : " & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'This is a bond with an odd long first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLPRICE : " & "OLFOSL : " & result.ToString())      '102.104790915433
        File.AppendAllText("OFLPRICE.csv", vbCrLf & "OLFOSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'This is an example of a bond paying interest every 26 weeks.
        Settlement = #10/4/2014#
        Maturity = #12/12/2029#
        Issue = #7/30/2014#
        FirstCoupon = #3/18/2015#
        LastCoupon = #2/28/2029#
        Rate = 0.125
        Yield = 0.11
        Redemption = 100
        Frequency = 182
        Basis = "9"

        result = wct.OFLPRICE(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLPRICE : " & "RPI : " & result.ToString())      '110.846098828263
        File.AppendAllText("OFLPRICE.csv", vbCrLf & "RPI : " & result.ToString())

    End Sub

    Public Shared Sub test_OFLYIELD()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim Rate As Double
        Dim Price As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a bond with an odd short first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 102.00036224598
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("OFLYIELD : " & "OSFOSL : " & result.ToString())      '0.02875
        File.WriteAllText("OFLYIELD.csv", vbCrLf & "OSFOSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This is a bond with an odd long first coupon and an odd long last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 102.103433425767
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("OFLYIELD : " & "OLFOLL : " & result.ToString())      '0.02875
        File.AppendAllText("OFLYIELD.csv", vbCrLf & "OLFOLL : " & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'This is a bond with an odd long first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 102.104790915433
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("OFLYIELD : " & "OLFOSL : " & result.ToString())      '0.02875
        File.AppendAllText("OFLYIELD.csv", vbCrLf & "OLFOSL : " & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'This is an example of a bond paying interest every 26 weeks.
        Settlement = #10/4/2014#
        Maturity = #12/12/2029#
        Issue = #7/30/2014#
        FirstCoupon = #3/18/2015#
        LastCoupon = #2/28/2029#
        Rate = 0.125
        Price = 110.846098828263
        Redemption = 100
        Frequency = 182
        Basis = "9"

        result = wct.OFLYIELD(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Redemption, Frequency, Basis)

        Console.WriteLine("OFLYIELD : " & "RPI : " & result.ToString())      '0.11
        File.AppendAllText("OFLYIELD.csv", vbCrLf & "RPI : " & result.ToString())

    End Sub

    Public Shared Sub test_OFLFACTORS()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Price As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Redemption As Double
        Dim Basis As String
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim factors As OutputRow_BONDOFL

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a bond with an odd short first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = wct.NULL_DOUBLE
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFLFACTORS : " & "OSFOSL A1 : " & .A1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL A2 : " & .A2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL DSC : " & .DSC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL E : " & .E.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL N : " & .N.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NCL : " & .NCL.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NCF : " & .NCF.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL DLC1 : " & .DLC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL DLC2 : " & .DLC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NLL1 : " & .NLL1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NLL2 : " & .NLL2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL DFC1 : " & .DFC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL DFC2 : " & .DFC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NLF1 : " & .NLF1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL NLF2 : " & .NLF2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL Nqf : " & .Nqf.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL quasistart : " & .quasistart.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL quasicoupfirs t : " & .quasicoupfirst.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL quasicouplast : " & .quasicouplast.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL quasimaturity : " & .quasimaturity.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL C : " & .C.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL LC : " & .LC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL FC : " & .FC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL P : " & .P.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL AI : " & .AI.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOSL Y : " & .Y.ToString())
        End With
        File.WriteAllText("OFLFACTORS.csv", "OSFOSL : " & factors.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This is a bond with an odd long first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 101.999004756314
        Yield = wct.NULL_DOUBLE
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFLFACTORS : " & "OLFOSL A1 : " & .A1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL A2 : " & .A2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL DSC : " & .DSC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL E : " & .E.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL N : " & .N.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NCL : " & .NCL.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NCF : " & .NCF.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL DLC1 : " & .DLC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL DLC2 : " & .DLC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NLL1 : " & .NLL1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NLL2 : " & .NLL2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL DFC1 : " & .DFC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL DFC2 : " & .DFC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NLF1 " & .NLF1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL NLF2 : " & .NLF2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL Nqf : " & .Nqf.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL quasistart : " & .quasistart.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL quasicoupfirst : " & .quasicoupfirst.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL quasicouplast : " & .quasicouplast.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL quasimaturity : " & .quasimaturity.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL C : " & .C.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL LC : " & .LC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL FC : " & .FC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL P : " & .P.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL AI : " & .AI.ToString())
            Console.WriteLine("OFLFACTORS : " & "OLFOSL Y : " & .Y.ToString())
        End With
        File.AppendAllText("OFLFACTORS.csv", vbCrLf & "OLFOSL : " & factors.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'This is a bond with an odd short first coupon and an odd long last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 102.104790915433
        Yield = wct.NULL_DOUBLE
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFLFACTORS : " & "OSFOLL A1 : " & .A1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL A2 : " & .A2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL DSC : " & .DSC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL E : " & .E.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL N " & .N.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NCL : " & .NCL.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NCF : " & .NCF.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL DLC1 : " & .DLC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL DLC2 : " & .DLC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NLL1 : " & .NLL1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NLL2 : " & .NLL2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL DFC1 : " & .DFC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL DFC2 : " & .DFC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NLF1 " & .NLF1.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL NLF2 : " & .NLF2.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL Nqf : " & .Nqf.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL quasistart : " & .quasistart.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL quasicoupfirst : " & .quasicoupfirst.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL quasicouplast : " & .quasicouplast.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL quasimaturity : " & .quasimaturity.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL C : " & .C.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL LC : " & .LC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL FC : " & .FC.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL P : " & .P.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL AI : " & .AI.ToString())
            Console.WriteLine("OFLFACTORS : " & "OSFOLL Y : " & .Y.ToString())
        End With
        File.AppendAllText("OFLFACTORS.csv", vbCrLf & "OSFOLL : " & factors.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'This is an example of a bond paying interest every 26 weeks.
        Settlement = #10/4/2014#
        Maturity = #12/12/2029#
        Issue = #7/30/2014#
        FirstCoupon = #3/18/2015#
        LastCoupon = #2/28/2029#
        Rate = 0.125
        Price = wct.NULL_DOUBLE
        Yield = 0.11
        Redemption = 100
        Frequency = 182
        Basis = "9"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFLFACTORS : " & "RPI A1 : " & .A1.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI A2 : " & .A2.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI DSC : " & .DSC.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI E : " & .E.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI N " & .N.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NCL : " & .NCL.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NCF : " & .NCF.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI DLC1 : " & .DLC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI DLC2 : " & .DLC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NLL1 : " & .NLL1.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NLL2 : " & .NLL2.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI DFC1 : " & .DFC1.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI DFC2 : " & .DFC2.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NLF1 " & .NLF1.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI NLF2 : " & .NLF2.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI Nqf : " & .Nqf.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI quasistart : " & .quasistart.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI quasicoupfirst : " & .quasicoupfirst.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI quasicouplast : " & .quasicouplast.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI quasimaturity : " & .quasimaturity.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI C : " & .C.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI LC : " & .LC.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI FC : " & .FC.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI P : " & .P.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI AI : " & .AI.ToString())
            Console.WriteLine("OFLFACTORS : " & "RPI Y : " & .Y.ToString())
        End With
        File.AppendAllText("OFLFACTORS.csv", vbCrLf & "RPI : " & factors.ToString())

    End Sub

    Public Shared Sub test_OFL()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Price As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Redemption As Double
        Dim Basis As String
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim factors As OutputRow_BONDOFL

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a bond with an odd short first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = wct.NULL_DOUBLE
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFL : OSFOSL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.WriteAllText("OFL.csv", "OSFOSL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            Price = .P
            Yield = wct.NULL_DOUBLE
            Console.WriteLine("OFL : OSFOSL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "OSFOSL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This is a bond with an odd long first coupon and an odd short last coupon.
        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        Issue = #6/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 101.999004756314
        Yield = wct.NULL_DOUBLE
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFL : OLFOSL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "OLFOSL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            Yield = .Y
            Price = wct.NULL_DOUBLE
            Console.WriteLine("OFL : OLFOSL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "OLFOSL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

        '=====================
        ' EXAMPLE# 3
        '=====================

        'This is a bond with an odd short first coupon and an odd long last coupon.
        Settlement = #3/4/2013#
        Maturity = #11/28/2022#
        Issue = #12/7/2012#
        FirstCoupon = #3/15/2013#
        LastCoupon = #3/15/2022#
        Rate = 0.03125
        Price = 102.104790915433
        Yield = wct.NULL_DOUBLE
        Redemption = 100
        Frequency = 2
        Basis = "1"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFL : OSFOLL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "OSFOLL Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            Yield = .Y
            Price = wct.NULL_DOUBLE
            Console.WriteLine("OFL : OSFOLL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "OSFOLL Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

        '=====================
        ' EXAMPLE# 4
        '=====================

        'This is an example of a bond paying interest every 26 weeks.
        Settlement = #10/4/2014#
        Maturity = #12/12/2029#
        Issue = #7/30/2014#
        FirstCoupon = #3/18/2015#
        LastCoupon = #2/28/2029#
        Rate = 0.125
        Price = wct.NULL_DOUBLE
        Yield = 0.11
        Redemption = 100
        Frequency = 182
        Basis = "9"

        factors = wct.OFLFACTORS(Settlement, Maturity, Issue, FirstCoupon, LastCoupon, Rate, Price, Yield, Redemption, Frequency, Basis)
        With factors
            Console.WriteLine("OFL : RPI Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "RPI Price : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            Price = .P
            Yield = wct.NULL_DOUBLE
            Console.WriteLine("OFL : RPI Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
            File.AppendAllText("OFL.csv", vbCrLf & "RPI Yield : " & wct.OFL(Rate, Yield, Price, Redemption, Convert.ToInt32(Frequency), .A1, .A2, .DSC, .E, .N, If(.NCF = 1, True, False), If(.NCL = 1, True, False), .DLC1, .DLC2, .NLL1, .NLL2, .DFC1, .DFC2, .NLF1, .NLF2, Convert.ToInt32(.Nqf)).ToString())
        End With

    End Sub

    Public Shared Sub test_BONDCF()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim result As XLeratorDLL_financial.FinancialTypes.BONDCF_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'In this example we generate the cash flows for bond which pays regular period interest and is maturing on 2034-06-15. 
        'The settlement date is 2014-05-01, the yield is 2.76%, the coupon rate is 2.50%, the redemption value is 100, 
        'the coupon is paid twice-yearly and the day-count convention is actual/actual.
        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        result = wct.BONDCF(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        'Write the table output to a csv
        Dim retStr1 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName & ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next

        Console.WriteLine("BONDCF : " & retStr1.ToString())
        File.WriteAllText("BONDCF.csv", retStr1.ToString())

    End Sub

    Public Shared Sub test_STEPCF()
        Dim Settlement As DateTime
        Dim Maturity As DateTime
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim Issue As DateTime
        Dim FirstCoupon As DateTime
        Dim LastCoupon As DateTime
        Dim Steps As System.Array
        Dim result As XLeratorDLL_financial.FinancialTypes.BONDCF_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'In this example we generate the cash flows for bond maturing on 2034-06-15. The settlement date is 2014-05-01, the yield is 2.76%. 
        'The coupon is paid twice-yearly, the redemption value is 100 and the day-count convention is actual/actual. 
        'The step-rate schedule is passed in using a derived table.

        Settlement = New DateTime(2014, 5, 1)
        Maturity = New DateTime(2034, 6, 15)

        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        Steps = New Object(,) {
                                {New DateTime(2029, 12, 15), New DateTime(2024, 12, 15), New DateTime(2020, 12, 15), New DateTime(2016, 12, 15), New DateTime(2012, 12, 15)},
                                {0.015, 0.014, 0.013, 0.012, 0.011}
                                }

        result = wct.STEPCF(Settlement, Maturity, Steps, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        'Write the table output to a csv
        Dim retStr1 As StringBuilder = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName + ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next 
        Next
        Console.WriteLine("STEPCF : " + retStr1.ToString())
        File.WriteAllText("STEPCF.csv", retStr1.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example we use the same data but use two typed arrays instead of 2-dimnsional object array.

        Settlement = New DateTime(2014, 5, 1)
        Maturity = New DateTime(2034, 6, 15)
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        Dim Steps_date As DateTime() = {New DateTime(2029, 12, 15), New DateTime(2024, 12, 15), New DateTime(2020, 12, 15), New DateTime(2016, 12, 15), New DateTime(2012, 12, 15)}
        Dim Steps_rate As Double() = {0.015, 0.014, 0.013, 0.012, 0.011}

        result = wct.STEPCF(Settlement, Maturity, Steps_date, Steps_rate, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        'Write the table output to a csv
        retStr1 = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName + ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("STEPCF : " + retStr1.ToString())
        File.WriteAllText("STEPCF.csv", retStr1.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example, we have a bond with zero first step. We use a ado datatable which typically would be populated with input data queried from a database.

        'SELECT
        '   *
        'FROM
        '   wct.STEPCF(
        '        '2016-07-01' --@Settlement
        '       ,'2044-06-15' --@Maturity
        '       ,'SELECT * FROM #t' --@Step
        '       ,0.0290       --@Yield
        '       ,100          --@Redemption
        '       ,2            --@Frequency
        '       ,1            --@Basis
        '       ,NULL         --@Issue
        '       ,NULL         --@FirstCoupon
        '       ,NULL         --@LastCoupon
        '       )
        Settlement = New DateTime(2016, 7, 1)
        Maturity = New DateTime(2044, 6, 15)
        Yield = 0.029
        Redemption = 100
        Frequency = 2
        Basis = "1"
        Issue = wct.NULL_DATE
        FirstCoupon = wct.NULL_DATE
        LastCoupon = wct.NULL_DATE

        'SELECT
        '   *
        'INTO
        '   #t
        'FROM (VALUES
        '   ('2040-12-15', .018),
        '   ('2036-12-15', .017),
        '   ('2032-12-15', .016),
        '   ('2028-12-15', .015),
        '   ('2024-12-15', .014)
        '   )n(date_step, rate_step)
        Dim Steps_data As DataTable = New DataTable("Steps_data")
        Steps_data.Columns.Add("date_step", (New DateTime()).GetType())
        Steps_data.Columns.Add("rate_step", (New Double()).GetType())
        Steps_data.Rows.Add(New DateTime(2040, 12, 15), 0.018)
        Steps_data.Rows.Add(New DateTime(2036, 12, 15), 0.017)
        Steps_data.Rows.Add(New DateTime(2032, 12, 15), 0.016)
        Steps_data.Rows.Add(New DateTime(2028, 12, 15), 0.015)
        Steps_data.Rows.Add(New DateTime(2024, 12, 15), 0.014)

        result = wct.STEPCF(Settlement, Maturity, Steps_data, Yield, Redemption, Frequency, Basis, Issue, FirstCoupon, LastCoupon)

        'Write the table output to a csv
        retStr1 = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName + ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("STEPCF : " + retStr1.ToString())
        File.WriteAllText("STEPCF.csv", retStr1.ToString())

    End Sub

    Public Shared Sub test_COUPDAYBS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Double

        result = wct.COUPDAYBS(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPDAYBS : " & result.ToString())
        File.WriteAllText("COUPDAYBS.csv", result.ToString())

    End Sub

    Public Shared Sub test_COUPDAYS()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Double

        result = wct.COUPDAYS(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPDAYS : " & result.ToString())
        File.WriteAllText("COUPDAYS.csv", result.ToString())

    End Sub

    Public Shared Sub test_COUPDAYSNC()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Double

        result = wct.COUPDAYSNC(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPDAYSNC : " & result.ToString())
        File.WriteAllText("COUPDAYSNC.csv", result.ToString())

    End Sub

    Public Shared Sub test_COUPNUM()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Double

        result = wct.COUPNUM(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPNUM : " & result.ToString())
        File.WriteAllText("COUPNUM.csv", result.ToString())

    End Sub

    Public Shared Sub test_COUPNCD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Date

        result = wct.COUPNCD(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPNCD : " & result.ToShortDateString)
        File.WriteAllText("COUPNCD.csv", result.ToString())

    End Sub

    Public Shared Sub test_COUPPCD()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date = #12/16/2007#
        Dim Maturity As Date = #3/1/2027#
        Dim Frequency As Double = 2
        Dim Basis As String = "1"
        Dim result As Date

        result = wct.COUPPCD(Settlement, Maturity, Frequency, Basis)

        Console.WriteLine("COUPPCD : " & result.ToShortDateString)
        File.WriteAllText("COUPPCD.csv", result.ToString())

    End Sub

    Public Shared Sub test_CFCONVEXITY()
        Dim Disc_rate As Double
        Dim VDate As Date
        Dim CFAmt As Double()
        Dim CFDate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Disc_rate = 0.050602946
        VDate = #12/21/2012#
        CFDate = New Date() {#5/15/2013#, #11/15/2013#, #5/15/2014#, #11/15/2014#, #5/15/2015#, #11/15/2015#, #5/15/2016#, #11/15/2016#, #5/15/2017#, #11/15/2017#}
        CFAmt = New Double() {2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5}

        result = wct.CFCONVEXITY(Disc_rate, VDate, CFAmt, CFDate)

        Console.WriteLine("CFOVNEXITY : " & result.ToString())     '22.6933974996726
        File.WriteAllText("CFOVNEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_CFDURATION()
        Dim Disc_rate As Double
        Dim VDate As Date
        Dim CFAmt As Double()
        Dim CFDate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Disc_rate = 0.050602946
        VDate = #12/21/2012#
        CFDate = New Date() {#5/15/2013#, #11/15/2013#, #5/15/2014#, #11/15/2014#, #5/15/2015#, #11/15/2015#, #5/15/2016#, #11/15/2016#, #5/15/2017#, #11/15/2017#}
        CFAmt = New Double() {2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5}

        result = wct.CFDURATION(Disc_rate, VDate, CFAmt, CFDate)

        Console.WriteLine("CFDURATION : " & result.ToString())     '4.38864089754494
        File.WriteAllText("CFDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_CFMDURATION()
        Dim Disc_rate As Double
        Dim VDate As Date
        Dim CFAmt As Double()
        Dim CFDate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Disc_rate = 0.050602946
        VDate = #12/21/2012#
        CFDate = New Date() {#5/15/2013#, #11/15/2013#, #5/15/2014#, #11/15/2014#, #5/15/2015#, #11/15/2015#, #5/15/2016#, #11/15/2016#, #5/15/2017#, #11/15/2017#}
        CFAmt = New Double() {2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 2.5, 102.5}
        result = wct.CFMDURATION(Disc_rate, VDate, CFAmt, CFDate)

        Console.WriteLine("CFMDURATION : " & result.ToString())     '4.17725927216746
        File.WriteAllText("CFMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_CONVEXITY()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Frequency As Integer
        Dim Basis As String
        Dim Par As Double
        Dim Redemption As Double
        Dim IssueDate As Date
        Dim FirstInterestDate As Date
        Dim LastInterestDate As Date
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #12/12/2011#
        Maturity = #4/20/2018#
        Rate = 0.02
        Yield = 0.0217
        Frequency = 2
        Basis = "1"
        Par = 100
        Redemption = 100
        IssueDate = wct.NULL_DATE
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE

        result = wct.CONVEXITY(Settlement, Maturity, Rate, Yield, Frequency, Basis, Par, Redemption, IssueDate, FirstInterestDate, LastInterestDate)

        Console.WriteLine("CONVEXITY : " & result.ToString())      '39.3347575344594
        File.WriteAllText("CONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_DURATION()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        Settlement = #10/10/2014#
        Maturity = #11/30/2033#
        Rate = 0.03
        Yield = 0.0315
        Frequency = 2
        Basis = "1"

        result = wct.DURATION(Settlement, Maturity, Rate, Yield, Frequency, Basis)

        Console.WriteLine("DURATION : " & result.ToString())       '14.4816608104738
        File.WriteAllText("DURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_MDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #10/10/2014#
        Maturity = #11/30/2033#
        Rate = 0.03
        Yield = 0.0315
        Frequency = 2
        Basis = "1"

        result = wct.MDURATION(Settlement, Maturity, Rate, Yield, Frequency, Basis)

        Console.WriteLine("MDURATION : " & result.ToString())       '14.2571113073825
        File.WriteAllText("MDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFCCONVEXITY()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        IssueDate = #5/1/2014#
        FirstCouponDate = #6/15/2014#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFCCONVEXITY(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFCCONVEXITY : " & result.ToString())       '2.91355672454477
        File.WriteAllText("OFCCONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFCDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        IssueDate = #5/1/2014#
        FirstCouponDate = #6/15/2014#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFCDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFCDURATION : " & result.ToString())       '15.8299356059947
        File.WriteAllText("OFCDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFCMDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        IssueDate = #5/1/2014#
        FirstCouponDate = #6/15/2014#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFCMDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFCMDURATION : " & result.ToString())       '15.6144561116539
        File.WriteAllText("OFCMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFLCONVEXITY()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        IssueDate = #12/7/2012#
        FirstCouponDate = #3/15/2013#
        LastCouponDate = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLCONVEXITY(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLCONVEXITY : " & result.ToString())       '0.715521292648933
        File.WriteAllText("OFLCONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFLDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        IssueDate = #12/7/2012#
        FirstCouponDate = #3/15/2013#
        LastCouponDate = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLDURATION : " & result.ToString())       '7.98265325301858
        File.WriteAllText("OFLDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OFLMDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim IssueDate As Date
        Dim FirstCouponDate As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #3/4/2013#
        Maturity = #4/28/2022#
        IssueDate = #12/7/2012#
        FirstCouponDate = #3/15/2013#
        LastCouponDate = #3/15/2022#
        Rate = 0.03125
        Yield = 0.02875
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OFLMDURATION(Settlement, Maturity, IssueDate, FirstCouponDate, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OFLMDURATION : " & result.ToString())       '7.86952877685135
        File.WriteAllText("OFLMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OLCCONVEXITY()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim Settlement As Date
        Dim Maturity As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        Settlement = #10/1/2014#
        Maturity = #12/15/2034#
        LastCouponDate = #9/15/2034#
        Rate = 0.0425
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OLCCONVEXITY(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OLCCONVEXITY : " & result.ToString())       '2.3881623279505
        File.WriteAllText("OLCCONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_OLCDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #10/1/2014#
        Maturity = #12/15/2034#
        LastCouponDate = #9/15/2034#
        Rate = 0.0425
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OLCDURATION(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OLCDURATION : " & result.ToString())       '13.8535032850962
        File.WriteAllText("OLCDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_OLCMDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim LastCouponDate As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #10/1/2014#
        Maturity = #12/15/2034#
        LastCouponDate = #9/15/2034#
        Rate = 0.0425
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.OLCMDURATION(Settlement, Maturity, LastCouponDate, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("OLCMDURATION : " & result.ToString())       '13.5818659657806
        File.WriteAllText("OLCMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_RPICONVEXITY()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.RPICONVEXITY(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("RPICONVEXITY : " & result.ToString())       '2.88538118059157
        File.WriteAllText("RPICONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_RPIDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.RPIDURATION(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("RPIDURATION : " & result.ToString())       '15.6780086644317
        File.WriteAllText("RPIDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_RPIMDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #5/1/2014#
        Maturity = #6/15/2034#
        Rate = 0.025
        Yield = 0.0276
        Redemption = 100
        Frequency = 2
        Basis = "1"

        result = wct.RPIMDURATION(Settlement, Maturity, Rate, Yield, Redemption, Frequency, Basis)

        Console.WriteLine("RPIMDURATION : " & result.ToString())       '15.4645972227576
        File.WriteAllText("RPIMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_STEPCONVEXITY()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim CoupDates As Date()
        Dim Coupons As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #4/21/2014#
        Maturity = #1/15/2019#
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "0"
        CoupDates = New Date() {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Coupons = New Double() {0.05, 0.055, 0.06}

        result = wct.STEPCONVEXITY(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons)

        Console.WriteLine("STEPCONVEXITY : " & result.ToString())       '0.202295869292095
        File.WriteAllText("STEPCONVEXITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_STEPDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim CoupDates As Date()
        Dim Coupons As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #4/21/2014#
        Maturity = #1/15/2019#
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "0"
        CoupDates = New Date() {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Coupons = New Double() {0.05, 0.055, 0.06}

        result = wct.STEPDURATION(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons)

        Console.WriteLine("STEPDURATION : " & result.ToString())       '4.18487140205481
        File.WriteAllText("STEPDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_STEPMDURATION()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Yield As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim CoupDates As Date()
        Dim Coupons As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #4/21/2014#
        Maturity = #1/15/2019#
        Yield = 0.04
        Redemption = 100
        Frequency = 2
        Basis = "0"
        CoupDates = New Date() {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Coupons = New Double() {0.05, 0.055, 0.06}

        result = wct.STEPMDURATION(Settlement, Maturity, Yield, Redemption, Frequency, Basis, CoupDates, Coupons)

        Console.WriteLine("STEPMDURATION : " & result.ToString())       '4.10281510005374
        File.WriteAllText("STEPMDURATION.csv", result.ToString())

    End Sub

    Public Shared Sub test_ACCINTACT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Par As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #10/29/2014#
        Maturity = #11/1/2034#
        Rate = 0.11
        Par = 100
        Frequency = 2
        Basis = "1"

        result = wct.ACCINTACT(Settlement, Maturity, Rate, Par, Frequency, Basis)

        Console.WriteLine("ACCINTACT : " & result.ToString())      '5.45479452054794
        File.WriteAllText("ACCINTACT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ACCRINT()
        Dim Issue As Date
        Dim FirstInterest As Date
        Dim Settlement As Date
        Dim Rate As Double
        Dim Par As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim CalcMethod As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Issue = #3/1/2008#
        FirstInterest = #8/31/2008#
        Settlement = #3/3/2008#
        Rate = 0.1
        Par = 1000
        Frequency = 2
        Basis = "0"
        CalcMethod = False

        result = wct.ACCRINT(Issue, FirstInterest, Settlement, Rate, Par, Frequency, Basis, CalcMethod)

        Console.WriteLine("ACCRINT : " & result.ToString())        '0.555555555555556
        File.WriteAllText("ACCRINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ACCRINTM()
        Dim Issue As Date
        Dim Settlement As Date
        Dim Rate As Double
        Dim Par As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Issue = #4/1/2008#
        Settlement = #6/15/2008#
        Rate = 0.1
        Par = 1000
        Basis = "3"
        result = wct.ACCRINTM(Issue, Settlement, Rate, Par, Basis)

        Console.WriteLine("ACCRINTM : " & result.ToString())       '20.5479452054795
        File.WriteAllText("ACCRINTM.csv", result.ToString())

    End Sub

    Public Shared Sub test_AIFACTOR()
        Dim Basis As BasisType
        Dim Rate As Double
        Dim Maturity As Date
        Dim Settlement As Date
        Dim Frequency As Integer
        Dim FirstInterestDate As Date
        Dim LastInterestDate As Date
        Dim IssueDate As Date
        Dim Holidays As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Regular periodic Interest
        Basis = BasisType.Bond
        Rate = 0.05
        Maturity = #5/15/2015#
        Settlement = #2/15/2012#
        Frequency = 2
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE
        IssueDate = wct.NULL_DATE

        result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate)

        Console.WriteLine("AIFACTOR : " & result.ToString())       '0.0125
        File.WriteAllText("AIFACTOR.csv", "BOND RPI : " & result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Compare different daycount conventions
        Rate = 0.05
        Maturity = #5/15/2015#
        Settlement = #2/29/2012#
        Frequency = 2
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE
        IssueDate = wct.NULL_DATE

        result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate)

        For i As Integer = 0 To 9
            Console.WriteLine("AIFACTOR : BOND_DC : daycount convention " & i.ToString() & " : " & wct.AIFACTOR(i.ToString(), Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate).ToString())
            File.AppendAllText("AIFACTOR.csv", vbCrLf & "BOND_DC : daycount convention " & i.ToString() & " : " & wct.AIFACTOR(i.ToString(), Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate).ToString() & vbCrLf)
        Next

        '=====================
        ' EXAMPLE# 3
        '=====================

        'BUS/252 example
        Basis = BasisType.BUS252
        Rate = 0.05
        Maturity = #6/30/2012#
        Settlement = #1/31/2012#
        Frequency = 2
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE
        IssueDate = wct.NULL_DATE
        Holidays = New Date() {#1/1/2011#, #3/7/2011#, #3/8/2011#, #4/21/2011#, #4/22/2011#, #5/1/2011#, #6/23/2011#, #9/7/2011#, #10/12/2011#, #11/2/2011#, #11/15/2011#, #12/25/2011#, #1/1/2012#, #2/20/2012#, #2/21/2012#, #4/6/2012#, #4/21/2012#, #5/1/2012#, #6/7/2012#, #9/7/2012#, #10/12/2012#, #11/2/2012#, #11/15/2012#, #12/25/2012#}

        result = wct.AIFACTOR(Basis, Rate, Maturity, Settlement, Frequency, FirstInterestDate, LastInterestDate, IssueDate, Holidays)

        Console.WriteLine("AIFACTOR : BUS/252 : " & result.ToString())     '0.00414877286812127
        File.AppendAllText("AIFACTOR.csv", vbCrLf & "BUS/252 : " & result.ToString())

    End Sub

    Public Shared Sub test_AIFACTOR_IAM()
        Dim Basis As String
        Dim Rate As Double
        Dim IssueDate As Date
        Dim Settlement As Date
        Dim Maturity As Date
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Basis = "2"
        Rate = 0.005
        IssueDate = #3/1/2012#
        Settlement = #5/21/2012#
        Maturity = #7/1/2012#

        result = wct.AIFACTOR_IAM(Basis, Rate, IssueDate, Settlement, Maturity)

        Console.WriteLine("AIFACTOR_IAM : " & result.ToString())       '0.001125
        File.WriteAllText("AIFACTOR_IAM.csv", result.ToString())

    End Sub

    Public Shared Sub test_AIFACTOR_OFC()
        Dim Basis As BasisType()
        Dim Rate As Double
        Dim IssueDate As Date
        Dim Settlement As Date
        Dim FirstInterestDate As Date
        Dim Frequency As Integer
        Dim Holidays As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Returns a result for each of the interest bases
        Basis = New BasisType() {BasisType.Bond, _
                                 BasisType.Actual, _
                                 BasisType.A360, _
                                 BasisType.A365, _
                                 BasisType.EBOND, _
                                 BasisType.ISDA_30_360, _
                                 BasisType.NLACT, _
                                 BasisType.NL360, _
                                 BasisType.NL365, _
                                 BasisType.A364, _
                                 BasisType.Bond_NON_EOM, _
                                 BasisType.Actual_NON_EOM, _
                                 BasisType.A360_NON_EOM, _
                                 BasisType.A365_NON_EOM, _
                                 BasisType.EBOND_NON_EOM, _
                                 BasisType.ISDA_30_360_NON_EOM, _
                                 BasisType.NLACT_NON_EOM, _
                                 BasisType.NL360_NON_EOM, _
                                 BasisType.NL365_NON_EOM, _
                                 BasisType.A364_NON_EOM, _
                                 BasisType.BUS252, _
                                 BasisType.Actual_ISDA, _
                                 BasisType.Actual_ISMA, _
                                 BasisType.Actual_365L, _
                                 BasisType.Actual_AFB, _
                                 BasisType.BUS252_NON_EOM}
        Holidays = New Date() {#1/1/2011#, #3/7/2011#, #3/8/2011#, #4/21/2011#, #4/22/2011#, #5/1/2011#, #6/23/2011#, #9/7/2011#, #10/12/2011#, #11/2/2011#, #11/15/2011#, #12/25/2011#, #1/1/2012#, #2/20/2012#, #2/21/2012#, #4/6/2012#, #4/21/2012#, #5/1/2012#, #6/7/2012#, #9/7/2012#, #10/12/2012#, #11/2/2012#, #11/15/2012#, #12/25/2012#}
        Rate = 0.1
        IssueDate = #7/15/2011#
        settlement = #6/30/2012#
        FirstInterestDate = #6/30/2012#
        Frequency = 2

        File.WriteAllText("AIFACTOR_OFC.csv", "")
        For i As Integer = 0 To Basis.GetUpperBound(0)

            Select Case Basis(i)
                Case BasisType.BUS252, BasisType.BUS252_NON_EOM
                    result = wct.AIFACTOR_OFC(Basis(i), Rate, IssueDate, Settlement, FirstInterestDate, Frequency, Holidays) * 10000
                Case Else
                    result = wct.AIFACTOR_OFC(Basis(i), Rate, IssueDate, Settlement, FirstInterestDate, Frequency) * 10000
            End Select

            Console.WriteLine("AIFACTOR_OFC : " & Basis(i).ToString() & " : " & result.ToString())
            File.AppendAllText("AIFACTOR_OFC.csv", Basis(i).ToString() & " : " & result.ToString() & vbCrLf)

        Next

    End Sub

    Public Shared Sub test_AIFACTOR_OLC()
        Dim Basis As BasisType()
        Dim Rate As Double
        Dim LastCouponDate As Date
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Frequency As Integer
        Dim Holidays As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Returns a result for each of the interest bases
        Basis = New BasisType() {BasisType.Bond, _
                                 BasisType.Actual, _
                                 BasisType.A360, _
                                 BasisType.A365, _
                                 BasisType.EBOND, _
                                 BasisType.ISDA_30_360, _
                                 BasisType.NLACT, _
                                 BasisType.NL360, _
                                 BasisType.NL365, _
                                 BasisType.A364, _
                                 BasisType.Bond_NON_EOM, _
                                 BasisType.Actual_NON_EOM, _
                                 BasisType.A360_NON_EOM, _
                                 BasisType.A365_NON_EOM, _
                                 BasisType.EBOND_NON_EOM, _
                                 BasisType.ISDA_30_360_NON_EOM, _
                                 BasisType.NLACT_NON_EOM, _
                                 BasisType.NL360_NON_EOM, _
                                 BasisType.NL365_NON_EOM, _
                                 BasisType.A364_NON_EOM, _
                                 BasisType.BUS252, _
                                 BasisType.Actual_ISDA, _
                                 BasisType.Actual_ISMA, _
                                 BasisType.Actual_365L, _
                                 BasisType.Actual_AFB, _
                                 BasisType.BUS252_NON_EOM}
        Holidays = New Date() {#1/1/2011#, #3/7/2011#, #3/8/2011#, #4/21/2011#, #4/22/2011#, #5/1/2011#, #6/23/2011#, #9/7/2011#, #10/12/2011#, #11/2/2011#, #11/15/2011#, #12/25/2011#, #1/1/2012#, #2/20/2012#, #2/21/2012#, #4/6/2012#, #4/21/2012#, #5/1/2012#, #6/7/2012#, #9/7/2012#, #10/12/2012#, #11/2/2012#, #11/15/2012#, #12/25/2012#}
        Rate = 0.1
        LastCouponDate = #7/15/2011#
        Settlement = #6/30/2012#
        Maturity = #6/30/2012#
        Frequency = 2

        File.WriteAllText("AIFACTOR_OLC.csv", "")
        For i As Integer = 0 To Basis.GetUpperBound(0)

            Select Case Basis(i)
                Case BasisType.BUS252, _
                     BasisType.BUS252_NON_EOM
                    result = wct.AIFACTOR_OLC(Basis(i), Rate, LastCouponDate, Settlement, Maturity, Frequency, Holidays) * 10000
                Case Else
                    result = wct.AIFACTOR_OLC(Basis(i), Rate, LastCouponDate, Settlement, Maturity, Frequency) * 10000
            End Select

            Console.WriteLine("AIFACTOR_OLC : " & Basis(i).ToString() & " : " & result.ToString())
            File.AppendAllText("AIFACTOR_OLC.csv", Basis(i).ToString() & " : " & result.ToString() & vbCrLf)

        Next

    End Sub

    Public Shared Sub test_AIFACTOR_RPI()
        Dim Basis As BasisType()
        Dim Rate As Double
        Dim PrevCoupDate As Date
        Dim NextCoupDate As Date
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Frequency As Integer
        Dim Holidays As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Returns a result for each of the interest bases
        Basis = New BasisType() {BasisType.Bond, _
                                 BasisType.Actual, _
                                 BasisType.A360, _
                                 BasisType.A365, _
                                 BasisType.EBOND, _
                                 BasisType.ISDA_30_360, _
                                 BasisType.NLACT, _
                                 BasisType.NL360, _
                                 BasisType.NL365, _
                                 BasisType.A364, _
                                 BasisType.Bond_NON_EOM, _
                                 BasisType.Actual_NON_EOM, _
                                 BasisType.A360_NON_EOM, _
                                 BasisType.A365_NON_EOM, _
                                 BasisType.EBOND_NON_EOM, _
                                 BasisType.ISDA_30_360_NON_EOM, _
                                 BasisType.NLACT_NON_EOM, _
                                 BasisType.NL360_NON_EOM, _
                                 BasisType.NL365_NON_EOM, _
                                 BasisType.A364_NON_EOM, _
                                 BasisType.BUS252, _
                                 BasisType.Actual_ISDA, _
                                 BasisType.Actual_ISMA, _
                                 BasisType.Actual_365L, _
                                 BasisType.Actual_AFB, _
                                 BasisType.BUS252_NON_EOM}
        Holidays = New Date() {#1/1/2011#, #3/7/2011#, #3/8/2011#, #4/21/2011#, #4/22/2011#, #5/1/2011#, #6/23/2011#, #9/7/2011#, #10/12/2011#, #11/2/2011#, #11/15/2011#, #12/25/2011#, #1/1/2012#, #2/20/2012#, #2/21/2012#, #4/6/2012#, #4/21/2012#, #5/1/2012#, #6/7/2012#, #9/7/2012#, #10/12/2012#, #11/2/2012#, #11/15/2012#, #12/25/2012#}
        Rate = 0.1
        PrevCoupDate = #11/1/2003#
        Settlement = #5/1/2004#
        NextCoupDate = #5/1/2004#
        Maturity = #11/1/2009#
        Frequency = 2

        File.WriteAllText("AIFACTOR_RPI.csv", "")
        For i As Integer = 0 To Basis.GetUpperBound(0)
            Select Case Basis(i)
                Case BasisType.BUS252, BasisType.BUS252_NON_EOM
                    result = wct.AIFACTOR_RPI(Basis(i), Rate, PrevCoupDate, Settlement, NextCoupDate, Frequency, Maturity, Holidays) * 10000

                Case Else
                    result = wct.AIFACTOR_RPI(Basis(i), Rate, PrevCoupDate, Settlement, NextCoupDate, Frequency, Maturity) * 10000
            End Select

            Console.WriteLine("AIFACTOR_RPI : " & Basis(i).ToString() & " : " & result.ToString())
            File.AppendAllText("AIFACTOR_RPI.csv", Basis(i).ToString() & " : " & result.ToString() & vbCrLf)

        Next

    End Sub

    Public Shared Sub test_BONDINT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim Par As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #1/15/2006#
        Maturity = #12/15/2010#
        Rate = 0.04
        Par = 100
        Frequency = 2
        Basis = "1"

        result = wct.BONDINT(Settlement, Maturity, Rate, Par, Frequency, Basis)

        Console.WriteLine("BONDINT : " & result.ToString())        '0.340659340659341
        File.WriteAllText("BONDINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_COMPINT()
        Dim Basis As String
        Dim Rate As Double
        Dim Issue As Date
        Dim Settlement As Date
        Dim Maturity As Date
        Dim CompFreq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Basis = "1"
        Rate = 0.0175
        Issue = #11/15/2013#
        Settlement = #11/29/2013#
        Maturity = #11/15/2016#
        CompFreq = 12

        result = wct.COMPINT(Basis, Rate, Issue, Settlement, Maturity, CompFreq)

        Console.WriteLine("COMPINT : " & result.ToString())        '0.000680555555555573
        File.WriteAllText("COMPINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDCOMPINT()
        Dim Basis As String
        Dim Rate As Double
        Dim Issue As Date
        Dim Settlement As Date
        Dim Maturity As Date
        Dim FirstCoupon As Date
        Dim LastCoupon As Date
        Dim CompFreq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Basis = "1"
        Rate = 0.0175
        Issue = #11/15/2013#
        Settlement = #11/29/2013#
        Maturity = #11/30/2016#
        FirstCoupon = #11/30/2013#
        LastCoupon = wct.NULL_DATE
        CompFreq = 12

        result = wct.ODDCOMPINT(Basis, Rate, Issue, Settlement, Maturity, FirstCoupon, LastCoupon, CompFreq)

        Console.WriteLine("ODDCOMPINT : " & result.ToString())        '0.000680555555555556
        File.WriteAllText("ODDCOMPINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDFINT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Issue As Date
        Dim FirstCoupon As Date
        Dim Rate As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #11/1/2010#
        Maturity = #7/15/2020#
        Issue = #7/23/2010#
        FirstCoupon = #1/15/2011#
        Rate = 0.03
        Frequency = 2
        Basis = "1"

        result = wct.ODDFINT(Settlement, Maturity, Issue, FirstCoupon, Rate, Frequency, Basis)

        Console.WriteLine("ODDFINT : " & result.ToString())        '0.823369565217391
        File.WriteAllText("ODDFINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDLINT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim LastCoupon As Date
        Dim Rate As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #10/1/2014#
        Maturity = #12/15/2014#
        LastCoupon = #9/15/2014#
        Rate = 0.0225
        Frequency = 2
        Basis = "1"

        result = wct.ODDLINT(Settlement, Maturity, LastCoupon, Rate, Frequency, Basis)

        Console.WriteLine("ODDLINT : " & result.ToString())        '0.0994475138121547
        File.WriteAllText("ODDLINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_STEPACCINT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Frequency As Double
        Dim Basis As String
        Dim CoupDates As Date()
        Dim Coupons As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #4/21/2014#
        Maturity = #1/15/2019#
        Frequency = 2
        Basis = "0"
        CoupDates = New Date() {#1/15/2010#, #1/15/2013#, #1/15/2016#}
        Coupons = New Double() {0.05, 0.055, 0.06}

        result = wct.STEPACCINT(Settlement, Maturity, Frequency, Basis, CoupDates, Coupons)

        Console.WriteLine("STEPACCINT : " & result.ToString())     '1.46666666666667
        File.WriteAllText("STEPACCINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_PV()
        Dim rate As Double
        Dim nper As Double
        Dim pmt As Double
        Dim FV As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.08 / 12
        nper = 20 * 12
        pmt = 500
        FV = 0
        pay_type = 0

        result = wct.PV(rate, nper, pmt, FV, pay_type)

        Console.WriteLine("PV : " & result.ToString())     '-59777.1458511878
        File.WriteAllText("PV.csv", result.ToString())

    End Sub

    Public Shared Sub test_PMT()
        Dim rate As Double
        Dim nper As Double
        Dim pv As Double
        Dim FV As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.08 / 12
        nper = 20 * 12
        pv = -59777.1458511878
        FV = 0
        pay_type = 0

        result = wct.PMT(rate, nper, pv, FV, pay_type)

        Console.WriteLine("PMT : " & result.ToString())        '500
        File.WriteAllText("PMT.csv", result.ToString())

    End Sub

    Public Shared Sub test_RATE()
        Dim nper As Double
        Dim pmt As Double
        Dim pv As Double
        Dim FV As Double
        Dim Pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        nper = 20 * 12
        pmt = 500
        pv = -59777.1458511878
        FV = 0
        Pay_type = 0

        result = wct.RATE(nper, pmt, pv, FV, Pay_type, wct.NULL_DOUBLE)

        Console.WriteLine("RATE : " & result.ToString())       '0.0066666666666667
        File.WriteAllText("RATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_NPER()
        Dim rate As Double
        Dim pmt As Double
        Dim pv As Double
        Dim FV As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.08 / 12
        pmt = 500
        pv = -59777.1458511878
        FV = 0
        pay_type = 0

        result = wct.NPER(rate, pmt, pv, FV, pay_type)

        Console.WriteLine("NPER : " & result.ToString())
        File.WriteAllText("NPER.csv", result.ToString())

    End Sub

    Public Shared Sub test_LPMT()
        Dim PV As Double
        Dim LoanDate As Date
        Dim Rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim PmtPerYear As Integer
        Dim DaysInYear As Double
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'payment amount for a 50,000, 5-year loan starting on 1 November 
        'with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        PmtPerYear = 12
        DaysInYear = wct.NULL_DOUBLE
        FV = 0
        IntRule = "A"

        result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule)

        Console.WriteLine("LPMT : " & result.ToString())       '966.640076471413
        File.WriteAllText("LPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'payment amount for a 50,000, 5-year loan starting on 1 November 
        'with payments due every 4 weeks, starting on 1 December. 
        'The rate on the loan is 6 per cent.

        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 65
        PmtPerYear = 13
        DaysInYear = 365
        FV = 0
        IntRule = "A"

        result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule)

        Console.WriteLine("LPMT : " & "02 : " & result.ToString())       '892.076053138519
        File.AppendAllText("LPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'payment amount for a 50,000, 5-year loan starting on 1 November 
        'with payments due semi-monthly, starting on 1 December. 
        'The rate on the loan is 6 per cent and the loan has a balloon payment of 10,000 at the end.

        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 120
        PmtPerYear = 24
        DaysInYear = 360
        FV = 10000
        IntRule = "A"

        result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule)

        Console.WriteLine("LPMT : " & "03 : " & result.ToString())       '412.446978104019
        File.AppendAllText("LPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the payment amount for a 150,000 4-year loan at 12 percent interest 
        'using the US Rule made on 13 October with no payments due until 4 January and 
        'with payments due every 4 weeks after that.

        PV = 150000
        LoanDate = #10/13/2010#
        Rate = 0.12
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        PmtPerYear = 13
        DaysInYear = 365
        FV = 0
        IntRule = "U"

        result = wct.LPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, PmtPerYear, DaysInYear, FV, IntRule)

        Console.WriteLine("LPMT : " & "04 : " & result.ToString())       '412.446978104019
        File.AppendAllText("LPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ODDPV()
        Dim Rate As Double
        Dim NumPmts As Integer
        Dim pmt As Double
        Dim FV As Double
        Dim pmtperyear As Integer
        Dim LoanDate As Date
        Dim FirstPayDate As Date
        Dim DaysInYear As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the present value, using a rate of 6.0%, for a 5-year loan starting on 1 November 
        'with payments due on the first of every month, 
        'having a monthly payment of 966.64. 
        'The rate on the loan is 6 per cent.
        Rate = 0.06
        NumPmts = 60
        pmt = 966.64
        FV = 0
        pmtperyear = 12
        LoanDate = #11/1/2010#
        FirstPayDate = #12/1/2010#
        DaysInYear = wct.NULL_INT

        result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear)

        Console.WriteLine("ODDPV : " & result.ToString())      '-49999.9960444733
        File.WriteAllText("ODDPV.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the present value, using a rate of 6.0%, for a 5-year loan starting on 1 November 
        'with payments due every 4 weeks, 
        'starting on 1 December. 
        'Each payment is 892.08
        Rate = 0.06
        NumPmts = 65
        pmt = 892.08
        FV = 0
        pmtperyear = 13
        LoanDate = #11/1/2010#
        FirstPayDate = #12/1/2010#
        DaysInYear = wct.NULL_INT

        result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear)

        Console.WriteLine("ODDPV : " & "02 : " & result.ToString())      '-50000.2212177688
        File.AppendAllText("ODDPV.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the present value using a rate of 6.0% for a 5-year loan starting on 1 November 
        'with payments due semi-monthly, '
        'starting on 1 December. 
        'The loan has a balloon payment of 10,000 at the end and the periodic payment is 412.45.
        Rate = 0.06
        NumPmts = 120
        pmt = 412.45
        FV = 10000
        pmtperyear = 24
        LoanDate = #11/1/2010#
        FirstPayDate = #12/1/2010#
        DaysInYear = 360

        result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear)

        Console.WriteLine("ODDPV : " & "03: " & result.ToString())      '-50000.3121743558
        File.AppendAllText("ODDPV.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the present value using a rate of 12.0% for a 4-year loan made on 13 October 
        'with no payments due until 4 January and 
        'with payments of 3,708.09 due every 4 weeks after that.
        Rate = 0.12
        NumPmts = 52
        pmt = 3708.09
        FV = 0
        pmtperyear = 13
        LoanDate = #10/13/2010#
        FirstPayDate = #1/4/2011#
        DaysInYear = 365

        result = wct.ODDPV(Rate, NumPmts, pmt, FV, pmtperyear, LoanDate, FirstPayDate, DaysInYear)

        Console.WriteLine("ODDPV : " & "04 : " & result.ToString())      '-150084.652551611
        File.AppendAllText("ODDPV.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ODDFPMT()
        Dim rate As Double
        Dim nper As Integer
        Dim pv As Double
        Dim fv As Double
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'periodic payment for an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
        'The price of the annuity is 11,500 and there is no cash value at the end of the annuity. 
        'The first period is 1 and 5/6th longer than the other periods.
        rate = 0.005
        nper = 36
        pv = -11500
        fv = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod)

        Console.WriteLine("ODDFPMT : " & result.ToString())        '351.309392324512
        File.WriteAllText("ODDFPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'periodic payment for an annuity assuming a periodic rate of 0.5%, with 180 periodic payments. 
        'The price of the annuity is 250,000 and there is a 50,000 cash value at the end of the annuity. 
        'The first period is one-half as long as the other periods.
        rate = 0.005
        nper = 180
        pv = -250000
        fv = 50000
        firstperiod = 0.5

        result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod)

        Console.WriteLine("ODDFPMT : " & "02 : " & result.ToString())        '1932.45924676706
        File.AppendAllText("ODDFPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'weekly payment for an automobile lease with a term of 3 years and an annual interest rate of 25%. 
        'The amount to be financed is 11,000 and the residual value at the end of the lease is 3,500. 
        'The first payment is due 2014-11-25.
        rate = 0.25 * 7 / 365
        nper = 156
        pv = -11000
        fv = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFPMT(rate, nper, pv, fv, firstperiod)

        Console.WriteLine("ODDFPMT : " & "03 : " & result.ToString())        '85.5110616087362
        File.AppendAllText("ODDFPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ODDFPV()
        Dim rate As Double
        Dim nper As Integer
        Dim pmt As Double
        Dim FV As Double
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'present value of an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
        'The periodic payment is 351.31 and there is no cash value at the end of the annuity. 
        'The first period is 1 and 5/6th longer than the other periods.
        rate = 0.005
        nper = 36
        pmt = 351.31
        FV = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod)

        Console.WriteLine("ODDFPV : " & result.ToString())         '-11500.0198920617
        File.WriteAllText("ODDFPV.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'present value of an annuity assuming a periodic rate of 0.5%, with 180 periodic payments of 1932.46. 
        'There is a 50,000 cash value at the end of the annuity. 
        'The first period is one-half as long as the other periods.
        rate = 0.005
        nper = 180
        pmt = 1932.46
        FV = 50000
        firstperiod = 0.5

        result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod)

        Console.WriteLine("ODDFPV : " & result.ToString())         '-250000.089483625
        File.AppendAllText("ODDFPV.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'present value of an automobile lease with a term of 3 years and an annual interest rate of 25%. 
        'Then weekly payment is 85.51 and the residual value at the end of the lease is 3,500. 
        'The first payment is due 2014-11-25.
        rate = 0.25 * 7 / 365
        nper = 156
        pmt = 85.51
        FV = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFPV(rate, nper, pmt, FV, firstperiod)

        Console.WriteLine("ODDFPV : " & result.ToString())         '-10999.8839704582
        File.AppendAllText("ODDFPV.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ODDFRATE()
        Dim nper As Integer
        Dim pmt As Double
        Dim pv As Double
        Dim fv As Double
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'rate for an annuity with 36 periodic payments of 351.31 and a current value of 11,500. 
        'There is no cash value at the end of the annuity. 
        'The first period is 1 and 5/6th longer than the other periods.
        nper = 36
        pmt = 351.31
        pv = -11500
        fv = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod)

        Console.WriteLine("ODDFRATE : " & result.ToString())            '0.00500009249040195
        File.WriteAllText("ODDFRATE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'rate for an annuity with a current value of 250,000 and 180 periodic payments of 1932.46. 
        'There is a 50,000 cash value at the end of the annuity. 
        'The first period is one-half as long as the other periods.
        nper = 180
        pmt = 1932.46
        pv = -250000
        fv = 50000
        firstperiod = 0.5

        result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod)

        Console.WriteLine("ODDFRATE : " & result.ToString())            '0.00500000422661209
        File.AppendAllText("ODDFRATE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'annual rate for an automobile lease having a current value of 11000 with a term of 3 years and weekly payments of 85.51. 
        'The residual value at the end of the lease is 3,500. 
        'The first payment is due 2014-11-25.
        nper = 156
        pmt = 85.51
        pv = -11000
        fv = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFRATE(nper, pmt, pv, fv, firstperiod) * 365 / 7

        Console.WriteLine("ODDFRATE : " & result.ToString())            '0.249993317585699
        File.AppendAllText("ODDFRATE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_IPMT()
        Dim rate As Double
        Dim per As Double
        Dim nper As Double
        Dim pv As Double
        Dim fv As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06 / 12
        per = 13
        nper = 12 * 20
        pv = -400000
        fv = 100000
        pay_type = 0

        result = wct.IPMT(rate, per, nper, pv, fv, pay_type)

        Console.WriteLine("IPMT : " & result.ToString())       '1959.95301768065
        File.WriteAllText("IPMT.csv", result.ToString())

    End Sub

    Public Shared Sub test_PPMT()
        Dim rate As Double
        Dim per As Double
        Dim nper As Double
        Dim pv As Double
        Dim fv As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06 / 12
        per = 13
        nper = 12 * 20
        pv = -400000
        fv = 100000
        pay_type = 0

        result = wct.PPMT(rate, per, nper, pv, fv, pay_type)

        Console.WriteLine("PPMT : " & result.ToString())       '689.340157753871
        File.WriteAllText("PPMT.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDFIPMT()
        Dim rate As Double
        Dim per As Integer
        Dim nper As Integer
        Dim pv As Double
        Dim FV As Double
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.005
        per = 1
        nper = 36
        pv = -11500
        FV = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFIPMT : " & result.ToString())           '105.636223805979
        File.WriteAllText("ODDFIPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        rate = 0.005
        per = 1
        nper = 180
        pv = -250000
        FV = 50000
        firstperiod = 0.5

        result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFIPMT : " & result.ToString())           '624.220697042766
        File.AppendAllText("ODDFIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        rate = 0.25 * 7 / 365.0
        per = 1
        nper = 156
        pv = -11000
        FV = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFIPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFIPMT : " & result.ToString())           '90.5657016076089
        File.AppendAllText("ODDFIPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ODDFPPMT()
        Dim rate As Double
        Dim per As Integer
        Dim nper As Integer
        Dim pv As Double
        Dim FV As Double
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.005
        per = 1
        nper = 36
        pv = -11500
        FV = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFPPMT : " & result.ToString())           '245.673168518522
        File.WriteAllText("ODDFPPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        rate = 0.005
        per = 1
        nper = 180
        pv = -250000
        FV = 50000
        firstperiod = 0.5

        result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFPPMT : " & result.ToString())           '1308.23854972428
        File.AppendAllText("ODDFPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        rate = 0.25 * 7 / 365.0
        per = 1
        nper = 156
        pv = -11000
        FV = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFPPMT(rate, per, nper, pv, FV, firstperiod)

        Console.WriteLine("ODDFPPMT : " & result.ToString())           '-5.0546399988707
        File.AppendAllText("ODDFPPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_CUMIPMT()
        Dim rate As Double
        Dim nper As Double
        Dim pv As Double
        Dim start_period As Integer
        Dim end_period As Integer
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.0075
        nper = 360
        pv = -125000
        start_period = 349
        end_period = 360
        pay_type = 0

        result = wct.CUMIPMT(rate, nper, pv, start_period, end_period, pay_type)

        Console.WriteLine("CUMIPMT : " & result.ToString())           '568.35255468914
        File.WriteAllText("CUMIPMT.csv", result.ToString())

    End Sub

    Public Shared Sub test_CUMPRINC()
        Dim rate As Double
        Dim nper As Double
        Dim pv As Double
        Dim start_period As Integer
        Dim end_period As Integer
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.0075
        nper = 360
        pv = -125000
        start_period = 349
        end_period = 360
        pay_type = 0

        result = wct.CUMPRINC(rate, nper, pv, start_period, end_period, pay_type)

        Console.WriteLine("CUMPRINC : " & result.ToString())           '11500.9866994826
        File.WriteAllText("CUMPRINC.csv", result.ToString())

    End Sub

    Public Shared Sub test_CUMODDFIPMT()
        Dim rate As Double
        Dim nper As Integer
        Dim pv As Double
        Dim FV As Double
        Dim startperiod As Integer
        Dim endperiod As Integer
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.005
        nper = 36
        pv = -11500
        FV = 0
        startperiod = 1
        endperiod = 10
        firstperiod = 1 + 5 / 6

        result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFIPMT : " & result.ToString())           '558.349885303963
        File.WriteAllText("CUMODDFIPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        rate = 0.005
        nper = 180
        pv = -250000
        FV = 50000
        startperiod = 171
        endperiod = 180
        firstperiod = 0.5

        result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFIPMT : " & result.ToString())           '2953.57105763922
        File.AppendAllText("CUMODDFIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        rate = 0.25 * 7 / 365.0
        nper = 156
        pv = -11000
        FV = 3500
        startperiod = 1
        endperiod = 10
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.CUMODDFIPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFIPMT : " & result.ToString())           '559.725417921967
        File.AppendAllText("CUMODDFIPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_CUMODDFPPMT()
        Dim rate As Double
        Dim nper As Integer
        Dim pv As Double
        Dim FV As Double
        Dim startperiod As Integer
        Dim endperiod As Integer
        Dim firstperiod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.005
        nper = 36
        pv = -11500
        FV = 0
        startperiod = 1
        endperiod = 10
        firstperiod = 1 + 5 / 6

        result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFPPMT : " & result.ToString())           '2954.74403794116
        File.WriteAllText("CUMODDFPPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        rate = 0.005
        nper = 180
        pv = -250000
        FV = 50000
        startperiod = 171
        endperiod = 180
        firstperiod = 0.5

        result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFPPMT : " & result.ToString())           '66371.0214100313
        File.AppendAllText("CUMODDFPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        rate = 0.25 * 7 / 365.0
        nper = 156
        pv = -11000
        FV = 3500
        startperiod = 1
        endperiod = 10
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.CUMODDFPPMT(rate, nper, pv, FV, startperiod, endperiod, firstperiod)

        Console.WriteLine("CUMODDFPPMT : " & result.ToString())           '295.385198165395
        File.AppendAllText("CUMODDFPPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_FV()
        Dim rate As Double
        Dim nper As Double
        Dim pmt As Double
        Dim pv As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.12
        Nper = 12
        pmt = -1000
        pv = -5000
        Pay_type = 0

        result = wct.FV(rate, nper, pmt, pv, pay_type)

        Console.WriteLine("FV : " & result.ToString())         '43613.0132339597
        File.WriteAllText("FV.csv", result.ToString())

    End Sub

    Public Shared Sub test_PVGA()
        Dim pmt As Double
        Dim pgr As Double
        Dim nper As Double
        Dim rate As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pmt = 3191.76
        pgr = 0.03
        nper = 40
        rate = 0.07
        pay_type = 1

        result = wct.PVGA(pmt, pgr, nper, rate, pay_type)

        Console.WriteLine("PVGA : " & result.ToString())           '66780.481480667
        File.WriteAllText("PVGA.csv", result.ToString())

    End Sub

    Public Shared Sub test_PMTGA()
        Dim fv As Double
        Dim pgr As Double
        Dim nper As Integer
        Dim rate As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        fv = 1000000
        pgr = 0.03
        nper = 40
        rate = 0.07
        pay_type = 1

        result = wct.PMTGA(fv, pgr, nper, rate, pay_type)

        Console.WriteLine("PMTGA : " & result.ToString())           '3191.75519827819
        File.WriteAllText("PMTGA.csv", result.ToString())

    End Sub

    Public Shared Sub test_FVGA()
        Dim pmt As Double
        Dim pgr As Double
        Dim nper As Integer
        Dim rate As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pmt = 3191.76
        pgr = 0.03
        nper = 40
        rate = 0.07
        pay_type = 1

        result = wct.FVGA(pmt, pgr, nper, rate, pay_type)

        Console.WriteLine("FVGA : " & result.ToString())           '1000001.50441419
        File.WriteAllText("FVGA.csv", result.ToString())

    End Sub

    Public Shared Sub test_NPERGA()
        Dim fv As Double
        Dim pgr As Double
        Dim pmt As Double
        Dim rate As Double
        Dim pay_type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        fv = 1000000
        pgr = 0.03
        pmt = 3191.76
        rate = 0.07
        pay_type = 1

        result = wct.NPERGA(fv, pgr, pmt, rate, pay_type)

        Console.WriteLine("NPERGA : " & result.ToString())         '40
        File.WriteAllText("NPERGA.csv", result.ToString())

    End Sub

    Public Shared Sub test_ODDFPMTSCHED()
        Dim rate As Double
        Dim nper As Integer
        Dim pv As Double
        Dim fv As Double
        Dim firstperiod As Double
        Dim result As ODDFPMTSCHED_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Schedule for an annuity assuming a periodic rate of 0.5%, with 36 periodic payments. 
        'The price of the annuity is 11,500 and there is no cash value at the end of the annuity. 
        'The first period is 1 and 5/6th longer than the other periods.
        rate = 0.005
        nper = 36
        pv = -11500
        fv = 0
        firstperiod = 1 + 5 / 6

        result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod)

        'Write the table output to a csv
        Dim retStr1 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName & ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ODDFPMTSCHED : " & retStr1.ToString())
        File.WriteAllText("ODDFPMTSCHED.csv", retStr1.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Schedule for an annuity assuming a periodic rate of 0.5%, with 180 periodic payments. 
        'The price of the annuity is 250,000 and there is a 50,000 cash value at the end of the annuity. 
        'The first period is one-half as long as the other periods.
        rate = 0.005
        nper = 180
        pv = -250000
        fv = 50000
        firstperiod = 0.5

        result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod)

        'Write the table output to a csv
        Dim retStr2 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr2.Append(col.ColumnName & ",")
        Next
        retStr2.Remove(retStr2.Length - 1, 1)
        retStr2.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr2.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr2.Append(",")
                Else
                    retStr2.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ODDFPMTSCHED : " & retStr2.ToString())
        File.AppendAllText("ODDFPMTSCHED.csv", vbCrLf & retStr2.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Schedule for a weekly payment for an automobile lease with a term of 3 years and an annual interest rate of 25%. 
        'The amount to be financed is 11,000 and the residual value at the end of the lease is 3,500. 
        'The first payment is due 2014-11-25.
        rate = 0.25 * 7 / 365
        nper = 156
        pv = -11000
        fv = 3500
        firstperiod = (#11/25/2014# - #11/13/2014#).TotalDays / 7

        result = wct.ODDFPMTSCHED(rate, nper, pv, fv, firstperiod)

        'Write the table output to a csv
        Dim retStr3 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr3.Append(col.ColumnName & ",")
        Next
        retStr3.Remove(retStr3.Length - 1, 1)
        retStr3.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr3.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr3.Append(",")
                Else
                    retStr3.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ODDFPMTSCHED : " & retStr3.ToString())
        File.AppendAllText("ODDFPMTSCHED.csv", vbCrLf & retStr3.ToString())

    End Sub

    Public Shared Sub test_PMTSCHED()
        Dim pv As Double
        Dim pmt As Double
        Dim nper As Integer
        Dim fv As Double
        Dim pay_type As Integer
        Dim result As PMTSCHED_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        pv = 100000
        pmt = 8606.64
        nper = 12
        fv = 0
        pay_type = 0

        result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type)

        'Write the table output to a csv
        Dim retStr1 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr1.Append(col.ColumnName & ",")
        Next
        retStr1.Remove(retStr1.Length - 1, 1)
        retStr1.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr1.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr1.Append(",")
                Else
                    retStr1.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("PMTSCHED : " & retStr1.ToString())
        File.WriteAllText("PMTSCHED.csv", vbCrLf & retStr1.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        pv = 32000
        pmt = 600.95
        nper = 60
        fv = 0
        pay_type = 0

        result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type)

        'Write the table output to a csv
        Dim retStr2 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr2.Append(col.ColumnName & ",")
        Next
        retStr2.Remove(retStr2.Length - 1, 1)
        retStr2.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr2.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr2.Append(",")
                Else
                    retStr2.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("PMTSCHED : " & retStr2.ToString())
        File.AppendAllText("PMTSCHED.csv", vbCrLf & retStr2.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        pv = 250000
        pmt = 6533.74
        nper = 36
        fv = 50000
        pay_type = 0

        result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type)

        'Write the table output to a csv
        Dim retStr3 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr3.Append(col.ColumnName & ",")
        Next
        retStr3.Remove(retStr3.Length - 1, 1)
        retStr3.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr3.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr3.Append(",")
                Else
                    retStr3.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("PMTSCHED : " & retStr3.ToString())
        File.AppendAllText("PMTSCHED.csv", vbCrLf & retStr3.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        pv = 10000
        pmt = 869.95
        nper = 12
        fv = 0
        pay_type = 1

        result = wct.PMTSCHED(pv, pmt, nper, fv, pay_type)

        'Write the table output to a csv
        Dim retStr4 As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr4.Append(col.ColumnName & ",")
        Next
        retStr4.Remove(retStr4.Length - 1, 1)
        retStr4.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr4.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr4.Append(",")
                Else
                    retStr4.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("PMTSCHED : " & retStr4.ToString())
        File.AppendAllText("PMTSCHED.csv", vbCrLf & retStr4.ToString())

    End Sub

    Public Shared Sub test_IRR()
        Dim CF As Double()
        Dim per As Integer()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        CF = New Double() {-10000, 2000, 1500, 3000, 3800, 2500, 2500}
        per = New Integer() {1, 2, 5, 6, 10, 14, 14}

        result = wct.IRR(CF, per, wct.NULL_DOUBLE)

        Console.WriteLine("IRR : " & result.ToString())            '0.0586199750678698
        File.WriteAllText("IRR.csv", result.ToString())

        'Calculate the NPV value using the IRR.
        If Not wct.IsNull(result) Then
            Console.WriteLine("IRR : NPV : " & Math.Round(wct.NPV(result, CF, per), 4).ToString("F4"))         '0.0000
            File.AppendAllText("IRR.csv", vbCrLf & "NPV : " & Math.Round(wct.NPV(result, CF, per), 4).ToString("F4"))
        End If

    End Sub

    Public Shared Sub test_MIRR()
        Dim cf As Double()
        Dim per As Integer()
        Dim FinanceRate As Double
        Dim ReinvestRate As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-1000, -4000, -6000, -10000, -12000, 1000, 7000, 12000, 15000, 22000}
        per = New Integer() {0, 1, 2, 4, 6, 6, 8, 11, 14, 17}
        FinanceRate = 0.008333
        ReinvestRate = 0.01

        result = wct.MIRR(cf, per, FinanceRate, ReinvestRate)

        Console.WriteLine("MIRR : " & result.ToString())           '0.0366852516799792
        File.WriteAllText("MIRR.csv", result.ToString())

    End Sub

    Public Shared Sub test_XIRR()
        Dim cf As Double()
        Dim cfdate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        cfdate = New Date() {#11/30/2011#, #3/15/2012#, #7/18/2012#, #11/30/2012#, #1/23/2013#, #4/30/2013#, #2/6/2014#}

        result = wct.XIRR(cf, cfdate, wct.NULL_DOUBLE)

        Console.WriteLine("XIRR : " & result.ToString())           '0.249256702747075
        File.WriteAllText("XIRR.csv", result.ToString())

        'Calculate the XNPV value using the XIRR.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XIRR : XNPV : " & Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"))         '0.0000
            File.AppendAllText("XIRR.csv", vbCrLf & "XNPV : " & Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"))
        End If

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important -- in this example the dates are in descending order in the Array
        ReDim cf(10)
        ReDim cfdate(10)
        cfdate(0) = #3/31/2017#
        cf(0) = -49436.82
        For i As Integer = 1 To 10
            cfdate(i) = wct.EOMONTH(cfdate(0), -i)
            cf(i) = 5000
        Next

        result = wct.XIRR(cf, cfdate, wct.NULL_DOUBLE)

        Console.WriteLine("XIRR : " & result.ToString())           '-0.0245351885742908
        File.AppendAllText("XIRR.csv", vbCrLf & result.ToString())

        'Calculate the XNPV value using the XIRR.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XIRR : XNPV : " & Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"))         '0.0000
            File.AppendAllText("XIRR.csv", vbCrLf & "XNPV : " & Math.Round(wct.XNPV(result, cf, cfdate), 4).ToString("F4"))
        End If

    End Sub

    Public Shared Sub test_XIRR30360()
        Dim cf As Double()
        Dim cfdate As Date()
        Dim freq As Integer
        Dim days360type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        cfdate = New Date() {#11/30/2011#, #3/15/2012#, #7/18/2012#, #11/30/2012#, #1/23/2013#, #4/30/2013#, #2/6/2014#}
        freq = 2
        days360type = 0

        result = wct.XIRR30360(cf, cfdate, freq, days360type, wct.NULL_DOUBLE)

        Console.WriteLine("XIRR30360 : " & result)          '0.235942247765748
        File.WriteAllText("XIRR30360.csv", result.ToString())

        'Calculate the XNPV30360 value using the XIRR30360.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XNPV30360 : " & wct.XNPV30360(result, cf, cfdate, freq, days360type))         '-6.57348937238567E-05
            File.AppendAllText("XIRR30360.csv", vbCrLf & wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString())
        End If

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important -- in this example the dates are in descending order in the Array
        ReDim cf(10)
        ReDim cfdate(10)
        cfdate(0) = #3/31/2017#
        cf(0) = -49436.82
        For i As Integer = 1 To 10
            cfdate(i) = wct.EOMONTH(cfdate(0), -i)
            cf(i) = 5000
        Next
        freq = 2
        days360type = 0

        result = wct.XIRR30360(cf, cfdate, freq, days360type, wct.NULL_DOUBLE)

        Console.WriteLine("XIRR30360 : " & result)           '-0.024570360778599
        File.AppendAllText("XIRR30360.csv", vbCrLf & result.ToString())

        'Calculate the XNPV30360 value using the XIRR30360.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XNPV30360 : " & wct.XNPV30360(result, cf, cfdate, freq, days360type))         '5.5313139455393E-06
            File.AppendAllText("XIRR30360.csv", vbCrLf & wct.XNPV30360(result, cf, cfdate, freq, days360type).ToString())
        End If

    End Sub

    Public Shared Sub test_XIRRT()
        Dim cf As Double()
        Dim T As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        T = New Double() {0, 0.29041095890411, 0.632876712328767, 1.0027397260274, 1.15068493150685, 1.41643835616438, 2.18904109589041}

        result = wct.XIRRT(cf, T, wct.NULL_DOUBLE)

        Console.WriteLine("XIRRT : " & result)           '0.24925670279801
        File.WriteAllText("XIRRT.csv", result.ToString())

        'Calculate the XNPVT value using the XIRRT.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XIRRT : XNPVT : " & wct.XNPVT(result, cf, T))         '-9.19715239433572E-06
            File.AppendAllText("XIRRT.csv", vbCrLf & "XNPVT : " & wct.XNPVT(result, cf, T).ToString())
        End If

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important
        cf = New Double() {90000, 75000, 37500, 12500, -2500, -50000, -100000}
        T = New Double() {2.18754277891855, 1.41514598540146, 1.14963503649635, 1, 0.631147540983607, 0.289617486338798, 0}

        result = wct.XIRRT(cf, T, wct.NULL_DOUBLE)

        Console.WriteLine("XIRRT : " & result)           '0.249465703484356
        File.AppendAllText("XIRRT.csv", vbCrLf & result.ToString())

        'Calculate the XNPVT value using the XIRRT.
        If Not wct.IsNull(result) Then
            Console.WriteLine("XIRRT : XNPVT : " & wct.XNPVT(result, cf, T))         '-9.2460832092911E-06
            File.AppendAllText("XIRRT.csv", vbCrLf & "XNPVT : " & wct.XNPVT(result, cf, T).ToString())
        End If

    End Sub

    Public Shared Sub test_XMIRR()
        Dim cf As Double()
        Dim cfdate As Date()
        Dim financerate As Double
        Dim reinvestrate As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-1000, -4000, -6000, -10000, -12000, 1000, 7000, 12000, 15000, 22000}
        cfdate = New Date() {#1/31/2011#, #3/15/2011#, #4/19/2011#, #6/6/2011#, #7/2/2011#, #7/28/2011#, #8/26/2011#, #11/16/2011#, #3/17/2012#, #6/19/2012#}
        financerate = 0.1
        reinvestrate = 0.12

        result = wct.XMIRR(cf, cfdate, financerate, reinvestrate)           '0.555762831092004

        Console.WriteLine("XMIRR : " & result)
        File.WriteAllText("XMIRR.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This changes the date to which the cash flows are discounted to 12/31/2010 from 1/31/2011.
        ReDim Preserve cf(cf.GetUpperBound(0) + 1)
        ReDim Preserve cfdate(cfdate.GetUpperBound(0) + 1)
        cfdate(cfdate.GetUpperBound(0)) = #12/31/2010#

        result = wct.XMIRR(cf, cfdate, financerate, reinvestrate)           '0.524881753702055

        Console.WriteLine("XMIRR : " & result)
        File.AppendAllText("XMIRR.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_AMORTIZECASHFLOWS()
        Dim cf As Double()
        Dim cfdate As Date()
        Dim result As AMORTIZECASHFLOWS_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-1350, 18.89, 18.89, 18.89, 18.89, 18.89, 18.89, 18.89, 1418.89}
        cfdate = New Date() {#1/15/2013#, #4/15/2013#, #7/15/2013#, #11/15/2013#, #1/15/2014#, #4/15/2014#, #7/15/2014#, #11/15/2014#, #1/15/2015#}

        result = wct.AMORTIZECASHFLOWS(cf, cfdate)

        'Write the table output to a csv
        Dim retStr As New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("AMORTIZECASHFLOWS : " & retStr.ToString())
        File.WriteAllText("AMORTIZECASHFLOWS.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_EFV()
        Dim startper As Double
        Dim per As Double
        Dim endper As Double
        Dim cashflowrate As Double
        Dim endrate As Double
        Dim cashflow As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        startper = 0
        per = 1
        endper = 12
        cashflowrate = 0.00142 / 12.0
        endrate = 0.00246 / 12.0
        cashflow = 100000

        result = wct.EFV(startper, per, endper, cashflowrate, endrate, cashflow)            '100234.416482003

        Console.WriteLine("EFV : " & result)
        File.WriteAllText("EFV.csv", result.ToString())

    End Sub

    Public Shared Sub test_ENPV()
        Dim rate As Double
        Dim cf As Double()
        Dim per As Integer()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.0075
        cf = New Double() {-10000, 2000, 1500, 3000, 3800, 2500, 2500}
        per = New Integer() {1, 2, 5, 6, 10, 14, 14}

        result = wct.ENPV(rate, cf, per)

        Console.WriteLine("ENPV : " & result)            '4420.95266037402
        File.WriteAllText("ENPV.csv", result.ToString())

    End Sub

    Public Shared Sub test_EPV()
        Dim rate As Double
        Dim startPer As Double
        Dim endper As Double
        Dim cashflow As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.01 / 12
        startPer = 0
        endper = 1
        cashflow = 10000

        result = wct.EPV(rate, startPer, endper, cashflow)

        Console.WriteLine("EPV : " & result)         '9991.67360532889
        File.WriteAllText("EPV.csv", result.ToString())

    End Sub

    Public Shared Sub test_NFV()
        Dim rate As Double
        Dim cf As Double()
        Dim per As Integer()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.0075
        cf = New Double() {-10000, 2000, 1500, 3000, 3800, 2500, 2500}
        per = New Integer() {1, 2, 5, 6, 10, 14, 14}

        result = wct.NFV(rate, cf, per)

        Console.WriteLine("NFV : " & result)         '4871.93602805044
        File.WriteAllText("NFV.csv", result.ToString())

    End Sub

    Public Shared Sub test_NPV()
        Dim rate As Double
        Dim cf As Double()
        Dim per As Integer()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.0075
        cf = New Double() {-10000, 2000, 1500, 3000, 3800, 2500, 2500}
        per = New Integer() {1, 2, 5, 6, 10, 14, 14}
        result = wct.NPV(rate, cf, per)

        Console.WriteLine("NPV : " & result)         '4388.04234280299
        File.WriteAllText("NPV.csv", result.ToString())

    End Sub

    Public Shared Sub test_XDCF()
        Dim discrate As Double
        Dim vdate As Date
        Dim cf As Double()
        Dim cfdate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        discrate = 0.06
        vdate = #12/21/2012#
        cf = New Double() {-87268.05, 70231.96, -457.02, -40603.18, 83598.77, -30994.43, 34666.49, 49029.24, -93685.98, -59199.55}
        cfdate = New Date() {#4/19/2016#, #11/13/2019#, #6/13/2021#, #10/29/2016#, #3/16/2021#, #3/15/2014#, #3/30/2022#, #4/30/2020#, #2/25/2018#, #4/10/2015#}

        result = wct.XDCF(discrate, vdate, cf, cfdate)

        Console.WriteLine("XDCF : " & result)            ' -103643.043186434
        File.WriteAllText("XDCF.csv", result.ToString())

    End Sub

    Public Shared Sub test_XFV()
        Dim startdate As Date
        Dim cashflowdate As Date
        Dim enddate As Date
        Dim cashflowrate As Double
        Dim endrate As Double
        Dim cashflow As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        startdate = #2/1/2011#
        cashflowdate = #3/1/2011#
        enddate = #2/1/2012#
        cashflowrate = 0.00142
        endrate = 0.00246
        cashflow = 100000

        result = wct.XFV(startdate, cashflowdate, enddate, cashflowrate, endrate, cashflow)

        Console.WriteLine("XFV : " & result)
        File.WriteAllText("XFV.csv", result.ToString())

    End Sub

    Public Shared Sub test_XNFV()
        Dim rate As Double
        Dim cf As Double()
        Dim cfdate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06
        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        cfdate = New Date() {#11/30/2011#, #3/15/2012#, #7/18/2012#, #11/30/2012#, #1/23/2013#, #4/30/2013#, #2/6/2014#}

        result = wct.XNFV(rate, cf, cfdate)

        Console.WriteLine("XNFV : " & result)         '49496.1812211834
        File.WriteAllText("XNFV.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important -- in this example the dates are in descending order in the Array
        ReDim cf(10)
        ReDim cfdate(10)
        cfdate(0) = #3/31/2017#
        cf(0) = -49436.82
        For i As Integer = 1 To 10
            cfdate(i) = wct.EOMONTH(cfdate(0), -i)
            cf(i) = 5000
        Next
        rate = -0.0025

        result = wct.XNFV(rate, cf, cfdate)

        Console.WriteLine("XNFV : " & result)         '506.061147540191
        File.AppendAllText("XNFV.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_XNPV()
        Dim rate As Double
        Dim cf As Double()
        Dim cfdate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06
        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        cfdate = New Date() {#11/30/2011#, #3/15/2012#, #7/18/2012#, #11/30/2012#, #1/23/2013#, #4/30/2013#, #2/6/2014#}

        result = wct.XNPV(rate, cf, cfdate)

        Console.WriteLine("XNPV : " & result)         '43568.8517245775
        File.WriteAllText("XNPV.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important -- in this example the dates are in descending order in the Array
        ReDim cf(10)
        ReDim cfdate(10)
        cfdate(0) = #3/31/2017#
        cf(0) = -49436.82
        For i As Integer = 1 To 10
            cfdate(i) = wct.EOMONTH(cfdate(0), -i)
            cf(i) = 5000
        Next
        rate = -0.0025

        result = wct.XNPV(rate, cf, cfdate)

        Console.WriteLine("XNPV : " & result)         '507.117283780321
        File.AppendAllText("XNPV.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_XPV()
        Dim rate As Double
        Dim startdate As Date
        Dim enddate As Date
        Dim cashflow As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.01
        startdate = #2/1/2011#
        enddate = #3/1/2011#
        cashflow = 10000

        result = wct.XPV(rate, startdate, enddate, cashflow)

        Console.WriteLine("XPV : " & result)         '9992.36978197548
        File.WriteAllText("XPV.csv", result.ToString())

    End Sub

    Public Shared Sub test_XNPV30360()
        Dim rate As Double
        Dim cf As Double()
        Dim cfdate As Date()
        Dim freq As Integer
        Dim days360type As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06
        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        cfdate = New Date() {#11/30/2011#, #3/15/2012#, #7/18/2012#, #11/30/2012#, #1/23/2013#, #4/30/2013#, #2/6/2014#}
        freq = 2
        days360type = 0

        result = wct.XNPV30360(rate, cf, cfdate, freq, days360type)

        Console.WriteLine("XNPV30360 : " & result)          '43346.5387538406
        File.WriteAllText("XNPV30360.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important -- in this example the dates are in descending order in the Array
        ReDim cf(10)
        ReDim cfdate(10)
        cfdate(0) = #3/31/2017#
        cf(0) = -49436.82
        For i As Integer = 1 To 10
            cfdate(i) = wct.EOMONTH(cfdate(0), -i)
            cf(i) = 5000
        Next
        freq = 2
        days360type = 0
        rate = -0.0025

        result = wct.XNPV30360(rate, cf, cfdate, freq, days360type)

        Console.WriteLine("XNPV30360 : " & result)           '506.880388631596
        File.AppendAllText("XNPV30360.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_XNPVT()
        Dim rate As Double
        Dim cf As Double()
        Dim T As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = 0.06
        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}
        T = New Double() {0, 0.29041095890411, 0.632876712328767, 1.0027397260274, 1.15068493150685, 1.41643835616438, 2.18904109589041}

        result = wct.XNPVT(rate, cf, T)

        Console.WriteLine("XNPVT : " & result)           '43568.8517245775
        File.WriteAllText("XNPVT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Order is not important
        cf = New Double() {90000, 75000, 37500, 12500, -2500, -50000, -100000}
        T = New Double() {2.18754277891855, 1.41514598540146, 1.14963503649635, 1, 0.631147540983607, 0.289617486338798, 0}

        result = wct.XNPVT(rate, cf, T)

        Console.WriteLine("XNPVT : " & result)           '43582.4812123601
        File.AppendAllText("XNPVT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_EMDIETZ()
        Dim tranamt As Double()
        Dim trandate As Date()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        tranamt = New Double() {20000, 4000, 1750, -3500, -2500, 1250, -3750, -18500}
        trandate = New Date() {#12/31/2009#, #1/6/2010#, #1/28/2010#, #2/3/2010#, #2/18/2010#, #3/1/2010#, #3/15/2010#, #3/31/2010#}

        result = wct.EMDIETZ(tranamt, trandate)

        Console.WriteLine("EMDIETZ : " & result)         '0.0584871328307772
        File.WriteAllText("EMDIETZ.csv", result.ToString())

    End Sub

    Public Shared Sub test_GTWRR()
        Dim tranamt As Double()
        Dim trandate As Date()
        Dim mv As Boolean()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        '(EMV - CF) / BMV
        tranamt = New Double() {100000.0, 5500.0, -5254.0, 105000.0, 4556.0, -9754.0, 110250.0, 886.0, -9525.0, 9775.0, -983.0, 115762.5, -6734.0, 3913.0, 1284.0, -6015.0, 121550.63}
        trandate = New Date() {#12/31/2011#, #1/10/2012#, #1/20/2012#, #1/31/2012#, #2/10/2012#, #2/20/2012#, #2/29/2012#, #3/10/2012#, #3/10/2012#, #3/20/2012#, #3/20/2012#, #3/31/2012#, #4/10/2012#, #4/20/2012#, #4/30/2012#, #4/30/2012#, #4/30/2012#}
        mv = New Boolean() {True, False, False, True, False, False, True, False, False, False, False, True, False, False, False, False, True}

        result = wct.GTWRR(tranamt, trandate, mv, 1)

        Console.WriteLine("GTWRR : " & result)           '0.346944952211272
        File.WriteAllText("GTWRR.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'EMV / (BMV + CF)
        result = wct.GTWRR(tranamt, trandate, mv, 2)

        Console.WriteLine("GTWRR : " & result)           '0.362813519089841
        File.AppendAllText("GTWRR.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        '(EMV + W) / (BMV + D)
        result = wct.GTWRR(tranamt, trandate, mv, 0)

        Console.WriteLine("GTWRR : " & result)           '0.327310011739417
        File.AppendAllText("GTWRR.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_MDIETZ()
        Dim trandate As DateTime()
        Dim tranamt As Double()
        Dim StartDate As DateTime
        Dim EndDate As DateTime
        Dim BMV As Double
        Dim EMV As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'calculate the performance on an account

        'SELECT ''01/06/2010'',4000 UNION ALL
        'SELECT ''01/28/2010'',1750 UNION ALL
        'SELECT ''02/03/2010'',-3500 UNION ALL
        'SELECT ''02/18/2010'',-2500 UNION ALL
        'SELECT ''03/01/2010'',1250 UNION ALL
        'SELECT ''03/15/2010'',-3750'

        trandate = {New DateTime(2010, 1, 6), New DateTime(2010, 1, 28), New DateTime(2010, 2, 3), New DateTime(2010, 2, 18), New DateTime(2010, 3, 1), New DateTime(2010, 3, 15)}
        tranamt = {4000.0, 1750.0, -3500.0, -2500.0, 1250.0, -3750.0}
        StartDate = New DateTime(2009, 12, 31)
        EndDate = New DateTime(2010, 3, 31)
        BMV = 20000
        EMV = 18500

        result = wct.MDIETZ(tranamt, trandate, StartDate, EndDate, BMV, EMV)

        Console.WriteLine("MDIETZ : " & result)           '0.0584871328307772
        File.WriteAllText("MDIETZ.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Date        Description     Value
        '12/31/2009  Market Value    20,000
        '1/6/2010    Deposit         4,000
        '1/28/2010   Deposit         1,750
        '2/3/2010    Withdrawal      -3,500
        '2/18/2010   Withdrawal      -2,500
        '3/1/2010    Deposit         1,250
        '3/15/2010   Withdrawal      -3,750
        '3/31/2010   Market Value    18,500

        trandate = {New DateTime(2009, 12, 31), New DateTime(2010, 1, 6), New DateTime(2010, 1, 28), New DateTime(2010, 2, 3), New DateTime(2010, 2, 18), New DateTime(2010, 3, 1), New DateTime(2010, 3, 15), New DateTime(2010, 3, 31)}
        tranamt = {20000.0, 4000.0, 1750.0, -3500.0, -2500.0, 1250.0, -3750.0, 18500.0}

        result = wct.MDIETZ(tranamt, trandate, StartDate, EndDate, BMV, EMV)

        Console.WriteLine("MDIETZ : " & result)           '0.0584871328307772
        File.AppendAllText("MDIETZ.csv", result.ToString())

    End Sub

    Public Shared Sub test_LMDIETZ()
        Dim tranamt As Double()
        Dim trandate As Date()
        Dim mv As Boolean()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        tranamt = New Double() {498987.32, -993.58, 1000.3, -954.15, 839.55, 36124.27, 493997.45, 503977.19, -930.18, 828.23, -938.37, 1090.9, -48246.35, 498937.42, 509016.96, -922.09, 1090.67, -916.77, 1044.57, 35643.6, 503926.79, 514107.13}
        trandate = New Date() {#6/28/2012#, #7/5/2012#, #7/10/2012#, #7/16/2012#, #7/20/2012#, #7/25/2012#, #7/25/2012#, #7/31/2012#, #8/3/2012#, #8/9/2012#, #8/14/2012#, #8/20/2012#, #8/24/2012#, #8/24/2012#, #8/29/2012#, #9/4/2012#, #9/7/2012#, #9/13/2012#, #9/18/2012#, #9/24/2012#, #9/24/2012#, #9/28/2012#}
        mv = New Boolean() {True, False, False, False, False, False, True, True, False, False, False, False, False, True, True, False, False, False, False, False, True, True}

        result = wct.LMDIETZ(tranamt, trandate, mv)

        Console.WriteLine("LMDIETZ : " & result)         '-0.0272594273584181
        File.WriteAllText("LMDIETZ.csv", result.ToString())

    End Sub

    Public Shared Sub test_TWROR()
        Dim tranamt As Double()
        Dim trandate As Date()
        Dim ttype As String()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        tranamt = New Double() {-100000.0, -111111.0, 10000.0, -110666.31, -500.0, -89759.65, -20000.0, -90298.21, -500.0, -94079.6, 500.0, 11000.0, -8000.0, 660.0, -94173.68}
        trandate = New Date() {#1/31/2012#, #2/14/2012#, #2/14/2012#, #2/29/2012#, #2/29/2012#, #3/15/2012#, #3/15/2012#, #3/31/2012#, #3/31/2012#, #4/15/2012#, #4/15/2012#, #4/15/2012#, #4/15/2012#, #4/15/2012#, #4/30/2012#}
        ttype = New String() {"M", "M", "", "M", "B", "M", "", "M", "E", "M", "B", "", "", "E", "M"}

        result = wct.TWROR(tranamt, trandate, ttype)

        Console.WriteLine("TWROR : " & result)           '-0.0808595339518893
        File.WriteAllText("TWROR.csv", result.ToString())

    End Sub

    Public Shared Sub test_TWRR()
        Dim tranamt As Double()
        Dim trandate As Date()
        Dim mv As Boolean()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        tranamt = New Double() {15000, -15477.3, 436.49, 0, -15115.7, 0, -495.97, -14878.1, 297.92, -609.07, -14197.3, 157.06, -880.28, -14275.1, 0, 0}
        trandate = New Date() {#1/3/2012#, #1/3/2012#, #1/3/2012#, #1/3/2012#, #1/10/2012#, #1/10/2012#, #1/10/2012#, #1/17/2012#, #1/17/2012#, #1/17/2012#, #1/24/2012#, #1/24/2012#, #1/24/2012#, #1/31/2012#, #1/31/2012#, #1/31/2012#}
        mv = New Boolean() {False, True, False, False, True, False, False, True, False, False, True, False, False, True, False, False}

        result = wct.TWRR(tranamt, trandate, mv)

        Console.WriteLine("TWRR : " & result)            '0.0246243629116802
        File.WriteAllText("TWRR.csv", result.ToString())

    End Sub

    Public Shared Sub test_BetaCoKurt()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}

        result = wct.BetaCoKurt(Ra, Rb)

        Console.WriteLine("BetaCoKurt : " & result)          '0.439447084098378
        File.WriteAllText("BetaCoKurt.csv", result.ToString())

    End Sub

    Public Shared Sub test_BetaCoSkew()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}

        result = wct.BetaCoSkew(Ra, Rb)

        Console.WriteLine("BetaCoSkew : " & result)          '0.703968314287678
        File.WriteAllText("BetaCoSkew.csv", result.ToString())

    End Sub

    Public Shared Sub test_BetaCoVar()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}

        result = wct.BetaCoVar(Ra, Rb)

        Console.WriteLine("BetaCoVar : " & result)          '0.234187219388425 
        File.WriteAllText("BetaCoVar.csv", result.ToString())

    End Sub

    Public Shared Sub test_DownsideDeviation()
        Dim Ra As Double()
        Dim MAR As Double
        Dim Full As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005
        Full = True

        result = wct.DownsideDeviation(Ra, MAR, Full)

        Console.WriteLine("DownsideDeviation : " & result)          '0.0076206946799754 
        File.WriteAllText("DownsideDeviation.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        Full = False

        result = wct.DownsideDeviation(Ra, MAR, Full)

        Console.WriteLine("DownsideDeviation : " & result)          '0.00988289278133041 
        File.AppendAllText("DownsideDeviation.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_DownsideFrequency()
        Dim Ra As Double()
        Dim MAR As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005

        result = wct.DownsideFrequency(Ra, MAR)

        Console.WriteLine("DownsideFrequency : " & result)          '0.594594594594595
        File.WriteAllText("DownsideFrequency.csv", result.ToString())

    End Sub

    Public Shared Sub test_DownsidePotential()
        Dim Ra As Double()
        Dim MAR As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005

        result = wct.DownsidePotential(Ra, MAR)

        Console.WriteLine("DownsidePotential : " & result)          '0.00474210810810811
        File.WriteAllText("DownsidePotential.csv", result.ToString())

    End Sub

    Public Shared Sub test_EQALPHA()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim bValue As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#}
        pValue = New Double() {194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29}
        bValue = New Double() {1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59}

        result = wct.EQALPHA(pDate, pValue, bValue)

        Console.WriteLine("EQALPHA : " & result)         '-0.000395022617631785
        File.WriteAllText("EQALPHA.csv", result.ToString())

    End Sub

    Public Shared Sub test_EQBETA()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim bValue As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#}
        pValue = New Double() {194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29}
        bValue = New Double() {1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59}

        result = wct.EQBETA(pDate, pValue, bValue)

        Console.WriteLine("EQBETA : " & result)         '0.902353333082278
        File.WriteAllText("EQBETA.csv", result.ToString())

    End Sub

    Public Shared Sub test_EQVOLATILITY()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim scale As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/18/2012#, #12/17/2012#, #12/14/2012#, #12/13/2012#, #12/12/2012#, #12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#, #10/31/2012#}
        pValue = New Double() {195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68}
        scale = 252

        result = wct.EQVOLATILITY(pDate, pValue, scale)

        Console.WriteLine("EQVOLATILITY : " & result)         '0.141437834070616
        File.WriteAllText("EQVOLATILITY.csv", result.ToString())

    End Sub

    Public Shared Sub test_FinCoKurt()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}

        result = wct.FinCoKurt(Ra, Rb)

        Console.WriteLine("FinCoKurt : " & result)          '7.86818968887937E-10
        File.WriteAllText("FinCoKurt.csv", result.ToString())

    End Sub

    Public Shared Sub test_FinCoSkew()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}

        result = wct.FinCoSkew(Ra, Rb)

        Console.WriteLine("FinCoSkew : " & result)          '1.77056901645221E-08
        File.WriteAllText("FinCoSkew.csv", result.ToString())

    End Sub

    Public Shared Sub test_INFORATIO()
        Dim r As Double()
        Dim rb As Double()
        Dim scale As Double
        Dim geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        r = New Double() {0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135}
        rb = New Double() {0.0115, 0.0119, -0.0041, -0.0063, 0.0004, 0.0065, 0.0003, 0.0029, 0.0033, 0.0016, -0.0017, -0.0047, 0.0002, 0.0043, 0.0079, -0.0052, -0.002, 0.013, 0.0023, 0.0007, 0.0199, 0.0048, -0.0016, -0.0139, -0.004, 0.0001, 0.0017, -0.0122, -0.0237, 0.0079, 0.0022, -0.0094, 0.0109}
        scale = 252
        geometric = False

        result = wct.INFORATIO(r, rb, scale, geometric)

        Console.WriteLine("INFORATIO : " & result)           '-1.46734740387312
        File.WriteAllText("INFORATIO.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        geometric = True
        result = wct.INFORATIO(r, rb, scale, geometric)

        Console.WriteLine("INFORATIO : " & result)          '-1.6901913539965
        File.AppendAllText("INFORATIO.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_INFORATIO2()
        Dim vDate As Date()
        Dim pr As Double()
        Dim prb As Double()
        Dim scale As Double
        Dim geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        vDate = New Date() {#12/18/2012#, #12/17/2012#, #12/14/2012#, #12/13/2012#, #12/12/2012#, #12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#, #10/31/2012#}
        pr = New Double() {195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68}
        prb = New Double() {1446.79, 1430.36, 1413.58, 1419.45, 1428.48, 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59, 1412.16}
        scale = 252
        geometric = False

        result = wct.INFORATIO2(vDate, pr, prb, scale, geometric)

        Console.WriteLine("INFORATIO2 : " & result)           '-1.47787598470777
        File.WriteAllText("INFORATIO2.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        geometric = True

        result = wct.INFORATIO2(vDate, pr, prb, scale, geometric)

        Console.WriteLine("INFORATIO2 : " & result)          '-1.70044976017884
        File.AppendAllText("INFORATIO2.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_MAXDD()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/31/2011#, #1/31/2012#, #2/29/2012#, #3/31/2012#, #4/30/2012#, #5/31/2012#, #6/30/2012#, #7/31/2012#, #8/31/2012#, #9/30/2012#, #10/31/2012#, #11/30/2012#, #12/31/2012#, #1/31/2013#, #2/28/2013#, #3/31/2013#, #4/30/2013#, #5/31/2013#, #6/30/2013#, #7/31/2013#, #8/31/2013#, #9/30/2013#, #10/31/2013#, #11/30/2013#, #12/31/2013#, #1/31/2014#, #2/28/2014#, #3/31/2014#, #4/30/2014#, #5/31/2014#, #6/30/2014#, #7/31/2014#, #8/31/2014#, #9/30/2014#, #10/31/2014#, #11/30/2014#, #12/31/2014#}
        pValue = New Double() {109.39, 102.46, 102.98, 102.38, 113.23, 93.06, 90.13, 79.55, 108.59, 108.39, 103.31, 101.83, 131.3, 127.31, 92.56, 65.05, 108.19, 95.3, 76.85, 84.59, 82.59, 107.36, 101.79, 143.82, 124.53, 109.66, 95.17, 108.4, 104.61, 84.24, 86.99, 92.34, 99.92, 128.53, 108.63, 101.59, 82.25}

        result = wct.MAXDD(pDate, pValue)

        Console.WriteLine("MAXDD : " & result)           '-0.504569687738005
        File.WriteAllText("MAXDD.csv", result.ToString())

    End Sub

    Public Shared Sub test_MAXDD2()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#1/31/2012#, #2/29/2012#, #3/31/2012#, #4/30/2012#, #5/31/2012#, #6/30/2012#, #7/31/2012#, #8/31/2012#, #9/30/2012#, #10/31/2012#, #11/30/2012#, #12/31/2012#, #1/31/2013#, #2/28/2013#, #3/31/2013#, #4/30/2013#, #5/31/2013#, #6/30/2013#, #7/31/2013#, #8/31/2013#, #9/30/2013#, #10/31/2013#, #11/30/2013#, #12/31/2013#, #1/31/2014#, #2/28/2014#, #3/31/2014#, #4/30/2014#, #5/31/2014#, #6/30/2014#, #7/31/2014#, #8/31/2014#, #9/30/2014#, #10/31/2014#, #11/30/2014#, #12/31/2014#}
        pValue = New Double() {-0.063351, 0.005075, -0.005826, 0.105978, -0.178133, -0.031485, -0.117386, 0.365053, -0.001842, -0.046868, -0.014326, 0.289404, -0.030388, -0.272956, -0.297213, 0.663182, -0.119142, -0.193599, 0.100716, -0.023643, 0.299915, -0.051882, 0.412909, -0.134126, -0.119409, -0.132136, 0.139014, -0.034963, -0.194723, 0.032645, 0.061501, 0.082088, 0.286329, -0.154828, -0.064807, -0.190373}

        result = wct.MAXDD2(pDate, pValue)

        Console.WriteLine("MAXDD2 : " & result)           '-0.504569891864632
        File.WriteAllText("MAXDD2.csv", result.ToString())

    End Sub

    Public Shared Sub test_MOIC()
        Dim cf As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        cf = New Double() {-100000, -50000, -2500, 12500, 37500, 75000, 90000}

        result = wct.MOIC(cf)

        Console.WriteLine("MOIC : " & result)            '1.40983606557377
        File.WriteAllText("MOIC.csv", result.ToString())

    End Sub

    Public Shared Sub test_Omega()
        Dim Ra As Double()
        Dim MAR As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005

        result = wct.Omega(Ra, MAR)

        Console.WriteLine("Omega : " & result)          '0.644154156550285
        File.WriteAllText("Omega.csv", result.ToString())

    End Sub

    Public Shared Sub test_OmegaExcessReturns()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim MAR As Double
        Dim freq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}
        MAR = 0.005
        freq = 12

        result = wct.OmegaExcessReturn(Ra, Rb, MAR, freq)

        Console.WriteLine("OmegaExcessReturn : " & result)          '0.0378469204243685
        File.WriteAllText("OmegaExcessReturn.csv", result.ToString())

    End Sub

    Public Shared Sub test_OmegaSharpeRatio()
        Dim Ra As Double()
        Dim MAR As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005

        result = wct.OmegaSharpeRatio(Ra, MAR)

        Console.WriteLine("OmegaSharpeRatio : " & result)          '-0.355845843449715
        File.WriteAllText("OmegaSharpeRatio.csv", result.ToString())

    End Sub

    Public Shared Sub test_SemiDeviation()
        Dim Ra As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}

        result = wct.SemiDeviation(Ra)

        Console.WriteLine("SemiDeviation : " & result)          '0.00660706168061328
        File.WriteAllText("SemiDeviation.csv", result.ToString())

    End Sub

    Public Shared Sub test_SemiVariance()
        Dim Ra As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}

        result = wct.SemiVariance(Ra)

        Console.WriteLine("SemiVariance : " & result)          '0.00922003188116091
        File.WriteAllText("SemiVariance.csv", result.ToString())

    End Sub

    Public Shared Sub test_SHARPE()
        Dim r As Double()
        Dim rf As Double()
        Dim scale As Double
        Dim geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        r = New Double() {0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135}
        ReDim rf(r.GetUpperBound(0))
        For i As Integer = 0 To r.GetUpperBound(0)
            rf(i) = 0.001 / 252
        Next
        scale = 252
        geometric = False

        result = wct.SHARPE(r, rf, scale, geometric)

        Console.WriteLine("SHARPE : " & result)           '0.630146290350507
        File.WriteAllText("SHARPE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        r = New Double() {0.009416, 0.013579, 0.009315, 0.018145, 0.007068, 0.006657, 0.006028, 0.004719, 0.018247, 0.001731, 0.002923, 0.018072, 0.0102, 0.014619, 0.017633, 0.010065, 0.012939, 0.008044, 0.017269, 0.010842, 0.009544, 0.008508, 0.000504, 0.015876, 0.00385, 0.002251, 0.013098, 0.013069, 0.006818, 0.013184, 0.013902, 0.007957, 0.003052, 0.013372, 0.014231, 0.011182}
        rf = New Double() {0.004986, 0.005009, 0.00495, 0.005082, 0.005112, 0.005066, 0.004967, 0.005018, 0.005011, 0.004863, 0.004825, 0.004895, 0.00507, 0.005088, 0.004956, 0.005025, 0.004736, 0.004933, 0.004969, 0.004814, 0.004959, 0.005083, 0.004977, 0.004992, 0.005003, 0.004997, 0.004815, 0.005026, 0.005013, 0.005044, 0.004822, 0.005019, 0.00487, 0.005153, 0.004994, 0.004966}
        scale = 12

        result = wct.SHARPE(r, rf, scale, geometric)

        Console.WriteLine("SHARPE : " & result)           '3.58446805338759
        File.AppendAllText("SHARPE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        r = New Double() {0.0173, -0.001, 0.0099, -0.0177, 0.035, -0.0142, -0.0153, 0.0008, -0.0005, -0.0695, -0.0133, 0.0151, 0.0072, -0.004, 0.0367, 0.0239, -0.0148, -0.0171, 0.0097, 0.0082, 0.0108, 0.0204, 0.0347, -0.0282, -0.0213, 0.0097, -0.0271, 0.0203, 0.032, -0.0268, -0.0081, -0.0263, -0.0145, -0.0088, 0.0361, -0.0158, -0.013, -0.0152, 0.0154, -0.0026, 0.0269, 0.0091, 0.0053, 0.0224, 0.0052, -0.0024, 0.0167, 0.0103, 0.0523, -0.0185}
        rf = New Double() {0.000195, 0.000194, 0.000192, 0.000193, 0.00019, 0.000192, 0.000193, 0.000192, 0.000192, 0.000193, 0.000193, 0.00019, 0.000189, 0.000189, 0.000199, 0.000189, 0.000191, 0.000193, 0.000192, 0.000195, 0.000192, 0.000192, 0.000192, 0.000194, 0.000192, 0.00019, 0.00019, 0.000194, 0.00019, 0.000192, 0.000192, 0.000195, 0.000196, 0.000191, 0.000192, 0.000193, 0.000196, 0.00019, 0.000195, 0.000191, 0.000188, 0.000191, 0.000191, 0.000194, 0.000193, 0.000194, 0.000191, 0.000193, 0.000191, 0.000189}
        scale = 52
        geometric = True

        result = wct.SHARPE(r, rf, scale, geometric)

        Console.WriteLine("SHARPE : " & result)           '0.478892083019103
        File.AppendAllText("SHARPE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_SHARPE2()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim RF As Double()
        Dim scale As Double
        Dim Geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/18/2012#, #12/17/2012#, #12/14/2012#, #12/13/2012#, #12/12/2012#, #12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#, #10/31/2012#}
        pValue = New Double() {195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68}
        ReDim RF(pValue.GetUpperBound(0))
        For i As Integer = 0 To pValue.GetUpperBound(0)
            RF(i) = 0.001 / 252
        Next
        scale = 252
        Geometric = False

        result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric)

        Console.WriteLine("SHARPE2 : " & result)         '0.618904269463314
        File.WriteAllText("SHARPE2.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        pDate = New Date() {#12/31/2010#, #1/31/2011#, #2/28/2011#, #3/31/2011#, #4/30/2011#, #5/31/2011#, #6/30/2011#, #7/31/2011#, #8/31/2011#, #9/30/2011#, #10/31/2011#, #11/30/2011#, #12/31/2011#, #1/31/2012#, #2/29/2012#, #3/31/2012#, #4/30/2012#, #5/31/2012#, #6/30/2012#, #7/31/2012#, #8/31/2012#, #9/30/2012#, #10/31/2012#, #11/30/2012#, #12/31/2012#, #1/31/2013#, #2/28/2013#, #3/31/2013#, #4/30/2013#, #5/31/2013#, #6/30/2013#, #7/31/2013#, #8/31/2013#, #9/30/2013#, #10/31/2013#, #11/30/2013#, #12/31/2013#}
        pValue = New Double() {100000.0, 100457.78, 101518.83, 102055.61, 103216.4, 103970.62, 104915.46, 105640.67, 107643.46, 108017.35, 109334.07, 110373.01, 111550.51, 112898.34, 113909.43, 114682.07, 114877.46, 115731.44, 117754.23, 118578.8, 120634.65, 121411.91, 123689.75, 124274.22, 125307.78, 126672.4, 128171.03, 129291.6, 131008.02, 132464.56, 133378.55, 133769.46, 135724.66, 136462.27, 137830.14, 139773.62, 142001.01}
        RF = New Double() {0.005202, 0.004891, 0.005097, 0.004959, 0.005014, 0.004995, 0.005035, 0.004929, 0.005027, 0.004865, 0.005033, 0.005042, 0.004938, 0.005016, 0.00499, 0.005084, 0.004889, 0.004898, 0.005133, 0.004719, 0.004908, 0.00496, 0.004986, 0.004853, 0.004995, 0.00499, 0.00497, 0.004814, 0.00503, 0.005208, 0.005126, 0.00493, 0.004932, 0.00486, 0.004946, 0.004971, 0.00495}
        scale = 12
        Geometric = False

        result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric)

        Console.WriteLine("SHARPE2 : " & result)         '3.76729327551418
        File.AppendAllText("SHARPE2.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3 
        '=====================

        pDate = New Date() {#12/17/2012#, #12/10/2012#, #12/3/2012#, #11/26/2012#, #11/19/2012#, #11/12/2012#, #11/5/2012#, #10/31/2012#, #10/22/2012#, #10/15/2012#, #10/8/2012#, #10/1/2012#, #9/24/2012#, #9/17/2012#, #9/10/2012#, #9/4/2012#, #8/27/2012#, #8/20/2012#, #8/13/2012#, #8/6/2012#, #7/30/2012#, #7/23/2012#, #7/16/2012#, #7/9/2012#, #7/2/2012#, #6/25/2012#, #6/18/2012#, #6/11/2012#, #6/4/2012#, #5/29/2012#, #5/21/2012#, #5/14/2012#, #5/7/2012#, #4/30/2012#, #4/23/2012#, #4/16/2012#, #4/9/2012#, #4/2/2012#, #3/26/2012#, #3/19/2012#, #3/12/2012#, #3/5/2012#, #2/27/2012#, #2/21/2012#, #2/13/2012#, #2/6/2012#, #1/30/2012#, #1/23/2012#, #1/17/2012#, #1/9/2012#, #1/3/2012#}
        pValue = New Double() {195.08, 191.76, 191.95, 190.07, 193.49, 186.94, 189.64, 192.59, 192.43, 192.52, 206.89, 209.67, 206.55, 205.08, 205.91, 198.63, 194.0, 196.91, 200.34, 198.42, 196.81, 194.7, 190.8, 184.41, 189.77, 193.9, 192.04, 197.39, 193.46, 187.46, 192.63, 194.2, 199.44, 202.38, 204.18, 197.06, 200.22, 202.86, 205.99, 202.86, 203.39, 198.07, 196.28, 195.24, 190.96, 189.97, 190.43, 187.31, 185.4, 176.19, 179.52}
        RF = New Double() {0.000191, 0.000192, 0.000192, 0.000193, 0.000194, 0.000191, 0.000192, 0.000194, 0.000188, 0.000195, 0.00019, 0.000194, 0.000196, 0.000189, 0.000193, 0.000194, 0.000192, 0.000191, 0.000192, 0.000191, 0.000194, 0.000196, 0.000191, 0.000193, 0.000194, 0.000192, 0.000192, 0.000193, 0.000195, 0.00019, 0.000191, 0.000191, 0.000194, 0.000193, 0.000191, 0.00019, 0.000191, 0.000194, 0.000193, 0.000196, 0.000195, 0.000189, 0.000191, 0.000194, 0.000193, 0.000193, 0.00019, 0.000193, 0.000188, 0.000193, 0.000191}
        scale = 52
        Geometric = True

        result = wct.SHARPE2(pDate, pValue, RF, scale, Geometric)

        Console.WriteLine("SHARPE2 : " & result)         '0.47823560953273
        File.AppendAllText("SHARPE2.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_SORTINO()
        Dim R As Double()
        Dim MAR As Double
        Dim Full As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        R = New Double() {0.0089, 0.0012, -0.002, 0.01, -0.0002, 0.02, 0.03, 0.01, -0.003, 0.01, 0.0102, -0.01}
        MAR = Math.Pow(1.1, 1 / 12) - 1
        Full = True

        result = wct.SORTINO(R, MAR, Full)

        Console.WriteLine("SORTINO : " & result)         '-0.11938313404921
        File.WriteAllText("SORTINO.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        Full = False

        result = wct.SORTINO(R, MAR, Full)

        Console.WriteLine("SORTINO : " & result)         '-0.0770614816651062
        File.AppendAllText("SORTINO.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_SORTINO2()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim MAR As Double
        Dim Full As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/31/2013#, #1/31/2014#, #2/28/2014#, #3/31/2014#, #4/30/2014#, #5/31/2014#, #6/30/2014#, #7/31/2014#, #8/31/2014#, #9/30/2014#, #10/31/2014#, #11/30/2014#, #12/31/2014#}
        pValue = New Double() {1000000.0, 1008900.0, 1010110.68, 1008090.46, 1018171.36, 1017967.73, 1038327.08, 1069476.9, 1080171.66, 1076931.15, 1087700.46, 1098795.01, 1087807.06}
        MAR = Math.Pow(1.1, 1 / 12) - 1
        Full = True

        result = wct.SORTINO2(pDate, pValue, MAR, Full)

        Console.WriteLine("SORTINO2 : " & result)         '-0.11938313404921
        File.WriteAllText("SORTINO2.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        Full = False

        result = wct.SORTINO2(pDate, pValue, MAR, Full)

        Console.WriteLine("SORTINO2 : " & result)         '-0.0770614816651062
        File.AppendAllText("SORTINO2.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_SpecificRisk()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim Rf As Double()
        Dim Freq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}
        Rf = New Double() {0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332}
        Freq = 12

        result = wct.SpecificRisk(Ra, Rb, Rf, Freq)

        Console.WriteLine("SpecificRisk : " & result)            '0.0324179638746894
        File.WriteAllText("SpecificRisk.csv", result.ToString())

    End Sub

    Public Shared Sub test_SystematicRisk()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim Rf As Double()
        Dim Freq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}
        Rf = New Double() {0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332}
        Freq = 12

        result = wct.SystematicRisk(Ra, Rb, Rf, Freq)

        Console.WriteLine("SystematicRisk : " & result)            '0.00514725126745008
        File.WriteAllText("SystematicRisk.csv", result.ToString())

    End Sub

    Public Shared Sub test_TotalRisk()
        Dim Ra As Double()
        Dim Rb As Double()
        Dim Rf As Double()
        Dim Freq As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        Rb = New Double() {-0.003929, -0.001701, 0.003165, -0.006487, -0.004653, 0.009577, 0.00588, 0.005089, 0.005233, -0.004338, -0.006109, -0.002222, 0.005451, -0.008099, -0.000299, -0.009809, 0.008875, -0.002681, 0.000312, 0.00936, 0.005434, 0.008157, -0.006766, 0.005742, -0.005063, 0.004357, -0.00044, -0.004866, -0.007649, -0.002073, 0.00399, -0.00613, -0.00351, 0.001172, -0.000024, 0.002596, -0.009025}
        Rf = New Double() {0.00142, 0.00075, 0.001579, 0.00017, 0.001034, 0.00071, 0.000256, 0.00101, 0.001522, 0.001119, 0.001232, 0.001374, 0.001406, 0.000624, 0.00164, 0.001427, 0.001618, 0.000937, 0.0008, 0.001733, 0.000325, 0.000535, 0.001325, 0.001152, 0.001246, 0.000372, 0.001045, 0.001419, 0.001383, 0.000783, 0.0007, 0.001188, 0.000771, 0.000742, 0.001119, 0.001116, 0.001332}
        Freq = 12

        result = wct.TotalRisk(Ra, Rb, Rf, Freq)

        Console.WriteLine("TotalRisk : " & result)            '0.0328240548590654
        File.WriteAllText("TotalRisk.csv", result.ToString())

    End Sub

    Public Shared Sub test_TREYNOR()
        Dim r As Double()
        Dim rb As Double()
        Dim Rf As Double
        Dim scale As Double
        Dim geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        r = New Double() {0.0107, 0.0097, -0.0012, -0.005, -0.0064, 0.0082, 0.0035, 0.0119, 0.0056, -0.0037, -0.0006, -0.0031, -0.0076, -0.0023, 0.0039, -0.0086, -0.0032, 0.0168, 0.0058, -0.006, 0.0182, 0.0059, 0.0018, -0.0149, -0.0049, -0.0021, -0.0024, -0.0055, -0.0158, 0.0048, 0.0036, -0.0188, 0.0135}
        rb = New Double() {0.0115, 0.0119, -0.0041, -0.0063, 0.0004, 0.0065, 0.0003, 0.0029, 0.0033, 0.0016, -0.0017, -0.0047, 0.0002, 0.0043, 0.0079, -0.0052, -0.002, 0.013, 0.0023, 0.0007, 0.0199, 0.0048, -0.0016, -0.0139, -0.004, 0.0001, 0.0017, -0.0122, -0.0237, 0.0079, 0.0022, -0.0094, 0.0109}
        Rf = 0.001 / 252
        scale = 252
        geometric = False

        result = wct.TREYNOR(r, rb, Rf, scale, geometric)

        Console.WriteLine("TREYNOR : " & result)           '0.000388271591630151
        File.WriteAllText("TREYNOR.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        r = New Double() {0.0173, -0.001, 0.0099, -0.0177, 0.035, -0.0142, -0.0153, 0.0008, -0.0005, -0.0695, -0.0133, 0.0151, 0.0072, -0.004, 0.0367, 0.0239, -0.0148, -0.0171, 0.0097, 0.0082, 0.0108, 0.0204, 0.0347, -0.0282, -0.0213, 0.0097, -0.0271, 0.0203, 0.032, -0.0268, -0.0081, -0.0263, -0.0145, -0.0088, 0.0361, -0.0158, -0.013, -0.0152, 0.0154, -0.0026, 0.0269, 0.0091, 0.0053, 0.0224, 0.0052, -0.0024, 0.0167, 0.0103, 0.0523, -0.0185}
        rb = New Double() {0.0157, -0.0032, 0.0013, 0.005, 0.0362, -0.0145, -0.0243, 0.0016, -0.0148, 0.0032, -0.0221, 0.0141, -0.0133, -0.0038, 0.0194, 0.0223, -0.0032, -0.005, 0.0087, 0.0107, 0.0036, 0.0171, 0.0043, 0.0016, -0.0055, 0.0203, -0.0058, 0.013, 0.0373, -0.0302, 0.0174, -0.043, -0.0115, -0.0244, 0.018, 0.006, -0.0199, -0.0074, 0.0081, -0.005, 0.0243, 0.0009, 0.0028, 0.0033, 0.0138, -0.0017, 0.0217, 0.0007, 0.0204, 0.0088}
        Rf = 0.001 / 52
        scale = 52
        geometric = True

        result = wct.TREYNOR(r, rb, Rf, scale, geometric)

        Console.WriteLine("TREYNOR : " & result)          '0.103411484610504
        File.AppendAllText("TREYNOR.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_TREYNOR2()
        Dim pDate As Date()
        Dim pValue As Double()
        Dim bValue As Double()
        Dim Rf As Double
        Dim scale As Double
        Dim geometric As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        pDate = New Date() {#12/18/2012#, #12/17/2012#, #12/14/2012#, #12/13/2012#, #12/12/2012#, #12/11/2012#, #12/10/2012#, #12/7/2012#, #12/6/2012#, #12/5/2012#, #12/4/2012#, #12/3/2012#, #11/30/2012#, #11/29/2012#, #11/28/2012#, #11/27/2012#, #11/26/2012#, #11/23/2012#, #11/21/2012#, #11/20/2012#, #11/19/2012#, #11/16/2012#, #11/15/2012#, #11/14/2012#, #11/13/2012#, #11/12/2012#, #11/9/2012#, #11/8/2012#, #11/7/2012#, #11/6/2012#, #11/5/2012#, #11/2/2012#, #11/1/2012#, #10/31/2012#}
        pValue = New Double() {195.69, 193.62, 191.76, 191.99, 192.95, 194.2, 192.62, 191.95, 189.7, 188.65, 189.36, 189.48, 190.07, 191.53, 191.98, 191.23, 192.88, 193.49, 190.29, 189.2, 190.35, 186.94, 185.85, 185.51, 188.32, 189.25, 189.64, 190.1, 191.16, 194.22, 193.29, 192.59, 196.29, 193.68}
        bValue = New Double() {1446.79, 1430.36, 1413.58, 1419.45, 1428.48, 1427.84, 1418.55, 1418.07, 1413.94, 1409.28, 1407.05, 1409.46, 1416.18, 1415.95, 1409.93, 1398.94, 1406.29, 1409.15, 1391.03, 1387.81, 1386.89, 1359.88, 1353.33, 1355.49, 1374.53, 1380.0, 1379.85, 1377.51, 1394.53, 1428.39, 1417.26, 1414.2, 1427.59, 1412.16}
        Rf = 0.001 / 252
        scale = 252
        geometric = False

        result = wct.TREYNOR2(pDate, pValue, bValue, Rf, scale, geometric)

        Console.WriteLine("TREYNOR2 : " & result)           '0.000381122001119534
        File.WriteAllText("TREYNOR2.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        pDate = New Date() {#12/17/2012#, #12/10/2012#, #12/3/2012#, #11/26/2012#, #11/19/2012#, #11/12/2012#, #11/5/2012#, #10/31/2012#, #10/22/2012#, #10/15/2012#, #10/8/2012#, #10/1/2012#, #9/24/2012#, #9/17/2012#, #9/10/2012#, #9/4/2012#, #8/27/2012#, #8/20/2012#, #8/13/2012#, #8/6/2012#, #7/30/2012#, #7/23/2012#, #7/16/2012#, #7/9/2012#, #7/2/2012#, #6/25/2012#, #6/18/2012#, #6/11/2012#, #6/4/2012#, #5/29/2012#, #5/21/2012#, #5/14/2012#, #5/7/2012#, #4/30/2012#, #4/23/2012#, #4/16/2012#, #4/9/2012#, #4/2/2012#, #3/26/2012#, #3/19/2012#, #3/12/2012#, #3/5/2012#, #2/27/2012#, #2/21/2012#, #2/13/2012#, #2/6/2012#, #1/30/2012#, #1/23/2012#, #1/17/2012#, #1/9/2012#, #1/3/2012#}
        pValue = New Double() {195.08, 191.76, 191.95, 190.07, 193.49, 186.94, 189.64, 192.59, 192.43, 192.52, 206.89, 209.67, 206.55, 205.08, 205.91, 198.63, 194.0, 196.91, 200.34, 198.42, 196.81, 194.7, 190.8, 184.41, 189.77, 193.9, 192.04, 197.39, 193.46, 187.46, 192.63, 194.2, 199.44, 202.38, 204.18, 197.06, 200.22, 202.86, 205.99, 202.86, 203.39, 198.07, 196.28, 195.24, 190.96, 189.97, 190.43, 187.31, 185.4, 176.19, 179.52}
        bValue = New Double() {1435.81, 1413.58, 1418.07, 1416.18, 1409.15, 1359.88, 1379.85, 1414.2, 1411.94, 1433.19, 1428.59, 1460.93, 1440.67, 1460.15, 1465.77, 1437.92, 1406.58, 1411.13, 1418.16, 1405.87, 1390.99, 1385.97, 1362.66, 1356.78, 1354.68, 1362.16, 1335.02, 1342.84, 1325.66, 1278.04, 1317.82, 1295.22, 1353.39, 1369.1, 1403.36, 1378.53, 1370.26, 1398.08, 1408.47, 1397.11, 1404.17, 1370.87, 1369.63, 1365.74, 1361.23, 1342.64, 1344.9, 1316.33, 1315.38, 1289.09, 1277.81}
        Rf = 0.001 / 52
        scale = 52
        geometric = True

        result = wct.TREYNOR2(pDate, pValue, bValue, Rf, scale, geometric)

        Console.WriteLine("TREYNOR2 : " & result)           '0.103301285778773
        File.AppendAllText("TREYNOR2.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_UpsideFrequency()
        Dim Ra As Double()
        Dim MAR As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005

        result = wct.UpsideFrequency(Ra, MAR)

        Console.WriteLine("UpsideFrequency : " & result)          '0.405405405405405
        File.WriteAllText("UpsideFrequency.csv", result.ToString())

    End Sub

    Public Shared Sub test_UpsidePotentialRatio()
        Dim Ra As Double()
        Dim MAR As Double
        Dim Full As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005
        Full = False

        result = wct.UpsidePotentialRatio(Ra, MAR, Full)

        Console.WriteLine("UpsidePotentialRatio : " & result)          '0.762408352161206
        File.WriteAllText("UpsidePotentialRatio.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        Full = True

        result = wct.UpsidePotentialRatio(Ra, MAR, Full)

        Console.WriteLine("UpsidePotentialRatio : " & result)          '0.400835983716186
        File.AppendAllText("UpsidePotentialRatio.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_UpsideRisk()
        Dim Ra As Double()
        Dim MAR As Double
        Dim Full As Boolean
        Dim State As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Ra = New Double() {0.001378, 0.028677, 0.005801, 0.01442, 0.00229, 0.014905, 0.008594, 0.011531, 0.008268, 0.013993, 0.009147, -0.00316, -0.00595, 0.013398, 0.002847, -0.009544, 0.002516, 0.004626, -0.002141, 0.009247, -0.01253, 0.00441, 0.01626, 0.013207, -0.008561, 0.012357, -0.002057, 0.008217, -0.013439, 0.004391, -0.008997, -0.001878, -0.014381, -0.001885, -0.001121, 0.000816, -0.003088}
        MAR = 0.005
        Full = False
        State = "POTENTIAL"

        result = wct.UpsideRisk(Ra, MAR, Full, State)

        Console.WriteLine("UpsideRisk : " & result)          '0.0075348
        File.WriteAllText("UpsideRisk.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        State = "RISK"

        result = wct.UpsideRisk(Ra, MAR, Full, State)

        Console.WriteLine("UpsideRisk : " & result)          '0.00916366023668126
        File.AppendAllText("UpsideRisk.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        State = "VARIANCE"

        result = wct.UpsideRisk(Ra, MAR, Full, State)

        Console.WriteLine("UpsideRisk : " & result)          '8.39726689333333E-05 
        File.AppendAllText("UpsideRisk.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_CUMLIPMT()
        Dim PV As Double
        Dim LoanDate As Date
        Dim Rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim StartPer As Integer
        Dim EndPer As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first year’s cumulative interest for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        StartPer = 1
        EndPer = 12
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLIPMT : " & result)            '2759.52255542426
        File.WriteAllText("CUMLIPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last year’s interest for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        StartPer = 49
        EndPer = 60
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLIPMT : " & result)            '368.355536073601
        File.AppendAllText("CUMLIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the cumulative interest payments for the first two periods on a 150,000 4-year loan 
        'at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
        'and with payments due every 4 weeks after that.

        PV = 150000
        LoanDate = #10/13/2010#
        Rate = 0.12
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        Pmtpyr = 13
        StartPer = 1
        EndPer = 2
        DaysInYr = 365
        FV = 0
        IntRule = "U"

        result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLIPMT : " & result)            '5473.97260273973
        File.AppendAllText("CUMLIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same cumulative interest payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.CUMLIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLIPMT : " & result)            '5477.51646742669
        File.AppendAllText("CUMLIPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_CUMLPPMT()
        Dim PV As Double
        Dim LoanDate As Date
        Dim Rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim StartPer As Integer
        Dim EndPer As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first year’s cumulative principal for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        StartPer = 1
        EndPer = 12
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLPPMT : " & result)            '8840.15836223269
        File.WriteAllText("CUMLPPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last year’s principal payments for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        StartPer = 49
        EndPer = 60
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLPPMT : " & result)            '11231.3253815834
        File.AppendAllText("CUMLPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the cumulative principal payments for the first two periods on a 150,000 4-year loan 
        'at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
        'and with payments due every 4 weeks after that.

        PV = 150000
        LoanDate = #10/13/2010#
        Rate = 0.12
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        Pmtpyr = 13
        StartPer = 1
        EndPer = 2
        DaysInYr = 365
        FV = 0
        IntRule = "U"

        result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLPPMT : " & result)            '1942.2129309029
        File.AppendAllText("CUMLPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same cumulative principal payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.CUMLPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, StartPer, EndPer, DaysInYr, FV, IntRule)

        Console.WriteLine("CUMLPPMT : " & result)            '1938.83811031561
        File.AppendAllText("CUMLPPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_EFFECT()
        Dim NominalRate As Double
        Dim npery As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        NominalRate = 0.04
        npery = 12

        result = wct.EFFECT(NominalRate, npery)

        Console.WriteLine("EFFECT : " & result)
        File.WriteAllText("EFFECT.csv", result.ToString())

    End Sub

    Public Shared Sub test_FVSCHEDULE()
        Dim rate As Double()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        rate = New Double() {0.00025, 0.00025, 0.000167, 0.0005, 0.000333, 0.0005, 0.00075, 0.00025, 0.0005, 0.00025, 0.000333, 0.000417}

        result = wct.FVSCHEDULE(rate) * 100000

        Console.WriteLine("FVSCHEDULE : " & result)          '100450.914303657
        File.WriteAllText("FVSCHEDULE.csv", result.ToString())

    End Sub

    Public Shared Sub test_LIPMT()
        Dim PV As Double
        Dim LoanDate As Date
        Dim Rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim Per As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        Per = 1
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LIPMT : " & result)            '250
        File.WriteAllText("LIPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        Per = 60
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LIPMT : " & result)            '4.80915460931035
        File.AppendAllText("LIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the first interest payment for a 150,000 4-year loan 
        'at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
        'and with payments due every 4 weeks after that.

        PV = 150000
        LoanDate = #10/13/2010#
        Rate = 0.12
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        Pmtpyr = 13
        Per = 1
        DaysInYr = 365
        FV = 0
        IntRule = "U"

        result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LIPMT : " & result)            '3708.09276682132
        File.AppendAllText("LIPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.LIPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LIPMT : " & result)            '4093.15068493151
        File.AppendAllText("LIPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_LPPMT()
        Dim PV As Double
        Dim LoanDate As Date
        Dim Rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim Per As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first principal payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        Per = 1
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LPPMT : " & result)            '716.640076471413
        File.WriteAllText("LPPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last principal payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        Rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        Per = 60
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LPPMT : " & result)            '961.830921862103
        File.AppendAllText("LPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the first principal payment for a 150,000 4-year loan 
        'at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January 
        'and with payments due every 4 weeks after that.

        PV = 150000
        LoanDate = #10/13/2010#
        Rate = 0.12
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        Pmtpyr = 13
        Per = 1
        DaysInYr = 365
        FV = 0
        IntRule = "U"

        result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LPPMT : " & result)            '0
        File.AppendAllText("LPPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same principal payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.LPPMT(PV, LoanDate, Rate, FirstPayDate, NumPmts, Pmtpyr, Per, DaysInYr, FV, IntRule)

        Console.WriteLine("LPPMT : " & result)            '-384.973396060356
        File.AppendAllText("LPPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_LPMTSCHED()
        Dim PV As Double
        Dim LoanDate As Date
        Dim pmt As Double
        Dim rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As InterestRuleType
        Dim Decimals As Integer
        Dim LastPmtNum As Integer
        Dim result As LPMTSCHED_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'amortization schedule for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The periodic payment is 966.64.
        PV = 50000
        LoanDate = #11/1/2010#
        rate = wct.NULL_DOUBLE
        pmt = 966.64
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = 0
        IntRule = InterestRuleType.Actuarial
        Decimals = 2
        LastPmtNum = wct.NULL_INT

        result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("LPMTSCHED : " & retStr.ToString())
        File.WriteAllText("LPMTSCHED.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'amortization schedule for a 50,000, 5-year loan starting on 1 November with payments of 1,161.41 
        'due on the fifteenth of every month starting 15-Apr-2011. 
        PV = 50000
        LoanDate = #11/1/2010#
        rate = wct.NULL_DOUBLE
        pmt = 1161.41
        FirstPayDate = #4/15/2011#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = 0
        IntRule = InterestRuleType.Actuarial
        Decimals = 2
        LastPmtNum = wct.NULL_INT

        result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum)

        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("LPMTSCHED : " & retStr.ToString())
        File.AppendAllText("LPMTSCHED.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example, the US Rule is applied, so there is no negative amortization. 
        'The monthly payment is 1160.82, and all other parameters are from the previous example.
        PV = 50000
        LoanDate = #11/1/2010#
        rate = wct.NULL_DOUBLE
        pmt = 1160.82
        FirstPayDate = #4/15/2011#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = 0
        IntRule = InterestRuleType.US
        Decimals = 2
        LastPmtNum = wct.NULL_INT

        result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum)

        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("LPMTSCHED : " & retStr.ToString())
        File.AppendAllText("LPMTSCHED.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example, we have a 7,125,000 loan that pays monthly and is being amortized over 25 years, 
        'but the loan is paid off in full at the end of 10 years. 
        'The monthly payment is 41,554.10
        PV = 7125000
        LoanDate = #11/18/2008#
        rate = wct.NULL_DOUBLE
        pmt = 41554.1
        FirstPayDate = #12/1/2008#
        NumPmts = 300
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = 0
        IntRule = InterestRuleType.Actuarial
        Decimals = 2
        LastPmtNum = 120

        result = wct.LPMTSCHED(PV, LoanDate, pmt, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, Decimals, LastPmtNum)

        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("LPMTSCHED : " & retStr.ToString())
        File.AppendAllText("LPMTSCHED.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_LRATE()
        Dim PV As Double
        Dim LoanDate As Date
        Dim pmt As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim DaysInYr As Double
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        PV = 50000
        LoanDate = #11/1/2010#
        pmt = 966.64
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_DOUBLE
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE)

        Console.WriteLine("LRATE : " & result)           '0.06
        File.WriteAllText("LRATE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        PV = 50000
        LoanDate = #11/1/2010#
        pmt = 892.42
        FirstPayDate = #12/1/2010#
        NumPmts = 65
        Pmtpyr = 13
        DaysInYr = wct.NULL_DOUBLE
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE)

        Console.WriteLine("LRATE : " & result)           '0.0601605094753396
        File.AppendAllText("LRATE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        PV = 50000
        LoanDate = #11/1/2010#
        pmt = 412.45
        FirstPayDate = #12/1/2010#
        NumPmts = 120
        Pmtpyr = 24
        DaysInYr = wct.NULL_DOUBLE
        FV = 10000
        IntRule = wct.NULL_STRING
        result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE)

        Console.WriteLine("LRATE : " & result)           '0.0600022174769416
        File.AppendAllText("LRATE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        PV = 150000
        LoanDate = #10/13/2010#
        pmt = 3710.27
        FirstPayDate = #1/4/2011#
        NumPmts = 52
        Pmtpyr = 13
        DaysInYr = wct.NULL_DOUBLE
        FV = wct.NULL_DOUBLE
        IntRule = wct.NULL_STRING

        result = wct.LRATE(PV, LoanDate, pmt, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule, wct.NULL_DOUBLE)

        Console.WriteLine("LRATE : " & result)           '0.120281649456623
        File.AppendAllText("LRATE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_NOMINAL()
        Dim Rate As Double
        Dim npery As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Rate = 0.0407415429197906
        npery = 12

        result = wct.NOMINAL(Rate, npery)

        Console.WriteLine("NOMINAL : " & result)         '0.0400000000000009
        File.WriteAllText("NOMINAL.csv", result.ToString())

    End Sub

    Public Shared Sub test_NUMPMTS()
        Dim firstpaymentdate As Date
        Dim lastpaymentdate As Date
        Dim pmtperyr As Integer
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        firstpaymentdate = #1/31/2014#
        lastpaymentdate = #12/31/2019#
        pmtperyr = 12

        result = wct.NUMPMTS(firstpaymentdate, lastpaymentdate, pmtperyr)

        Console.WriteLine("NUMPMTS : " & result)         '72
        File.WriteAllText("NUMPMTS.csv", result.ToString())

    End Sub

    Public Shared Sub test_TOTALINT()
        Dim nper As Double
        Dim pmt As Double
        Dim pv As Double
        Dim fv As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        nper = 15
        pmt = 215
        pv = 3000
        fv = 0

        result = wct.TOTALINT(nper, pmt, pv, fv)

        Console.WriteLine("TOTALINT : " & result)            '225
        File.WriteAllText("TOTALINT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        nper = 48
        pmt = 595.02
        pv = 30000
        fv = 5000

        result = wct.TOTALINT(nper, pmt, pv, fv)

        Console.WriteLine("TOTALINT : " & result)            '3560.96
        File.AppendAllText("TOTALINT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_AMORTRATE()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim rate As Double
        Dim FaceAmount As Double
        Dim CleanPrice As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As BasisType
        Dim IssueDate As Date
        Dim FirstInterestDate As Date
        Dim LastInterestDate As Date
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'We buy 1,000,000 in face value of a bond on 2012-05-03 at a price of 999000. 
        'The bond matures on 2012-06-30 and accrues interest using the Actual/365 day-count method. The interest rate is 5%
        Settlement = #5/3/2015#
        Maturity = #6/30/2015#
        rate = 0.05
        FaceAmount = 1000000.0
        CleanPrice = 999000.0
        Redemption = wct.NULL_DOUBLE
        Frequency = 2
        Basis = BasisType.A365
        IssueDate = wct.NULL_DATE
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE

        result = wct.AMORTRATE(Settlement, Maturity, rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        Console.WriteLine("AMORTRATE : " & result)           '0.000154306279086793
        File.WriteAllText("AMORTRATE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2 
        '=====================

        'In this example we change the basis to the US 30/360 day-count convention.
        Basis = BasisType.Bond

        result = wct.AMORTRATE(Settlement, Maturity, rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        Console.WriteLine("AMORTRATE : " & result)           '0.000156512494013327
        File.AppendAllText("AMORTRATE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_AMORTSCHED()
        Dim PV As Double
        Dim LoanDate As Date
        Dim rate As Double
        Dim FirstPayDate As Date
        Dim NumPmts As Integer
        Dim Pmtpyr As Integer
        Dim DaysInYr As Integer
        Dim FV As Double
        Dim IntRule As InterestRuleType
        Dim result As AMORTSCHED_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'amortization schedule for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. 
        'The rate on the loan is 6 per cent.
        PV = 50000
        LoanDate = #11/1/2010#
        rate = 0.06
        FirstPayDate = #12/1/2010#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = InterestRuleType.Actuarial

        result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("AMORTSCHED : " & retStr.ToString())
        File.WriteAllText("AMORTSCHED.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This is an example of a loan that negatively amortizes principal using the actuarial rule.. 
        PV = 50000
        LoanDate = #11/1/2010#
        rate = 0.12
        FirstPayDate = #4/15/2011#
        NumPmts = 60
        Pmtpyr = 12
        DaysInYr = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        IntRule = InterestRuleType.Actuarial

        result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule)

        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("AMORTSCHED : " & retStr.ToString())
        File.AppendAllText("AMORTSCHED.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'The same loan, except that the US Rule is applied, so there is no negative amortization.
        IntRule = InterestRuleType.US

        result = wct.AMORTSCHED(PV, LoanDate, rate, FirstPayDate, NumPmts, Pmtpyr, DaysInYr, FV, IntRule)

        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("AMORTSCHED : " & retStr.ToString())
        File.AppendAllText("AMORTSCHED.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_Balloon()
        Dim OutstandingAmount As Double
        Dim InterestBasis As InterestBasisType
        Dim InterestRate As Double
        Dim PaymentFrequency As Integer
        Dim MaturityDate As Date
        Dim ReferenceDate As Date
        Dim PrevPayDate As Date
        Dim StartDate As Date
        Dim FirstPayDate As Date
        Dim GracePeriodStartDate As Date
        Dim GracePeriodEndDate As Date
        Dim result As Balloon_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a simple 5-year loan with quarterly payments of interest.
        OutstandingAmount = 100000
        InterestBasis = InterestBasisType.A365
        InterestRate = 0.04
        PaymentFrequency = 3
        MaturityDate = #9/15/2019#
        ReferenceDate = #9/15/2014#
        PrevPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE

        result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Balloon : " & retStr.ToString())
        File.WriteAllText("Balloon.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example, we modify the first payment date so that the interest will not be paid until 31-Mar-2015.
        FirstPayDate = #3/15/2015#

        result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Balloon : " & retStr.ToString())
        File.AppendAllText("Balloon.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'No interest payments in 2018
        GracePeriodStartDate = #1/1/2018#
        GracePeriodEndDate = #1/1/2019#

        result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Balloon : " & retStr.ToString())
        File.AppendAllText("Balloon.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Get rid of the first payment date and set previous payment date.
        FirstPayDate = wct.NULL_DATE
        PrevPayDate = #8/15/2014#

        result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Balloon : " & retStr.ToString())
        File.AppendAllText("Balloon.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Eliminate the grace period and the previous payment date and add a start date.
        PrevPayDate = wct.NULL_DATE
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE
        StartDate = #9/15/2011#

        result = wct.Balloon(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Balloon : " & retStr.ToString())
        File.AppendAllText("Balloon.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_Bullet()
        Dim OutstandingAmount As Double
        Dim InterestBasis As InterestBasisType
        Dim InterestRate As Double
        Dim Frequency As Integer
        Dim MaturityDate As Date
        Dim ReferenceDate As Date
        Dim result As Bullet_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        OutstandingAmount = 6000000
        InterestBasis = InterestBasisType.A360
        InterestRate = 0.07
        Frequency = 3
        MaturityDate = #7/5/2015#
        ReferenceDate = #6/30/2014#

        result = wct.Bullet(OutstandingAmount, InterestBasis, InterestRate, Frequency, MaturityDate, ReferenceDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("Bullet : " & retStr.ToString())
        File.WriteAllText("Bullet.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_CDRCashFlow()
        Dim PrinAmt As Double
        Dim InterestRate As Double
        Dim NumPmts As Integer
        Dim LastPmtNum As Integer
        Dim PmtPerYr As Integer
        Dim InterestOnly As Boolean
        Dim PrinPaymentMultiple As Integer
        Dim FirstPrinPayNo As Integer
        Dim PmtPayPct As Double
        Dim retStr As StringBuilder
        Dim result As CDRCashflow_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim LSRates_array As System.Array = {{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim CPRRates_array As System.Array = {{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CDRRates_array As System.Array = {{1, 0.1}, {61, 0.08}, {181, 0.05}}

        'In this example we amortize a 100,000,000 loan with a 5% interest rate over 360 months 

        PrinAmt = 100000000
        InterestRate = 0.05
        NumPmts = 360
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 12
        InterestOnly = False
        PrinPaymentMultiple = wct.NULL_INT
        FirstPrinPayNo = wct.NULL_INT
        PmtPayPct = wct.NULL_DOUBLE

        result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CDRCashflow : " & retStr.ToString())
        File.WriteAllText("CDRCashflow.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example, we have a 10-year, 20,000,000 loan, with monthly payments calculated as though it were a 25-year loan.

        PrinAmt = 20000000
        InterestRate = 0.05
        NumPmts = 300
        LastPmtNum = 120
        PmtPerYr = 12

        result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CDRCashflow : " & retStr.ToString())
        File.AppendAllText("CDRCashflow.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example we have a 10-year, 25,000,000 interest-only loan with quarterly payments

        PrinAmt = 25000000
        InterestRate = 0.04
        NumPmts = 40
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 4

        Dim LSRates_mo As Integer() = {1, 91, 181, 271, 361}                '{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim LSRates_rate As Double() = {0.15, 0.16, 0.14, 0.17, 0.13}       '{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim CPRRates_mo As Integer() = {1, 25, 61, 181}                     '{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CPRRates_rate As Double() = {0.002, 0.012, 0.02, 0.06}          '{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CDRRates_mo As Integer() = {1, 61, 181}                         '{{1, 0.1}, {61, 0.08}, {181, 0.05}}
        Dim CDRRates_rate As Double() = {0.1, 0.08, 0.05}                   '{{1, 0.1}, {61, 0.08}, {181, 0.05}}

        result = wct.CDRCashflow(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_mo, LSRates_rate, CPRRates_mo, CPRRates_rate, CDRRates_mo, CDRRates_rate, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CDRCashflow : " & retStr.ToString())
        File.AppendAllText("CDRCashflow.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantCashFlow()
        Dim OutstandingAmount As Double
        Dim LastPrinPayAmount As Double
        Dim InterestRate As Double
        Dim PaymentFrequency As Integer
        Dim MaturityDate As Date
        Dim ReferenceDate As Date
        Dim PrevPayDate As Date
        Dim StartDate As Date
        Dim FirstPayDate As Date
        Dim GracePeriodStartDate As Date
        Dim GracePeriodEndDate As Date
        Dim result As ConstantCashFlow_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a simple a 300,000 loan with quarterly payments and an interest rate of 6.0%.
        OutstandingAmount = 300000
        LastPrinPayAmount = 0
        InterestRate = 0.06
        PaymentFrequency = 3
        MaturityDate = #12/15/2019#
        ReferenceDate = #12/15/2014#
        PrevPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.WriteAllText("ConstantCashFlow.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Modify the terms to reflect a final principal payment of 172,000.
        LastPrinPayAmount = 172000

        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlow.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Add a first payment date
        FirstPayDate = #6/15/2015#
        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlow.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'No payemnts in 2018
        GracePeriodStartDate = #1/1/2018#
        GracePeriodEndDate = #1/1/2019#

        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlow.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Don't use first payment date; previous payment date is 2014-11-15
        FirstPayDate = wct.NULL_DATE
        PrevPayDate = #11/15/2014#

        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlow.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 6
        '=====================

        'Get rid of the grace period an add a starte date.
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE
        StartDate = #10/15/2014#

        result = wct.ConstantCashFlow(OutstandingAmount, LastPrinPayAmount, InterestRate, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlow : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlow.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantCashFlowFR()
        Dim OutstandingAmount As Double
        Dim LastPrinPayAmount As Double
        Dim PaymentFrequency As Integer
        Dim MaturityDate As Date
        Dim ReferenceDate As Date
        Dim PrevPayDate As Date
        Dim StartDate As Date
        Dim FirstPayDate As Date
        Dim GracePeriodStartDate As Date
        Dim GracePeriodEndDate As Date
        Dim FutureDates As Date()
        Dim FutureRates As Double()
        Dim result As ConstantCashFlowFR_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        FutureDates = New Date() {#12/31/2014#, #1/31/2015#, #2/28/2015#, #3/31/2015#, #4/30/2015#, #5/31/2015#, #6/30/2015#, #7/31/2015#, #8/31/2015#, #9/30/2015#, #10/31/2015#, #11/30/2015#, #12/31/2015#, #1/31/2016#, #2/29/2016#, #3/31/2016#, #4/30/2016#, #5/31/2016#, #6/30/2016#, #7/31/2016#, #8/31/2016#, #9/30/2016#, #10/31/2016#, #11/30/2016#, #12/31/2016#, #1/31/2017#, #2/28/2017#, #3/31/2017#, #4/30/2017#, #5/31/2017#, #6/30/2017#, #7/31/2017#, #8/31/2017#, #9/30/2017#, #10/31/2017#, #11/30/2017#, #12/31/2017#, #1/31/2018#, #2/28/2018#, #3/31/2018#, #4/30/2018#, #5/31/2018#, #6/30/2018#, #7/31/2018#, #8/31/2018#, #9/30/2018#, #10/31/2018#, #11/30/2018#, #12/31/2018#, #1/31/2019#, #2/28/2019#, #3/31/2019#, #4/30/2019#, #5/31/2019#, #6/30/2019#, #7/31/2019#, #8/31/2019#, #9/30/2019#, #10/31/2019#, #11/30/2019#, #12/31/2019#}
        FutureRates = New Double() {0.025, 0.025102, 0.025205, 0.025308, 0.02541, 0.025513, 0.025615, 0.025718, 0.02582, 0.025923, 0.026025, 0.026128, 0.026231, 0.026333, 0.026436, 0.026539, 0.026641, 0.026744, 0.026847, 0.026949, 0.027052, 0.027155, 0.027257, 0.02736, 0.027463, 0.027566, 0.027668, 0.027771, 0.027874, 0.027977, 0.028079, 0.028182, 0.028285, 0.028388, 0.028491, 0.028594, 0.028696, 0.028799, 0.028902, 0.029005, 0.029108, 0.029211, 0.029314, 0.029417, 0.02952, 0.029623, 0.029726, 0.029829, 0.029932, 0.030035, 0.030138, 0.030241, 0.030344, 0.030447, 0.03055, 0.030653, 0.030756, 0.030859, 0.030962, 0.031065, 0.031168}

        'This is a simple a 300,000 loan with quarterly payments and an interest rate of 6.0%.
        OutstandingAmount = 300000
        LastPrinPayAmount = 0
        PaymentFrequency = 3
        MaturityDate = #12/15/2019#
        ReferenceDate = #12/15/2014#
        PrevPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.WriteAllText("ConstantCashFlowFR.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Modify the terms to reflect a final principal payment of 172,000.
        LastPrinPayAmount = 172000
        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Add a first payment date & set the last principal payment amount to 0.
        FirstPayDate = #6/15/2015#
        LastPrinPayAmount = 0

        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'No payemnts in 2018
        GracePeriodStartDate = #1/1/2018#
        GracePeriodEndDate = #1/1/2019#

        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Don't use first payment date; previous payment date is 2014-11-15
        FirstPayDate = wct.NULL_DATE
        PrevPayDate = #11/15/2014#

        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 6
        '=====================

        'Get rid of the grace period an add a starte date.
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE
        StartDate = #10/15/2014#

        result = wct.ConstantCashFlowFR(OutstandingAmount, LastPrinPayAmount, PaymentFrequency, MaturityDate, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate, FutureDates, FutureRates)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantCashFlowFR : " & retStr.ToString())
        File.AppendAllText("ConstantCashFlowFR.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantPaymentAmount()
        Dim OutstandingAmount As Double
        Dim InterestBasis As InterestBasisType
        Dim InterestRate As Double
        Dim PaymentFrequency As Integer
        Dim PaymentAmount As Double
        Dim ReferenceDate As Date
        Dim PrevPayDate As Date
        Dim StartDate As Date
        Dim FirstPayDate As Date
        Dim GracePeriodStartDate As Date
        Dim GracePeriodEndDate As Date
        Dim result As ConstantPaymentAmount_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a simple 300,000 loan with quarterly payments of 17,500 and an interest rate of 5%.
        OutstandingAmount = 300000
        InterestBasis = InterestBasisType.A360
        InterestRate = 0.05
        PaymentFrequency = 3
        PaymentAmount = 17500
        ReferenceDate = #12/15/2014#
        PrevPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPaymentAmount : " & retStr.ToString())
        File.WriteAllText("ConstantPaymentAmount.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Add a first payment date so that the first payment will not be paid until 30-June-2015.
        FirstPayDate = #6/30/2015#

        result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPaymentAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'No payemnts in 2018
        GracePeriodStartDate = #1/1/2018#
        GracePeriodEndDate = #1/1/2019#

        result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPaymentAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Don't use first payment date; previous payment date is 2014-11-15
        FirstPayDate = wct.NULL_DATE
        PrevPayDate = #11/15/2014#

        result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPaymentAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Get rid of the grace period, the previous payment date and add a start date.
        GracePeriodStartDate = wct.NULL_DATE
        GracePeriodEndDate = wct.NULL_DATE
        PrevPayDate = wct.NULL_DATE
        StartDate = #10/15/2014#

        result = wct.ConstantPaymentAmount(OutstandingAmount, InterestBasis, InterestRate, PaymentFrequency, PaymentAmount, ReferenceDate, PrevPayDate, StartDate, FirstPayDate, GracePeriodStartDate, GracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPaymentAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPaymentAmount.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantPrincipal()
        Dim OutstandingAmount As Double
        Dim InterestBasis As String
        Dim InterestRate As Double
        Dim PrincipalPaymentFrequency As Integer
        Dim InterestPaymentFrequency As Integer
        Dim LastPrinPayAmount As Double
        Dim MaturityDate As DateTime
        Dim ReferenceDate As DateTime
        Dim PrevPrincipalPayDate As DateTime
        Dim PrevInterestPayDate As DateTime
        Dim StartDate As DateTime
        Dim FirstPrincipalPayDate As DateTime
        Dim FirstInterestPayDate As DateTime
        Dim PrincipalGracePeriodStartDate As DateTime
        Dim PrincipalGracePeriodEndDate As DateTime
        Dim InterestGracePeriodStartDate As DateTime
        Dim InterestGracePeriodEndDate As DateTime
        Dim retStr As StringBuilder
        Dim result As ConstantPrincipal_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a simple 5-year 100,000 loan with quarterly payments of principal and interest and an interest rate of 4.5%.

        OutstandingAmount = 100000
        InterestBasis = "30/360"
        InterestRate = 0.045
        PrincipalPaymentFrequency = 3
        InterestPaymentFrequency = 3
        LastPrinPayAmount = 10000.0
        MaturityDate = New DateTime(2019, 10, 1)
        ReferenceDate = New DateTime(2014, 10, 1)
        PrevPrincipalPayDate = wct.NULL_DATE
        PrevInterestPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipal : " & retStr.ToString())
        File.WriteAllText("ConstantPrincipal.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'monthly payments of principal and quarterly payments of interest

        OutstandingAmount = 100000
        InterestBasis = "30/360"
        InterestRate = 0.045
        PrincipalPaymentFrequency = 1
        InterestPaymentFrequency = 3
        LastPrinPayAmount = 10000.0
        MaturityDate = New DateTime(2019, 10, 1)
        ReferenceDate = New DateTime(2014 - 10 - 1)

        result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipal : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipal.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.

        OutstandingAmount = 100000
        InterestBasis = "30/360"
        InterestRate = 0.045
        PrincipalPaymentFrequency = 1
        InterestPaymentFrequency = 3
        LastPrinPayAmount = 10000.0
        MaturityDate = New DateTime(2019, 10, 1)
        ReferenceDate = New DateTime(2014, 10, 1)
        FirstPrincipalPayDate = New DateTime(2015, 3, 1)
        FirstInterestPayDate = New DateTime(2015, 1, 1)

        result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipal : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipal.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'there are no payments in 2018

        OutstandingAmount = 100000
        InterestBasis = "30/360"
        InterestRate = 0.045
        PrincipalPaymentFrequency = 1
        InterestPaymentFrequency = 3
        LastPrinPayAmount = 10000.0
        MaturityDate = New DateTime(2019, 10, 1)
        ReferenceDate = New DateTime(2014, 10, 1)
        ',NULL              --@PrevPrincipalPayDate
        ',NULL              --@PrevInterestPayDate
        ',NULL              --@StartDate
        FirstPrincipalPayDate = New DateTime(2015, 3, 1)
        FirstInterestPayDate = New DateTime(2015, 1, 1)
        PrincipalGracePeriodStartDate = New DateTime(2018, 1, 1)
        PrincipalGracePeriodEndDate = New DateTime(2019, 1, 1)
        InterestGracePeriodStartDate = New DateTime(2018, 1, 1)
        InterestGracePeriodEndDate = New DateTime(2019, 1, 1)

        result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipal : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipal.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'The principal payment frequency to every 5 months, the interest payment frequency to every 6 months, remove the grace periods, and calculate the first payment dates from the previous interest and previous principal payemnt dates.

        OutstandingAmount = 100000
        InterestBasis = "30/360"
        InterestRate = 0.045
        PrincipalPaymentFrequency = 5
        InterestPaymentFrequency = 6
        LastPrinPayAmount = 10000.0
        MaturityDate = New DateTime(2019, 10, 1)
        ReferenceDate = New DateTime(2014, 10, 1)
        PrevPrincipalPayDate = New DateTime(2014, 6, 1)
        PrevInterestPayDate = New DateTime(2014, 5, 1)
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipal(OutstandingAmount, InterestBasis, InterestRate, PrincipalPaymentFrequency, InterestPaymentFrequency, LastPrinPayAmount, MaturityDate, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipal : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipal.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantPrincipalAmount()
        Dim OutstandingAmount As Double
        Dim InterestBasis As InterestBasisType
        Dim InterestRate As Double
        Dim FreqPayPrincipal As Integer
        Dim FreqPayInterest As Integer
        Dim PrinPayAmount As Double
        Dim ReferenceDate As Date
        Dim PrevPrincipalPayDate As Date
        Dim PrevInterestPayDate As Date
        Dim StartDate As Date
        Dim FirstPrincipalPayDate As Date
        Dim FirstInterestPayDate As Date
        Dim PrincipalGracePeriodStartDate As Date
        Dim PrincipalGracePeriodEndDate As Date
        Dim InterestGracePeriodStartDate As Date
        Dim InterestGracePeriodEndDate As Date
        Dim result As ConstantPrincipalAmount_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a 100,000 loan with quarterly payments of principal and interest and an interest rate of 3.0%. 
        'The principal payment is 5,250.
        OutstandingAmount = 100000
        InterestBasis = InterestBasisType.A365
        InterestRate = 0.03
        FreqPayPrincipal = 3
        FreqPayInterest = 3
        PrinPayAmount = 5250.0
        ReferenceDate = #10/1/2014#
        PrevPrincipalPayDate = wct.NULL_DATE
        PrevInterestPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalAmount : " & retStr.ToString())
        File.WriteAllText("ConstantPrincipalAmount.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Modify the loan to have monthly payments of principal and semi-annual payments of interest
        FreqPayPrincipal = 1
        FreqPayInterest = 6

        result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Modify the loan so that first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.
        FirstPrincipalPayDate = #3/1/2015#
        FirstInterestPayDate = #1/1/2015#

        result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'No payments in 2016
        PrincipalGracePeriodStartDate = #1/1/2016#
        PrincipalGracePeriodEndDate = #1/1/2017#
        InterestGracePeriodStartDate = #1/1/2016#
        InterestGracePeriodEndDate = #1/1/2017#

        result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Modify the loan so that the principal payment frequency will be every 5 months and 
        'the first payment dates are calculated from the previous interest and previous principal payment dates.
        FreqPayPrincipal = 5
        ReferenceDate = #10/1/2014#
        PrevPrincipalPayDate = #6/1/2014#
        PrevInterestPayDate = #9/1/2014#
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipalAmount(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, PrinPayAmount, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalAmount : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalAmount.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ConstantPrincipalRate()
        Dim OutstandingAmount As Double
        Dim InterestBasis As InterestBasisType
        Dim InterestRate As Double
        Dim FreqPayPrincipal As Integer
        Dim FreqPayInterest As Integer
        Dim AmortizationRate As Double
        Dim MinimumPayment As Double
        Dim ReferenceDate As Date
        Dim PrevPrincipalPayDate As Date
        Dim PrevInterestPayDate As Date
        Dim StartDate As Date
        Dim FirstPrincipalPayDate As Date
        Dim FirstInterestPayDate As Date
        Dim PrincipalGracePeriodStartDate As Date
        Dim PrincipalGracePeriodEndDate As Date
        Dim InterestGracePeriodStartDate As Date
        Dim InterestGracePeriodEndDate As Date
        Dim result As ConstantPrincipalRate_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This is a 100,000 loan with quarterly payments of principal and interest and an interest rate of 3.0%. 
        'The principal payments are calculated as 13.0% of the outstanding principal balance, with a minimum payment of 2,500.00.
        OutstandingAmount = 100000
        InterestBasis = InterestBasisType.A365
        InterestRate = 0.03
        FreqPayPrincipal = 3
        FreqPayInterest = 3
        AmortizationRate = 0.13
        MinimumPayment = 2500
        ReferenceDate = #10/1/2014#
        PrevPrincipalPayDate = wct.NULL_DATE
        PrevInterestPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalRate : " & retStr.ToString())
        File.WriteAllText("ConstantPrincipalRate.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Modify the loan to have monthly payments of principal and semi-annual payments of interest
        FreqPayPrincipal = 1
        FreqPayInterest = 6

        result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalRate : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Modify the loan so that first interest payment isn't due until January of 2015 and the first principal payment isn't due until March of 2015.
        FirstPrincipalPayDate = #3/1/2015#
        FirstInterestPayDate = #1/1/2015#

        result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalRate : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'No payments in 2016
        PrincipalGracePeriodStartDate = #1/1/2016#
        PrincipalGracePeriodEndDate = #1/1/2017#
        InterestGracePeriodStartDate = #1/1/2016#
        InterestGracePeriodEndDate = #1/1/2017#

        result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalRate : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Modify the loan so that the principal payment frequency will be every 5 months and 
        'the first payment dates are calculated from the previous interest and previous principal payment dates.
        FreqPayPrincipal = 5
        ReferenceDate = #10/1/2014#
        PrevPrincipalPayDate = #6/1/2014#
        PrevInterestPayDate = #9/1/2014#
        StartDate = wct.NULL_DATE
        FirstPrincipalPayDate = wct.NULL_DATE
        FirstInterestPayDate = wct.NULL_DATE
        PrincipalGracePeriodStartDate = wct.NULL_DATE
        PrincipalGracePeriodEndDate = wct.NULL_DATE
        InterestGracePeriodStartDate = wct.NULL_DATE
        InterestGracePeriodEndDate = wct.NULL_DATE

        result = wct.ConstantPrincipalRate(OutstandingAmount, InterestBasis, InterestRate, FreqPayPrincipal, FreqPayInterest, AmortizationRate, MinimumPayment, ReferenceDate, PrevPrincipalPayDate, PrevInterestPayDate, StartDate, FirstPrincipalPayDate, FirstInterestPayDate, PrincipalGracePeriodStartDate, PrincipalGracePeriodEndDate, InterestGracePeriodStartDate, InterestGracePeriodEndDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("ConstantPrincipalRate : " & retStr.ToString())
        File.AppendAllText("ConstantPrincipalRate.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_CONSTPRINAMORT()
        Dim pv As Double
        Dim rate As Double
        Dim LoanDate As Date
        Dim NumPmtsPerYear As Integer
        Dim FirstPaymentDate As Date
        Dim DaysInYear As Integer
        Dim NumberOfPayments As Integer
        Dim LastPaymentNumber As Integer
        Dim FirstPrinPayNo As Integer
        Dim FV As Double
        Dim PPMT As Double
        Dim eom As Boolean
        Dim result As CONSTPRINAMORT_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'A 1,000,000 loan dated 2014-05-14 with 50 monthly payments commencing on 2014-06-15. The interest rate is 6.0%.
        pv = 1000000
        rate = 0.06
        LoanDate = #5/15/2014#
        NumPmtsPerYear = 12
        FirstPaymentDate = #6/15/2014#
        DaysInYear = 360
        NumberOfPayments = 50
        LastPaymentNumber = wct.NULL_INT
        FirstPrinPayNo = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        PPMT = wct.NULL_DOUBLE
        eom = True

        result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CONSTPRINAMORT : " & retStr.ToString())
        File.WriteAllText("CONSTPRINAMORT.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Same loan except the first payment date is changed to 2014-06-30
        FirstPaymentDate = #6/30/2014#

        result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CONSTPRINAMORT : " & retStr.ToString())
        File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example there are 26 payments per year and the principal is amortized as though the loan is maturing in 5 years, 
        'but it will be paid off, in full, in 2 years
        pv = 1000000
        rate = 0.06
        LoanDate = #5/15/2014#
        NumPmtsPerYear = 26
        FirstPaymentDate = #6/30/2014#
        DaysInYear = 365
        NumberOfPayments = 130
        LastPaymentNumber = 52
        FirstPrinPayNo = wct.NULL_INT
        FV = wct.NULL_DOUBLE
        PPMT = wct.NULL_DOUBLE
        eom = True

        result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CONSTPRINAMORT : " & retStr.ToString())
        File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example, no interest is due until the 14th installment and then the principal will be 
        'paid down by 25,000 with each payment with balance paid off at maturity.
        pv = 1000000
        rate = 0.06
        LoanDate = #5/15/2014#
        NumPmtsPerYear = 26
        FirstPaymentDate = #6/30/2014#
        DaysInYear = 365
        NumberOfPayments = 52
        LastPaymentNumber = wct.NULL_INT
        FirstPrinPayNo = 14
        FV = wct.NULL_DOUBLE
        PPMT = 25000
        eom = True

        result = wct.CONSTPRINAMORT(pv, rate, LoanDate, NumPmtsPerYear, FirstPaymentDate, DaysInYear, NumberOfPayments, LastPaymentNumber, FirstPrinPayNo, FV, PPMT, eom)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("CONSTPRINAMORT : " & retStr.ToString())
        File.AppendAllText("CONSTPRINAMORT.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_NPD()
        Dim SettleDate As Date
        Dim FirstPayDate As Date
        Dim PmtPerYr As Integer
        Dim NumPmts As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the next payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
        SettleDate = #11/1/2010#
        FirstPayDate = #9/15/2010#
        PmtPerYr = 12
        NumPmts = 60

        result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts)

        Console.WriteLine("NPD : " & result)         '11/15/2010
        File.WriteAllText("NPD.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #7/29/2009#
        PmtPerYr = 26

        result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPD : " & result)         '11/3/2010
        File.AppendAllText("NPD.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #3/15/2009#
        PmtPerYr = 24

        result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPD : " & result)         '10/31/2010
        File.AppendAllText("NPD.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the maturity date on a loan by entering a date far in the future.
        SettleDate = #12/31/2099#
        FirstPayDate = #3/15/2009#
        PmtPerYr = 26
        NumPmts = 78

        result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts)

        Console.WriteLine("NPD : " & result)         '2/26/2012
        File.AppendAllText("NPD.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
        SettleDate = #12/6/2010#
        FirstPayDate = #12/15/2010#
        PmtPerYr = 12

        result = wct.NPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPD : " & result)         '12/15/2010
        File.AppendAllText("NPD.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_NPNO()
        Dim SettleDate As Date
        Dim FirstPayDate As Date
        Dim PmtPerYr As Integer
        Dim NumPmts As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the next payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
        SettleDate = #11/1/2010#
        FirstPayDate = #9/15/2010#
        PmtPerYr = 12
        NumPmts = 60

        result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr, NumPmts)

        Console.WriteLine("NPNO : " & result)         '3
        File.WriteAllText("NPNO.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #7/29/2009#
        PmtPerYr = 26

        result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPNO : " & result)         '34
        File.AppendAllText("NPNO.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #3/15/2009#
        PmtPerYr = 24

        result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPNO : " & result)         '40
        File.AppendAllText("NPNO.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
        SettleDate = #12/6/2010#
        FirstPayDate = #12/15/2010#
        PmtPerYr = 12

        result = wct.NPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("NPNO : " & result)         '1
        File.AppendAllText("NPNO.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_PAYMENTPERIODS()
        Dim ReferenceDate As Date
        Dim PaymentFrequency As Integer
        Dim PrevPayDate As Date
        Dim StartDate As Date
        Dim FirstPayDate As Date
        Dim InterimGracePeriodStartDate As Date
        Dim InterimGracePeriodEndDate As Date
        Dim MaturityDate As Date
        Dim result As OutputRow_PAYMENTPERIODS

        '=====================
        ' EXAMPLE# 1
        '=====================

        ' loan with quarterly payments, a Reference Date of 2014-09-15 and a Maturity Date of 2019-09-15.
        ReferenceDate = #9/15/2014#
        PaymentFrequency = 3
        PrevPayDate = wct.NULL_DATE
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        InterimGracePeriodStartDate = wct.NULL_DATE
        InterimGracePeriodEndDate = wct.NULL_DATE
        MaturityDate = #9/15/2019#

        result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate)

        With result
            Console.WriteLine("InitialGracePeriod:" & .InitialGracePeriod)
            Console.WriteLine("InterimGracePeriodMonthStart : " & .InterimGracePeriodMonthStart)
            Console.WriteLine("InterimGracePeriodMonthEnd : " & .InterimGracePeriodMonthEnd)
            Console.WriteLine("MonthsUntilFirstPayment" & .MonthsUntilFirstPayment)
            Console.WriteLine("InterimGracePeriod : " & .InterimGracePeriod)
            Console.WriteLine("NumberOfPayments : " & .NumberOfPayments)
        End With
        File.WriteAllText("PAYMENTPERIODS.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Add a first payment date
        FirstPayDate = #3/15/2015#

        result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate)

        With result
            Console.WriteLine("InitialGracePeriod:" & .InitialGracePeriod)
            Console.WriteLine("InterimGracePeriodMonthStart : " & .InterimGracePeriodMonthStart)
            Console.WriteLine("InterimGracePeriodMonthEnd : " & .InterimGracePeriodMonthEnd)
            Console.WriteLine("MonthsUntilFirstPayment" & .MonthsUntilFirstPayment)
            Console.WriteLine("InterimGracePeriod : " & .InterimGracePeriod)
            Console.WriteLine("NumberOfPayments : " & .NumberOfPayments)
        End With
        File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'No payments in 2018
        InterimGracePeriodStartDate = #1/1/2018#
        InterimGracePeriodEndDate = #1/1/2019#

        result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate)

        With result
            Console.WriteLine("InitialGracePeriod:" & .InitialGracePeriod)
            Console.WriteLine("InterimGracePeriodMonthStart : " & .InterimGracePeriodMonthStart)
            Console.WriteLine("InterimGracePeriodMonthEnd : " & .InterimGracePeriodMonthEnd)
            Console.WriteLine("MonthsUntilFirstPayment" & .MonthsUntilFirstPayment)
            Console.WriteLine("InterimGracePeriod : " & .InterimGracePeriod)
            Console.WriteLine("NumberOfPayments : " & .NumberOfPayments)
        End With
        File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Clear out every date except Reference Date and Maturity Date and add a previous payment date.
        PrevPayDate = #8/15/2014#
        StartDate = wct.NULL_DATE
        FirstPayDate = wct.NULL_DATE
        InterimGracePeriodStartDate = wct.NULL_DATE
        InterimGracePeriodEndDate = wct.NULL_DATE

        result = wct.PAYMENTPERIODS(ReferenceDate, PaymentFrequency, PrevPayDate, StartDate, FirstPayDate, InterimGracePeriodStartDate, InterimGracePeriodEndDate, MaturityDate)

        With result
            Console.WriteLine("InitialGracePeriod:" & .InitialGracePeriod)
            Console.WriteLine("InterimGracePeriodMonthStart : " & .InterimGracePeriodMonthStart)
            Console.WriteLine("InterimGracePeriodMonthEnd : " & .InterimGracePeriodMonthEnd)
            Console.WriteLine("MonthsUntilFirstPayment" & .MonthsUntilFirstPayment)
            Console.WriteLine("InterimGracePeriod : " & .InterimGracePeriod)
            Console.WriteLine("NumberOfPayments : " & .NumberOfPayments)
        End With
        File.AppendAllText("PAYMENTPERIODS.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_PERIODRATE()
        Dim Nominal As Double
        Dim ComPeriod As Double
        Dim PayPeriod As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Nominal = 0.07
        ComPeriod = 2           'semi-annual compounding
        PayPeriod = 12          'monthly compounding

        result = wct.PERIODRATE(Nominal, ComPeriod, PayPeriod)

        Console.WriteLine("PERIODRATE : " & result)          '0.0690004739713013
        File.WriteAllText("PERIODRATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_PPD()
        Dim SettleDate As Date
        Dim FirstPayDate As Date
        Dim PmtPerYr As Integer
        Dim NumPmts As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the previous payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
        SettleDate = #11/1/2010#
        FirstPayDate = #9/15/2010#
        PmtPerYr = 12
        NumPmts = 60

        result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr, NumPmts)

        Console.WriteLine("PPD : " & result)         '10/15/2010
        File.WriteAllText("PPD.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the previous payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #7/29/2009#
        PmtPerYr = 26

        result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPD : " & result)         '10/20/2010
        File.AppendAllText("PPD.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the previous payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #3/15/2009#
        PmtPerYr = 24

        result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPD : " & result)         '10/15/2010
        File.AppendAllText("PPD.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the previous payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
        SettleDate = #12/6/2010#
        FirstPayDate = #12/15/2010#
        PmtPerYr = 12
        result = wct.PPD(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPD : " & If(wct.IsNull(result), "NULL", result.ToString()))         'NULL
        File.AppendAllText("PPD.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_PPNO()
        Dim SettleDate As Date
        Dim FirstPayDate As Date
        Dim PmtPerYr As Integer
        Dim NumPmts As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the previous payment date on 11/1/2010 for a loan that commenced on 8/31/2008, paying interest monthly on the 15th of the month, starting 9/15/2008.
        SettleDate = #11/1/2010#
        FirstPayDate = #9/15/2010#
        PmtPerYr = 12
        NumPmts = 60

        result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr, NumPmts)

        Console.WriteLine("PPNO : " & result)         '2
        File.WriteAllText("PPNO.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 7/15/2009, paying interest every two weeks starting on 7/29/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #7/29/2009#
        PmtPerYr = 26

        result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPNO : " & result)         '33
        File.AppendAllText("PPNO.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the next payment date on 10/29/2010 for a loan that commenced on 2/28/2009, paying interest every semi-monthly starting on 3/15/2009.
        SettleDate = #10/29/2010#
        FirstPayDate = #3/15/2009#
        PmtPerYr = 24

        result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPNO : " & result)         '39
        File.AppendAllText("PPNO.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate the next payment date on 12/6/2010 for a loan that commenced on 12/06/2010, paying interest monthly on the 15th of the month, starting 12/15/2010.
        SettleDate = #12/6/2010#
        FirstPayDate = #12/15/2010#
        PmtPerYr = 12

        result = wct.PPNO(SettleDate, FirstPayDate, PmtPerYr)

        Console.WriteLine("PPNO : " & result)         'NULL
        File.AppendAllText("PPNO.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_SMMAmort()
        Dim PrinAmt As Double
        Dim InterestRate As Double
        Dim NumPmts As Integer
        Dim LastPmtNum As Integer
        Dim PmtPerYr As Integer
        Dim InterestOnly As Boolean
        Dim PrinPaymentMultiple As Integer
        Dim FirstPrinPayNo As Integer
        Dim PmtPayPct As Double
        Dim retStr As StringBuilder
        Dim result As XLeratorDLL_financial.FinancialTypes.CDRCashflow_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim LSRates_array As System.Array = New Double(,) {{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim CPRRates_array As System.Array = New Double(,) {{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CDRRates_array As System.Array = New Double(,) {{1, 0.1}, {61, 0.08}, {181, 0.05}}

        'In this example we amortize a 100,000,000 loan with a 5% interest rate over 360 months 

        PrinAmt = 100000000
        InterestRate = 0.05
        NumPmts = 360
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 12
        InterestOnly = False
        PrinPaymentMultiple = wct.NULL_INT
        FirstPrinPayNo = wct.NULL_INT
        PmtPayPct = wct.NULL_DOUBLE

        result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SMMAmort : ex1 : " + retStr.ToString())
        File.WriteAllText("SMMAmort.csv", "ex1 : " + retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example, we have a 10-year, 20,000,000 loan, with monthly payments calculated as though it were a 25-year loan.

        PrinAmt = 20000000
        InterestRate = 0.05
        NumPmts = 300
        LastPmtNum = 120
        PmtPerYr = 12

        result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SMMAmort : ex2 : " + retStr.ToString())
        File.AppendAllText("SMMAmort.csv", vbCrLf + "ex2 : " + retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example we have a 10-year, 25,000,000 interest-only loan with quarterly payments

        PrinAmt = 25000000
        InterestRate = 0.04
        NumPmts = 40
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 4

        Dim LSRates_mo As Integer() = {1, 91, 181, 271, 361}                '{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim LSRates_rate As Double() = {0.15, 0.16, 0.14, 0.17, 0.13}       '{{1, 0.15}, {91, 0.16}, {181, 0.14}, {271, 0.17}, {361, 0.13}}
        Dim CPRRates_mo As Integer() = {1, 25, 61, 181}                     '{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CPRRates_rate As Double() = {0.002, 0.012, 0.02, 0.06}          '{{1, 0.002}, {25, 0.012}, {61, 0.02}, {181, 0.06}}
        Dim CDRRates_mo As Integer() = {1, 61, 181}                         '{{1, 0.1}, {61, 0.08}, {181, 0.05}}
        Dim CDRRates_rate As Double() = {0.1, 0.08, 0.05}                   '{{1, 0.1}, {61, 0.08}, {181, 0.05}}

        result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_mo, LSRates_rate, CPRRates_mo, CPRRates_rate, CDRRates_mo, CDRRates_rate, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SMMAmort : ex3 : " + retStr.ToString())
        File.AppendAllText("SMMAmort.csv", vbCrLf + "ex3 : " + retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example we have a 10,000,000 loan with monthly payments of interest and quarterly payments of principal. The loan matures in 120 months.

        PrinAmt = 10000000
        InterestRate = 0.06
        NumPmts = 120
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 12
        InterestOnly = False
        PrinPaymentMultiple = 3
        FirstPrinPayNo = wct.NULL_INT
        PmtPayPct = wct.NULL_DOUBLE

        result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SMMAmort : ex4 : " + retStr.ToString())
        File.AppendAllText("SMMAmort.csv", vbCrLf + "ex4 : " + retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'In this example we have a 10,000,000 loan with monthly payments of interest and quarterly payments of principal. The loan matures in 120 months.

        PrinAmt = 10000000
        InterestRate = 0.06
        NumPmts = 119
        LastPmtNum = wct.NULL_INT
        PmtPerYr = 12
        InterestOnly = False
        PrinPaymentMultiple = 3
        FirstPrinPayNo = 2
        PmtPayPct = wct.NULL_DOUBLE

        result = wct.SMMAmort(PrinAmt, InterestRate, NumPmts, LastPmtNum, PmtPerYr, LSRates_array, CPRRates_array, CDRRates_array, InterestOnly, PrinPaymentMultiple, FirstPrinPayNo, PmtPayPct)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If (i < result.Columns.Count - 1) Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SMMAmort : ex5 : " + retStr.ToString())
        File.AppendAllText("SMMAmort.csv", vbCrLf + "ex5 : " + retStr.ToString())

    End Sub

    Public Shared Sub test_UNEQUALLOANPAYMENTS()
        Dim pv As Double
        Dim rate As Double
        Dim loandate As Date
        Dim interestfrequency As Integer
        Dim firstpaymentdate As Date
        Dim daysinyear As Integer
        Dim prinpaymentmultiple As Integer
        Dim firstprinpayno As Integer
        Dim numberofpayments As Integer
        Dim lastpaymentnumber As Integer
        Dim fv As Double
        Dim IsRegPay As Boolean
        Dim result As UNEQUALLOANPAYMENTS_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'A 1,000,000 loan dated 2014-01-15 with monthly payments commencing on 2014-02-15 and 
        'quarterly principal payments commencing with the third interest payment. 
        'Notice that the quarterly payments are constant at 130,690.30 but 
        'that interest payments very from quarter to quarter though they are the same for each month between principal payments.
        pv = 1000000
        rate = 0.12
        loandate = #1/15/2014#
        interestfrequency = 12
        firstpaymentdate = #2/15/2014#
        daysinyear = wct.NULL_INT
        prinpaymentmultiple = 3
        firstprinpayno = 3
        numberofpayments = 24
        lastpaymentnumber = wct.NULL_INT
        fv = 0
        IsRegPay = True

        result = wct.UNEQUALLOANPAYMENTS(pv, rate, loandate, interestfrequency, firstpaymentdate, daysinyear, prinpaymentmultiple, firstprinpayno, numberofpayments, lastpaymentnumber, fv, IsRegPay)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("UNEQUALLOANPAYMENTS : " & retStr.ToString())
        File.WriteAllText("UNEQUALLOANPAYMENTS.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'This loan, has monthly payments and is interest only for the first 6 months, with payments calculated assuming a 25-year maturity but payoff is at the end of 5 years.
        pv = 1000000
        rate = 0.05
        loandate = #1/15/2014#
        interestfrequency = 12
        firstpaymentdate = #2/15/2014#
        daysinyear = wct.NULL_INT
        prinpaymentmultiple = 1
        firstprinpayno = 7
        numberofpayments = 300
        lastpaymentnumber = 60
        fv = 0
        IsRegPay = True

        result = wct.UNEQUALLOANPAYMENTS(pv, rate, loandate, interestfrequency, firstpaymentdate, daysinyear, prinpaymentmultiple, firstprinpayno, numberofpayments, lastpaymentnumber, fv, IsRegPay)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("UNEQUALLOANPAYMENTS : " & retStr.ToString())
        File.AppendAllText("UNEQUALLOANPAYMENTS.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_R78IPMT()
        Dim LoanAmount As Double
        Dim LoanStartDate As Date
        Dim Rate As Double
        Dim FirstPaymentDate As Date
        Dim NumPmts As Integer
        Dim PmtPerYear As Integer
        Dim NumPeriod As Integer
        Dim DaysInYear As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
        LoanAmount = 50000
        LoanStartDate = #11/1/2010#
        Rate = 0.06
        FirstPaymentDate = #12/1/2010#
        NumPmts = 60
        PmtPerYear = 12
        NumPeriod = 1
        DaysInYear = wct.NULL_INT
        FV = 0
        IntRule = "A"

        result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78IPMT : " & result)         '262.242773386387
        File.WriteAllText("R78IPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last interest payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
        LoanAmount = 50000
        LoanStartDate = #11/1/2010#
        Rate = 0.06
        FirstPaymentDate = #12/1/2010#
        NumPmts = 60
        PmtPerYear = 12
        NumPeriod = 60
        DaysInYear = wct.NULL_INT
        FV = 0
        IntRule = "A"

        result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78IPMT : " & result)         '4.37071288977312
        File.AppendAllText("R78IPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the first interest payment for a 150,000 4-year loan at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January and with payments due every 4 weeks after that.
        LoanAmount = 150000
        LoanStartDate = #10/13/2010#
        Rate = 0.12
        FirstPaymentDate = #1/4/2011#
        NumPmts = 52
        PmtPerYear = 13
        NumPeriod = 1
        DaysInYear = 365
        FV = 0
        IntRule = "U"

        result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78IPMT : " & result)         '1615.88014621541
        File.AppendAllText("R78IPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.R78IPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78IPMT : " & result)         '1616.04600080377
        File.AppendAllText("R78IPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_R78PPMT()
        Dim LoanAmount As Double
        Dim LoanStartDate As Date
        Dim Rate As Double
        Dim FirstPaymentDate As Date
        Dim NumPmts As Integer
        Dim PmtPerYear As Integer
        Dim NumPeriod As Integer
        Dim DaysInYear As Integer
        Dim FV As Double
        Dim IntRule As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the first principal payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
        LoanAmount = 50000
        LoanStartDate = #11/1/2010#
        Rate = 0.06
        FirstPaymentDate = #12/1/2010#
        NumPmts = 60
        PmtPerYear = 12
        NumPeriod = 1
        DaysInYear = wct.NULL_INT
        FV = 0
        IntRule = "A"

        result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78PPMT : " & result)         '704.397303085026
        File.WriteAllText("R78PPMT.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Calculate the last principal payment for a 50,000, 5-year loan starting on 1 November with payments due on the first of every month. The rate on the loan is 6 per cent.
        LoanAmount = 50000
        LoanStartDate = #11/1/2010#
        Rate = 0.06
        FirstPaymentDate = #12/1/2010#
        NumPmts = 60
        PmtPerYear = 12
        NumPeriod = 60
        DaysInYear = wct.NULL_INT
        FV = 0
        IntRule = "A"

        result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78PPMT : " & result)         '962.26936358164
        File.AppendAllText("R78PPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate the first principal payment for a 150,000 4-year loan at 12 percent interest using the US Rule made on 13 October with no payments due until 4 January and with payments due every 4 weeks after that.
        LoanAmount = 150000
        LoanStartDate = #10/13/2010#
        Rate = 0.12
        FirstPaymentDate = #1/4/2011#
        NumPmts = 52
        PmtPerYear = 13
        NumPeriod = 1
        DaysInYear = 365
        FV = 0
        IntRule = "U"

        result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78PPMT : " & result)         '2092.2126206059
        File.AppendAllText("R78PPMT.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Let’s look at the same interest payment, but this time we will specify the actuarial method as the interest rule.
        IntRule = "A"

        result = wct.R78PPMT(LoanAmount, LoanStartDate, Rate, FirstPaymentDate, NumPmts, PmtPerYear, NumPeriod, DaysInYear, FV, IntRule)

        Console.WriteLine("R78PPMT : " & result)         '2092.13128806738
        File.AppendAllText("R78PPMT.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_R78PAYOFF()
        Dim LoanAmount As Double
        Dim FV As Double
        Dim IntAmt As Double
        Dim NumPmts As Integer
        Dim PeriodNo As Integer
        Dim pmt As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the payoff amount, using the Rule-of-78, for a 24,000 loan that originated on 8/15/2009 at a rate of 5.9%, with payments on the first of every month for 36 months, commencing on 9/1/2009, paid off on 12/1/2010. The monthly payment is 727.37
        NumPmts = 36
        PeriodNo = 16
        pmt = 727.37
        LoanAmount = 24000
        IntAmt = wct.TOTALINT(NumPmts, pmt, LoanAmount, FV)

        result = wct.R78PAYOFF(IntAmt, NumPmts, PeriodNo, pmt)

        Console.WriteLine("R78PAYOFF : " & result)           '14585.7051351351
        File.WriteAllText("R78PAYOFF.csv", result.ToString())

    End Sub

    Public Shared Sub test_R78REBATE()
        Dim IntAmt As Double
        Dim NumPmts As Integer
        Dim PeriodNo As Integer
        Dim pmt As Double
        Dim prin As Double
        Dim fv As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Calculate the rebate amount, using the Rule of 78, for a 24,000 loan that originated on 8/15/2009 at a rate of 5.9%, with payments on the first of every month for 36 months, commencing on 9/1/2009, paid off on 12/1/2010. The monthly payment is 727.37
        prin = 24000
        pmt = 727.37
        NumPmts = 36
        PeriodNo = 16
        IntAmt = wct.TOTALINT(NumPmts, pmt, prin, fv)

        result = wct.R78REBATE(IntAmt, NumPmts, PeriodNo)

        Console.WriteLine("R78REBATE : " & result)           '689.064864864865
        File.WriteAllText("R78REBATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_DB()
        Dim Cost As Double
        Dim Salvage As Double
        Dim Life As Double
        Dim Per As Integer
        Dim MonthsInFirstYear As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Cost = 60000
        Salvage = 0
        Life = 4
        Per = 1
        MonthsInFirstYear = 3

        result = wct.DB(Cost, Salvage, Life, Per, MonthsInFirstYear)

        Console.WriteLine("DB : " & result)          '15000
        File.WriteAllText("DB.csv", result.ToString())

    End Sub

    Public Shared Sub test_DDB()
        Dim Cost As Double
        Dim Salvage As Double
        Dim Life As Double
        Dim Per As Integer
        Dim Factor As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Cost = 600000
        Salvage = 0
        Life = 4
        Per = 1
        Factor = 2.25

        result = wct.DDB(Cost, Salvage, Life, Per, Factor)

        Console.WriteLine("DDB : " & result)         '337500
        File.WriteAllText("DDB.csv", result.ToString())

    End Sub

    Public Shared Sub test_SLN()
        Dim Cost As Double
        Dim Salvage As Double
        Dim Life As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Cost = 600000
        Salvage = 25000
        Life = 40

        result = wct.SLN(Cost, Salvage, Life)

        Console.WriteLine("SLN : " & result)         '14375
        File.WriteAllText("SLN.csv", result.ToString())

    End Sub

    Public Shared Sub test_SYD()
        Dim Cost As Double
        Dim Salvage As Double
        Dim Life As Double
        Dim Per As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Cost = 30000
        Salvage = 7500
        Life = 10
        Per = 1

        result = wct.SYD(Cost, Salvage, Life, Per)

        Console.WriteLine("SYD : " & result)         '4090.90909090909
        File.WriteAllText("SYD.csv", result.ToString())

    End Sub

    Public Shared Sub test_VDB()
        Dim Cost As Double
        Dim Salvage As Double
        Dim Life As Double
        Dim start_period As Double
        Dim end_period As Double
        Dim Factor As Double
        Dim No_switch As Boolean
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Cost = 2400
        Salvage = 300
        Life = 10
        start_period = 6
        end_period = 7
        Factor = 1.5
        No_switch = True

        result = wct.VDB(Cost, Salvage, Life, start_period, end_period, Factor, No_switch)

        Console.WriteLine("VDB : " & result)         '135.773825625
        File.WriteAllText("VDB.csv", result.ToString())

    End Sub

    Public Shared Sub test_DFINTERP()
        Dim dfdate As Date()
        Dim df As Double()
        Dim newdate As Date
        Dim startdate As Date
        Dim RV As ZCReturnValues
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        dfdate = New Date() {#1/16/2013#, #1/17/2013#, #1/24/2013#, #1/31/2013#, #2/18/2013#, #3/18/2013#, #4/17/2013#, #6/19/2013#, #9/18/2013#, #12/18/2013#, #3/19/2014#, #6/18/2014#, #9/17/2014#, #12/17/2014#, #1/19/2015#, #1/18/2016#, #1/17/2017#, #1/17/2018#, #1/17/2020#, #1/17/2023#, #1/19/2043#}
        df = New Double() {0.999995555575309, 0.99999111117037, 0.999956112706425, 0.999916450742048, 0.999804481000583, 0.999574621744643, 0.999241679910437, 0.998800609148515, 0.998022836090921, 0.997197057207847, 0.996311568695976, 0.995354720378904, 0.994289565586446, 0.993104681356623, 0.992402694592988, 0.98508478388398, 0.973098042807202, 0.955265832115111, 0.906604451702898, 0.820620615064395, 0.385646181323946}
        newdate = #3/20/2013#
        startdate = #1/15/2013#
        'return the interpolated discount factor
        RV = ZCReturnValues.DF

        result = wct.DFINTERP(dfdate, df, newdate, startdate, RV)

        Console.WriteLine("DFINTERP : " & result)            '0.999555003248605
        File.WriteAllText("DFINTERP.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Discount Factor
        newdate = #6/30/2013#
        RV = ZCReturnValues.DF

        result = wct.DFINTERP(dfdate, df, newdate, startdate, RV)

        Console.WriteLine("DFINTERP : " & result)            '0.998709482483852
        File.AppendAllText("DFINTERP.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Zero COupon
        RV = ZCReturnValues.Zero

        result = wct.DFINTERP(dfdate, df, newdate, startdate, RV)

        Console.WriteLine("DFINTERP : " & result)            '0.00283941624767274
        File.AppendAllText("DFINTERP.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Continuously compounded zero coupon
        RV = ZCReturnValues.CCZero

        result = wct.DFINTERP(dfdate, df, newdate, startdate, RV)

        Console.WriteLine("DFINTERP : " & result)            '0.00284125037852567
        File.AppendAllText("DFINTERP.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ED_FUT_CONV_ADJ_HL()
        Dim Price As Double
        Dim Vol As Double
        Dim T1 As Double
        Dim T2 As Double
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Price = 99.675
        Vol = 0.0008
        T1 = wct.ED_FUTYF("M3", #3/25/2013#, Nothing)
        T2 = wct.NULL_DOUBLE

        result = wct.ED_FUT_CONV_ADJ_HL(Price, Vol, T1, T2)

        Console.WriteLine("D_FUT_CONV_ADJ_HL : " & result)           '0.003294
        File.WriteAllText("D_FUT_CONV_ADJ_HL.csv", result.ToString())

    End Sub

    Public Shared Sub test_INTERPDFACT()
        Dim dfdate As Date()
        Dim df As Double()
        Dim iStartDate As Date
        Dim iEndDate As Date
        Dim StartDate As Date
        Dim result As INTERPDFACT_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        dfdate = New Date() {#1/16/2013#, #1/17/2013#, #1/24/2013#, #1/31/2013#, #2/18/2013#, #3/18/2013#, #4/17/2013#, #6/19/2013#, #9/18/2013#, #12/18/2013#, #3/19/2014#, #6/18/2014#, #9/17/2014#, #12/17/2014#, #1/19/2015#, #1/18/2016#, #1/17/2017#, #1/17/2018#, #1/17/2020#, #1/17/2023#, #1/19/2043#}
        df = New Double() {0.999995555575309, 0.99999111117037, 0.999956112706425, 0.999916450742048, 0.999804481000583, 0.999574621744643, 0.999241679910437, 0.998800609148515, 0.998022836090921, 0.997197057207847, 0.996311568695976, 0.995354720378904, 0.994289565586446, 0.993104681356623, 0.992402694592988, 0.98508478388398, 0.973098042807202, 0.955265832115111, 0.906604451702898, 0.820620615064395, 0.385646181323946}
        iStartDate = #1/16/2013#
        iEndDate = #2/13/2013#
        StartDate = #1/15/2013#

        result = wct.INTERPDFACT(dfdate, df, iStartDate, iEndDate, StartDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("INTERPDFACT : " & retStr.ToString())
        File.WriteAllText("INTERPDFACT.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_SWAPCURVE()
        Dim mDate As Date()
        Dim cRate As Double()
        Dim iType As String()
        Dim StartDate As Date
        Dim Frequency As Integer
        Dim SpotDate As Date
        Dim CashBasis As String
        Dim FuturesBasis As String
        Dim SwapsBasis As String
        Dim InterpMethod As String
        Dim DateRoll As String
        Dim result As SWAPCURVE_table
        Dim retStr As StringBuilder
        Dim curve As Object(,)
        Dim futures As Object(,)

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #3/7/2013#
        SpotDate = #3/11/2013#
        Frequency = 2
        CashBasis = "2"
        FuturesBasis = "2"
        SwapsBasis = "0"
        InterpMethod = "L"
        DateRoll = "M"
        curve = New Object(,) {{"1M", "3M", "6M", "1Y", "2Y", "3Y", "4Y", "5Y", "7Y", "10Y", "30Y"}, {0.0023, 0.0028, 0.0044, 0.0031, 0.0039, 0.0054, 0.0074, 0.01, 0.015, 0.0207, 0.0304}, {"C", "C", "C", "S", "S", "S", "S", "S", "S", "S", "S"}}

        'Convert the tenor codes into dates
        For i As Integer = 0 To curve.GetUpperBound(1)
            curve(0, i) = wct.TENOR2DATE(curve(0, i).ToString(), StartDate, SpotDate)
        Next

        result = wct.SWAPCURVE(curve, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SWAPCURVE : " & retStr.ToString())
        File.WriteAllText("SWAPCURVE.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Use one of the overloads
        ReDim mDate(curve.GetUpperBound(1))
        ReDim cRate(curve.GetUpperBound(1))
        ReDim iType(curve.GetUpperBound(1))
        For i As Integer = 0 To curve.GetUpperBound(1)
            mDate(i) = DirectCast(curve(0, i), Date)
            cRate(i) = DirectCast(curve(1, i), Double)
            iType(i) = DirectCast(curve(2, i), String)
        Next

        result = wct.SWAPCURVE(mDate, cRate, iType, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SWAPCURVE : " & retStr.ToString())
        File.AppendAllText("SWAPCURVE.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Add the futures to the existing curve

        futures = New Object(,) {{"M3", "U3", "Z3", "H4", "M4", "U4", "Z4"}, {99.705, 99.685, 99.645, 99.61, 99.56, 99.495, 99.405}, {"F", "F", "F", "F", "F", "F", "F"}}
        Dim r As Integer = curve.GetUpperBound(1) + 1
        ReDim Preserve curve(curve.GetUpperBound(0), curve.GetUpperBound(1) + futures.GetUpperBound(1) + 1)
        For i As Integer = 0 To futures.GetUpperBound(1)
            curve(0, i + r) = wct.ED_FUT2DATE(futures(0, i).ToString(), StartDate)
            curve(1, i + r) = (100 - DirectCast(futures(1, i), Double)) / 100
            curve(2, i + r) = futures(2, i)
        Next

        result = wct.SWAPCURVE(curve, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SWAPCURVE : " & retStr.ToString())
        File.AppendAllText("SWAPCURVE.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Use one of the overloads
        ReDim mDate(curve.GetUpperBound(1))
        ReDim cRate(curve.GetUpperBound(1))
        ReDim iType(curve.GetUpperBound(1))
        For i As Integer = 0 To curve.GetUpperBound(1)
            mDate(i) = DirectCast(curve(0, i), Date)
            cRate(i) = DirectCast(curve(1, i), Double)
            iType(i) = DirectCast(curve(2, i), String)
        Next

        result = wct.SWAPCURVE(mDate, cRate, iType, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("SWAPCURVE : " & retStr.ToString())
        File.AppendAllText("SWAPCURVE.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_ZEROCOUPON()
        Dim vDate As Date
        Dim RV As String
        Dim StartDate As Date
        Dim Frequency As Integer
        Dim SpotDate As Date
        Dim CashBasis As String
        Dim FuturesBasis As String
        Dim SwapsBasis As String
        Dim InterpMethod As String
        Dim DateRoll As String
        Dim result As Double
        Dim curve As Object(,)
        Dim futures As Object(,)

        '=====================
        ' EXAMPLE# 1
        '=====================

        vDate = #9/11/2013#
        RV = "ZC"
        StartDate = #3/7/2013#
        Frequency = 2
        SpotDate = #3/11/2013#
        CashBasis = "2"
        FuturesBasis = "2"
        SwapsBasis = "0"
        InterpMethod = "L"
        DateRoll = "MP"
        curve = New Object(,) {{"1M", "3M", "6M", "1Y", "2Y", "3Y", "4Y", "5Y", "7Y", "10Y", "30Y"}, {0.0023, 0.0028, 0.0044, 0.0031, 0.0039, 0.0054, 0.0074, 0.01, 0.015, 0.0207, 0.0304}, {"C", "C", "C", "S", "S", "S", "S", "S", "S", "S", "S"}}

        'Convert the tenor codes into dates
        For i As Integer = 0 To curve.GetUpperBound(1)
            curve(0, i) = wct.TENOR2DATE(curve(0, i).ToString(), StartDate, SpotDate)
        Next

        result = wct.ZEROCOUPON(curve, vDate, RV, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        Console.WriteLine("ZEROCOUPON : " & result)          '0.00441592127318121
        File.WriteAllText("ZEROCOUPON.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Add the futures to the existing curve

        futures = New Object(,) {{"M3", "U3", "Z3", "H4", "M4", "U4", "Z4"}, {99.705, 99.685, 99.645, 99.61, 99.56, 99.495, 99.405}, {"F", "F", "F", "F", "F", "F", "F"}}
        Dim r As Integer = curve.GetUpperBound(1) + 1
        ReDim Preserve curve(curve.GetUpperBound(0), curve.GetUpperBound(1) + futures.GetUpperBound(1) + 1)
        For i As Integer = 0 To futures.GetUpperBound(1)
            curve(0, i + r) = wct.ED_FUT2DATE(futures(0, i).ToString(), StartDate)
            curve(1, i + r) = (100 - DirectCast(futures(1, i), Double)) / 100
            curve(2, i + r) = futures(2, i)
        Next

        result = wct.ZEROCOUPON(curve, vDate, RV, StartDate, Frequency, SpotDate, CashBasis, FuturesBasis, SwapsBasis, InterpMethod, DateRoll)

        Console.WriteLine("ZEROCOUPON : " & result)          '0.00296280638269212
        File.AppendAllText("ZEROCOUPON.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_NELSONSIEGEL()
        Dim Maturity As Double
        Dim T As Double()
        Dim R As Double()
        Dim coefficients As NScoef
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Maturity = 1
        T = New Double() {1, 2, 3, 5, 7, 10, 20, 30}
        R = New Double() {0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406}
        coefficients = XLeratorDLL_financial.XLeratorDLL_financial.NSCOEF(T, R)

        With coefficients
            result = wct.NELSONSIEGEL(Maturity, .B0, .B1, .B2, .Tau)
        End With

        Console.WriteLine("NELSONSIEGEL : " & result)            '0.00298376016908077
        File.WriteAllText("NELSONSIEGEL.csv", result.ToString())

    End Sub

    Public Shared Sub test_NSCOEF()
        Dim T As Double()
        Dim R As Double()
        Dim result As NScoef

        '=====================
        ' EXAMPLE# 1
        '=====================

        T = New Double() {1, 2, 3, 5, 7, 10, 20, 30}
        R = New Double() {0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406}

        result = wct.NSCOEF(T, R)

        With result
            Console.WriteLine("NSCOEF - B0 : " & .B0)            '0.0466701067626488
            Console.WriteLine("NSCOEF - B1 : " & .B1)            '-0.0409398575156674
            Console.WriteLine("NSCOEF - B2 : " & .B2)            '-0.0635075128703469
            Console.WriteLine("NSCOEF - Tau : " & .Tau)          '1.6545
            Console.WriteLine("NSCOEF - RMSE : " & .RMSE)        '9.8912932859058E-07
        End With
        File.WriteAllText("NSCOEF.csv", result.ToString())

    End Sub

    Public Shared Sub test_NSCOEF2()
        Dim T As Double()
        Dim R As Double()
        Dim NumSteps As Integer
        Dim Tau_min As Double
        Dim Tau_max As Double
        Dim B0_min As Double
        Dim B0_max As Double
        Dim B1_min As Double
        Dim B1_max As Double
        Dim B2_min As Double
        Dim B2_max As Double
        Dim result As NScoef

        '=====================
        ' EXAMPLE# 1
        '=====================

        T = New Double() {1, 2, 3, 5, 7, 10, 20, 30}
        R = New Double() {0.0028, 0.0056, 0.0085, 0.0164, 0.0235, 0.0299, 0.0382, 0.0406}
        NumSteps = 900
        Tau_min = 0.5
        Tau_max = 9.5
        B0_min = -0.5
        B0_max = 0.15
        B1_min = -0.15
        B1_max = 0.3
        B2_min = -0.3
        B2_max = 0.3

        result = wct.NSCOEF2(T, R, NumSteps, Tau_min, Tau_max, B0_min, B0_max, B1_min, B1_max, B2_min, B2_max)

        With result
            Console.WriteLine("NSCOEF2 - B0 : " & .B0)            '0.0466701067626488
            Console.WriteLine("NSCOEF2 - B1 : " & .B1)            '-0.0409398575156674
            Console.WriteLine("NSCOEF2 - B2 : " & .B2)            '-0.0635075128703469
            Console.WriteLine("NSCOEF2 - Tau : " & .Tau)          '1.6545
            Console.WriteLine("NSCOEF2 - RMSE : " & .RMSE)        '9.8912932859058E-07
        End With
        File.WriteAllText("NSCOEF2.csv", result.ToString())

    End Sub

    Public Shared Sub test_ED_FUTYF()
        Dim DeliveryCode As String
        Dim StartDate As Date
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        DeliveryCode = "M3"
        StartDate = #3/25/2013#
        Basis = "2"

        result = wct.ED_FUTYF(DeliveryCode, StartDate, Basis)

        Console.WriteLine("ED_FUTYF : " & result)            '0.238888888888889
        File.WriteAllText("ED_FUTYF.csv", result.ToString())

    End Sub

    Public Shared Sub test_ED_FUT2DATE()
        Dim DeliveryCode As String
        Dim StartDate As Date
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        DeliveryCode = "M3"
        StartDate = #3/25/2013#

        result = wct.ED_FUT2DATE(DeliveryCode, StartDate)

        Console.WriteLine("ED_FUT2DATE : " & result)         '6/19/2013
        File.WriteAllText("ED_FUT2DATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_TENOR2DATE()
        Dim Tenors As String()
        Dim startDate As Date
        Dim spotDate As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        Tenors = New String() {"ON", "TN", "SN", "1W", "2W", "1M", "3M", "6M", "1Y", "2Y", "5Y", "10Y"}
        startDate = #3/25/2013#
        spotDate = wct.NULL_DATE
        File.WriteAllText("TENOR2DATE.tsv", "")
        For i As Integer = 0 To Tenors.GetUpperBound(0)
            Console.WriteLine("TENOR2DATE : " & Tenors(i) & vbTab & wct.TENOR2DATE(Tenors(i), startDate, spotDate))
            File.AppendAllText("TENOR2DATE.tsv", Tenors(i) & vbTab & wct.TENOR2DATE(Tenors(i), startDate, spotDate).ToString() & vbCrLf)
        Next

        '=====================
        ' EXAMPLE# 2
        '=====================

        'Add some holidays
        Dim Holidays As New List(Of Date)
        For i As Integer = 2013 To 2023
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 1, 1), "D", 0, "G"))        'New Year's Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 1, 1), "Mon").AddDays(14))  'MLK Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 2, 1), "Mon").AddDays(14))  'President's Day
            Holidays.Add(wct.LASTWEEKDAY(DateSerial(i, 5, 1), "Mon"))               'Memorial Day
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 7, 4), "D", 0, "G"))        'Independence Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 9, 1), "Mon"))              'Labor Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 10, 1), "Mon").AddDays(7)) 'Columbus Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 11, 1), "Thu").AddDays(21)) 'Thanksgiving
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 12, 25), "D", 0, "G"))      'Christmas Day
        Next
        startDate = #3/27/2013#

        For i As Integer = 0 To Tenors.GetUpperBound(0)
            Console.WriteLine("TENOR2DATE 2 : " & Tenors(i) & vbTab & wct.TENOR2DATE(Tenors(i), startDate, spotDate, Holidays))
            File.AppendAllText("TENOR2DATE.tsv", Tenors(i) & vbTab & wct.TENOR2DATE(Tenors(i), startDate, spotDate, Holidays).ToString() & vbCrLf)
        Next

    End Sub

    Public Shared Sub test_BUSDAYS()
        Dim Holidays As List(Of Date)
        Dim StartDate As Date()
        Dim EndDate As Date()

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = New Date() {#8/1/2015#, #11/1/2014#, #3/16/2010#, #10/21/2009#, #7/22/2014#, #2/3/2013#, #4/26/2013#, #1/7/2013#, #8/4/2015#, #12/16/2013#, #1/2/2015#, #12/2/2009#, #6/14/2014#, #1/15/2010#, #1/19/2015#, #9/20/2014#, #11/8/2009#, #7/28/2011#, #11/7/2010#, #1/8/2013#, #7/12/2012#, #7/3/2012#}
        EndDate = New Date() {#11/19/2014#, #4/12/2015#, #12/12/2015#, #11/24/2011#, #10/13/2015#, #2/27/2012#, #1/23/2015#, #6/7/2011#, #9/23/2010#, #8/25/2015#, #8/7/2013#, #10/7/2011#, #10/23/2015#, #3/13/2015#, #6/7/2010#, #12/31/2012#, #9/1/2014#, #9/24/2015#, #6/8/2011#, #11/26/2014#, #2/4/2015#, #8/30/2012#}

        Holidays = New List(Of Date)
        For i As Integer = 2010 To 2020
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 1, 1), "D", 0, "G"))        'New Year's Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 1, 1), "Mon").AddDays(14))  'MLK Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 2, 1), "Mon").AddDays(14))  'President's Day
            Holidays.Add(wct.LASTWEEKDAY(DateSerial(i, 5, 1), "Mon"))               'Memorial Day
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 7, 4), "D", 0, "G"))        'Independence Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 9, 1), "Mon"))              'Labor Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 10, 1), "Mon").AddDays(7))  'Columbus Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 11, 1), "Thu").AddDays(21)) 'Thanksgiving
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 12, 25), "D", 0, "G"))      'Christmas Day
        Next

        File.WriteAllText("BUSDAYS.tsv", "")
        For i As Integer = 0 To StartDate.GetUpperBound(0)
            Console.WriteLine("BUSDAYS : " & StartDate(i) & vbTab & EndDate(i) & vbTab & wct.BUSDAYS(StartDate(i), EndDate(i), Holidays) & vbTab & (EndDate(i) - StartDate(i)).TotalDays)
            File.AppendAllText("BUSDAYS.tsv", StartDate(i) & vbTab & EndDate(i) & vbTab & wct.BUSDAYS(StartDate(i), EndDate(i), Holidays) & vbTab & (EndDate(i) - StartDate(i)).TotalDays & vbCrLf)
        Next

    End Sub

    Public Shared Sub test_BUSDAYSWE()
        Dim Holidays As Date()
        Dim StartDate As Date
        Dim EndDate As Date
        Dim WE1 As Integer
        Dim WE2 As Integer
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        'In this example we calculate the number of business days from Wednesday to Wednesday assuming that the only weekend day is Sunday.
        StartDate = #5/1/2014#
        EndDate = #5/8/2014#
        WE1 = 0

        result = wct.BUSDAYSWE(StartDate, EndDate, "", WE1, wct.NULL_INT)

        Console.WriteLine("BUSDAYSWE : " & result)       '6
        File.WriteAllText("BUSDAYSWE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example, we add some holidays, and we calculate the number number of business days in the month of May, assuming that the only weekend day is Sunday.
        Holidays = New Date() {#1/1/2014#, #1/20/2014#, #2/17/2014#, #5/26/2014#, #7/4/2014#, #9/1/2014#, #10/13/2014#, #11/11/2014#, #11/27/2014#, #12/25/2014#}
        StartDate = #5/1/2014#
        EndDate = #6/1/2014#

        result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, wct.NULL_INT)

        Console.WriteLine("BUSDAYSWE : " & result)          '26
        File.AppendAllText("BUSDAYSWE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'In this example, we calculate the number of business days for each month in 2014 assuming that the only weekend day is Sunday.
        For i As Integer = 1 To 12
            StartDate = DateSerial(2014, i, 1)
            EndDate = StartDate.AddMonths(1)

            result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, wct.NULL_INT)

            Console.WriteLine(MonthName(StartDate.Month, False) & vbTab & result)
            File.AppendAllText("BUSDAYSWE.csv", vbCrLf & MonthName(StartDate.Month, False) & vbTab & result)
        Next

        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example, the weekend days are Monday and Tuesday.
        WE1 = 1
        WE2 = 2
        For i As Integer = 1 To 12
            StartDate = DateSerial(2014, i, 1)
            EndDate = StartDate.AddMonths(1)

            result = wct.BUSDAYSWE(StartDate, EndDate, Holidays, WE1, WE2)

            Console.WriteLine(MonthName(StartDate.Month, False) & vbTab & result)
            File.AppendAllText("BUSDAYSWE.csv", vbCrLf & MonthName(StartDate.Month, False) & vbTab & result)
        Next

    End Sub

    Public Shared Sub test_BUSINESSDATE()
        Dim Holidays As List(Of Date)
        Dim StartDate As Date
        Dim DatePart As String
        Dim Num As Integer
        Dim DateRollRule As String
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Create some holidays
        Holidays = New List(Of Date)
        For i As Integer = 2010 To 2020
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 1, 1), "D", 0, "G"))        'New Year's Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 1, 1), "Mon").AddDays(14))  'MLK Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 2, 1), "Mon").AddDays(14))  'President's Day
            Holidays.Add(wct.LASTWEEKDAY(DateSerial(i, 5, 1), "Mon"))               'Memorial Day
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 7, 4), "D", 0, "G"))        'Independence Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 9, 1), "Mon"))              'Labor Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 10, 1), "Mon").AddDays(7))  'Columbus Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 11, 1), "Thu").AddDays(21)) 'Thanksgiving
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 12, 25), "D", 0, "G"))      'Christmas Day
        Next

        'In this example, we calculate 2 business days from 2012-05-04.
        StartDate = #5/4/2012#
        DatePart = "D"
        Num = 2
        DateRollRule = "F"

        result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays)

        Console.WriteLine("BUSINESSDATE : " & result)            '5/8/2012
        File.WriteAllText("BUSINESSDATE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'calculate 5 business days from 2012-05-21
        StartDate = #5/21/2012#
        Num = 5

        result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays)

        Console.WriteLine("BUSINESSDATE : " & result)            '5/29/2012
        File.AppendAllText("BUSINESSDATE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Calculate 3 months from 2012-02-28
        StartDate = #2/28/2012#
        DatePart = "M"
        Num = 3

        result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays)

        Console.WriteLine("BUSINESSDATE : " & result)            '5/29/2012
        File.AppendAllText("BUSINESSDATE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_BUSINESSDATEWE()
        Dim Holidays As List(Of Date)
        Dim StartDate As Date
        Dim DatePart As String
        Dim Num As Integer
        Dim DateRollRule As String
        Dim WE1 As Integer
        Dim WE2 As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Create some holidays
        Holidays = New List(Of Date)
        For i As Integer = 2010 To 2020
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 1, 1), "D", 0, "G"))        'New Year's Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 1, 1), "Mon").AddDays(14))  'MLK Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 2, 1), "Mon").AddDays(14))  'President's Day
            Holidays.Add(wct.LASTWEEKDAY(DateSerial(i, 5, 1), "Mon"))               'Memorial Day
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 7, 4), "D", 0, "G"))        'Independence Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 9, 1), "Mon"))              'Labor Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 10, 1), "Mon").AddDays(7))  'Columbus Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 11, 1), "Thu").AddDays(21)) 'Thanksgiving
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 12, 25), "D", 0, "G"))      'Christmas Day
        Next

        'In this example, we calculate 2 business days from 2012-05-04.
        StartDate = #5/4/2012#
        DatePart = "D"
        Num = 2
        DateRollRule = "F"
        WE1 = wct.NULL_INT
        WE2 = wct.NULL_INT

        result = wct.BUSINESSDATEWE(StartDate, DatePart, Num, DateRollRule, Holidays, WE1, WE2)

        Console.WriteLine("BUSINESSDATEWE : " & result)            '5/8/2012
        File.WriteAllText("BUSINESSDATEWE", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'What if we want to count Saturday as a business day?
        WE1 = 6

        result = wct.BUSINESSDATEWE(StartDate, DatePart, Num, DateRollRule, Holidays, WE1, WE2)

        Console.WriteLine("BUSINESSDATEWE : " & result)            '5/7/2012
        File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'calculate 5 business days from 2012-05-21
        WE1 = wct.NULL_INT
        StartDate = #5/21/2012#
        Num = 5

        result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays)

        Console.WriteLine("BUSINESSDATE : " & result)            '5/29/2012
        File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf & result.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Calculate 3 months from 2012-02-28
        StartDate = #2/28/2012#
        DatePart = "M"
        Num = 3

        result = wct.BUSINESSDATE(StartDate, DatePart, Num, DateRollRule, Holidays)

        Console.WriteLine("BUSINESSDATE : " & result)            '5/29/2012
        File.AppendAllText("BUSINESSDATEWE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_T360()
        Dim Maturity As Date
        Dim Settlement As Date
        Dim CFDate As Date
        Dim Freq As Integer
        Dim Method30360 As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Maturity = #3/16/2022#
        Settlement = #3/12/2015#
        CFDate = #3/31/2015#
        Freq = 2
        Method30360 = 0

        result = wct.T360(Maturity, Settlement, CFDate, Freq, Method30360)

        Console.WriteLine("T360 : " & result)
        File.WriteAllText("T360.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        Maturity = #7/3/2018#
        CFDate = Maturity
        Settlement = #3/18/2015#
        Do
            Console.WriteLine("T360 : " & CFDate & vbTab & wct.T360(Maturity, Settlement, CFDate, Freq, Method30360))
            File.AppendAllText("T360.csv", vbCrLf & CFDate & vbTab & wct.T360(Maturity, Settlement, CFDate, Freq, Method30360))
            CFDate = CFDate.AddMonths(-6)
        Loop While CFDate > Settlement

    End Sub

    Public Shared Sub test_CALCDATE()
        Dim Y As Integer
        Dim M As Integer
        Dim D As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        Y = 2011
        D = 7
        M = 2

        result = wct.CALCDATE(Y, M, D)

        Console.WriteLine("CALCDATE : " & result)            '2/7/2011
        File.WriteAllText("CALCDATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_DATEFLOAT()
        Dim Y As Integer
        Dim M As Integer
        Dim D As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Y = 2011
        D = 7
        M = 2

        result = wct.DATEFLOAT(Y, M, D)

        Console.WriteLine("DATEFLOAT : " & result)            '40579
        File.WriteAllText("DATEFLOAT.csv", result.ToString())

    End Sub

    Public Shared Sub test_DATEINT()
        Dim Y As Integer
        Dim M As Integer
        Dim D As Integer
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        Y = 2011
        D = 7
        M = 2

        result = wct.DATEINT(Y, M, D)

        Console.WriteLine("DATEINT : " & result)            '40579
        File.WriteAllText("DATEINT.csv", result.ToString())

    End Sub

    Public Shared Sub test_DAYS360()
        Dim StartDate As Date()
        Dim EndDate As Date()

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = New Date() {#1/15/2007#, #1/15/2007#, #1/15/2007#, #9/30/2007#, #9/30/2007#, #9/30/2007#, #1/15/2007#, #1/31/2007#, #2/28/2007#, #8/31/2006#, #2/28/2007#, #2/14/2007#, #2/26/2007#, #2/29/2008#, #2/29/2008#, #2/29/2008#, #2/28/2007#, #10/31/2007#, #8/31/2007#, #2/29/2008#, #8/31/2008#, #2/28/2009#}
        EndDate = New Date() {#1/30/2007#, #2/15/2007#, #7/15/2007#, #3/31/2008#, #10/31/2007#, #9/30/2008#, #1/31/2007#, #2/28/2007#, #3/31/2007#, #2/28/2007#, #8/31/2007#, #2/28/2007#, #2/29/2008#, #2/28/2009#, #3/30/2008#, #3/31/2008#, #3/5/2007#, #11/28/2007#, #2/29/2008#, #8/31/2008#, #2/28/2009#, #8/31/2009#}

        Console.WriteLine("DAYS360 : " & vbTab & "Start Date" & vbTab & "End Date" & vbTab & "30/360 US" & vbTab & "30E/360" & "30E360 ISDA" & vbTab & "Actual")
        File.WriteAllText("DAYS360.tsv", "Start Date" & vbTab & "End Date" & vbTab & "30/360 US" & vbTab & "30E/360" & "30E360 ISDA" & vbTab & "Actual")

        For i As Integer = 0 To StartDate.GetUpperBound(0)
            Console.WriteLine("DAYS360 : " & vbTab & StartDate(i) & vbTab & EndDate(i) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 0) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 1) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 2) & vbTab & (EndDate(i) - StartDate(i)).TotalDays)
            File.AppendAllText("DAYS360.tsv", vbCrLf & StartDate(i) & vbTab & EndDate(i) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 0) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 1) & vbTab & wct.DAYS360(StartDate(i), EndDate(i), 2) & vbTab & (EndDate(i) - StartDate(i)).TotalDays)
        Next

    End Sub

    Public Shared Sub test_DAYSINMONTH()
        Dim StartDate As Date
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #9/18/2014#

        result = wct.DAYSINMONTH(StartDate)

        Console.WriteLine("DAYSINMONTH : " & result)         '30
        File.WriteAllText("DAYSINMONTH.csv", result.ToString())

    End Sub

    Public Shared Sub test_DAYSINYEAR()
        Dim StartDate As Date
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #9/18/2014#

        result = wct.DAYSINYEAR(StartDate)

        Console.WriteLine("DAYSINYEAR : " & result)          '365
        File.WriteAllText("DAYSINYEAR.csv", result.ToString())

    End Sub

    Public Shared Sub test_DAYSNL()
        Dim StartDate As Date()
        Dim EndDate As Date()

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = New Date() {#1/15/2007#, #1/15/2007#, #1/15/2007#, #9/30/2007#, #9/30/2007#, #9/30/2007#, #1/15/2007#, #1/31/2007#, #2/28/2007#, #8/31/2006#, #2/28/2007#, #2/14/2007#, #2/26/2007#, #2/29/2008#, #2/29/2008#, #2/29/2008#, #2/28/2007#, #10/31/2007#, #8/31/2007#, #2/29/2008#, #8/31/2008#, #2/28/2009#}
        EndDate = New Date() {#1/30/2007#, #2/15/2007#, #7/15/2007#, #3/31/2008#, #10/31/2007#, #9/30/2008#, #1/31/2011#, #2/28/2011#, #3/31/2011#, #2/28/2011#, #8/31/2011#, #2/28/2011#, #2/29/2012#, #2/28/2013#, #3/30/2012#, #3/31/2012#, #3/5/2011#, #11/28/2011#, #2/29/2012#, #8/31/2012#, #2/28/2012#, #8/31/2012#}

        Console.WriteLine("DAYSNL : " & vbTab & "Start Date" & vbTab & "End Date" & vbTab & "DAYSNL" & vbTab & "Actual")
        File.WriteAllText("DAYSNL.tsv", "Start Date" & vbTab & "End Date" & vbTab & "DAYSNL" & vbTab & "Actual")

        For i As Integer = 0 To StartDate.GetUpperBound(0)
            Console.WriteLine(StartDate(i) & vbTab & EndDate(i) & vbTab & wct.DAYSNL(StartDate(i), EndDate(i)) & vbTab & (EndDate(i) - StartDate(i)).TotalDays)
            File.AppendAllText("DAYSNL.tsv", vbCrLf & StartDate(i) & vbTab & EndDate(i) & vbTab & wct.DAYSNL(StartDate(i), EndDate(i)) & vbTab & (EndDate(i) - StartDate(i)).TotalDays)
        Next

    End Sub

    Public Shared Sub test_EASTER()
        Dim Y As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        Y = 2016

        result = wct.EASTER(Y)

        Console.WriteLine("EASTER : " & result)          '03/27/2016
        File.WriteAllText("EASTER.csv", result.ToString())

    End Sub

    Public Shared Sub test_EDATE()
        Dim StartDate As Date
        Dim Months As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #8/31/2016#
        Months = 6

        result = wct.EDATE(StartDate, Months)

        Console.WriteLine("EDATE : " & result)
        File.WriteAllText("EDATE.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        result = wct.EDATE(StartDate, -Months)

        Console.WriteLine("EDATE : " & result)
        File.AppendAllText("EDATE.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_EOMONTH()
        Dim StartDate As Date
        Dim Months As Integer
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #8/15/2016#
        Months = 6

        result = wct.EOMONTH(StartDate, Months)

        Console.WriteLine("EOMONTH : " & result)
        File.WriteAllText("EOMONTH.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        result = wct.EOMONTH(StartDate, -Months)

        Console.WriteLine("EOMONTH : " & result)
        File.AppendAllText("EOMONTH.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_FIRSTWEEKDAY()
        Dim startDate As Date
        Dim DayOfWeek As String
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'First Wednesday in January 2013
        startDate = #1/1/2013#
        DayOfWeek = "4"

        result = wct.FIRSTWEEKDAY(startDate, DayOfWeek)

        Console.WriteLine("FIRSTWEEKDAY : " & result)            '01/02/2013
        File.WriteAllText("FIRSTWEEKDAY.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        DayOfWeek = "Wed"

        result = wct.FIRSTWEEKDAY(startDate, DayOfWeek)

        Console.WriteLine("FIRSTWEEKDAY : " & result)            '01/02/2013
        File.AppendAllText("FIRSTWEEKDAY.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_ISREGULARPAY()
        Dim IssueDate As DateTime
        Dim FirstPaymentDate As DateTime
        Dim PmtPerYear As Integer
        Dim retStr As StringBuilder
        Dim result As Boolean

        '=====================
        ' EXAMPLE# 1
        '=====================

        'A loan with weekly repayments with the first payment 7 days after the issue date.

        IssueDate = New DateTime(2014, 1, 13)
        FirstPaymentDate = New DateTime(2014, 1, 20)
        PmtPerYear = 52

        result = wct.ISREGULARPAY(IssueDate, FirstPaymentDate, PmtPerYear)

        Console.WriteLine("ISREGULARPAY ex1 : " & result)                'True
        File.WriteAllText("ISREGULARPAY.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'The following example uses 2013-12-31 as @IssueDate and then compares all the days in January 2014 
        '(using the SeriesDate function from the XLeratorDB math library) 
        'as @FirstPaymentDate and pivots the results for each possible value of @PmtPerYear.

        Dim pmt As Integer() = {1, 2, 3, 4, 6, 12, 13, 24, 26, 52, 365}
        retStr = New StringBuilder()
        retStr.Append("FirstPaymentDate")
        For i As Integer = 0 To 10
            retStr.Append(",")
            retStr.Append(pmt(i).ToString())
        Next
        retStr.AppendLine()

        IssueDate = New DateTime(2013, 12, 31)
        FirstPaymentDate = New DateTime(2013, 12, 31)
        Do
            retStr.Append(FirstPaymentDate.ToString())
            For i As Integer = 0 To 10
                result = wct.ISREGULARPAY(IssueDate, FirstPaymentDate, pmt(i))
                retStr.Append(result.ToString())
                If i < 10 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
            FirstPaymentDate = FirstPaymentDate.AddDays(1)
        Loop Until FirstPaymentDate > New DateTime(2014, 1, 31)

        Console.WriteLine("ISREGULARPAY ex2 : " & retStr.ToString())
        File.AppendAllText("ISREGULARPAY.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_LASTWEEKDAY()
        Dim startDate As Date
        Dim DayOfWeek As String
        Dim result As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Last Wednesday in January 2013
        startDate = #1/1/2013#
        DayOfWeek = "4"

        result = wct.LASTWEEKDAY(startDate, DayOfWeek)

        Console.WriteLine("LASTWEEKDAY : " & result)         '01/30/2013
        File.WriteAllText("LASTWEEKDAY.csv", result.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        DayOfWeek = "Wed"

        result = wct.LASTWEEKDAY(startDate, DayOfWeek)

        Console.WriteLine("LASTWEEKDAY : " & result)         '01/30/2013 
        File.AppendAllText("LASTWEEKDAY.csv", vbCrLf & result.ToString())

    End Sub

    Public Shared Sub test_NBD()
        Dim Holidays As List(Of Date)
        Dim result As String

        '=====================
        ' EXAMPLE# 1
        '=====================

        'Create some holidays
        Holidays = New List(Of Date)
        For i As Integer = 2010 To 2020
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 1, 1), "D", 0, "G"))        'New Year's Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 1, 1), "Mon").AddDays(14))  'MLK Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 2, 1), "Mon").AddDays(14))  'President's Day
            Holidays.Add(wct.LASTWEEKDAY(DateSerial(i, 5, 1), "Mon"))               'Memorial Day
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 7, 4), "D", 0, "G"))        'Independence Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 9, 1), "Mon"))              'Labor Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 10, 1), "Mon").AddDays(7))  'Columbus Day
            Holidays.Add(wct.FIRSTWEEKDAY(DateSerial(i, 11, 1), "Thu").AddDays(21)) 'Thanksgiving
            Holidays.Add(wct.BUSINESSDATE(DateSerial(i, 12, 25), "D", 0, "G"))      'Christmas Day
        Next

        result = wct.NBD(Holidays)

        Console.WriteLine("NBD : " & result)
        File.WriteAllText("NBD.csv", result.ToString())

    End Sub

    Public Shared Sub test_NUMMONTHS()
        Dim StartDate As Date
        Dim EndDate As Date
        Dim result As Integer

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #9/18/2014#
        EndDate = #10/1/2014#

        result = wct.NUMMONTHS(StartDate, EndDate)

        Console.WriteLine("NUMMONTHS : " & result)
        File.WriteAllText("NUMMONTHS.csv", result.ToString())

    End Sub

    Public Shared Sub test_YEARFRAC()
        Dim StartDate As Date
        Dim EndDate As Date

        '=====================
        ' EXAMPLE# 1
        '=====================

        StartDate = #8/30/2015#
        EndDate = #4/30/2016#

        File.WriteAllText("YEARFRAC.csv", "")
        For i As Integer = 0 To 4
            Console.WriteLine("YEARFRAC " & StartDate & " to " & EndDate & " Basis = " & i & " : " & wct.YEARFRAC(StartDate, EndDate, i.ToString()))
            File.AppendAllText("YEARFRAC.csv", StartDate & " to " & EndDate & " Basis = " & i & " : " & wct.YEARFRAC(StartDate, EndDate, i.ToString()) & vbCrLf)
        Next

    End Sub

    Public Shared Sub test_DOLLARDE()
        Dim Fraction_dollar As Double
        Dim Fraction As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Fraction_dollar = 1.59
        Fraction = 64

        result = wct.DOLLARDE(Fraction_dollar, Fraction)            '1.921875

        Console.WriteLine("DOLLARDE : " & result)
        File.WriteAllText("DOLLARDE.csv", result.ToString())

    End Sub

    Public Shared Sub test_DOLLARFR()
        Dim Decimal_dollar As Double
        Dim Fraction As Integer
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Decimal_dollar = 1.921875
        Fraction = 64

        result = wct.DOLLARFR(Decimal_dollar, Fraction)

        Console.WriteLine("DOLLARFR : " & result)            '1.59
        File.WriteAllText("DOLLARFR.csv", result.ToString())

    End Sub

    Public Shared Sub test_BONDAMORT()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Rate As Double
        Dim FaceAmount As Double
        Dim CleanPrice As Double
        Dim Redemption As Double
        Dim Frequency As Double
        Dim Basis As String
        Dim IssueDate As Date
        Dim FirstInterestDate As Date
        Dim LastInterestDate As Date
        Dim result As BONDAMORT_table
        Dim retStr As StringBuilder

        '=====================
        ' EXAMPLE# 1
        '=====================

        'We buy 1,000,000 in face value of a bond on 2012-05-03 at a price of 999000. 
        'The bond matures on 2012-06-30 and accrues interest using the Actual/365 day-count method. 
        'The interest rate is 5%.
        Settlement = #5/3/2012#
        Maturity = #6/30/2012#
        Rate = 0.05
        FaceAmount = 1000000
        CleanPrice = 999000
        Redemption = wct.NULL_DOUBLE
        Frequency = 2
        Basis = "3"
        IssueDate = wct.NULL_DATE
        FirstInterestDate = wct.NULL_DATE
        LastInterestDate = wct.NULL_DATE

        result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

       'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("BONDAMORT : " & retStr.ToString())
        File.WriteAllText("BONDAMORT.csv", retStr.ToString())

        '=====================
        ' EXAMPLE# 2
        '=====================

        'In this example we will change the basis to 0, to reflect the US 30/360 day-count convention.
        'Notice the row for the 31-May-12. There is no daily coupon accrual, reflecting the market convention for this day-count convention.
        'Since there is no daily coupon accrual, there is no need to adjust the interest for this day and there is no daily amortization.
        Basis = "0"

        result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("BONDAMORT : " & retStr.ToString())
        File.AppendAllText("BONDAMORT.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 3
        '=====================

        'Let’s look at what happens with a US 30/360 bond over a February month end.
        'Notice that for 2012-02-29 the daily coupon is actually two days of coupon interest (again, reflecting market practice). 
        'Thus, the daily amortization also needs to reflect two days of amortization.
        Settlement = #2/15/2012#
        Maturity = #3/15/2012#

        result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("BONDAMORT : " & retStr.ToString())
        File.AppendAllText("BONDAMORT.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 4
        '=====================

        'Here’s an example for NL/365 in which Feb-29 is not included in the date calculations (NL meaning No Leap year).
        'On 29-Feb-12 there is no coupon accrual and therefore there is no amortization.
        Settlement = #2/15/2012#
        Maturity = #3/15/2012#
        Basis = "7"

        result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
        Next
        Console.WriteLine("BONDAMORT : " & retStr.ToString())
        File.AppendAllText("BONDAMORT.csv", vbCrLf & retStr.ToString())

        '=====================
        ' EXAMPLE# 5
        '=====================

        'Let’s look at a bond which accrues on an actual/actual basis.
        'Notice the on 2012-03-16 the dly_coup changed from 137.3626 to 135.8696 and the dly_amort changed from 1.6317 to 3.1250. 
        'The dly_eff_rate, however, only changed slightly, from 138.9943 to 138.9945 and the (calculated) amort_rate did not change at all. 
        'In fact, if we were to look at the last 10 days of the amortization schedule we would see that the (calculated) amortization rate is still the same.
        Settlement = #3/10/2012#
        Maturity = #3/15/2022#
        CleanPrice = 990000.0
        Basis = "1"

        result = wct.BONDAMORT(Settlement, Maturity, Rate, FaceAmount, CleanPrice, Redemption, Frequency, Basis, IssueDate, FirstInterestDate, LastInterestDate)

        'Write the table output to a csv
        'We only write out the rows where amort_date < 2012-03-20
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName & ",")
        Next
        retStr.Remove(retStr.Length - 1, 1)
        retStr.AppendLine()
        For Each row As DataRow In result.Rows
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString())
                If i < result.Columns.Count - 1 Then
                    retStr.Append(",")
                Else
                    retStr.AppendLine()
                End If
            Next
            If row.Field(Of Date)("amort_date") = #3/19/2012# Then
                Exit For
            End If
        Next
        Console.WriteLine("BONDAMORT : " & retStr.ToString())
        File.AppendAllText("BONDAMORT.csv", vbCrLf & retStr.ToString())

    End Sub

    Public Shared Sub test_INTRATE()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Investment As Double
        Dim Redemption As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #2/2/2008#
        Maturity = #5/8/2008#
        Investment = 98975
        Redemption = 100000
        Basis = "0"

        result = wct.INTRATE(Settlement, Maturity, Investment, Redemption, Basis)

        Console.WriteLine("INTRATE : " & result)
        File.WriteAllText("INTRATE.csv", result.ToString())

    End Sub

    Public Shared Sub test_RECEIVED()
        Dim Settlement As Date
        Dim Maturity As Date
        Dim Investment As Double
        Dim Discount As Double
        Dim Basis As String
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        Settlement = #2/1/2008#
        Maturity = #5/2/2008#
        Investment = 98975
        Discount = 0.0475
        Basis = "0"

        result = wct.RECEIVED(Settlement, Maturity, Investment, Discount, Basis)

        Console.WriteLine("RECEIVED : " & result)
        File.WriteAllText("RECEIVED.csv", result.ToString())

    End Sub

    Public Shared Sub test_BondPriceFromZeroes()
        Dim result As Double

        '=====================
        ' EXAMPLE# 1
        '=====================

        'SELECT
        '   *
        'INTO
        '   #ccz
        'FROM (VALUES
        '    (0.011,0.0031936941106789)
        '   ,(0.25,0.00319251640948496)
        '   ,(0.50,0.0042807233899723)
        '   ,(1,0.00544437632630534)
        '   ,(2,0.00727274510130153)
        '   ,(3,0.00859818036980457)
        '   ,(4,0.0101034030456584)
        '   ,(5,0.0116083325279126)
        '   ,(7,0.0144258819802952)
        '   ,(10,0.0163078262966277)
        '   ,(20,0.0203301487469184)
        '   ,(30,0.0250383019093584)
        '   )n(T,z)
        Dim ccz_T As Double() = {0.011, 0.25, 0.5, 1, 2, 3, 4, 5, 7, 10, 20, 30}
        Dim ccz_z As Double() = {0.0031936941106789, 0.00319251640948496, 0.0042807233899723, 0.00544437632630534, 0.00727274510130153, 0.00859818036980457, 0.0101034030456584, 0.0116083325279126, 0.0144258819802952, 0.0163078262966277, 0.0203301487469184, 0.0250383019093584}

        'SELECT wct.BondPricefromZeroes(
        '        '2016-11-28' --@Settlement
        '       ,'2023-10-01' --@Maturity
        '       ,0.0525       --@Rate
        '       ,.03276       --@CurveSpread
        '       ,100          --@Redemption
        '       ,2            --@Frequency
        '       ,0            --@Basis
        '       ,NULL         --@LastCouponDate
        '       ,NULL         --@FirstCouponDate
        '       ,NULL         --@IssueDate
        '       ,'SELECT * FROM #ccz'      --@ZeroRate
        '       ,NULL         --@CurveType
        '       ,NULL         --@CurveStartDate
        '       ,NULL         --@CurveDayCount
        '       ,NULL         --@CurveFrequency
        '       ,NULL         --@InterpMethod
        '       ) as [Price]

        result = wct.BondPricefromZeroes(New DateTime(2016, 11, 28), New DateTime(2023, 10, 1), 0.0525, 0.03276, 100.0, 2, "0", wct.NULL_DATE, wct.NULL_DATE, wct.NULL_DATE, ccz_T, ccz_z, "", wct.NULL_DATE)

        Console.WriteLine("BondPriceFromZeroes : " & result.ToString())
        File.WriteAllText("BondPriceFromZeroes.csv", result.ToString())


        '=====================
        ' EXAMPLE# 2
        '=====================

        '--Put the step information into the #step table
        'SELECT
        '    date_step
        '   ,rate_step
        'INTO
        '   #step
        'FROM (VALUES
        '    ('2020-10-01', .0425)
        '   ,('2024-10-01', .0625)
        '   )n(date_step, rate_step)

        '--The bond to be evaluated
        'SELECT wct.BondPricefromZeroes(
        '        '2016-11-28' --@Settlement
        '       ,'2026-10-01' --@Maturity
        '       ,'SELECT date_step, rate_step FROM #step'       --@Rate
        '       ,.00836       --@Spread
        '       ,100          --@Redemption
        '       ,2            --@Frequency
        '       ,0            --@Basis
        '       ,NULL         --@LastCouponDate
        '       ,NULL         --@FirstCouponDate
        '       ,NULL         --@IssueDate
        '       ,'SELECT * FROM #ccz'             --@ZeroRate
        '       ,NULL         --@CurveType
        '       ,NULL         --@CurveStartDate
        '       ,NULL         --@CurveDayCount
        '       ,NULL         --@CurveFrequency
        '       ,NULL         --@InterpMethod
        '       ) [Price]

        Dim step_datestep As Date() = {New DateTime(2020, 10, 1), New DateTime(2024, 10, 1)}
        Dim step_ratestep As Double() = {0.0425, 0.0625}

        result = wct.BondPricefromZeroes(New DateTime(2016, 11, 28), New DateTime(2026, 10, 1), step_datestep, step_ratestep, 0.00836, 100.0, 2, "0", wct.NULL_DATE, wct.NULL_DATE, wct.NULL_DATE, ccz_T, ccz_z, "", wct.NULL_DATE)

        Console.WriteLine("BondPriceFromZeroes : " & result.ToString())
        File.WriteAllText("BondPriceFromZeroes.csv", result.ToString())


        '=====================
        ' EXAMPLE# 3 - ILists
        '=====================

        '--Establish the CMT curve
        'SELECT
        '   *
        'INTO
        '   #par
        'FROM (VALUES
        '        (0.25,0.00396)
        '       ,(0.5,0.00520)
        '       ,(1,0.00614)
        '       ,(2,0.00823)
        '       ,(3,0.00987)
        '       ,(4,0.01138)
        '       ,(5,0.01290)
        '       ,(7,0.01605)
        '       ,(10,0.01839)
        '       ,(20,0.02216)
        '       ,(30,0.02593)
        '   )n(T,r)

        Dim curve_T As Double() = {0.25, 0.5, 1, 2, 3, 4, 5, 7, 10, 20, 30}
        Dim curve_Rates As Double() = {0.00396, 0.0052, 0.00614, 0.00823, 0.00987, 0.01138, 0.0129, 0.01605, 0.01839, 0.02216, 0.02593}

        '--Convert the CMT curve to continuously compounded zeroes
        'SELECT
        '   *
        'INTO
        '   #z
        'FROM
        '   wct.CMTCURVE('SELECT * FROM #par','S',2)
        'WHERE
        '   bootstrap = 'False'

        Dim z As XLeratorDLL_financial.OptionTypes.CMTCurve_table = wct.CMTCurve(curve_T, curve_Rates, "S", 2)

        '--Enter some bonds into a table
        'SELECT
        '   *
        'INTO
        '   #bonds
        'FROM (VALUES
        '    ('A','2025-11-03',0.0333, 226.7)
        '   ,('B','2023-05-12',0.0447, 252)
        '   ,('C','2029-07-17',0.0654, 413.6)
        '   ,('D','2022-08-06',0.0673, 264.8)
        '   ,('E','2030-02-18',0.0649, 404.1)
        '   ,('F','2024-08-17',0.047, 237.4)
        '   ,('G','2023-04-07',0.0488, 253.6)
        '   ,('H','2026-05-29',0.0584, 227.2)
        '   ,('I','2023-11-06',0.0426, 245.1)
        '   ,('J','2027-04-20',0.0572, 338.9)
        '   )n(id_bond,maturity,rate,spread)

        Dim bonds As String = "'A','2025-11-03',0.0333, 226.7" + "|" + _
                              "'B','2023-05-12',0.0447, 252" + "|" + _
                              "'C','2029-07-17',0.0654, 413.6" + "|" + _
                              "'D','2022-08-06',0.0673, 264.8" + "|" + _
                              "'E','2030-02-18',0.0649, 404.1" + "|" + _
                              "'F','2024-08-17',0.047, 237.4" + "|" + _
                              "'G','2023-04-07',0.0488, 253.6" + "|" + _
                              "'H','2026-05-29',0.0584, 227.2" + "|" + _
                              "'I','2023-11-06',0.0426, 245.1" + "|" + _
                              "'J','2027-04-20',0.0572, 338.9"
                       '   )n(id_bond,maturity,rate,spread)

        'SELECT
        '    b.id_bond
        '   ,ROUND(
        '       wct.BondPricefromZeroes(
        '           '2016-11-28'    --@Settlement
        '          ,b.maturity             --@Maturity
        '          ,b.rate                 --@Rate
        '          ,b.spread / 10000       --@Spread
        '          ,NULL            --@Redemption
        '          ,2               --@Frequency
        '          ,0               --@Basis
        '          ,NULL            --@LastCouponDate
        '          ,NULL            --@FirstCouponDate
        '          ,NULL            --@IssueDate
        '          ,'SELECT * FROM #z'     --@ZeroRate
        '          ,NULL            --@CurveType
        '          ,NULL            --@CurveStartDate
        '          ,NULL            --@CurveDayCount
        '          ,NULL            --@CurveFrequency
        '          ,NULL            --@InterpMethod
        '          )
        '       ,3) as [Price]
        'FROM
        '   #bonds b

        Dim retStr As New StringBuilder()
        retStr.Append("id_bond" + vbTab + "Price")

        For Each bond As String In bonds.Split("|"c)
            Dim col As String() = bond.Split(","c)
            result = wct.BondPricefromZeroes(New DateTime(2016, 11, 28), _
                                             DateTime.Parse(col(1).Replace("'", "")), _
                                             Double.Parse(col(2)),
                                             Double.Parse(col(3)) / 10000.0, _
                                             wct.NULL_DOUBLE, _
                                             2, _
                                             "0", _
                                             wct.NULL_DATE, _
                                             wct.NULL_DATE, _
                                             wct.NULL_DATE, _
                                             z, _
                                             "", _
                                             wct.NULL_DATE)
            retStr.AppendLine()
            retStr.Append(col(0).Replace("'", "") + vbTab + Math.Round(result, 3).ToString())
        Next

        Console.WriteLine(vbCrLf)
        Console.WriteLine("BondPriceFromZeroes : " + vbCrLf + retStr.ToString())
        File.WriteAllText("BondPriceFromZeroes.csv", retStr.ToString())


        '========================
        ' EXAMPLE# 3 - 2d arrays
        '========================

        '2d-array contains 10 rows with 4 cols, in (col,row) format
        Dim bonds2 As Object(,) = { _
                                   {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J"}, _
                                   {New DateTime(2025, 11, 3), New DateTime(2023, 5, 12), New DateTime(2029, 7, 17), New DateTime(2022, 8, 6), New DateTime(2030, 2, 18), New DateTime(2024, 8, 17), New DateTime(2023, 4, 7), New DateTime(2026, 5, 29), New DateTime(2023, 11, 6), New DateTime(2027, 4, 20)}, _
                                   {0.0333, 0.0447, 0.0654, 0.0673, 0.0649, 0.047, 0.0488, 0.0584, 0.0426, 0.0572},
                                   {226.7, 252.0, 413.6, 264.8, 404.1, 237.4, 253.6, 227.2, 245.1, 338.9} _
                                  }
        retStr.Clear()
        retStr.Append("id_bond" + vbTab + "Price")
        For r As Integer = 0 To bonds2.GetUpperBound(1)
            result = wct.BondPricefromZeroes(New DateTime(2016, 11, 28), _
                                             DirectCast(bonds2(1, r), Date), _
                                             DirectCast(bonds2(2, r), Double), _
                                             DirectCast(bonds2(3, r), Double) / 10000.0, _
                                             wct.NULL_DOUBLE, _
                                             2, _
                                             "0", _
                                             wct.NULL_DATE, _
                                             wct.NULL_DATE, _
                                             wct.NULL_DATE, _
                                             z, _
                                             "", _
                                             wct.NULL_DATE)
            retStr.AppendLine()
            retStr.Append(bonds2(0, r).ToString() + vbTab + Math.Round(result, 3).ToString())
        Next

        Console.WriteLine(vbCrLf)
        Console.WriteLine("BondPriceFromZeroes : " + vbCr + retStr.ToString())
        File.WriteAllText("BondPriceFromZeroes.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_CMTCurve()
        Dim retStr As StringBuilder
        Dim result As CMTCurve_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'SELECT
        '   *
        'FROM  
        '   wct.CMTCurve('
        '       SELECT T, r
        '       FROM (VALUES (1, .035), (2, .042), (3, .047), (4, .052))n(T, r)
        '       ORDER BY T'   --@Curve
        '       ,'L'          --@InterpMethod
        '       ,1            --@Freq
        '       )

        result = wct.CMTCurve({{1.0, 2.0, 3.0, 4.0}, {0.035, 0.042, 0.047, 0.052}}, "L", 1)

        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("CMTCurve : " + vbCrLf + retStr.ToString())
        File.WriteAllText("CMTCurve.csv", retStr.ToString())


        '=====================
        ' EXAMPLE# 2
        '=====================

        'SELECT
        '   T
        '   ,r/100 as R
        'INTO
        '   #cmt
        'FROM (VALUES
        '    (0.0833333333333333,0.44)
        '   ,(0.25,0.51)
        '   ,(0.5,0.62)
        '   ,(1,0.85)
        '   ,(2,1.2)
        '   ,(3,1.47)
        '   ,(5,1.93)
        '   ,(7,2.25)
        '   ,(10,2.45)
        '   ,(20,2.79)
        '   ,(30,3.06)
        '   )n(T,r)

        'SELECT
        '   *
        'FROM  
        '   wct.CMTCurve('
        '       SELECT T,R
        '       FROM #cmt' --@Curve
        '       ,NULL     --@InterpMethod
        '       ,NULL     --@Freq
        '       )

        Dim curve As New System.Data.DataTable  ' input datatable values are hard-coded in this example, they would typically be populated from a db query
        curve.Columns.AddRange({New System.Data.DataColumn("T", System.Type.GetType("System.Double")), New System.Data.DataColumn("r", System.Type.GetType("System.Double"))})
        curve.Rows.Add(0.0833333333333333, 0.44 / 100.0)
        curve.Rows.Add(0.25, 0.51 / 100.0)
        curve.Rows.Add(0.5, 0.62 / 100.0)
        curve.Rows.Add(1.0, 0.85 / 100.0)
        curve.Rows.Add(2.0, 1.2 / 100.0)
        curve.Rows.Add(3.0, 1.47 / 100.0)
        curve.Rows.Add(5.0, 1.93 / 100.0)
        curve.Rows.Add(7.0, 2.25 / 100.0)
        curve.Rows.Add(10.0, 2.45 / 100.0)
        curve.Rows.Add(20.0, 2.79 / 100.0)
        curve.Rows.Add(30.0, 3.06 / 100.0)

        result = wct.CMTCurve(curve)

        'Write the table output to a csv
        retStr.Clear()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("CMTCurve : " + vbCrLf + retStr.ToString())
        File.WriteAllText("CMTCurve.csv", retStr.ToString())


        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example we use the same data as in the previous example, but no we are going to shift the par curve up and down by 25 basis points and then return the continuously compounded zeroes for par rates and the shifted rates. 

        'SELECT
        '    x.Ztype
        '   ,k.T
        '   ,k.cczero
        'INTO
        '   #z
        'FROM (VALUES
        '    ('Z', 'SELECT T,r FROM #cmt')
        '   ,('Zplus','SELECT T,r+.0025 FROM #cmt')
        '   ,('Zminus','SELECT T,r-.0025 FROM #cmt')
        '   )x(Ztype,ZSQL)
        'CROSS APPLY
        '   wct.CMTCURVE(x.ZSQL,'S',2)k
        'WHERE
        '   bootstrap = 'False'

        'SELECT T, Z, zPlus, Zminus
        'FROM (SELECT * FROM #z) pvt
        'PIVOT(MAX(cczero) for Ztype in(Z,Zplus,Zminus))d

        Dim zplus As System.Data.DataTable = curve.Clone()
        Dim zminus As System.Data.DataTable = curve.Clone()
        For Each row As DataRow In curve.Rows
            zplus.Rows.Add(row(0), DirectCast(row(1), Double) + 0.0025)
            zminus.Rows.Add(row(0), DirectCast(row(1), Double) - 0.0025)
        Next

        Dim resultPlus As CMTCurve_table = wct.CMTCurve(zplus, "s", 2)
        Dim resultMinus As CMTCurve_table = wct.CMTCurve(zminus, "s", 2)

        Dim d As New System.Data.DataTable  ' input datatable values are hard-coded in this example, they would typically be populated from a db query
        d.Columns.AddRange({New System.Data.DataColumn("T", System.Type.GetType("System.Double")), _
                            New System.Data.DataColumn("Z", System.Type.GetType("System.Double")), _
                            New System.Data.DataColumn("ZPlus", System.Type.GetType("System.Double")), _
                            New System.Data.DataColumn("ZMinus", System.Type.GetType("System.Double")) _
                           })
        For r As Integer = 0 To result.Rows.Count - 1
            If result(r).bootstrap = False Then
                d.Rows.Add(result(r).T, result(r).cczero, resultPlus(r).cczero, resultMinus(r).cczero)
            End If
        Next

        'Write the table output to a csv
        retStr.Clear()
        For Each col As DataColumn In d.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In d.Rows
            retStr.AppendLine()
            For i As Integer = 0 To d.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("CMTCurve : " + vbCrLf + retStr.ToString())
        File.WriteAllText("CMTCurve.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_LogNormalIRLattice()
        Dim retStr As StringBuilder
        Dim result As LogNormalIRLattice_table

        '=====================
        ' EXAMPLE# 1
        '=====================

        'This example is taken from The Handbook of Fixed Income Securities, Eighth Edition Edited by Frank J. Fabozzi with Steven V. Mann, Chapter 40 pp 875 – 876. We start with the following par curve.

        '--Rates used in the OAS calculation
        'SELECT
        '    T
        '   ,par
        '   ,spot
        '   ,df
        '   ,-LOG(df)/T as ccZero
        'INTO
        '   #z
        'FROM (
        '   SELECT
        '        T
        '       ,par
        '       ,spot
        '       ,POWER(1+spot,-T) as df
        '   FROM (VALUES
        '        (1,0.035,0.035)
        '       ,(2,0.042,0.0421480257395637)
        '       ,(3,0.047,0.0473524471924105)
        '      ,(4,0.052,0.0527059539733534)
        '       )n(T,par,spot)
        '   )nn

        '                        col0              :          col1              :                                 col2
        Dim n As Double(,) = {{1.0, 2.0, 3.0, 4.0}, {0.035, 0.042, 0.047, 0.052}, {0.035, 0.0421480257395637, 0.0473524471924105, 0.0527059539733534}}  '2d-array contains 4 rows with 3 cols, in (col,row) format
        '                      row0 row1 row2 row3 | row0   row1   row2   row3  |  row0   row1                row2                row3
        Dim cczero As Object(,)
        ReDim cczero(1, n.GetUpperBound(1))
        For r As Integer = 0 To n.GetUpperBound(1)
            cczero(0, r) = n(0, r)
            cczero(1, r) = -Math.Log(Math.Pow(1.0 + n(2, r), -n(0, r))) / n(0, r)
        Next

        'Using the bond from the Fabozzi example we are given the Option-adjusted Spread as 35 basis points for a bond that matures in 4 years that is redeemable at par starting with the next coupon date. The bond has 6.5% coupon which is paid annually and the volatility is 10%.

        '--The bond to be evaluated
        'SELECT
        '    date_pmt
        '   ,num_step
        '   ,num_node
        '   ,rate_fwd
        '   ,rate_calibrated
        '   ,T
        '   ,delta
        '   ,df
        '   ,df_calibrated
        '   ,cczero
        '   ,PVCF
        '   ,coupon
        '   ,price_call
        '   ,price_put
        'FROM
        '   wct.LogNormalIRLattice(
        '    '2016-11-28' --@Settlement
        '   ,'2020-11-28' --@Maturity
        '   ,.065         --@Rate
        '   ,.0035        --@Spread
        '   ,NULL         --@Redemption
        '   ,1            --@Frequency
        '   ,NULL         --@Basis
        '   ,NULL         --@LastCouponDate
        '   ,NULL         --@FirstCouponDate
        '   ,NULL         --@IssueDate
        '   ,'SELECT T, cczero FROM #z'    --@CCZero
        '   ,NULL         --@CurveType
        '   ,NULL         --@CurveStartDate
        '   ,NULL         --@CurveDayCount
        '   ,1            --@CurveFrequency
        '   ,'L'          --@CurveInterpMethod
        '   ,0.10         --@Vol
        '   ,'SELECT ''2017-11-28'',100' --@OptionSched
        '   )

        Dim OptionSched As Object(,) = {{New Date(2017, 11, 28)}, {100.0}}          '2d-array contains 1 row with 2 cols, in (col,row) format (3rd col defaults to 'C'all)

        result = wct.LogNormalIRLattice(New DateTime(2016, 11, 28), _
                                        New DateTime(2020, 11, 28), _
                                        0.065, _
                                        0.0035, _
                                        wct.NULL_DOUBLE, _
                                        1, _
                                        "", _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        cczero, _
                                        "", _
                                        wct.NULL_DATE, _
                                        "", _
                                        1, _
                                        "L", _
                                        0.1, _
                                        OptionSched)


        'Write the table output to a csv
        retStr = New StringBuilder()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("LogNormalIRLattice : " + vbCrLf + retStr.ToString())
        File.WriteAllText("LogNormalIRLattice.csv", retStr.ToString())


        '=====================
        ' EXAMPLE# 2
        '=====================

        'We will use the same curve information as from the previous example. In this example (base on Exhibit 40-16, p. 873 in Fabozzi) we want to calculate the price for a step-up callable note with 4 years to maturity, callable in 2 years at 100 with a 10% volatility and an OAS of 88.7.

        '--Put the step information into the #step table
        'SELECT
        '    date_step
        '   ,rate_step
        'INTO
        '   #step
        'FROM (VALUES
        '    ('2016-11-28', .0425)
        '   ,('2018-11-28', .0750)
        '   )n(date_step, rate_step)

        Dim steps As Object(,) = {{New DateTime(2016, 11, 28), New DateTime(2018, 11, 28)}, {0.0425, 0.075}}    '2d-array contains 2 row with 2 cols, in (col,row) format

        '--The bond to be evaluated
        'SELECT
        '    date_pmt
        '   ,num_step
        '   ,num_node
        '   ,rate_fwd
        '   ,rate_calibrated
        '   ,T
        '   ,delta
        '   ,df
        '   ,df_calibrated
        '   ,cczero
        '   ,PVCF
        '   ,coupon
        '   ,price_call
        '   ,price_put
        'FROM
        '   wct.LogNormalIRLattice(
        '    '2016-11-28' --@Settlement
        '   ,'2020-11-28' --@Maturity
        '   ,'SELECT date_step, rate_step FROM #step'    --@Rate
        '   ,.00887       --@Spread
        '   ,NULL         --@Redemption
        '   ,1        --@Frequency
        '   ,NULL         --@Basis
        '   ,NULL         --@LastCouponDate
        '   ,NULL         --@FirstCouponDate
        '   ,NULL         --@IssueDate
        '   ,'SELECT T, cczero FROM #z'    --@CCZero
        '   ,NULL         --@CurveType
        '   ,NULL         --@CurveStartDate
        '   ,NULL         --@CurveDayCount
        '   ,1        --@CurveFrequency
        '   ,'L'          --@CurveInterpMethod
        '   ,0.10         --@Vol
        '   ,'SELECT ''2018-11-28'',100' --@OptionSched
        '   )

        OptionSched = {{New Date(2018, 11, 28)}, {100.0}}               '2d-array contains 1 row with 2 cols, in (col,row) format (3rd col defaults to 'C'all)

        result = wct.LogNormalIRLattice(New DateTime(2016, 11, 28), _
                                        New DateTime(2020, 11, 28), _
                                        steps, _
                                        0.00887, _
                                        wct.NULL_DOUBLE, _
                                        1, _
                                        "", _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        cczero, _
                                        "", _
                                        wct.NULL_DATE, _
                                        "", _
                                        1, _
                                        "L", _
                                        0.1, _
                                        OptionSched)

        'Write the table output to a csv
        retStr.Clear()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("LogNormalIRLattice : " + vbCrLf + retStr.ToString())
        File.WriteAllText("LogNormalIRLattice.csv", retStr.ToString())


        '=====================
        ' EXAMPLE# 3
        '=====================

        ' This is similar to Example #1, except that we make the bond putable rather than callable and the OAS is 117.

        '--The bond to be evaluated
        'SELECT
        '    date_pmt
        '   ,num_step
        '   ,num_node
        '   ,rate_fwd
        '   ,rate_calibrated
        '   ,T
        '   ,delta
        '   ,df
        '   ,df_calibrated
        '   ,cczero
        '   ,PVCF
        '   ,coupon
        '   ,price_call
        '   ,price_put
        'FROM
        '   wct.LogNormalIRLattice(
        '    '2016-11-28' --@Settlement
        '   ,'2020-11-28' --@Maturity
        '   ,.065         --@Rate
        '   ,.0117        --@Spread
        '   ,NULL         --@Redemption
        '   ,1            --@Frequency
        '   ,NULL         --@Basis
        '   ,NULL         --@LastCouponDate
        '   ,NULL         --@FirstCouponDate
        '   ,NULL         --@IssueDate
        '   ,'SELECT T, cczero FROM #z'    --@CCZero
        '   ,NULL         --@CurveType
        '   ,NULL         --@CurveStartDate
        '   ,NULL         --@CurveDayCount
        '   ,1            --@CurveFrequency
        '   ,'L'          --@CurveInterpMethod
        '   ,0.10         --@Vol
        '   ,'SELECT ''2017-11-28'',100, ''P''' --@OptionSched
        '   )

        OptionSched = {{New Date(2017, 11, 28)}, {100.0}, {"P"}}            '2d-array contains 1 row with 3 cols, in (col,row) format

        result = wct.LogNormalIRLattice(New DateTime(2016, 11, 28), _
                                        New DateTime(2020, 11, 28), _
                                        0.065, _
                                        0.0117, _
                                        wct.NULL_DOUBLE, _
                                        1, _
                                        "", _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        cczero, _
                                        "", _
                                        wct.NULL_DATE, _
                                        "", _
                                        1, _
                                        "L", _
                                        0.1, _
                                        OptionSched)

        'Write the table output to a csv
        retStr.Clear()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("LogNormalIRLattice : " + vbCrLf + retStr.ToString())
        File.WriteAllText("LogNormalIRLattice.csv", retStr.ToString())


        '=====================
        ' EXAMPLE# 4
        '=====================

        'In this example, we will calculate the price from the spread using CMT curve, which unlike the previous examples, compounds semi-annually. We use the CMTURVE function to convert the par rates into continuously compounded zeroes.
        'The bond matures on 2026-03-15, has a coupon rate of 7.0% paid semi-annually and a spread of 605.9. The bond is callable based on the following schedule. 

        'exdate         strike
        '2017-03-15     103.50
        '2018-03-15     103.00
        '2019-03-15     102.50
        '2020-03-15     102.00
        '2021-03-15     101.50
        '2022-03-15     101.00
        '2023-03-15     100.50
        '2024-03-15     100.00

        '--Establish the CMT curve
        'SELECT
        '   *
        'INTO
        '   #par
        'FROM (VALUES
        '    (0.25,0.00396)
        '   ,(0.5,0.00520)
        '   ,(1,0.00614)
        '   ,(2,0.00823)
        '   ,(3,0.00987)
        '   ,(4,0.01138)
        '   ,(5,0.01290)
        '   ,(7,0.01605)
        '   ,(10,0.01839)
        '   ,(20,0.02216)
        '   ,(30,0.02593)
        '   )n(T,r)

        Dim par As Double(,) = {{0.25, 0.5, 1.0, 2.0, 3.0, 4.0, 5.0, 7.0, 10.0, 20.0, 30.0}, _
                                {0.00396, 0.0052, 0.00614, 0.00823, 0.00987, 0.01138, 0.0129, 0.01605, 0.01839, 0.02216, 0.02593} _
                               }

        '--Convert the CMT curve to continuously compounded zeroes
        'SELECT
        '   *
        'INTO
        '   #z
        'FROM
        '   wct.CMTCURVE('SELECT * FROM #par','S',2)
        'WHERE
        '   bootstrap = 'False'

        Dim cmt As CMTCurve_table = wct.CMTCurve(par, "S", 2)
        Dim z As New System.Data.DataTable
        z.Columns.AddRange({New System.Data.DataColumn("T", System.Type.GetType("System.Double")), New System.Data.DataColumn("cczero", System.Type.GetType("System.Double"))})
        For r As Integer = 0 To cmt.Rows.Count - 1
            If cmt(r).bootstrap = False Then z.Rows.Add(cmt(r).T, cmt(r).cczero)
        Next

        '--Put the call schedules into a table
        'SELECT
        '   CAST(exdate as datetime) as exdate, strike
        'INTO
        '   #calls
        'FROM (VALUES
        '    ('2017-03-15',103.50)
        '   ,('2018-03-15',103.00)
        '   ,('2019-03-15',102.50)
        '   ,('2020-03-15',102.00)
        '   ,('2021-03-15',101.50)
        '   ,('2022-03-15',101.00)
        '   ,('2023-03-15',100.50)
        '   ,('2024-03-15',100.00)
        '   )n(exdate,strike)  

        Dim calls As New System.Data.DataTable()
        calls.Columns.AddRange({New System.Data.DataColumn("exdate", System.Type.GetType("System.DateTime")), New System.Data.DataColumn("strike", System.Type.GetType("System.Double"))})
        calls.Rows.Add({New DateTime(2017, 3, 15), 103.5})
        calls.Rows.Add({New DateTime(2018, 3, 15), 103.0})
        calls.Rows.Add({New DateTime(2019, 3, 15), 102.5})
        calls.Rows.Add({New DateTime(2020, 3, 15), 102.0})
        calls.Rows.Add({New DateTime(2021, 3, 15), 101.5})
        calls.Rows.Add({New DateTime(2022, 3, 15), 101.0})
        calls.Rows.Add({New DateTime(2023, 3, 15), 100.5})
        calls.Rows.Add({New DateTime(2024, 3, 15), 100.0})

        '--Variables to guarantee consistency in the function calls
        'DECLARE @Settlement as datetime = CAST('2016-11-28' as datetime)
        'DECLARE @StartDate as datetime = CAST('2016-11-28' as datetime)
        'DECLARE @Interp as CHAR(1) = 'S'
        'DECLARE @vol as float = 0.4248
        'DECLARE @rate_coupon as float = 0.07
        'DECLARE @date_maturity as date = cast('2026-03-15' as date)
        'DECLARE @Spread as float = 605.9 / 10000
        'DECLARE @dcc as varchar(2) = '1'
        'DECLARE @typeCurve as char(2) = 'CC'
        Dim Settlement As DateTime = New DateTime(2016, 11, 28)
        Dim StartDate As DateTime = New DateTime(2016, 11, 28)
        Dim Interp As String = "S"
        Dim Vol As Double = 0.4248
        Dim rate_coupon As Double = 0.07
        Dim date_maturity As DateTime = New DateTime(2026, 3, 15)
        Dim Spread As Double = 605.9 / 10000
        Dim dcc As String = "1"
        Dim typeCurve As String = "CC"

        'SELECT
        '    date_pmt
        '   ,num_step
        '   ,num_node
        '   ,rate_fwd
        '   ,rate_calibrated
        '   ,T
        '   ,delta
        '   ,df
        '   ,df_calibrated
        '   ,cczero
        '   ,PVCF
        '   ,coupon
        '   ,price_call
        '   ,price_put
        'FROM
        '    wct.LogNormalIRLattice(
        '    @Settlement --@Settlement
        '   ,@date_maturity --@Maturity
        '   ,@rate_coupon --@Rate
        '   ,@Spread      --@Spread
        '   ,100          --@Redemption
        '   ,2            --@Frequency
        '   ,1            --@Basis
        '   ,NULL         --@LastCouponDate
        '   ,NULL         --@FirstCouponDate
        '   ,NULL         --@IssueDate
        '   ,'SELECT t, cczero FROM #z'    --@CCZero
        '   ,@typecurve      --@CurveType
        '   ,@Settlement --@CurveStartDate
        '   ,@dcc         --@CurveDayCount
        '   ,NULL         --@CurveFrequency
        '   ,@Interp      --@CurveInterpMethod
        '   ,@vol         --@Vol
        '   ,'SELECT exdate,strike FROM #calls'   --@OptionSched
        '   )

        result = wct.LogNormalIRLattice(Settlement, _
                                        date_maturity, _
                                        rate_coupon, _
                                        Spread, _
                                        100.0, _
                                        2, _
                                        "1", _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        wct.NULL_DATE, _
                                        z, _
                                        typeCurve, _
                                        Settlement, _
                                        dcc, _
                                        wct.NULL_INT, _
                                        Interp, _
                                        Vol, _
                                        calls _
                                      )

        'Write the table output to a csv
        retStr.Clear()
        For Each col As DataColumn In result.Columns
            retStr.Append(col.ColumnName + vbTab)
        Next
        For Each row As DataRow In result.Rows
            retStr.AppendLine()
            For i As Integer = 0 To result.Columns.Count - 1
                retStr.Append(row(i).ToString() + vbTab)
            Next
        Next
        retStr.Replace(vbTab + vbCrLf, vbCrLf)  ' remove all trailing tabs at end of each row

        Console.WriteLine(vbCrLf)
        Console.WriteLine("LogNormalIRLattice : " + vbCrLf + retStr.ToString())
        File.WriteAllText("LogNormalIRLattice.csv", retStr.ToString())

    End Sub

    Public Shared Sub test_OAC()
        ' OAC - Option Adjusted Convexity - example coming soon
    End Sub

    Public Shared Sub test_OAD()
        ' OAC - Option Adjusted Duration - example coming soon
    End Sub

    Public Shared Sub test_OAS()
        ' OAC - Option Adjusted Spread - example coming soon
    End Sub

    Public Shared Sub test_PriceFromIRLattice()
        ' PriceFromIRLattice - example coming soon
    End Sub

    Public Shared Sub test_PriceFromZeroesTVF()
        ' PriceFromZeroesTVF - example coming soon
    End Sub

    Public Shared Sub test_ZSPREAD()
        ' ZSPREAD - Zero Volatility / Static spread - example coming soon
    End Sub


    Public Shared Sub test_XLDLL_FINANCIAL_VERSION()

        '=====================
        ' EXAMPLE# 1
        '=====================

        Dim result As String = wct.XLDLL_FINANCIAL_VERSION(wct.VersionInfoType.LicenseInformation)

        Console.WriteLine("XLDLL_FINANCIAL_VERSION : " & result)
        File.WriteAllText("XLDLL_FINANCIAL_VERSION.csv", result.ToString())

    End Sub

End Class


